/* ################# Хронология изменений ###################
 *  
 *  26.01.17   Поддержано хранения форматов строк вывода на экран во флеш-памяти, регулируется #define LCD_PROGMEM
 *  26.12.16   Обновление количества СТОПов происходит перед отправкой данных (см. предыдущий пункт)
 *             В методе RecvSerial3 поддержано удаление десятичных разделителей из полученной строки
 *  20.12.16   Поддержка отправки оперативных данных в момент изменения этапа процесса (метод SendOper)
 *  17.11.16   Поддержка обмена сообщениями с модулем ESP через Serial3
 *  23.08.16   В методе GetStateSerial время, прошедшее с момента старта, выводится в формате hh:mi:ss (для режима вывода сообщений в Serial port)
 *             Изменен номер клапана KLP_GLV_HVS=3, чтобы пользоваться одним клапаном
 *  22.08.16   Для процесса Ректификация при превышении температуры в ТСА возможно вернуться в предыдущее состояние при стабилизации температуры без перезагрузки
 *  21.08.16   Датчик нуля эмулируется по прерыванию таймера 3 (с частотой появления нуля в сети 50Гц)
 *  
*/


//#################################################################################################################
#define MQTT_BROKER
#ifdef MQTT_BROKER

unsigned char CurrentStage = 0;       // текущий этап процесса
unsigned long TimeStage    = 0;       // Время работы на текущем этапе
unsigned int CntStop       = 0;       // Счетчик СТОПОВ

unsigned long SecPrevBuf   = 0;       // время (Seconds) предыдущего обновления данных дисплея
unsigned long SecPrevOper  = 0;       // время (Seconds) предыдущего обновления оперативных данных
unsigned long SecCmdDisp   = 0;       // время (Seconds) прихода команды, приводящей к изменению информации на дисплее

bool FlagRefreshOnline     = false;   // признак необходимости обновления данных в online режиме
bool NeedRefresh           = false;   // признак необходимости обновления показаний дисплея

String RecvBufSerial       = "";      // буфер для приема сообщений от ESP

char lcd_mqtt_buf1[26];               // первая строка экрана для отправки по MQTT
char lcd_mqtt_buf2[26];               // вторая строка экрана для отправки по MQTT

char buf_pmem[30];                    // буфер для считывания данных из PROGMEM


// ########################################################
// # Описание форматов для передачи на ESP
// ########################################################

// оперативные данные
PROGMEM const char fmt_lcd1[]           =  "lcd1=%s";                 // первая строка дисплея
PROGMEM const char fmt_lcd2[]           =  "lcd2=%s";                 // вторая строка дисплея
PROGMEM const char fmt_klp[]            =  "klp=%d";
PROGMEM const char fmt_stage_0[]        =  "stage=НЕ ЗАПУЩЕН";        // этапы текущего процесса
PROGMEM const char fmt_stage_1[]        =  "stage=ЗАПУСК";
PROGMEM const char fmt_stage_2[]        =  "stage=РАЗГОН";
PROGMEM const char fmt_stage_3[]        =  "stage=НА СЕБЯ";
PROGMEM const char fmt_stage_4[]        =  "stage=ОТБОР ГОЛОВ";
PROGMEM const char fmt_stage_5[]        =  "stage=СТОП";
PROGMEM const char fmt_stage_6[]        =  "stage=ОТБОР ТЕЛА";
PROGMEM const char fmt_stage_7[]        =  "stage=ХВОСТЫ";
PROGMEM const char fmt_stage_8[]        =  "stage=ОЖИДАНИЕ";
PROGMEM const char fmt_stage_100[]      =  "stage=ЗАКОНЧЕН";
PROGMEM const char fmt_stage_101[]      =  "stage=АВАРИЯ";
PROGMEM const char fmt_stage_def[]      =  "stage=НЕИЗВЕСТНО";           // этапы текущего процесса
PROGMEM const char fmt_t_kub[]          =  "t_kub=%u.%u";                // температура в кубе
PROGMEM const char fmt_t_col[]          =  "t_col=%u.%u";                // температура в колонне
PROGMEM const char fmt_t_tsa[]          =  "t_tsa=%u.%u";                // температура в ТСА
PROGMEM const char fmt_work_time[]      =  "work_time=%02u:%02u:%02u";   // время текущего процесса
PROGMEM const char fmt_ProcShimSR[]     =  "ProcShimSR=%d";              // % ШИМ отбора отбора тела
PROGMEM const char fmt_tStabSR[]        =  "tStabSR=%u.%u";              // температура стабилизации колонны
PROGMEM const char fmt_CntStop[]        =  "CntStop=%u";                 // количество СТОПОВ
PROGMEM const char fmt_TimeStage[]      =  "TimeStage=%02u:%02u:%02u";   // время текущего этапа

// настройки
PROGMEM const char fmt_BeepKeyPress[]   =  "BeepKeyPress=%u";            // звук кнопок
PROGMEM const char fmt_BeepEndProc[]    =  "BeepEndProc=%u";             // звук окончания процесса
PROGMEM const char fmt_BeepStateProc[]  =  "BeepStateProc=%u";           // звук смены этапа
PROGMEM const char fmt_FlToUSART[]      =  "FlToUSART=%u";               // вывод в UART
PROGMEM const char fmt_tEndRectRazgon[] =  "tEndRectRazgon=%d.%u";       // температура окончания разгона
PROGMEM const char fmt_tEndRectOtbGlv[] =  "tEndRectOtbGlv=%u.%u";       // температура окончания отбора голов
PROGMEM const char fmt_tEndRectOtbSR[]  =  "tEndRectOtbSR=%u.%u";        // температура окончания отбора тела
PROGMEM const char fmt_tEndRect[]       =  "tEndRect=%u.%u";             // температура окончания ректификации
PROGMEM const char fmt_ShimGlv[]        =  "ShimGlv=%u";                 // ШИМ отбора голов
PROGMEM const char fmt_ProcShimGlv[]    =  "ProcShimGlv=%u";             // % ШИМ отбора голов
PROGMEM const char fmt_ShimSR[]         =  "ShimSR=%u";                  // ШИМ отбора тела
PROGMEM const char fmt_MinProcShimSR[]  =  "MinProcShimSR=%u";           // минимальный % ШИМ отбора тела
PROGMEM const char fmt_BegProcShimSR[]  =  "BegProcShimSR=%u";           // начальный % ШИМ отбора тела
PROGMEM const char fmt_tDeltaRect[]     =  "tDeltaRect=%u.%u";           // дельта ректификации
PROGMEM const char fmt_ProvodSR[]       =  "ProvodSR=%d";                // настройка окончания отбора голов

// ########################################################
// # Название изменяемых значений
// ########################################################
PROGMEM const char val_BeepKeyPress[]   =  "BeepKeyPress";            // звук кнопок
PROGMEM const char val_BeepEndProc[]    =  "BeepEndProc";             // звук окончания процесса
PROGMEM const char val_BeepStateProc[]  =  "BeepStateProc";           // звук смены этапа
PROGMEM const char val_FlToUSART[]      =  "FlToUSART";               // вывод в UART
PROGMEM const char val_tEndRectRazgon[] =  "tEndRectRazgon";          // температура окончания разгона
PROGMEM const char val_tEndRectOtbGlv[] =  "tEndRectOtbGlv";          // температура окончания отбора голов
PROGMEM const char val_tEndRectOtbSR[]  =  "tEndRectOtbSR";           // температура окончания отбора тела
PROGMEM const char val_tEndRect[]       =  "tEndRect";                // температура окончания ректификации
PROGMEM const char val_ShimGlv[]        =  "ShimGlv";                 // ШИМ отбора голов
PROGMEM const char val_ProcShimGlv[]    =  "ProcShimGlv";             // % ШИМ отбора голов
PROGMEM const char val_ShimSR[]         =  "ShimSR";                  // ШИМ отбора тела
PROGMEM const char val_MinProcShimSR[]  =  "MinProcShimSR";           // минимальный % ШИМ отбора тела
PROGMEM const char val_BegProcShimSR[]  =  "BegProcShimSR";           // начальный % ШИМ отбора тела
PROGMEM const char val_tDeltaRect[]     =  "tDeltaRect";              // дельта ректификации
PROGMEM const char val_ProvodSR[]       =  "ProvodSR";                // настройка окончания отбора голов
PROGMEM const char val_online[]         =  "online";                  // настройка обновления дисплея
PROGMEM const char val_save[]           =  "save";                    // сохранение в EEPROM
PROGMEM const char val_refresh[]        =  "refresh";                 // считывание значений настроек
PROGMEM const char val_key_d[]          =  "key_d";                   // кнопка ВНИЗ
PROGMEM const char val_key_u[]          =  "key_u";                   // кнопка ВВЕРХ
PROGMEM const char val_key_l[]          =  "key_l";                   // кнопка ВЛЕВО
PROGMEM const char val_key_r[]          =  "key_r";                   // кнопка ВПРАВО
PROGMEM const char val_key_s[]          =  "key_s";                   // кнопка ВЫБРАТЬ
PROGMEM const char val_IspReg[]         =  "IspReg";                  // текущий процесс
PROGMEM const char val_StateMachine[]   =  "StateMachine";            // этап процесса

#endif
//#################################################################################################################


//#########################################################
//# Описание форматов строк для вывода на экран
//#########################################################

#define LCD_PROGMEM  // Раскомментировать, если требуется хранить строки вывода на экран во Flash-памяти

#ifdef  LCD_PROGMEM

// Строки для процедуры Setup
PROGMEM const char lcd_setup_1[]         =  "debug=%u";
PROGMEM const char lcd_setup_2[]         =  "ds18B20Count=%u";
PROGMEM const char lcd_setup_3[]         =  "C,V,P =%i,%i,%i(%1i) ";
PROGMEM const char lcd_setup_4[]         =  "%s  RWR=%i";
PROGMEM const char lcd_setup_5[]         =  "Simpled=%u ";
PROGMEM const char lcd_setup_6[]         =  "Simpled=%u Slave";

// Строки для процедуры formTSAErr
PROGMEM const char lcd_TSAErr_1[]        =  "%02u:%02u:%02u TSA!";
PROGMEM const char lcd_TSAErr_2[]        =  "tT=%3i/%3i";

// Строки для процедуры DisplayRectif
PROGMEM const char lcd_rect_case_1_1[]   =  "%02u:%02u:%02u Rect";
PROGMEM const char lcd_rect_case_2_1[]   =  "%02u:%02u:%02u R RZG";
PROGMEM const char lcd_rect_case_2_2[]   =  "tK=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_2_3[]   =  "tN=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_3_1[]   =  "%02u:%02u:%02u R NSB";
PROGMEM const char lcd_rect_case_3_2[]   =  "tN=%3i(%3i)%4iW";
PROGMEM const char lcd_rect_case_4_1[]   =  "%02u:%02u:%02u R GLV";
PROGMEM const char lcd_rect_case_4_2[]   =  "tK=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_4_3[]   =  "Pr=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_4_4[]   =  "Pr=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_4_5[]   =  "Tm=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_4_6[]   =  "%02u:%02u:%02u RF GLV";
PROGMEM const char lcd_rect_case_4_7[]   =  "tK=%3i/%3i %4iW";
PROGMEM const char lcd_rect_case_4_8[]   =  "t=%5imin %4iW";
PROGMEM const char lcd_rect_case_5_1[]   =  "%02u:%02u:%02u R Stop";
PROGMEM const char lcd_rect_case_5_2[]   =  "tN=%3i/%3i %2i%%";
PROGMEM const char lcd_rect_case_6_1[]   =  "%02u:%02u:%02u RSt %2i%%";
PROGMEM const char lcd_rect_case_6_2[]   =  "tN=%3i/%3i/d%2i";
PROGMEM const char lcd_rect_case_7_1[]   =  "%02u:%02u:%02u R Hvost";
PROGMEM const char lcd_rect_case_7_2[]   =  "tK=%3i/%3i";
PROGMEM const char lcd_rect_case_8_1[]   =  "%02u:%02u:%02u R Wait";
PROGMEM const char lcd_rect_case_8_2[]   =  "tK=%3i/%3i";
PROGMEM const char lcd_rect_case_9_1[]   =  "%02u:%02u:%02u R Wait";
PROGMEM const char lcd_rect_case_9_2[]   =  "by Urov=%3i/%3i";
PROGMEM const char lcd_rect_case_100_1[] =  "%02u:%02u:%02u Rec End";
PROGMEM const char lcd_rect_case_100_2[] =  "tK=%3i/%3i End";
PROGMEM const char lcd_rect_case_102_1[] =  "%02u:%02u:%02u Pres";
PROGMEM const char lcd_rect_case_102_2[] =  "MPX5010=%3i/%3i";
PROGMEM const char lcd_rect_1[]          =  "D2 T=%3u,%3u,%3u";
PROGMEM const char lcd_rect_2[]          =  "%3immV=%4u,%4u";
PROGMEM const char lcd_rect_3[]          =  "%3imm t=%3i F%1i/%1i";
PROGMEM const char lcd_rect_4[]          =  "%3imm %4im F%1i/%1i";
PROGMEM const char lcd_rect_5[]          =  "%3immV=%4u F%1i/%1i";
PROGMEM const char lcd_rect_6[]          =  "%3imm reSt=%5i";

// Строки для процедуры DisplayRazvar
PROGMEM const char lcd_razv_1[]          =  "%02u:%02u:%02u HLD Ztr";
PROGMEM const char lcd_razv_2[]          =  "%02u:%02u:%02u Razvar";
PROGMEM const char lcd_razv_case_1_1[]   =  "t=%3i/%3i %4iW";
PROGMEM const char lcd_razv_case_2_1[]   =  "t=%3i ZASYP & Dn";
PROGMEM const char lcd_razv_case_3_1[]   =  "t=%3i/%3i %4iW";
PROGMEM const char lcd_razv_case_4_1[]   =  "t=%3i %4iW %4i";
PROGMEM const char lcd_razv_case_5_1[]   =  "tK=%3i/%3i %4iW";
PROGMEM const char lcd_razv_case_5_2[]   =  "tD=%3i/%3i %4iW";
PROGMEM const char lcd_razv_case_6_1[]   =  "t=%3i %4iW %4i";
PROGMEM const char lcd_razv_case_7_1[]   =  "tK=%3i/%3i HLD-1";
PROGMEM const char lcd_razv_case_8_1[]   =  "t=%3i SOLOD & Dn";
PROGMEM const char lcd_razv_case_9_1[]   =  "tK=%3i MIX %4i";
PROGMEM const char lcd_razv_case_10_1[]  =  "tK=%3i OSH %4i";
PROGMEM const char lcd_razv_case_11_1[]  =  "tK=%3i/400 HLD-2";
PROGMEM const char lcd_razv_case_12_1[]  =  "tK=%3i/%3i HLD-3";
PROGMEM const char lcd_razv_case_13_1[]  =  "tK=%3i/%3i BROG";
PROGMEM const char lcd_razv_case_14_1[]  =  "tK=%3i/%3i BR-H";
PROGMEM const char lcd_razv_case_100_1[] =  "t=%3i end";
PROGMEM const char lcd_razv_3[]          =  "D2 T=%3u,%3u,%3u";
PROGMEM const char lcd_razv_4[]          =  "OSH=%3u NPG=%4u";

// Строки для процедуры DisplayNBK
PROGMEM const char lcd_nbk_case_0_1[]    =  "%02u:%02u:%02u NBK";
PROGMEM const char lcd_nbk_case_0_2[]    =  "tK=%3i,%3imm";
PROGMEM const char lcd_nbk_case_2_1[]    =  "%02u:%02u:%02u NBK Rzg";
PROGMEM const char lcd_nbk_case_3_1[]    =  "%02u:%02u:%02u NBK Beg";
PROGMEM const char lcd_nbk_case_3_2[]    =  "UP to Beg %3imm";
PROGMEM const char lcd_nbk_case_6_1[]    =  "%02u:%02u:%02u NBK Wrk";
PROGMEM const char lcd_nbk_case_6_2[]    =  "Sp=%4i,%3imm";
PROGMEM const char lcd_nbk_case_5_1[]    =  "%02u:%02u:%02u NBK tmp";
PROGMEM const char lcd_nbk_case_5_2[]    =  "tD=%3i/970,s%4i";
PROGMEM const char lcd_nbk_case_100_1[]  =  "%02u:%02u:%02u NBK End";
PROGMEM const char lcd_nbk_case_100_2[]  =  "Sp=%4i,%3imm";
PROGMEM const char lcd_nbk_1[]           =  "D2 T=%3u,%3u,%3u";
PROGMEM const char lcd_nbk_2[]           =  "%4uW,%3imm %3i";

// Строки для процедуры DisplayBeer
PROGMEM const char lcd_beer_1[]           =  "%02u:%02u:%02u beer";
PROGMEM const char lcd_beer_2[]           =  "T=%3i/%3i P%2i/%2i";
PROGMEM const char lcd_beer_3[]           =  "Ts=%5i P=%2i/%2i";
PROGMEM const char lcd_beer_4[]           =  "Press UP to%2i/%2i";
PROGMEM const char lcd_beer_5[]           =  "end";
PROGMEM const char lcd_beer_6[]           =  "D2 T=%3u,%3u,%3u";
PROGMEM const char lcd_beer_7[]           =  "mix=%5i %4iW";

// Строки для процедуры formTRZGDistill
PROGMEM const char lcd_TRZGDistill_1[]    =  "tD=%3i/%3i %4iW";
PROGMEM const char lcd_TRZGDistill_2[]    =  "tK=%3i/%3i %4iW";

// Строки для процедуры DisplayDistDefl
PROGMEM const char lcd_DDefl_case_1_1[]   =  "%02u:%02u:%02u DD";
PROGMEM const char lcd_DDefl_case_2_1[]   =  "%02u:%02u:%02u DD Rzg";
PROGMEM const char lcd_DDefl_case_2_2[]   =  "tD=%3i/%3i %4iW";
PROGMEM const char lcd_DDefl_case_3_1[]   =  "%02u:%02u:%02u DD Rzg";
PROGMEM const char lcd_DDefl_case_3_2[]   =  "tK=%3i/%3i %4iW";
PROGMEM const char lcd_DDefl_case_4_1[]   =  "%02u:%02u:%02u DD BezD";
PROGMEM const char lcd_DDefl_case_4_2[]   =  "tD=%3i/%3i %4iW";
PROGMEM const char lcd_DDefl_case_5_1[]   =  "%02u:%02u:%02u DD 50%%";
PROGMEM const char lcd_DDefl_case_5_2[]   =  "tD=%3i/%3i %4iW";
PROGMEM const char lcd_DDefl_case_6_1[]   =  "%02u:%02u:%02u DD 100%%";
PROGMEM const char lcd_DDefl_case_6_2[]   =  "tD=%3i/%3i %4iW";
PROGMEM const char lcd_DDefl_case_7_1[]   =  "%02u:%02u:%02u DD Wait";
PROGMEM const char lcd_DDefl_case_7_2[]   =  "tK=%3i Wait=%4i";
PROGMEM const char lcd_DDefl_case_100_1[] =  "%02u:%02u:%02u DD end";
PROGMEM const char lcd_DDefl_case_100_2[] =  "tK=%3i end";
PROGMEM const char lcd_DDefl_1[]          =  "D2 T=%3u,%3u,%3u";
PROGMEM const char lcd_DDefl_2[]          =  "V%4u,%4u-%4iW";

// Строки для процедуры DisplaySimpleDistill
PROGMEM const char lcd_SDist_1[]          =  "%02u:%02u:%02u %1u-Otbor";
PROGMEM const char lcd_SDist_2[]          =  "%02u:%02u:%02u sGl";
PROGMEM const char lcd_SDist_case_2_1[]   =  "%02u:%02u:%02u %1u-O Rzg";
PROGMEM const char lcd_SDist_case_2_2[]   =  "%02u:%02u:%02u sGl Rzg";

#endif


// Считывание данных из PROGMEM
char* GetPMEM(
  char* ret_fmt,  // указатель на буфер для возвращаемого значения
  char* pmem_str  // указатель на строку (по имени переменной) из PROGMEM
)
{

int i;
int len;
  
  len = strlen_P(pmem_str);
  
  for (i=0; i<len; i++)
    ret_fmt[i] =  pgm_read_byte(pmem_str + i);
  
  ret_fmt[len] = '\0';

return ret_fmt;
}


// контроль нуля идет на 1-е прерывание (это pin 3)
// Управление симистором для фазового регулятора мощности реализовано через PIN 36
// АЦП от трансформатора подключен на pin А2 (раньше шел на pin A3)
// Датчики ds18b20 сидят на 3ь7 пине. (потому что на пины с 7 по 13 я подключил ULM2003, из-за желания использовать встроенный ШИМ в Ардуино)


// Кнопки подключены на следующие пины:
// почему на этих - да потому, что они рядом идут, и еще пин земли рядом, а гребенку 6 онтактов проще паять и подключать, чем каждый контакт одтельно
// PIN_SELECT 53
// PIN_RIGHT 51
// PIN_LEFT 49
// PIN_UP 45
// PIN_DOWN 47

// Уровень НПГ подключен по следующей схеме: Подтяджка - 10 ком, защита -330 ом, замыкание - 30, резистор 1.1К (верхниц уровень) - 128, 2.2К (рабочий уровень) - 256, 3,3К (нижний уровень)-266

// Почему-то иногда глючит дисплей - когда это происходит (на нем появляются иероглифы или перестает отображаться текст),
// нажать и держать одновременно клавишу влево и впрёаво, либо клавишей вправо перейти на 6-ю страницу, дисплей переинициализируется.
// одновременное нажатие кнопок вверх и вних позволчет перейти к следующему этапу процесса ректификации , также можно сделать это на 8 странице кнопками вверх и вниз однако это не рекомендуется работа не гаранитируется.

// Клапана для управления ШИМ подключены, начиная с PIN 22
// Всего максимум 5 клапанов, то есть на пины с 22 по 26 реализован
// программный ШИМ по количество полупериодов с контролем нуля.


// Дисплей на следующих пинах - почему на этих - да потому, что они рядом идут, и еще пин земли рядом, как раз для вывода R/W дисплея
// а гребенку из 7 контактов проще паять и подключать, чем каждый контакт одтельно
// LiquidCrystal lcd(52,50, 48, 46, 44, 42);

//#define TEST   // Раскомметировать, если нужно получать всяческие тестовые значения через Serial1
//#define TESTMEM   // Раскомметировать, если нужно получать всяческие тестовые значения через Serial1
//#define TESTGSM   // Раскомметировать, если нужно дублировать входящюю информацияю из serial2 (сотового телефона) в Serial1 (для отладки)//.
//#define TESTGSM1   // Раскомметировать, если нужно дублировать входящюю информацияю из serial2 (сотового телефона) в Serial1 (для отладки)
//#define TESTRM   // Раскомметировать, если нужно получать тестовые значения регулятора мощности через Serial1
//#define TESTERR   // Раскомметировать, если нужно получать тестовые значения ошибок через Serial1

//#define DEBUG  // Режим отладки, в этом режиме не считываются значения датчиков температуры, а они передаются через монитор порта в ардуине
// в формате: сколько прибавить секунд, Температура датчика 0, Температура датчика 1,Температура датчика 2
// например: 60,820,810,800
//  (монитор порта надо настроить так, чтобы он выдавал возврат каретки и перевод строки) 
#define PR_REWRITE_EEPROM 9  // Константа, которая содержит признак необходимости перезаписи энергонезависимой памяти (1-254). 
// При запуске программы, значение 0-го байта ЕЕПРОМ сравнивается с этим знначением, 
// и если они не совпадают, тогда энергонезависимая памиять переписывается текущими значениями переменных
// То есть для значений переменных из скетча в контроллер, ее значение надо поменять например с 9 до 10.
#define NUM_PHASE 0
unsigned char WiFiAP[12]="A2345678901";
unsigned char WiFiPass[12]="12345678901";
char my_phone[13]="+77777777777";
const char my_version[]="v3.41";
unsigned char idDevice[11]="0123456789";
unsigned char ip[4]={0,0,0,0};
unsigned int ipPort=20123;
char CorrectASC712=0;      // Признак того, как надо использовать датчик asc712 
                         // (0- не надо, 1 - только показывать мощность, 2-корректировать мощность через фактический ток, 3- ПИД регулирование через ток)

#define SIMPLED_VERSION 0       // Признак того, насколько упрощена версия
                                // 0 - стандартная
                                // 1 - с твердотельным реле в качестве регулятора мощности по брезинхему, с датчиком ASC712 30A в качестве датчика напряжения в сети
                                // с двумя релейным модуля для управления клапанами отбором голов и хвостов и товарного спирта.
                                // LCD keypad shield в качестве кнопок и экрана.
                                // 2 - то же, что и 1, но с цифровыми датчиками уровня спирта, голов, разлития жидкости
                                // 3 - то же, что и 2, но только с одним датчиком уровня спирта без датчика голов и разлития жидкости.
                                // 4 - то же что и 3, но с датчиком напряжения на трансформаторе.
                                // 5 - это почти нулевая версия, но разгон и двигатель управляются реле с низким уровнем и используется LCD keypad shield в качестве дисплея и кнопок.
                                // 6-  то же что и 5 но с поддержкой НПГ, с расчетом фактической мощности через датчик тока..
                                // 7 - slave конороллер, управляемый через внешние пины.
                                // 8 - То же что и 2, но с аналоговыми датчиками уровня спирта, голов, разлития жидкости.
                                // 9 - То же что и 5, но с расчетом фактической мощности через датчик тока.
                                // 20 - эксперимент с трехфазным нестабилизированным контроллером.
                                // 30 - то же что и 5 но с измененными пинами
                                        

#define KLP_HIGH 1              // Для клапанов с низким уровнем управления поменять 0 на 1

#define USE_LCD_KEYPAD_SHIELD 1  // 1 - использовать стандартный шильд с кнопкми ардуино, 0 - не использовать
//#define USE_PROVODIMOST_SR    2 // (теперь за это отвечает переменная UrovenProvodimostSR) 1 - использовать датчик проводимости спирта для контроля окончания голов, 0 - не использовать,
// 2 - использовать датчик проводимости как индикатор наполнения приемной емкости с головами.

int UrovenProvodimostSR=0;   // Уровень проводимости, меньше которого считаем, что закончились головы и начался спирт.
int UrovenBarda=250;        // Уровень сигнала, достижение котогого свидетельсвует о срабатывании датчика бардоотводчика.

#define PIN_PROVODIMOST_SR A7        // Пин для анализа проводимости спирта (аналоговый пин А7)
#define MAX_COUNT_PROVODIMOST_SR 6  // Количество срабатываний датчика, по достижении которых можно точно сказать, что головы закончились
// Каждое значение - это 5 секунд, то есть в данном случае датчик должен показывать 30 секунд проводимость менее 20


#define UROVEN_ALARM 50        // Уровень сигнала, достижение котогого свидетельсвует о срабатывании аналогового датчика (в обычном состоянии он выведен на значение около 1000 для датчиков на уменьшение напряжения)
                               // или на значение около 0-10 для датчиков на увеличение напряжения 
#define COUNT_ALARM 6          // Сколько должно держаться значение уровня, чтобы сработало предупреждение каждое значение - это полсекунды.
#define COUNT_ALARM_VODA 60    // Сколько должно держаться значение разлития воды, чтобы сработала тревога каждое значение - это полсекунды.
#define COUNT_ALARM_UROVEN 200 // Сколько должно держаться значение уровня, чтобы сработала тревога по уровню в приемной емкости каждое значение - это полсекунды.
#define COUNT_ALARM_UROVEN_FR 60 // Сколько должно держаться значение уровня, чтобы сработала тревога по уровню в приемной емкости при фракционной перегонке каждое значение - это полсекунды.

#ifdef DEBUG
#define COUNT_ALARM_UROVEN 20 // Сколько должно держаться значение уровня, чтобы сработала тревога по уровню в приемной емкости каждое значение - это полсекунды.
#endif

#define PIN_GAS_SENSOR A8        // Пин для анализа датчика загазованности (спирта)
#define USE_GAS_SENSOR 0        // Надо ли использовать датчик загазованности
#define UROVEN_GAS_SENSOR 700    // Уровень сигнала, достижение котогого свидетельсвует о срабатывании датчика спиртового пара
#define COUNT_GAS_SENSOR 6          // Сколько должно держаться значение уровня, чтобы сработала тревога каждое значение - это полсекунды.

#define TIME_PROGREV_GAS_SENSOR 60    // Время для програва датчика спиртовых паров газа.


#define USE_ALARM_UROVEN 0  // Нужно ли использовать датчик уровня в приемной емкости ардуино 1- использовать для остановки всег процесса
                            // 2 - датчик используется как уровень при котором останавливается отбор, но не останавливается ректификация
#define USE_ALARM_VODA   0  // Нужно ли использовать датчик разлития воды ардуино

//#define NPG_UROVEN_PIN A3         // Пин уровня НПГ. (закомментировать, если НПГ не используется во избежании ложных сработок)
#if NUM_PHASE==1
#define NPG_UROVEN_PIN_MIN A8         // Пин уровня НПГ. (закомментировать, если НПГ не используется во избежании ложных сработок)
#define NPG_UROVEN_PIN_WORK A9         // Пин уровня НПГ. (закомментировать, если НПГ не используется во избежании ложных сработок)
#endif


#define MAX_COUNT_NPG_ALARM 60     // Количество сработок осушения или переполнения, чтобы достоверно детектировать состояние НПГ (каждое значение -полсекунды)

#define PIN_MPX5010 A9      // Пин датчика давления MPX5010DP

#define PIN_DVIGATEL 28 // Пин управления двигателем в фазовом режиме (не исползуется).

#define PIN_NASOS_NBK 12  // Пин управления насосом НБК через ШИМ (двигатель не шаговый, подключать полевик, например IRL2703 или транзисторный ключ или драйвер полевика).

#define PIN_NASOS_NBK_BRD 44  // Пин управления насосом бардоотводчика НБК через ШИМ (двигатель не шаговый, подключать полевик, например IRL2703 или транзисторный ключ или драйвер полевика).

unsigned char zPS=0; // Число прерываний нуля за секунду.
unsigned char zPSOut=0; // Число прерываний нуля за секунду для вывода на экран.
#define PIN_LIGHT 10
unsigned char PeriodRefrServer=30;

int TimeStabKolonna=900; // Время стабилизации колонны (положительное значение - время от последнего изменения температуры, отрицательное - абсолютное время).

#define MAX_TABLE_T 200

unsigned char tableS10[MAX_TABLE_T]; //табличное значение времени задежки открытия триака

#define MAX_TABLE_SQRT 26
unsigned char tableSQRT[MAX_TABLE_SQRT];  //Значение корня
unsigned int tableSQ[MAX_TABLE_SQRT];     //Значение квадрата
int TekTemp=0; //Текущая температура для отображения


int timeNBK=0;   // Время работы насоса НБК в секундах со скоростью SpeedNKB. По истечении этого времени насос остановится
int MaxPressByPeriod=0;
char minPressNBK=160;
char deltaPressNBK=25;
char timePressNBK=180;
char UprNasosNBK=1;

#define PIN_MIXER     32  // Пин управления мешалокой 

int timeMIXER=0; // Время работы двигателя миксера в секундах. По истечении этого времени мешалка остановится

int time1=0;   // Таймер для отсчета секунд 1
int time2=0;   // Таймер для отсчета секунд 2
int time3=0;   // Таймер для отсчета секунд 3
unsigned char timeBRD=0; // Таймер для отсчета времени открытия бардо-отводчика.
unsigned char timeGPRS=0;   // Таймер для отсчета GPRS соединения
unsigned char timeWaitGPRS=0;   // Таймер для отсчета ответа соединения
unsigned char timeRefrServer=0;   // Таймер для отсчета ответа соединения
char flAutoDetectPowerTEN=0;
char flNeedRefrServer=0;// Флаг того, что надо обновить данные на сервере.
char ErrGPRSConnectInernet=0;
char ErrGPRSConnectServer=0;
char flNeedCall=0; // Состояние необходимости звонка по тревоге
                   // 0 - звонить не нужно
                   // 1 - надо позвонить
                   // 2 - дозваниваемся
                   // 3 - звонили, получили ответ  

char cmdGPRS=' ';// Посланная команда от сервера
char cmdGPRSIsp=' ';// Выполненная команда от сервера
unsigned char flGPRSState=0;    // Флаг состояния обработки команд.
char stateAfterDelete=0; // Семантика обработки команд после некотрых завершающих состояний.

#define UDP_PROTOKOL  // Призанк использования UDP протокола для обмена

int U_MPX5010=0;       // Текущее значение давления (в 10 мм р.ст, то есть 100, это 10 мм рст).
int P_MPX5010=0;       // Поправка для датчика 
int AlarmMPX5010=0;   // Давление при котором надо выдавать сингал тревоги.

char countAlrmNPG=0;  // Количество сработок тревоги НПГ.

#define USE_DIFAVTOMAT 1       // Константа, которая показывает, используется ли дифавтомат в работе системы нужно для того, чтобы при нормальном завершении процесса
// при плановом отключении дифавтомата не выдавалась тревога.

#define PEREGREV_ON 0  // Того, что надо использовать режим работы клапана в режиме защиты от перегрева. 1- использовать, 0-нет.

unsigned int NaprPeregrev=300;  // Какое среднеквадратичное надо поддерживать для защиты клапанов от перегрева.

#if PEREGREV_ON==0
#define PER_KLP_OPEN  1 // клапана открываем через полу-период (аналог диода), чтобы не перегревались и
#define PER_KLP_CLOSE 1 // чтобы не быо гидроударов
#else
#define PER_KLP_OPEN  1000 // клапана на воду переводим в фазовое управление, чтобы раз в 10 секунд на них подавалось полное напряжение, а затем напряжение
#define PER_KLP_CLOSE 0    // из контанты NAPR_PEREGREV 150 
#endif

#define PIN_READU A2         // Аналоговый Пин для чтения сетевого напряжения (тока) 
#define PIN_READI A4         // Аналоговый Пин для чтения тока 

#define SENSITIVE_ASC712 135 // Чувствительность датчика тока (показаний АЦП ардуино на 10 ампер тока 135 для 30А датчика, 205 для 20А датчика)
#define USE_ASC712 0           // Надо ли исполозовать датчик тока
#define PIN_DS18B20 37
#define RELAY_HIGH 1         // Какой сигнал подавать на релейные выходы мешалки и разгона 

#define ALL_OFF_HIGH 1         // Какой сигнал подавать на релейные выходы мешалки и разгона 

#define PIN_TRIAC  36    // Управление симистором реализовано через PIN 36

char flCorrASC712=0; // Флаг того, что надо расчитывать в данный момент среднеквадратичную мощность 1-мощность, 2-напряжение.

#if SIMPLED_VERSION==0
// Константа определяющая датчик потока воды.
#endif


//unsigned int Sec5;      // 1/5 секунды

// Номер телефона
// Номер телефона
const char sms_start[]="0410"; // SMS для старта (рестарта процесса), это большая русская буква А в формате UNICODE 
// (обязательно надо использовать русские буквы, чтобы SMS гарантированно пришел в формате UNICODE, другие программа 
// анализировать не может, точнее может, но надо заранее тогда знать, что придет последними 4 байтами в сообщении.
const char sms_stop[]="0411"; // SMS для остановки процесса, это большая русская буква Б в формате UNICODE
const char call_sms[]="0412"; // SMS для получения состояния в виде SMS  в любом случае, даже если включена отправка на сервер (большая русская буква В в UNICODE).
const char call_gprs[]="0413"; // SMS для отключения получения состояния в виде SMS  при звонке и перевода на получения состояния через GPRS (большая русская буква Г в UNICODE).


char lastSMSState=0; // Последнее отправленное состояние по SMS (Чтобы не отправлять его многократно)
char SMSOnCall=0;
char pdu_phone[14];  // Номер телефона в формате отправки SMS (заполняется автоматически)
unsigned char CntErrDs18=0;   // Количество глюков, зафиксированных от датчика 
int FindKt=0;         // Нужный процент открытия симистора (если отрицательный, значит 
char CntErrPower=0;   // Количество глюков, зафиксированных от датчика 
char NumErrDs18=0;   // Номер сглючившего датчика
char NumErr=0;       // Код ошибки 1 - глючит датчик ds18b20

unsigned char SpeedNBK=0;
#define MAX_ERR_DS18 120    // После тридцати глюков подряд (а это тридцать секунд) от датчиков температуры, считаем, что процесс надо остановить.
#define MAX_ERR_MPX5010 120 // После 120 значений датчика давления подряд (а это 60 секунд) от датчиков температуры, считаем, что процесс надо остановить.
char CountAlarmMPX5010=0;   // Количество глюков, зафиксированных от датчика 

char timeOpenBRD=2; // Скорость порта, деленная на 1200

//#define STAB_RM 1  // Поставить 1, если программе пытаться стабилизировать регулятор мощности, путем запрета вывода информации на дисплей в момент работы, чтения температуры и т.д.
// Поставить 0, если идет отладка программы без подключенных датчиков, контроллера детектора нуля, схемы измерения текущего напряжения в сети. 

#include <LiquidCrystal.h>

#include <OneWire.h>

#include <EEPROM.h> 


//(раскомментировать если используется тест свободной памяти. 
/*
#ifdef TESTMEM 
int freeRam () 
{
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}
#endif
*/

unsigned int ee_addr; // Переменная для хранения текущего адреса чтения/записи в EEPROM
unsigned char eeReadChar()
{
  unsigned char lByte = EEPROM.read(ee_addr);
  ee_addr++;
  return lByte;
}

void eeWriteChar(char p_value)
{
  if (EEPROM.read(ee_addr)!=p_value)
    EEPROM.write(ee_addr, p_value);
  ee_addr++;
}

void eeWriteInt(int p_value)
{
  unsigned char lByte = (unsigned char) (p_value);
  unsigned char hByte =(unsigned char) (p_value >> 8);

  // Для экономии ресурса флеш памяти, сначала проверяем ее содержимое и запись производим только если значение отличается.
  eeWriteChar(lByte);
  eeWriteChar(hByte);
}

//This function will read a 2 byte integer from the eeprom at the specified address and address + 1
unsigned int eeReadInt()
{
  unsigned char lByte = eeReadChar();
  unsigned char hByte = eeReadChar();
  return (unsigned int) (lByte) | (unsigned int)  (hByte << 8);
}


#define MAX_INDEX_INPUT 540

#define MAX_INDEX_BY_PERIOD 90
#define MIN_INDEX_BY_PERIOD 75

#define CNT_PERIOD 4  // Кооличество полу-периодов для обсчета среднеквадратичного

unsigned char ErrCountIndex=0;

#define BEEP_LONG 20   //Длительность сигнала оповещения о состояниях процесса

//int data_adc[MAX_INDEX_INPUT+1];
//#ifdef TESTRM
//int tic_adc[MAX_INDEX_INPUT+1];
//#endif

char lcd_buffer[33]; //Буфер для хранения данных для вывода на дисплей
char CntPeriod=0; //Номер обсчитываемого полу-периода
#define MY_RX_BUFFER_SIZE 70
#define MY_TX_BUFFER_SIZE 70
char my_tx_buffer[MY_TX_BUFFER_SIZE];
char my_rx_buffer[MY_RX_BUFFER_SIZE];
char my_gprs_buffer[MY_TX_BUFFER_SIZE];
char pos_rx_buffer=0;


unsigned long SqNaprT=0,SqNaprPrevT=0; //  Переменные для накапливания квадратов напряжений.

int NewErr;// Новая ошибка рассогласования
int OldErrOut;// Новая ошибка рассогласования
int OldErr;// Старая ошибка рассогласования
int It;     // Интегральная составляющая (без коэффициентов, чтобы избавиться от дробей, коэффициенты учтем в момент ее применения);
int Dt;     // Дифференциальная составляющая (без коэффициентов, чтобы избавиться от дробей, коэффициенты учтем в момент ее применения);

int NewErrTemp;// Новая ошибка рассогласования температуры
int OldErrTemp;// Старая ошибка рассогласования температуры
int OldErrTempOut;// Старая ошибка рассогласования температуры
int ItTemp;     // Интегральная составляющая температуры (без коэффициентов, чтобы избавиться от дробей, коэффициенты учтем в момент ее применения);
int DtTemp;     // Дифференциальная составляющая температуры (без коэффициентов, чтобы избавиться от дробей, коэффициенты учтем в момент ее применения);

int KtT;        // Коэффициент мощности ТЭНа в 1000 долей.

char PIDTemp[3]={20,10,30};    // Пропорциональный коэффициент пид-регулятора температуры [0]
                            // Интегральный коэффициент пид-регулятора температуры [1]
                            // Дифференциальный коэффициент пид-регулятора температуры [2]


int  FlState=0;  // Флаг состоянияч конечного автомата обработки нажатия кнопок клавиатуры 
unsigned char  IspReg=101; // Используемый в данный момент режим (по умолчанию пока ставим регулятор мощности)
char  DispPage=0; // Какую страницу отображать на дисплее
int TempTerm=200;
int Delta=5;
char FlToGSM=0;        // Признак того, что сотовый используется для вывода информации
char FlToUSART=0;      // Признак того, что CОМ-порт используется для вывода информации
char FlAvtonom=0;      // Признак того, что CОМ-порт используется для вывода информации
char BeepEndProcess=1; // Пищать динамиком по окончании процесса.
char BeepStateProcess=1; // Пищать динамиком при переходе между процессами.
char BeepKeyPress=1; // Пищать динамиком при нажатии на клавиши
int Temp1P=992;        // Температура первой (недробной) перегонки 
int Temp2P=964;        // Температура второй  перегонки  (дистилляция без дефлегматора)
int Temp3P=954;        // Температура третьей перегонки  (дистилляция без дефлегматора)
int TempKipenZator=800;// Температура начала закипания затора (либо это температура в верхней части заторника (если два термометра, либо в заторнике, если стоит один термометр)
char TempZSP=50;// Температура при которой можно засыпать несоложенку.
char TempZSPSld=63;// Температура при которой можно засыпать солод.
char TempHLDZatorBrog1=36;// Температура брожения
int PowerGlvDistil=800; // Мощность отбора голов при простой дистилляции
int PowerDistil=3000; // Мощность дистилляции
int PowerNBK=2400; // Мощность дистилляции
unsigned char SpeedNBKDst=0; // Скорость насоса НБК
int PowerRazvZerno=820; // Мощность, при которой будут развариваться зерновые (2 часа).
int PowerVarkaZerno=3000; // Мощность, при которой будут варится зерновые и доводиться до кипения (она меньше, чем мощность ТЭНов, потому что на полной мощности может быть факелирование).

int TempDeflBegDistil=700; //Температура в дефлегматоре, для начала подачи воды в холодильник.
int TempDefl=820; //Температура в дефлегматоре, которую надо поддерживать путем включения клапана дефлегматора
unsigned char DeltaDefl=20; //дельта Температуры в дефлегматоре, которую надо поддерживать путем включения клапана дефлегматора
int tEndDistDefl=985;       // Температура в кубе, при которой надо прекращать дистилляцию с дефлегматором 
int timerMinute=0;          // Время для таймера
int PowerMinute=0;          // Установленная мощность
int tEndRectRazgon=830;     // Температура окончания режима разгона 83.0 С +(температура в кубе) - (температура в колонне)
int PowerRect=1000;     // Мощность ректификации
int tStabSR;// Температура, относительно которой стабилизируется отбор СР
char ProcChimSR;// Текущий процент отбора СР
long SecTempPrev;// Знчение секудндомера, при котором было считана эта температура.
long SecTempPrev1;// Знчение секудндомера, при котором было считана эта температура.
long SecOstatok;// Знчение секудндомера, сколько еще осталось до стабилизации
int TempPrev=0;// Значение предыдущей температуры (для сверки с текущей (проверка стабилизации значение 344 для облегчения отладки)

int tEndRectOtbGlv=854;     // Температура окончания отбора голов 85.4 С
unsigned int timeChimRectOtbGlv=2000;  // Шим отбора голов (в полупериодах сетевого напряжения в данном случае это около 20 секунд)
char ProcChimOtbGlv=5;  // Процент ШИМ отбора голов (в данном случае это 5 процентов)
unsigned int timeChimRectOtbSR=1000;  // Шим отбора голов (в полупериодах сетевого напряжения в данном случае это около 10 секунд)
unsigned char tDeltaRect=3;  // Дельта ректификации (в десятых градуса Цельсия) в данном случае - это 10 десятых, то есть 1 градус Цельсия
unsigned int tEndRectOtbSR=965;  // Температура окончания отбора спирта-ректификата и начала отбора хвостов (в данном случе 96.5 С)
unsigned int tEndRect=995;       // Температура в кубе, при которой прекращается ректификация (в данном случе 99.5 С)
unsigned char minProcChimOtbSR=20;  // Минимальный ШИМ, за который нельзя выходить при отборе СР
unsigned char begProcChimOtbSR=40;  // Начальный ШИМ отбора СР
char ProcChimOtbCP=0;  // Процент ШИМ отбора голов при работе с царгой пастеризации положительные значения-отбор относительно отбора голов, отрицательные - абсолютное значение

char StateMachine=0; // Состояние конечного автомата процесса 
char flNeedReadAlarm;// Флаг того, что нужно анализировать состояние датчиков НПГ, Уровня в приемной емкости, Разлития по полу.
char flAlarmUroven=0; // Флаг того, что уровень в приемной емкости достик предела
char flAlarmMPX5010=0; // Флаг того, что уровень в приемной емкости достик предела
char flNoAlarmLowPower=0; // Флаг того, что не надо предупреждать о низком напряжении в сети (оно в общем-то важно только для ректификации и НБК).


char FlUsart=0; // Признак того, что надо вывести информацию на СОМ-порт
char flRing=0; // Признак наличия звонка от своего номера (запрос на СМС о состоянии процесса);

char flNeedAnalyse=0; // Флаг того, что надо анализировать результат измерения температуры
//char flStateProcess=0;
char flNeedScanKbd=0; // Флаг того, что надо читать состояние клавиш 
char flScanKbd; // Признак того, что в данный момент идет сканирование клавиатуры, состояние не менять.
char CountKeys;       // Переменная для защиты от дребезга
char secPressKeys;    // Признак того, сколько секунд нажата клавиша
unsigned char AddPressKeys=1;    // Признак того, сколько прибавлять значений, в зависимости от длительности нажатия клавиши.
unsigned char AddPressKeys10=10;   // Признак того, сколько прибавлять значений, в зависимости от длительности нажатия клавиши.
char CountState=0; // Флаг того, что сохраняем состояние процесса (каждое значение - это полсекунды)
char DispDopInfo=0; // Флаг отображения дополнительной информации (через раз)
                    // 1 - смените емкость
                    // 2 - емкость с головами.
                    // 3 - низкое напряжение для поддержания мощности
                    // 4 - датчик ds18b20
                    // 5 - датчик давления MPX5010


#define MAX_KLP 5       // Количество клапанов, которыми надо управлять по ШИМ. 

#define PIN_KLP_BEG 22  // Начальный пин для управления клапанами 

#define KLP_NPG  0      // Номер клапана для управления НПГ (при дистилляции) 
#define KLP_VODA 1      // Номер клапана для управления общей подачей воды в систему
#define KLP_DEFL 2      // Номер клапана для подачи воды в дефлегматор
#define KLP_DEFL_D 2    // Номер клапана для подачи воды в дефлегматор (при дистилляции с дефлегматором с паровым отбором)
#define KLP_GLV_HVS  3  // Номер клапана отбора головных и хвостовых фракций
#define KLP_BARDA  4  // Номер клапана слива барды
#define KLP_SR   3      // Номер клапана отбора ректификата
#define KLP_HLD  3      // Номер клапана холодильника (для дистилляции)
#define KLP_PB 0      // Номер клапана для слива польского буфера

#define TEMP_KUB  0      // Номер датчика  термометра в кубе
#define TEMP_RK20 1      // Номер датчика термометра в РК 20 см от насадки
#define TEMP_TSA  2      // Номер термометра в трубке связи с атмосферой
#define MAX_TEMP_TSA 650 // Максимальная температура в датчике ТСА
#define TEMP_DEFL 1      // Номер датчика  термометра в дефлегматоре
#define TEMP_TERMOSTAT 0 // Номер датчика  термометра термостата
#define TEMP_RAZVAR 0    // Номер датчика  термометра для разваривания зерновых

#define PIN_RZG_ON 30    // Пин для включения режима разгона             (может работать и без него)

#if SIMPLED_VERSION>=1
  #define USE_LCD_KEYPAD_SHIELD 1  // 1 - использовать стандартный шильд с кнопкми ардуино, 0 - не использовать
  // Пятая версия это почти нулева
  #undef PIN_MPX5010 A9      // Пин датчика давления MPX5010DP
  #if SIMPLED_VERSION!=5 && SIMPLED_VERSION!=6 && SIMPLED_VERSION!=8 && SIMPLED_VERSION!=9 && SIMPLED_VERSION<30
    #define PEREGREV_ON 0  // Не используем защиту клапанов от перегрева
    #define PER_KLP_OPEN  100 // Клапана на воду постоянно открыты
    #define PER_KLP_CLOSE 0  // из контанты NAPR_PEREGREV 150 
    #define USE_ALARM_UROVEN 0  // Нужно ли использовать датчик уровня в приемной емкости ардуино
    #define USE_ALARM_VODA   0  // Нужно ли использовать датчик разлития воды ардуино
    #define UROVEN_ALARM 1        // Датчик уровня переводим в цифровой режим
  #endif

  #define USE_GAS_SENSOR 0        // Пин для анализа датчика загазованности (спирта)
  #define UROVEN_GAS_SENSOR 1    // уровень газа переводим в цифровой режим
  
  #if SIMPLED_VERSION!=6
    #undef NPG_UROVEN_PIN A3         // Отключаем датчик НПГ елси не SIMPLED 6
  #else
    #define NPG_UROVEN_PIN A3         // Включаем датчик НПГ елси не SIMPLED 6
  #endif
  
  #define USE_ASC712 1        // Надо ли исполозовать датчик тока
  #define PIN_READI A2         // Аналоговый Пин для чтения тока 

#endif

// Версия урощена 2 - то же, то упрощенная версия, но с цифровыми датчиками уровня голов, спирта, разлития воды
#if SIMPLED_VERSION==2
#define USE_ALARM_UROVEN 1  // Нужно ли использовать датчик уровня в приемной емкости ардуино 
                            // 2, нужно использовать как признак стопа ректификации.
#define USE_ALARM_VODA   1  // Нужно ли использовать датчик разлития воды ардуино
#define PIN_READI A2         // Аналоговый Пин для чтения тока 

#endif

#if SIMPLED_VERSION==3
  #define USE_ALARM_UROVEN 2  // Нужно ли использовать датчик уровня в приемной емкости ардуино 2 - датчик используется как уровень при котором останавливается отбор, но не останавливается ректификация
  #define USE_ALARM_VODA   0  // Нужно ли использовать датчик разлития воды ардуино
  #define KLP_HIGH 0              // В простой версии 3 реле с низким уровнем включения
  #define PIN_READI A2         // Аналоговый Пин для чтения тока 
#endif

#if SIMPLED_VERSION==4
  #define USE_ALARM_UROVEN 2  // Нужно ли использовать датчик уровня в приемной емкости ардуино 2 - датчик используется как уровень при котором останавливается отбор, но не останавливается ректификация
  #define USE_ALARM_VODA   0  // Нужно ли использовать датчик разлития воды ардуино
  #define KLP_HIGH 0              // В простой версии 3 реле с низким уровнем включения
  #define USE_ASC712 0        // Надо ли исполозовать датчик тока
#endif

#if SIMPLED_VERSION==5 || SIMPLED_VERSION==6 || SIMPLED_VERSION==7 || SIMPLED_VERSION==8 || SIMPLED_VERSION==9 || SIMPLED_VERSION==20 || SIMPLED_VERSION>=30
  #define USE_ALARM_UROVEN 2  // Нужно ли использовать датчик уровня в приемной емкости ардуино 2 - датчик используется как уровень при котором останавливается отбор, но не останавливается ректификация
  #define USE_ALARM_VODA   1  // Нужно ли использовать датчик разлития воды ардуино
  #define KLP_HIGH 1              // В простой версии 5 вместо реле на клапана используются симисторы.
  #define USE_ASC712 0        // Надо ли исполозовать датчик тока
  #define PIN_TRIAC  10    // Управление симистором реализовано через PIN 10 (для удобства пайки)
  #define PIN_DS18B20 2    // датчики температуры подключены к PIN 2 (для удобства пайки)
  #define PIN_MPX5010 A1      // Пин датчика давления MPX5010DP к А1 (для удобства пайки)
  #define RELAY_HIGH  0         // Какой сигнал подавать на релейные выходы мешалки и разгона 
  #define KLP_HLD     2      // Номер клапана холодильника (для дистилляции), такой же, как и дефлегматор.
  #define KLP_DEFL_D  4    // Номер клапана дефлегматора дистиллятора - такой же, как и номер клапана отбора спирта, поскольку клапан отбора спирта не используется при дистилляции.
  // Версия Slave-регулятора мощности.
  #if SIMPLED_VERSION==7
    #define USE_ALARM_UROVEN 0 // Нужно ли использовать датчик уровня в приемной емкости ардуино 2 - датчик используется как уровень при котором останавливается отбор, но не останавливается ректификация
    #define USE_ALARM_VODA   0 // Нужно ли использовать датчик разлития воды ардуино
    #undef PIN_MPX5010 A1      // Пин датчика давления MPX5010DP к А1 (для удобства пайки)
    #define USE_SLAVE  1       // Переводим контроллер в Slave - режим
  #endif
  // Версия с датчиком тока (в основном это для отладки на моем оборудовании подключил датчик тока на A1)
  #if SIMPLED_VERSION==8
    #define PIN_READI A4         // Аналоговый Пин для чтения тока 
    #define USE_ASC712 1         // Надо ли исполозовать датчик тока
    #define PEREGREV_ON 0        // Отключаем защиту от перегрева клапанов.
    #define PER_KLP_OPEN  100 // Клапана на воду постоянно открыты
    #define PER_KLP_CLOSE 0  // из контанты NAPR_PEREGREV 150 
  #endif
  
  #if SIMPLED_VERSION==6 || SIMPLED_VERSION==9 || SIMPLED_VERSION==7 || SIMPLED_VERSION==5 || SIMPLED_VERSION==15
    #define PIN_READI A4         // Аналоговый Пин для чтения тока 
  #endif
  
 #if SIMPLED_VERSION>=30
  #define PIN_READI A4 
  #define PIN_TRIAC  36    // Управление симистором реализовано через PIN 36 (для удобства пайки)
  #define PIN_MIXER  38
  #define PIN_RZG_ON 34
  #define RELAY_HIGH 1         // Релейные выходы не используем, вместо них при необходимости симисторное управление.
  #define KLP_NPG  4      // Номер клапана для управления НПГ (при дистилляции) 
  #define KLP_VODA 3      // Номер клапана для управления общей подачей воды в систему
  #define KLP_DEFL 0      // Номер клапана для подачи воды в дефлегматор
  #define KLP_DEFL_D 0    // Номер клапана для подачи воды в дефлегматор (при дистилляции с дефлегматором с паровым отбором)
  #define KLP_GLV_HVS  1  // Номер клапана отбора головных и хвостовых фракций 
  #define KLP_BARDA  2  
  #define KLP_HLD     0      // Номер клапана холодильника (для дистилляции), такой же, как и дефлегматор.
  #define KLP_DEFL_D  2    // Номер клапана дефлегматора дистиллятора - такой же, как и номер клапана отбора спирта, поскольку клапан отбора спирта не используется при дистилляции.
  #define KLP_SR   2      // Номер клапана отбора ректификата
  #define KLP_PB 4      // Номер клапана для слива польского буфера


  #if NUM_PHASE>1
    #undef PIN_MPX5010 A9      // Пин датчика давления MPX5010DP
    #define PER_KLP_OPEN  100 // Клапана на воду постоянно открыты
    #define PER_KLP_CLOSE 0  // из контанты NAPR_PEREGREV 150 
    #define USE_ALARM_UROVEN 0  // Нужно ли использовать датчик уровня в приемной емкости ардуино
    #define USE_ALARM_VODA   0  // Нужно ли использовать датчик разлития воды ардуино
    #define PEREGREV_ON 0  // Не используем защиту клапанов от перегрева
  #endif
  
 
 #endif
  
  
  #if SIMPLED_VERSION==20
    #define USE_ALARM_UROVEN 0 // Нужно ли использовать датчик уровня в приемной емкости ардуино 2 - датчик используется как уровень при котором останавливается отбор, но не останавливается ректификация
    #define USE_ALARM_VODA   0 // Нужно ли использовать датчик разлития воды ардуино
    #define PIN_MPX5010 A1      // Пин датчика давления MPX5010DP к А1 (для удобства пайки)
  #endif
  
  
#endif

#define PIN_SLAVE_0 38
#define PIN_SLAVE_1 39
#define PIN_SLAVE_2 40
#define PIN_SLAVE_3 41


char SlaveON=0;

OneWire  ds(PIN_DS18B20);  // on pin 37
#define MAX_DS1820 5
unsigned char ds1820_devices;                  // Количество датчиков ds18b20 
unsigned char ds1820_rom_codes[MAX_DS1820][9]; // Коды датчиков ds18b20
char ds1820_popr[MAX_DS1820];                  // Поправки к температуре датчиков

// В этой переменной храним температуру в кубе для 
#define COUNT_CHIM 15
int tempK[COUNT_CHIM]={0  ,  880,  960,  1000,0,0,0,0,0,0,0,0,0,0,0};
char CHIM[COUNT_CHIM]={-1,   68,   25,  0,0,0,0,0,0,0,0,0,0,0,0};
// здесь храним количество значений в таблице (Отрицательные значения означают, что колнна находится в режиме само-обучения);
char CntCHIM=-4;
char flPopr; // Флаг поправки означает что в данный момент редактируется, температура или ШИМ 0-температура, 1- ШИМ.
#define MAX_CNT_PAUSE 16
char CntPause=8;
char tempP[MAX_CNT_PAUSE]={10,35,45,50,58,64,70,71,110,0,0,0,0,0,0,0};           // Температурные паузы в градусах (первое значение - это время работы насоса).
unsigned char timeP[MAX_CNT_PAUSE]={2,15,15,20,30,60,30,0,120,0,0,0,0,0,0,0};  // Время паузы в минутах (если 0, то это значит ожидаем действия пользователя (первое значение - это время паузы насоса). 

#define MAX_CNT_FRACTION_DIST 8
char CountFractionDist=0;
char CountFractionRect=0;
char TekFraction=0;
int TempFractionDist[MAX_CNT_FRACTION_DIST];
int PowerFractionDist[MAX_CNT_FRACTION_DIST];
unsigned char AngleFractionDist[MAX_CNT_FRACTION_DIST];
#define MAX_CNT_FRACTION_RECT 5

int TempFractionRect[MAX_CNT_FRACTION_RECT];
unsigned char AngleFractionRect[MAX_CNT_FRACTION_RECT];


char IncrementCHIM=0;
char DecrementCHIM=10;
int TimeAutoIncCHIM=600;
int TimeRestabKolonna=3600; // Время если в течении которого колонна не стабилизировалась, считаем новую температуру стабилизации

unsigned int KlOpen[MAX_KLP];  // Время закрытия клапанов (в полупериодах, то есть 100 - это примерно 1 секунда)
unsigned int KlClose[MAX_KLP];// Время открытия клапанов  (в полупериодах,то есть 100 - это примерно 1 секунда)
char KlState[MAX_KLP];         // Состояние клапанов (1 открыт - 0 закрыт) 
unsigned int  KlCount[MAX_KLP]; // Счетчик состояния клапанов
char KlReg[MAX_KLP];         // Режим работы
char KlSelect=0;  // Выбран режим прибавления - либо время открытия, либо время закрытия - это для ручного управления ШИМом, тестирования и прочего
char KlTek=0;     // Для отладки - режим какого клапана отображать на дисплее  - это для ручного управления ШИМом, тестирования и прочего .  
//char flSyncKLP=0; // Флаг того, что идет синхронизация клапанов, признак того, что состояние клапанов не менять.  
int Angle=0;
// Ну это понятно, это подключен LCDs


#if USE_LCD_KEYPAD_SHIELD
// Это пины стандратного шильда, если у Вас другой шильд, то поменять значения.
 # define PIN_START_1 53      // На этих двух пинах устанавливается заданный уровень при старте процесса и снимается при окончкни нагрева для включения индукционки через реле, например)
 # define PIN_START_2 52

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);   
#else
 # define PIN_START_1 4      // На этих двух пинах устанавливается заданный уровень при старте процесса и снимается при окончкни нагрева для включения индукционки через реле, например)
 # define PIN_START_2 5

LiquidCrystal lcd(52,50, 48, 46, 44, 42);

#endif

int MaxIOut;
int MaxVoltsOut;


int temps[MAX_DS1820];            // Температура датчиков  
int Maxtemps[MAX_DS1820];           // максимальная Температура датчиков  за процесс.

unsigned char nPopr=0;                     // Текущий датчик, которому вносится поправка

#define PIN_SOUND 11    // На 11 пине сидит спикер от компа (мощная штука, 15 ом, потребляет 300ма, поэтому ей пока не злоупотребляю, может заменить ее в дальнейшем на Пьезо?)


#define PIN_ALL_OFF 31  // Пин, при подаче напряжения на который вырыбаеся вообще все (например выключается УЗО) (может работать и без него)
#define PIN_RST_WDT 43  // Пин, на который периодически меняется значение с 0 на 1 (раз в 5 секунд обычно) служит для отслеживаения зависания Ардуины внешним устройством.

char flAllOff;           // Флаг того, что все выключено

unsigned char Counter1=0;
long Seconds;   // Текущее состояние времени с начала процесса (не беззнаковое, потому, что есть проверки на <0);
long SecondsEnd;// Время окончания процесса 

char NeedDisplaying;
char flNoPhase=1;  // Переменная признак отключения фазового управления клапанов (например на период чтения датчиков температуры)
int resultU;
char NeedGPRS;

unsigned char StateVolts=0;  // Состояние вычисление среднеквадратичного 
//0 - надо вычислить - запускается старт, 1 - вычисляется, 2 - вычислено, 3 - идет обработка рузультата
// 4 - Результат обработан, можно вычислять следующий
int index_input=0; // Текущий размер массива дла расчета среднеквадратичного
int prev_index_input=0; // Текущий размер массива дла расчета среднеквадратичного
int indexOut=0;    // Размер для вывода на экран

unsigned char flCrossZero=0;
//unsigned int VoltsZr;
unsigned long TekPower;
unsigned long TekPowerKLP;
unsigned int TimeOpenTriac=0;
unsigned int TimeOpenTriacFact=0;
unsigned int TimeOpenKLP=0;
unsigned int TicZero; // Количество тиков таймера от нуля до нуля.
//char FirstRun;
// Установка мощности (Пока в виде переменных)

int Power=3000;       // Номинальная мощность ТЭНов
int PowerPhase[3]={3000,0,0};
unsigned char KtPhase[3]={100,100,100};
int FactPower=0;       // Номинальная мощность ТЭНов
int UstPower;     // Нужно получить мощность от регулятора (текущая мощность на всех фазах)
int UstPwrPH1;     // Нужно получить мощность от регулятора (текущая мощность на первой фазе)
int UstPwrPH2;     // Нужно получить мощность от регулятора (текущая мощность на первой фазе)
int UstPwrPH3;     // Нужно получить мощность от регулятора (текущая мощность на первой фазе)
int R_TEN20;        // Сопротиввление ТЭНов, умноженное на 10;

int UstPowerReg=500;     // Мощность в режиме регулятора мощности

char flNeedTemp=0;    // Флаг того, что нужно читать температуру датчиков
char StepOut=0;       // Признак шага чтения температуры 0 - выдаем команду на запрос температуры
// 1 - читаем температуру

//#define USE_BRESENHAM 0// Надо ли использовать брезинхем для управлением двигателем.

int deltaPower=0; // Разница мощности при дистилляции с дефлегматором.

unsigned char b_size[3];  // Размер брезинхема
unsigned char b_value[3]; // Текущее значение алгоритма
int b_error[3]; // Переменная
unsigned char b_stepNumber[3]; // Номер шага

void bresenham_Init(char n, unsigned char sz)
{
  b_size[n] = sz;
  b_error[n] = b_size[n]/2;
  b_stepNumber[n] = 0;
  b_value[n] = 0;
}

boolean bresenham_getNext(char n) 
{
  boolean result;
  b_error[n] -= b_value[n];
  if ( b_error[n] < 0 ) 
  {
    b_error[n] += b_size[n];
    result = HIGH;
  }
  else 
  {
    result = LOW;
  }

  if ( ++b_stepNumber[n] >= b_size[n]) {
    b_stepNumber[n] = 0;
    b_error[n] = b_size[n]/2;
  }
  return result;
}		



// Пины, на которых сидят кнопки
#define PIN_SELECT 53
#define PIN_RIGHT 51
#define PIN_LEFT 49
#define PIN_UP 45
#define PIN_DOWN 47

#define PRESS_SELECT digitalRead(PIN_SELECT)==0
#define PRESS_RIGHT  digitalRead(PIN_RIGHT)==0
#define PRESS_LEFT  digitalRead(PIN_LEFT)==0
#define PRESS_UP     digitalRead(PIN_UP)==0
#define PRESS_DOWN   digitalRead(PIN_DOWN)==0

char KeyCode; // Код нажатой клавиши

#if USE_LCD_KEYPAD_SHIELD
#define PRESS_SELECT KeyCode==1
#define PRESS_RIGHT  KeyCode==2
#define PRESS_LEFT   KeyCode==3
#define PRESS_UP     KeyCode==4
#define PRESS_DOWN   KeyCode==5
#endif


#define PIN_ALARM_VODA   A5      // Пин, на котором сидит датчик разлития воды на полу (да какой датчик, два луженых провода).     
#define PIN_ALARM_UROVEN A6      // Пин, на котором сидит датчик уровня в приемной емкости (да какой датчик, два луженых провода).
unsigned char CountAlarmVoda=0;          // Счетчик срабатываний датчика (если сработал три раза, тогда тревога)
unsigned char CountAlarmUroven=0;        // Счетчик срабатываний датчика (если сработал три раза, тогда тревога)
char CountGasSensor=0;          // Счетчик срабатываний датчика паров спирта (если сработал три раза, тогда тревога)

#define PIN_TERMOSTAT_ON  28    // Пин термостата, который включается при превышении температуры 
#define PIN_TERMOSTAT_OFF 29    // Пин термостата, который отключается при превышении температуры 

unsigned int BeepTime;
char StateNPG=0; // Состояние автомата обработки НПГ
// 0 - не запускался
// 1 - инициализируется (наполнение)
// 2 - рабочий режим
// 3 - режим одного уровня тревоги (один раз считан нижний или верхний уровень)

int U_VODA=0,U_UROVEN=0,U_GAS=0,U_NPG=0,U_GLV=0; // Уровни воды, газа, НПГ, голов


char NpgDt;// 4- нет сработки датчика 1 - нижний уровень, 2 - рабочий, 3-верхний

void ReadStateDtNPG()
{
 #ifdef NPG_UROVEN_PIN   
  U_NPG=analogRead(NPG_UROVEN_PIN);    
  // Уровень НПГ подключен по следующей схеме: Подтяджка - 10 ком, защита -330 ом, замыкание - 30, резистор 1.1К (верхниц уровень) - 128, 2.2К (рабочий уровень) - 256, 3,3К (нижний уровень)-266
  if (U_NPG>=80) NpgDt=1;  // Нижний уровень
  if (U_NPG>=180) NpgDt=2;  // Рабочий уровень
  if (U_NPG>=240) NpgDt=3;  // Верхний уровень
  if (U_NPG>=500) NpgDt=4; // Нет сработки

  // Если находимся в режиме аварии, состояние НПГ всегда нет сработки
  if (IspReg>240) StateNPG=2;
#endif

#ifdef NPG_UROVEN_PIN_MIN   
  NpgDt=4; // Нет сработки
  // Датчик уровня в следущем виде 10 (первая цифра сработка минимума, 2 цифра сработка нормального уровня)
  U_NPG=analogRead(NPG_UROVEN_PIN_MIN);
  if (UrovenBarda>0)
  {
    if (U_NPG<=UrovenBarda) NpgDt=1; // Нижний уровень
    else
    {
      U_NPG=analogRead(NPG_UROVEN_PIN_WORK); 
      if (U_NPG<=UrovenBarda) NpgDt=2; 
    }
  }
  else
  {
    if (U_NPG>=-UrovenBarda) NpgDt=1; // Нижний уровень
    else
    {
       NpgDt=2; // Нет сработки
    }    
  }
  if (IspReg>240) StateNPG=2;
#endif
}

void ProcessNPG()
{

#ifndef NPG_UROVEN_PIN
#ifndef NPG_UROVEN_PIN_MIN
  StateNPG=2;
  return;
#endif
#endif

ReadStateDtNPG();

  switch (StateNPG)
  {
  case 0:// НПГ не запущен
    if (NpgDt==1) // Если сработка нижнего уровня, то тогда НПГ надо наполнить.
    {
      StateNPG=1;
      break;
    }
    StateNPG=2;
    break;
  case 1:// Инициализация НПГ - наполнение
    KlOpen[KLP_NPG]=PER_KLP_OPEN;
    KlClose[KLP_NPG]=PER_KLP_CLOSE;
    if (NpgDt==2 || NpgDt==3) // Дошли до рабочего уровня или до переполнения, тогда переходим в рабочий режим.
    {
#ifdef TEST
      sprintf(my_tx_buffer,"NAPOLN NPG=%i NpgDt=%i",U_NPG,NpgDt);
      Serial.println(my_tx_buffer);
#endif      

      // Повторное чтение для подтверждения датчика 
      ReadStateDtNPG();
      if (NpgDt==2 || NpgDt==3)  StateNPG=4; // Переводим в режим однократной сработки, если прошло подтверждение
    }
    break;
  case 2:// НПГ работает в штатном режиме.
    if (NpgDt==1 || NpgDt==3)  
    {
#ifdef TEST
      sprintf(my_tx_buffer,"Alarm NPG=%i NpgDt=%i",U_NPG,NpgDt);
      Serial.println(my_tx_buffer);
#endif      
      // Повторное чтение для подтверждения датчика 
      ReadStateDtNPG();
      if (NpgDt==1 || NpgDt==3) StateNPG=3; // Переводим в режим однократной сработки аварии, если прошло подтверждение
      countAlrmNPG=1;
    }

    if (NpgDt==2) 
    {
      // Сработка рабочего уровня НПГ
      KlOpen[KLP_NPG]=PER_KLP_OPEN;
      KlClose[KLP_NPG]=PER_KLP_CLOSE;
      countAlrmNPG=0;

    }

    if (NpgDt==4) // Нет никакой сработки - отключаем подачу воды
    {
      // Сработка рабочего уровня НПГ
      KlOpen[KLP_NPG]=0;
      KlClose[KLP_NPG]=10;
      countAlrmNPG=0;
    }
    break;
  case 3:  // Была однократная сработка датчиков осушения или переполнения

    if (NpgDt==1) 
    {
      countAlrmNPG++;
      ReadStateDtNPG();
#ifndef TEST
#ifndef DEBUG
      if (countAlrmNPG>MAX_COUNT_NPG_ALARM)
      {
        IspReg=253;      // Переводим контроллер в режим тревоги по осушению НПГ
      }
#endif
#endif

#ifdef TEST
      sprintf(my_tx_buffer,"Alarm pNPG=%i NpgDt=%i",U_NPG,NpgDt);
      Serial.println(my_tx_buffer);
#endif      
      break;
    }
    if (NpgDt==3) 
    {
      countAlrmNPG++;
      ReadStateDtNPG();
      // Сработка верхнего уровня НПГ, исклчение - переполнение НПГ.
#ifndef TEST
#ifndef DEBUG
      IspReg=254;      // Переводим контроллер в режим тревоги по осушению НПГ
#endif
#endif

#ifdef TEST
      sprintf(my_tx_buffer,"Alarm pNPG=%i NpgDt=%i",U_NPG,NpgDt);
      Serial.println(my_tx_buffer);
#endif      
      break;
    }
    StateNPG=2; // Если тревога не подтвердилась, переходим в обычный ражим.
    countAlrmNPG=0;
    break;

  case 4:// При наполнении НПГ была однократная сработка датчиков уровня или переполнения

    // Если подтверждаестся повторная сработка датчика, то токда считаем НПГ переведенным в рабочее состояние
    if (NpgDt==2 || NpgDt==3) // Дошли до рабочего уровня или до переполнения, тогда переходим в рабочий режим.
    {
#ifdef TEST
      sprintf(my_tx_buffer,"NAPOLN pNPG=%i NpgDt=%i",U_NPG,NpgDt);
      Serial.println(my_tx_buffer);
#endif      
      StateNPG=2; // Переводим НПГ в рабочее состояние.
    }
    break;
  } 
}

// Функция устанавливает пищалку на заданное время (в 1/125 секунды), то есть 125-это одна секунда.
void my_beep(unsigned int mBeepTime)
{
  analogWrite(PIN_SOUND, 10);  
  BeepTime=mBeepTime;
}

// Функция расчета квадратного корня.
// Значения корней от 0 до 90 и от 256 до 512 считаются по очень приблизительным формулам, поскольку такое значение среднеквадратичного в нормальной сети 220 вольт недопустимо
// Больше 512 значение не определено.
int my_sqrt(unsigned long tsQT)
{  char i;
   unsigned int sQT; // Временная переменная, чтобы поменьше сравнивать 4-байтных чисел.
   int res=666;
   char fl2=0;
    if (tsQT<64) 
    {
      tsQT=tsQT<<10;
      fl2=5;
    }
    else
    if (tsQT<256) 
    {
      tsQT=tsQT<<8;
      fl2=4;
    }
    else
    if (tsQT<1024) 
    {
      tsQT=(tsQT<<6);
      fl2=3;
    }
    else
    if (tsQT<4096) 
    {
      tsQT=(tsQT<<4);
      fl2=2;
    }
    else
     if (tsQT<tableSQ[0])
       {
        tsQT=tsQT<<2;
        fl2=1;
       }
       
   // Если попадаем в диапазон
   if (tsQT<tableSQ[0]) 
   {
     // Если меньше минимального значения, тогда расчет по крайне приближенной формуле
     res=tsQT*tableSQRT[0]/tableSQ[0];
   }
   else
   if (tsQT<=tableSQ[MAX_TABLE_SQRT-1])
   {
     sQT=tsQT;
     for(i=1;i<MAX_TABLE_SQRT;i++)
     {
       
       if (tableSQ[i-1]<=sQT && tableSQ[i]>=sQT)
        {
           if (tableSQ[i-1]==sQT) res=tableSQRT[i-1]; // На крайних значениях берем готовые цифры
             else
               if (tableSQ[i]==sQT) res=tableSQRT[i]; // На крайних значениях берем готовые цифры
                else res=(sQT-tableSQ[i-1])*(tableSQRT[i]-tableSQRT[i-1])/(tableSQ[i]-tableSQ[i-1]) + tableSQRT[i-1] +1; // На промежуточных вычисляем.
           break;
        }
     }
   }
   else
   {
     // Если больше максимального считаем по крайне приближенной формуле, исходя из максимума значений в 512
     res=(tsQT-tableSQ[MAX_TABLE_SQRT-1])*(512-tableSQRT[MAX_TABLE_SQRT-1])/(262144-tableSQ[MAX_TABLE_SQRT-1]) + tableSQRT[MAX_TABLE_SQRT-1] +1;
   }

   // Если перед корнем умножали на 4, результат уменьшим на 2
   return  res >> fl2;
}

void setup() 
{
  unsigned char i;
  // set up the LCD's number of columns and rows: 


//#################################################################################################################
#ifdef MQTT_BROKER
  
  // инициализация UART для обмена с модулем ESP, пины 15(RX) и 14(TX)
  Serial3.begin(9600);
  
#endif
//#################################################################################################################  
  
  Serial.begin(9600);
  Serial2.begin(9600);
  
  #if NUM_PHASE>0
    Serial1.begin(9600);
  #endif

  StateVolts=6;

 lcd.begin(16, 2);

  // Print a message to the LCD.
  //  my_lcdprint("hello, world!");
  //  analogReference(DEFAULT);

  //  #if USE_LCD_KEYPAD_SHIELD==0
  //    analogReference(INTERNAL2V56);
  //  #else
  // теперь опорно напряжение всегда делаем +5В, чтобы работа программы была одинаковой, как с использованием дисплей шильда, так и без него
  analogReference(DEFAULT);
  // #endif

 pinMode(PIN_TRIAC, OUTPUT);
 pinMode(PIN_ALL_OFF, OUTPUT);
 pinMode(PIN_RST_WDT, OUTPUT);
 
 pinMode(PIN_RZG_ON, OUTPUT);
 digitalWrite(PIN_RZG_ON,!RELAY_HIGH);

 pinMode(PIN_START_1, OUTPUT);
 digitalWrite(PIN_START_1,!RELAY_HIGH);
 pinMode(PIN_START_2, OUTPUT);
 digitalWrite(PIN_START_2,!RELAY_HIGH);
  
  pinMode(PIN_NASOS_NBK, OUTPUT);
  pinMode(PIN_NASOS_NBK_BRD, OUTPUT);
  pinMode(PIN_MIXER, OUTPUT);
  digitalWrite(PIN_MIXER,!RELAY_HIGH);
  digitalWrite(PIN_NASOS_NBK, LOW);  
  pinMode(PIN_TERMOSTAT_ON, OUTPUT);
  pinMode(PIN_TERMOSTAT_OFF, OUTPUT);
  
  if (UROVEN_ALARM==1)
  {
    pinMode(PIN_ALARM_VODA, INPUT);
    pinMode(PIN_ALARM_UROVEN, INPUT);
  }
 
 
 if (UrovenProvodimostSR==1) 
 {
   pinMode(PIN_PROVODIMOST_SR, INPUT);
 }
  
  #ifdef NPG_UROVEN_PIN_MIN
//    pinMode(NPG_UROVEN_PIN_MIN, INPUT);           // назначить выводу порт ввода
//    digitalWrite(NPG_UROVEN_PIN_MIN, HIGH);       // включить подтягивающий резистор
//    pinMode(NPG_UROVEN_PIN_WORK, INPUT);           // назначить выводу порт ввода
//    digitalWrite(NPG_UROVEN_PIN_WORK, HIGH);       // включить подтягивающий резистор
    
  #endif
  
 

  pinMode(PIN_SELECT, INPUT);           // назначить выводу порт ввода
  digitalWrite(PIN_SELECT, HIGH);       // включить подтягивающий резистор

  pinMode(PIN_RIGHT, INPUT);           // назначить выводу порт ввода
  digitalWrite(PIN_RIGHT, HIGH);       // включить подтягивающий резистор

  pinMode(PIN_LEFT, INPUT);           // назначить выводу порт ввода
  digitalWrite(PIN_LEFT, HIGH);       // включить подтягивающий резистор

  pinMode(PIN_UP, INPUT);           // назначить выводу порт ввода
  digitalWrite(PIN_UP, HIGH);       // включить подтягивающий резистор

  pinMode(PIN_DOWN, INPUT);           // назначить выводу порт ввода
  digitalWrite(PIN_DOWN, HIGH);       // включить подтягивающий резистор


  // Проходим по клапанам и устанавливаем им ШИМ и настраиваем пины
  for(i=0;i<MAX_KLP;i++)
  {
    #if SIMPLED_VERSION<30
      pinMode(PIN_KLP_BEG+i, OUTPUT);
      digitalWrite(PIN_KLP_BEG+i, !KLP_HIGH);  
    #else
      pinMode(PIN_KLP_BEG+i+i, OUTPUT);
      digitalWrite(PIN_KLP_BEG+i+i, !KLP_HIGH);  
    #endif
    KlOpen[i]=0;    
    KlClose[i]=10;   
    KlState[i]=0;          // Начальное состояние - клапан закрыт, счетчик 0
    KlCount[i]=0;  
  }
    #if NUM_PHASE>1
    for(i=0;i<NUM_PHASE;i++)
    {
     digitalWrite(PIN_KLP_BEG,HIGH);
     digitalWrite(PIN_KLP_BEG+4,HIGH);
     delay(1000);
     digitalWrite(PIN_KLP_BEG+4,LOW);
     digitalWrite(PIN_KLP_BEG,LOW);
     delay(1000); 
    }
    #endif

//  #if USE_ASC712==0
//  attachInterrupt(1, zero_crosss_int, RISING);  // Устанавливаем на 1-е прерывание функцию (это pin 3) обработку от детектора нулюя сетевого напряжения
//  #else
  // Сейчас всегда устанавливаем обработку на переход с ненуля на ноль, чтобы было немного больше времени на программу в прерывании нуля.
   //attachInterrupt(1, zero_crosss_int, FALLING);  // Устанавливаем на 1-е прерывание функцию (это pin 3) обработку от детектора нулюя сетевого напряжения
//  #endif
  //  attachInterrupt(0, zero_crosss_trans, RISING);  // Устанавливаем на 0-е прерывание функцию (это pin 2) обработку от детектора нуля трансформатора.

   // Таймер 2 используется для подсчета числа секунд и, возможно, программного небыстрого ШИМа
  // Прерываение по этому таймеру будет вызываться 125 раз в секунду.
  // Timer/Counter 2 initialization
  // Clock source: System Clock
  // Clock value: 62,500 kHz
  // Mode: Normal top=0xFF
  // OC2A output: Disconnected
  // OC2B output: Disconnected
  TCCR2A=0x00;
  TCCR2B=0x07;
  TCNT2=0x00;
  OCR2A=0x7D;
  OCR2B=0x00;  
  //  // Timer/Counter 1 Interrupt(s) initialization
  TIMSK2=0x02;


  // Таймер 3 используем для эмуляции датчика нуля
  TCCR3A = 0;
  TCCR3B = 0;
  TCNT3  = 0;
  // set compare match register for 100hz increments
  OCR3A = 2500-1;  //(must be <65536)
  // turn on CTC mode
  TCCR3B |= (1 << WGM32);
  // Set 64 prescaler
  TCCR3B |= (1 << CS31) | (1 << CS30);  
  // enable timer compare interrupt
  TIMSK3 |= (1 << OCIE3A);
 


  // Таймер 4 будет использоваться для подсчета числа тиков при вызове АЦП в расчете среднеквадратичного 
  // (поскольку непонятно, чего и сколько это занимает в Ардуино - исходный код разбирать неохота)
  // Timer/Counter 4 initialization
  // Clock source: System Clock
  // Clock value: 250,000 kHz
  // Mode: Normal top=0xFFFF
  // OC4A output: Discon.
  // OC4B output: Discon.
  // OC4C output: Discon.
  // Noise Canceler: Off
  // Input Capture on Falling Edge
  // Timer4 Overflow Interrupt: Off
  // Input Capture Interrupt: Off
  // Compare A Match Interrupt: Off
  // Compare B Match Interrupt: Off
  // Compare C Match Interrupt: Off
  TCCR4A=0x00;
  TCCR4B=0x03;
  TCNT4H=0x00;
  TCNT4L=0x00;
  ICR4H=0x00;
  ICR4L=0x00;
  OCR4AH=0x00;
  OCR4AL=0x00;
  OCR4BH=0x00;
  OCR4BL=0x00;
  OCR4CH=0x00;
  OCR4CL=0x00;
  // Timer/Counter 4 Interrupt(s) initialization
  TIMSK4=0x00;
  
  


  // Таймер 5 будет использоваться для управления симистором регулятора мощности по прерыванию COMPA.
  // Таймер 5 будет использоваться для управления симистороми клапанов по прерыванию COMPB.
  // Таймер 5 будет использоваться для управления симистором регулятора оборотов двигателя прерыванию COMPС.
  // Timer/Counter 5 initialization
  // Clock source: System Clock
  // Clock source: System Clock
  // Clock value: 250,000 kHz
  // Mode: Normal top=0xFFFF
  // OC5A output: Discon.
  // OC5B output: Discon.
  // OC5C output: Discon.
  // Noise Canceler: Off
  // Input Capture on Falling Edge
  // Timer5 Overflow Interrupt: Off
  // Input Capture Interrupt: Off
  // Compare A Match Interrupt: On
  // Compare B Match Interrupt: On
  // Compare C Match Interrupt: On

  TCCR5A=0x00;
  TCCR5B=0x03;
  TCNT5H=0x00;
  TCNT5L=0x00;
  ICR5H=0x00;
  ICR5L=0x00;
  OCR5AH=0x00;
  OCR5AL=0x02;
  OCR5BH=0x00;
  OCR5BL=0x00;
  OCR5CH=0x00;
  OCR5CL=0x00;
  // Timer/Counter 5 Interrupt(s) initialization
  //  TIMSK5=0x06;
  TIMSK5=0x0E;// Инициализируем прерывания таймера


  //  TekPower=(unsigned long) UstPower*220*220/ Power;
  nPopr=1;
  ds1820_devices=0;
  while(ds.search(ds1820_rom_codes[ds1820_devices])) {
    ds1820_devices++;
    if (ds1820_devices>=MAX_DS1820) break;
  }
  ds.reset_search();
  #if SIMPLED_VERSION>=30
    if (ds1820_devices<3 && NUM_PHASE<=1) ScanDS18b20Slave();
  #endif
  
  lcd.clear();
#ifdef DEBUG
  ds1820_devices=3;
  #ifdef LCD_PROGMEM
    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_setup_1),ds1820_devices); //запись в буфер текста и значений температуры в подготовленном
  #else
    sprintf(lcd_buffer,"debug=%u",ds1820_devices); //запись в буфер текста и значений температуры в подготовленном
  #endif
#else
  #ifdef LCD_PROGMEM
    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_setup_2),(int) ds1820_devices); //запись в буфер текста и значений температуры в подготовленном
  #else
    sprintf(lcd_buffer,"ds18B20Count=%u",(int) ds1820_devices); //запись в буфер текста и значений температуры в подготовленном
  #endif
#endif

  my_lcdprint(lcd_buffer);                      
  #ifdef LCD_PROGMEM
    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_setup_3),USE_ALARM_UROVEN,USE_ALARM_VODA,UrovenProvodimostSR,(int)nPopr); //запись в буфер текста и значений температуры в подготовленном
  #else
    sprintf(lcd_buffer,"C,V,P =%i,%i,%i(%1i) ",USE_ALARM_UROVEN,USE_ALARM_VODA,UrovenProvodimostSR,(int)nPopr); //запись в буфер текста и значений температуры в подготовленном
  #endif
  nPopr=0;                    
  lcd.setCursor(0, 1);
  my_lcdprint(lcd_buffer);  
  delay(2000);
  lcd.clear();
  #ifdef LCD_PROGMEM
    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_setup_4),my_version,PR_REWRITE_EEPROM); //запись в буфер текста и значений температуры в подготовленном
  #else
    sprintf(lcd_buffer,"%s  RWR=%i",my_version,PR_REWRITE_EEPROM); //запись в буфер текста и значений температуры в подготовленном
  #endif
  my_lcdprint(lcd_buffer);                 
  lcd.setCursor(0, 1);
  #ifndef USE_SLAVE
    #ifdef LCD_PROGMEM
      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_setup_5),SIMPLED_VERSION); //запись в буфер текста и значений температуры в подготовленном
    #else
     sprintf(lcd_buffer,"Simpled=%u ",SIMPLED_VERSION); //запись в буфер текста и значений температуры в подготовленном
    #endif
  #else
    #ifdef LCD_PROGMEM
      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_setup_6),SIMPLED_VERSION); //запись в буфер текста и значений температуры в подготовленном
    #else
      sprintf(lcd_buffer,"Simpled=%u Slave",SIMPLED_VERSION); //запись в буфер текста и значений температуры в подготовленном
    #endif
  #endif
  my_lcdprint(lcd_buffer);                      
  delay(1000);
  flNeedTemp=0;
  StepOut=0;
  
  // Читаем ранее сохраненные значения из энергонезависимой памяти.
  if (EEPROM.read(0)!=PR_REWRITE_EEPROM)
  { 
    writeEEPROM();
  }

  for(i=0;i<ds1820_devices;i++)
  {
    temps[i]=0;
    Maxtemps[i]=0;
    ds1820_popr[i]=0;                  // Поправки к температуре датчиков устанавливаем в ноль (в дальнейшем они сами считаются из eeprom, если были ранее записаны).
    // Выдадим номера датчиков.
    if (FlToUSART)
    {
      sprintf(my_tx_buffer,"ds18b20\t%1X\t%02X%02X%02X%02X%02X%02X%02X%02X",i,ds1820_rom_codes[i][0],ds1820_rom_codes[i][1],ds1820_rom_codes[i][2],
              ds1820_rom_codes[i][3],ds1820_rom_codes[i][4],ds1820_rom_codes[i][5],ds1820_rom_codes[i][6],ds1820_rom_codes[i][7]);
      
      Serial.println(my_tx_buffer);
    }
  }

  readEEPROM();

  #ifdef USE_SLAVE
    IspReg=103; // Для Slave режима всегда устанавливаем регулятор мощности при перезагрузке
    pinMode(PIN_SLAVE_0, INPUT);
    pinMode(PIN_SLAVE_1, INPUT);
    pinMode(PIN_SLAVE_2, INPUT);
    pinMode(PIN_SLAVE_3, INPUT);
  #else
    pinMode(PIN_SLAVE_0, OUTPUT);
    pinMode(PIN_SLAVE_1, OUTPUT);
    pinMode(PIN_SLAVE_2, OUTPUT);
    pinMode(PIN_SLAVE_3, OUTPUT);
    // Отключаем подчиненные контроллеры
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
  #endif
  
  #if SIMPLED_VERSION==20 || NUM_PHASE>1
    IspReg=103; // Для режима регулятора всегда устанавливаем регулятор мощности при перезагрузке
    Power=1000;
    UstPowerReg=0;
    UstPower=0;
    UstPwrPH1=0;
  #endif
  
  #ifndef DEBUG
    my_beep(10);
  #endif
  
  if (IspReg>=240) IspReg=101;
  // Выдаем тестовую команду
 // InitGSM();

  if (IspReg>=250) IspReg=101;

  GetPhonePDU();
#ifdef DEBUG
  //  Serial.begin(9600);
  Serial.println("start debug");
#endif

  CloseAllKLP();
//  digitalWrite(PIN_REG_ON,LOW);
  digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
  digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
  delay(100);
  digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
  flAllOff=0;

  digitalWrite(PIN_TERMOSTAT_ON,LOW);
  digitalWrite(PIN_TERMOSTAT_OFF,LOW);
  UstPower=0;
  StateNPG=0;
  StateMachine=0;

  bresenham_Init(0,200);
  bresenham_Init(1,125);
  bresenham_Init(2,125);
  
  b_value[0]=0;

  #ifdef TEST
    temps[TEMP_KUB]=799;
    GetCHIMOtbor();
    temps[TEMP_KUB]=880;
    GetCHIMOtbor();
    temps[TEMP_KUB]=890;
    GetCHIMOtbor();
    temps[TEMP_KUB]=900;
    GetCHIMOtbor();
    temps[TEMP_KUB]=960;
    GetCHIMOtbor();
    temps[TEMP_KUB]=980;
    GetCHIMOtbor();
  #endif

  // Устанавливаем значение таблицы открытия симистора
  tableS10[ 0]= 222;//tableK[ 0]= 5;
  tableS10[ 1]= 222;//tableK[ 1]= 10;
  tableS10[ 2]= 217;//tableK[ 2]= 15;
  tableS10[ 3]= 214;//tableK[ 3]= 20;
  tableS10[ 4]= 211;//tableK[ 4]= 25;
  tableS10[ 5]= 208;//tableK[ 5]= 30;
  tableS10[ 6]= 206;//tableK[ 6]= 35;
  tableS10[ 7]= 204;//tableK[ 7]= 40;
  tableS10[ 8]= 202;//tableK[ 8]= 45;
  tableS10[ 9]= 200;//tableK[ 9]= 50;
  tableS10[ 10]= 198;//tableK[ 10]= 55;
  tableS10[ 11]= 196;//tableK[ 11]= 60;
  tableS10[ 12]= 195;//tableK[ 12]= 65;
  tableS10[ 13]= 193;//tableK[ 13]= 70;
  tableS10[ 14]= 192;//tableK[ 14]= 75;
  tableS10[ 15]= 190;//tableK[ 15]= 80;
  tableS10[ 16]= 189;//tableK[ 16]= 85;
  tableS10[ 17]= 188;//tableK[ 17]= 90;
  tableS10[ 18]= 186;//tableK[ 18]= 95;
  tableS10[ 19]= 185;//tableK[ 19]= 100;
  tableS10[ 20]= 184;//tableK[ 20]= 105;
  tableS10[ 21]= 183;//tableK[ 21]= 110;
  tableS10[ 22]= 182;//tableK[ 22]= 115;
  tableS10[ 23]= 181;//tableK[ 23]= 120;
  tableS10[ 24]= 180;//tableK[ 24]= 125;
  tableS10[ 25]= 179;//tableK[ 25]= 130;
  tableS10[ 26]= 178;//tableK[ 26]= 135;
  tableS10[ 27]= 177;//tableK[ 27]= 140;
  tableS10[ 28]= 176;//tableK[ 28]= 145;
  tableS10[ 29]= 175;//tableK[ 29]= 150;
  tableS10[ 30]= 174;//tableK[ 30]= 155;
  tableS10[ 31]= 173;//tableK[ 31]= 160;
  tableS10[ 32]= 172;//tableK[ 32]= 165;
  tableS10[ 33]= 171;//tableK[ 33]= 170;
  tableS10[ 34]= 170;//tableK[ 34]= 175;
  tableS10[ 35]= 169;//tableK[ 35]= 180;
  tableS10[ 36]= 168;//tableK[ 36]= 185;
  tableS10[ 37]= 168;//tableK[ 37]= 190;
  tableS10[ 38]= 167;//tableK[ 38]= 195;
  tableS10[ 39]= 166;//tableK[ 39]= 200;
  tableS10[ 40]= 165;//tableK[ 40]= 205;
  tableS10[ 41]= 164;//tableK[ 41]= 210;
  tableS10[ 42]= 164;//tableK[ 42]= 215;
  tableS10[ 43]= 163;//tableK[ 43]= 220;
  tableS10[ 44]= 162;//tableK[ 44]= 225;
  tableS10[ 45]= 161;//tableK[ 45]= 230;
  tableS10[ 46]= 160;//tableK[ 46]= 235;
  tableS10[ 47]= 160;//tableK[ 47]= 240;
  tableS10[ 48]= 159;//tableK[ 48]= 245;
  tableS10[ 49]= 158;//tableK[ 49]= 250;
  tableS10[ 50]= 157;//tableK[ 50]= 255;
  tableS10[ 51]= 157;//tableK[ 51]= 260;
  tableS10[ 52]= 156;//tableK[ 52]= 265;
  tableS10[ 53]= 155;//tableK[ 53]= 270;
  tableS10[ 54]= 154;//tableK[ 54]= 275;
  tableS10[ 55]= 154;//tableK[ 55]= 280;
  tableS10[ 56]= 153;//tableK[ 56]= 285;
  tableS10[ 57]= 152;//tableK[ 57]= 290;
  tableS10[ 58]= 152;//tableK[ 58]= 295;
  tableS10[ 59]= 151;//tableK[ 59]= 300;
  tableS10[ 60]= 150;//tableK[ 60]= 305;
  tableS10[ 61]= 150;//tableK[ 61]= 310;
  tableS10[ 62]= 149;//tableK[ 62]= 315;
  tableS10[ 63]= 148;//tableK[ 63]= 320;
  tableS10[ 64]= 148;//tableK[ 64]= 325;
  tableS10[ 65]= 147;//tableK[ 65]= 330;
  tableS10[ 66]= 146;//tableK[ 66]= 335;
  tableS10[ 67]= 146;//tableK[ 67]= 340;
  tableS10[ 68]= 145;//tableK[ 68]= 345;
  tableS10[ 69]= 144;//tableK[ 69]= 350;
  tableS10[ 70]= 144;//tableK[ 70]= 355;
  tableS10[ 71]= 143;//tableK[ 71]= 360;
  tableS10[ 72]= 142;//tableK[ 72]= 365;
  tableS10[ 73]= 142;//tableK[ 73]= 370;
  tableS10[ 74]= 141;//tableK[ 74]= 375;
  tableS10[ 75]= 140;//tableK[ 75]= 380;
  tableS10[ 76]= 140;//tableK[ 76]= 385;
  tableS10[ 77]= 139;//tableK[ 77]= 390;
  tableS10[ 78]= 138;//tableK[ 78]= 395;
  tableS10[ 79]= 138;//tableK[ 79]= 400;
  tableS10[ 80]= 137;//tableK[ 80]= 405;
  tableS10[ 81]= 136;//tableK[ 81]= 410;
  tableS10[ 82]= 136;//tableK[ 82]= 415;
  tableS10[ 83]= 135;//tableK[ 83]= 420;
  tableS10[ 84]= 134;//tableK[ 84]= 425;
  tableS10[ 85]= 134;//tableK[ 85]= 430;
  tableS10[ 86]= 133;//tableK[ 86]= 435;
  tableS10[ 87]= 132;//tableK[ 87]= 440;
  tableS10[ 88]= 132;//tableK[ 88]= 445;
  tableS10[ 89]= 131;//tableK[ 89]= 450;
  tableS10[ 90]= 131;//tableK[ 90]= 455;
  tableS10[ 91]= 130;//tableK[ 91]= 460;
  tableS10[ 92]= 129;//tableK[ 92]= 465;
  tableS10[ 93]= 129;//tableK[ 93]= 470;
  tableS10[ 94]= 128;//tableK[ 94]= 475;
  tableS10[ 95]= 128;//tableK[ 95]= 480;
  tableS10[ 96]= 127;//tableK[ 96]= 485;
  tableS10[ 97]= 126;//tableK[ 97]= 490;
  tableS10[ 98]= 126;//tableK[ 98]= 495;
  tableS10[ 99]= 125;//tableK[ 99]= 500;
  tableS10[ 100]= 124;//tableK[ 100]= 505;
  tableS10[ 101]= 124;//tableK[ 101]= 510;
  tableS10[ 102]= 123;//tableK[ 102]= 515;
  tableS10[ 103]= 122;//tableK[ 103]= 520;
  tableS10[ 104]= 122;//tableK[ 104]= 525;
  tableS10[ 105]= 121;//tableK[ 105]= 530;
  tableS10[ 106]= 121;//tableK[ 106]= 535;
  tableS10[ 107]= 120;//tableK[ 107]= 540;
  tableS10[ 108]= 119;//tableK[ 108]= 545;
  tableS10[ 109]= 119;//tableK[ 109]= 550;
  tableS10[ 110]= 118;//tableK[ 110]= 555;
  tableS10[ 111]= 118;//tableK[ 111]= 560;
  tableS10[ 112]= 117;//tableK[ 112]= 565;
  tableS10[ 113]= 116;//tableK[ 113]= 570;
  tableS10[ 114]= 116;//tableK[ 114]= 575;
  tableS10[ 115]= 115;//tableK[ 115]= 580;
  tableS10[ 116]= 114;//tableK[ 116]= 585;
  tableS10[ 117]= 114;//tableK[ 117]= 590;
  tableS10[ 118]= 113;//tableK[ 118]= 595;
  tableS10[ 119]= 112;//tableK[ 119]= 600;
  tableS10[ 120]= 112;//tableK[ 120]= 605;
  tableS10[ 121]= 111;//tableK[ 121]= 610;
  tableS10[ 122]= 110;//tableK[ 122]= 615;
  tableS10[ 123]= 110;//tableK[ 123]= 620;
  tableS10[ 124]= 109;//tableK[ 124]= 625;
  tableS10[ 125]= 108;//tableK[ 125]= 630;
  tableS10[ 126]= 108;//tableK[ 126]= 635;
  tableS10[ 127]= 107;//tableK[ 127]= 640;
  tableS10[ 128]= 107;//tableK[ 128]= 645;
  tableS10[ 129]= 106;//tableK[ 129]= 650;
  tableS10[ 130]= 105;//tableK[ 130]= 655;
  tableS10[ 131]= 105;//tableK[ 131]= 660;
  tableS10[ 132]= 104;//tableK[ 132]= 665;
  tableS10[ 133]= 103;//tableK[ 133]= 670;
  tableS10[ 134]= 103;//tableK[ 134]= 675;
  tableS10[ 135]= 102;//tableK[ 135]= 680;
  tableS10[ 136]= 101;//tableK[ 136]= 685;
  tableS10[ 137]= 100;//tableK[ 137]= 690;
  tableS10[ 138]= 100;//tableK[ 138]= 695;
  tableS10[ 139]= 99;//tableK[ 139]= 700;
  tableS10[ 140]= 98;//tableK[ 140]= 705;
  tableS10[ 141]= 98;//tableK[ 141]= 710;
  tableS10[ 142]= 97;//tableK[ 142]= 715;
  tableS10[ 143]= 96;//tableK[ 143]= 720;
  tableS10[ 144]= 96;//tableK[ 144]= 725;
  tableS10[ 145]= 95;//tableK[ 145]= 730;
  tableS10[ 146]= 94;//tableK[ 146]= 735;
  tableS10[ 147]= 93;//tableK[ 147]= 740;
  tableS10[ 148]= 93;//tableK[ 148]= 745;
  tableS10[ 149]= 92;//tableK[ 149]= 750;
  tableS10[ 150]= 91;//tableK[ 150]= 755;
  tableS10[ 151]= 90;//tableK[ 151]= 760;
  tableS10[ 152]= 90;//tableK[ 152]= 765;
  tableS10[ 153]= 89;//tableK[ 153]= 770;
  tableS10[ 154]= 88;//tableK[ 154]= 775;
  tableS10[ 155]= 87;//tableK[ 155]= 780;
  tableS10[ 156]= 86;//tableK[ 156]= 785;
  tableS10[ 157]= 86;//tableK[ 157]= 790;
  tableS10[ 158]= 85;//tableK[ 158]= 795;
  tableS10[ 159]= 84;//tableK[ 159]= 800;
  tableS10[ 160]= 83;//tableK[ 160]= 805;
  tableS10[ 161]= 82;//tableK[ 161]= 810;
  tableS10[ 162]= 82;//tableK[ 162]= 815;
  tableS10[ 163]= 81;//tableK[ 163]= 820;
  tableS10[ 164]= 80;//tableK[ 164]= 825;
  tableS10[ 165]= 79;//tableK[ 165]= 830;
  tableS10[ 166]= 78;//tableK[ 166]= 835;
  tableS10[ 167]= 77;//tableK[ 167]= 840;
  tableS10[ 168]= 76;//tableK[ 168]= 845;
  tableS10[ 169]= 75;//tableK[ 169]= 850;
  tableS10[ 170]= 74;//tableK[ 170]= 855;
  tableS10[ 171]= 73;//tableK[ 171]= 860;
  tableS10[ 172]= 72;//tableK[ 172]= 865;
  tableS10[ 173]= 71;//tableK[ 173]= 870;
  tableS10[ 174]= 70;//tableK[ 174]= 875;
  tableS10[ 175]= 69;//tableK[ 175]= 880;
  tableS10[ 176]= 68;//tableK[ 176]= 885;
  tableS10[ 177]= 67;//tableK[ 177]= 890;
  tableS10[ 178]= 66;//tableK[ 178]= 895;
  tableS10[ 179]= 65;//tableK[ 179]= 900;
  tableS10[ 180]= 64;//tableK[ 180]= 905;
  tableS10[ 181]= 62;//tableK[ 181]= 910;
  tableS10[ 182]= 61;//tableK[ 182]= 915;
  tableS10[ 183]= 60;//tableK[ 183]= 920;
  tableS10[ 184]= 58;//tableK[ 184]= 925;
  tableS10[ 185]= 57;//tableK[ 185]= 930;
  tableS10[ 186]= 56;//tableK[ 186]= 935;
  tableS10[ 187]= 54;//tableK[ 187]= 940;
  tableS10[ 188]= 52;//tableK[ 188]= 945;
  tableS10[ 189]= 51;//tableK[ 189]= 950;
  tableS10[ 190]= 49;//tableK[ 190]= 955;
  tableS10[ 191]= 47;//tableK[ 191]= 960;
  tableS10[ 192]= 45;//tableK[ 192]= 965;
  tableS10[ 193]= 42;//tableK[ 193]= 970;
  tableS10[ 194]= 40;//tableK[ 194]= 975;
  tableS10[ 195]= 37;//tableK[ 195]= 980;
  tableS10[ 196]= 34;//tableK[ 196]= 985;
  tableS10[ 197]= 29;//tableK[ 197]= 990;
  tableS10[ 198]= 24;//tableK[ 198]= 995;
  tableS10[ 199]= 11;//tableK[ 199]= 1000;
    
    // Заполняем таблицу квадратов от 90 - минимаальное значени напряжение при котором работает блок питания контролера до
    // 255 - ну просто максимальное среднеквадратичное значение напряжения в сети.
    #ifdef TESTRM
      Serial.println("TABLESQRT");
    #endif
    for(i=0;i<MAX_TABLE_SQRT;i++)
    {  tableSQRT[i]=(int) i*5+125;
       tableSQ[i]=(unsigned int) tableSQRT[i]*tableSQRT[i];
       #ifdef TESTRM
          Serial.print((unsigned int)i);
          Serial.write(9);
          Serial.print((unsigned int)tableSQRT[i]);
          Serial.write(9);
          Serial.println((unsigned int)tableSQ[i]);
       #endif
    }
    #ifdef TESTRM
      Serial.println("END TABLESQRT");
    #endif


    #ifdef TEST
      Serial.println("TEST MY_SQRT");
      int j;

      for(j=0;j<=600;j++)
      {
          Serial.print((unsigned int) j);
          Serial.write(9);
          Serial.print((unsigned long) j*j );
          Serial.write(9);
          Serial.print((unsigned int) sqrt((unsigned long) j*j) );
          Serial.write(9);
          Serial.print(my_sqrt((unsigned long) j*j) );
          Serial.write(9);
          Serial.println((long)my_sqrt((unsigned long) j*j)- (long) sqrt((unsigned long) j*j));
      }
      Serial.println("END TEST MY_SQRT");
    #endif
    StateVolts=0;
    // Состояние модема - инициализация   
    flGPRSState=98;
    // После инициализации переходим в нулевое состояние.
    stateAfterDelete=0;
    // На инициализации даем максимум 120 секунд
    timeGPRS=120;
    MaxVoltsOut=0;
    MaxIOut=0;
    
}

// Прерывание управления симистором.
ISR(TIMER5_COMPA_vect) 
{
  #if USE_ASC712==0
  // Включаем симистор
  if (UstPower>0 && StateMachine<100)
  {
    // Если НПГ инициализирован, токда подаем питание на ТЭНы, чтобы их не спалить при не заполненном НПГ
    if (StateNPG>1) digitalWrite(PIN_TRIAC, HIGH);  
    // ждем 20 мкс и выключаем симистор
  }
  delayMicroseconds(20);
  if (TimeOpenTriacFact!=9999)
  {
    digitalWrite(PIN_TRIAC, LOW);
  }
  #else
    // Отключаем симистор только если установленная мощность меньше заданной
    if (UstPwrPH1<PowerPhase[0]) digitalWrite(PIN_TRIAC, LOW);
  #endif

}

// Прерывание управления симистором клапанов
ISR(TIMER5_COMPB_vect) 
{
  #if USE_ASC712==0 && SIMPLED_VERSION!=20
  char i;  
  for (i=0;i<MAX_KLP;i++)
  { // Если клапан в режиме ШИМ и Состояние клапана включен, то включаем его 
    #if SIMPLED_VERSION<30
       if (KlReg[i]==1 && KlState[i]==1) digitalWrite(PIN_KLP_BEG+i, KLP_HIGH); 
    #else
       if (KlReg[i]==1 && KlState[i]==1) digitalWrite(PIN_KLP_BEG+i+i, KLP_HIGH);    
    #endif 
  }
  
  if (TimeOpenKLP!=9999 && !flNoPhase) // Если клапана не в режиме максимальной мощности или фазовое управление отключено
  {
    delayMicroseconds(20);       
  
    for (i=0;i<MAX_KLP;i++)
    { // Если клапан в режиме ШИМ и Состояние клапана включен, то выключаем его 
      #if SIMPLED_VERSION<30
        if (KlReg[i]==1 && KlState[i]==1) digitalWrite(PIN_KLP_BEG+i, !KLP_HIGH);  
      #else
       if (KlReg[i]==1 && KlState[i]==1) digitalWrite(PIN_KLP_BEG+i+i, !KLP_HIGH);  
      #endif
    }
  }
  //  digitalWrite(PIN_KLP_BEG+1, HIGH);  
  //  // ждем 20 мкс и выключаем симистор
  //  delayMicroseconds(20);       
  //  digitalWrite(PIN_KLP_BEG+1, LOW); 
  #endif
  // Взводим таймер открытия фазы дефлегматора
  
 
}

// Прерывание управления симистором двигателя
ISR(TIMER5_COMPC_vect) 
{
  // Взводим таймер открытия фазы голов

}




// Процедура открывает клапана, которые необходимо открыть.

void OpenKLP()
{
  char i;  
  for (i=0;i<MAX_KLP;i++)
  { // Если клапан в режиме ШИМ и Состояние клапана включен, то включаем его 
    #if SIMPLED_VERSION<30
      if (KlReg[i]==1 && KlState[i]==1) digitalWrite(PIN_KLP_BEG+i, KLP_HIGH); 
    #else
      if (KlReg[i]==1 && KlState[i]==1) digitalWrite(PIN_KLP_BEG+i+i, KLP_HIGH); 
    #endif 
  }
}

char fl_read; // Признак того, что надо считать средреквадратичное


void ReadAlarm()
{
  boolean UprNBK;
  ProcessNPG();

 // Если уровень тревоги равен единице, то считаем, что датчик цифровой
  if  (USE_ALARM_VODA)
  {
      if (UROVEN_ALARM!=1)  U_VODA=analogRead(PIN_ALARM_VODA);    
                       else U_VODA=!digitalRead(PIN_ALARM_VODA); // Датчики влажности инверсные, то есть при не сработавшем состоянии у них на выходе 1, при сработке 0
                       
      if (U_VODA>=UROVEN_ALARM )
      {
        CountAlarmVoda++;
        
#ifdef TEST
        sprintf(my_tx_buffer,"ALARM U_VODA=%i cnt=%i",U_VODA,CountAlarmVoda);
        Serial.println(my_tx_buffer);
#endif      

        if (CountAlarmVoda>COUNT_ALARM)
        {
          DispDopInfo=2;
          if (IspReg==129) DispPage=4;
          
          if (flNeedCall==0) flNeedCall=1;
          // По прошествии минуты устанавливаем флаг сработки.
          if (CountAlarmVoda>COUNT_ALARM_VODA)
          {
            DispDopInfo=0;
    #ifndef TEST
    #ifndef DEBUG
    #ifndef TESTGSM
    #ifndef TESTGSM1
              IspReg=250;      // Переводим контроллер в режим тревоги, если не установлен тестовый режим.
    #endif
    #endif
    #endif
    #endif
          }
        }
      }
      else 
      {
        if (DispDopInfo==2)
        { flNeedCall=0; 
          DispDopInfo=0;
        }
        CountAlarmVoda=0;
      }
  }

  if (USE_ALARM_UROVEN)
  {
    if (UROVEN_ALARM!=1)  U_UROVEN=analogRead(PIN_ALARM_UROVEN);
                     else U_UROVEN=!digitalRead(PIN_ALARM_UROVEN); // Датчики влажности инверсные, то есть при не сработавшем состоянии у них на выходе 1, при сработке 0
                     
      // Для фракционной перегонки первой фракции считаем сработкой датчика уровня также и сработку датчика голов. Чтобы можно было отбирать головы через один клапан, тело через другой
      // без фракционника.
      if (IspReg==117 && TekFraction==0)
      {
          if (UROVEN_ALARM!=1)  U_UROVEN=U_UROVEN+analogRead(PIN_PROVODIMOST_SR);
                           else U_UROVEN=U_UROVEN || !digitalRead(PIN_PROVODIMOST_SR); // Датчики влажности инверсные, то есть при не сработавшем состоянии у них на выходе 1, при сработке 0
      
      }
        // Предупреждение по уровню генерируем только уже не произошла сработка уроня
        // Если произошла, то уже смысла анализировать нет
      if (U_UROVEN>=UROVEN_ALARM  && flAlarmUroven==0)
      {
        CountAlarmUroven++;
#ifdef TEST
        sprintf(my_tx_buffer,"ALARM U_UROVEN=%i cnt=%i",U_UROVEN,CountAlarmUroven);
        Serial.println(my_tx_buffer);
#endif      
        if (CountAlarmUroven>COUNT_ALARM )
        {
          DispDopInfo=1;
          if (flNeedCall==0) flNeedCall=1;
          // По прошествии минуты устанавливаем флаг сработки.
          if (CountAlarmUroven>COUNT_ALARM_UROVEN || ((IspReg==117 || IspReg==118) && CountAlarmUroven>COUNT_ALARM_UROVEN_FR) )
          {
            flAlarmUroven=1;
            flNeedAnalyse=1;
            DispDopInfo=0;
          }
        } 
      }
      else 
      {
        if (DispDopInfo==1)
        { flNeedCall=0; 
          DispDopInfo=0;
        }
        CountAlarmUroven=0;
      }
  }

  if (USE_GAS_SENSOR)
  {
    if (UROVEN_GAS_SENSOR!=1) U_GAS=analogRead(PIN_GAS_SENSOR);
                         else U_GAS=digitalRead(PIN_GAS_SENSOR); 
                         
    //      Если датчик еще не прогрелся, то выводим значение времени, оставшееся для прогрева датчика паров спирта
    if (Seconds<TIME_PROGREV_GAS_SENSOR) U_GAS=TIME_PROGREV_GAS_SENSOR-Seconds;
    else
    {

        if (U_GAS>=UROVEN_GAS_SENSOR) 
        {
          CountGasSensor++;
          #ifdef TEST
          sprintf(my_tx_buffer,"ALARM U_GAS=%i cnt=%i",U_GAS,CountGasSensor);
          Serial.println(my_tx_buffer);
          #endif  
    
          if (CountGasSensor>COUNT_GAS_SENSOR )
          {
            flNeedAnalyse=1;
            #ifndef TEST
            IspReg=252;      // Переводим контроллер в режим тревоги, если не установлен тестовый режим.
            #endif
          }
        }
        else
          CountGasSensor=0;
    }
  }
  

  #ifdef PIN_MPX5010
    U_MPX5010=analogRead(PIN_MPX5010);

    
    //      U_MPX5010=(U_MPX5010*4)/5-16;
    U_MPX5010=(U_MPX5010*4-160)/5;
    if (FlState!=253) U_MPX5010=U_MPX5010+P_MPX5010;
    
    if (U_MPX5010>MaxPressByPeriod) MaxPressByPeriod=U_MPX5010;
    
    if (U_MPX5010>=AlarmMPX5010 && AlarmMPX5010>0)
    {
      CountAlarmMPX5010++;
#ifdef TEST
      sprintf(my_tx_buffer,"ALARM MPX5010=%i cnt=%i",AlarmMPX5010,CountAlarmMPX5010);
      Serial.println(my_tx_buffer);
#endif      

      if (CountAlarmMPX5010>COUNT_ALARM)
      {
        DispDopInfo=5;
        if (flNeedCall==0) flNeedCall=1;
        // По прошествии минуты устанавливаем флаг сработки.
        if (CountAlarmUroven>MAX_ERR_MPX5010)
        {
          flAlarmMPX5010=1;
          flNeedAnalyse=1;
          DispDopInfo=0;
        }
      } 
    }
    else 
    {
      if (DispDopInfo==5) 
      {  DispDopInfo=0;
         flNeedCall=0;
      } 
      CountAlarmMPX5010=0;
    }
    
  #endif

  if (UrovenProvodimostSR!=0 || IspReg==112 || FlState==232 || IspReg==129) // Читаем датчик уровня голов, если установлено, что его надо читать, либо если в данный момент используется режим НБК.
  {
    if (UrovenProvodimostSR!=1) U_GLV=analogRead(PIN_PROVODIMOST_SR);
                             else U_GLV=!digitalRead(PIN_PROVODIMOST_SR);// Датчики влажности инверсные, то есть при не сработавшем состоянии у них на выходе 1, при сработке 0
  } 
  
  // Для НБК датчик голов служит датчиком заполнения барды.
  if (IspReg==112 || IspReg==129) // Если режим - НБК, то открываем слив барды
  {
    if (timeBRD==0) // Если датчик уже нужно анализировать
    {
      if (U_GLV>=UrovenBarda) // Если датчик НБК сработал, то открываем клапан слива барды
      {
         KlOpen[KLP_BARDA]=PER_KLP_OPEN;
         KlClose[KLP_BARDA]=PER_KLP_CLOSE;
         if (timeOpenBRD>0) timeBRD=timeOpenBRD;
                   else timeBRD=2;
         
      }
      else // Если не сработал, то закрываем.
      {
         if(IspReg!=129)
         { // Клапан барды в режиме тестирования не закрываем.е
           KlOpen[KLP_BARDA]=0;
           KlClose[KLP_BARDA]=100;
          UprNBK=LOW;
          if (UprNasosNBK==0 || UprNasosNBK==1) UprNBK=!UprNBK;
          digitalWrite(PIN_NASOS_NBK_BRD,UprNBK);  
         } 
      }
    }
  }
  
 
}


// Прерывание вызывается 125 раз в секунду
ISR(TIMER2_COMPA_vect) 
{
  boolean UprNBK;
  // Сброс таймера
  TCNT2=0x00;


  Counter1++;
  if (Counter1==62)
  {
    NeedDisplaying=true;
    flNeedReadAlarm=true;
    NeedGPRS=1;
  }

  // Насосом НБК все равно управляем ( в режим НБК кстати не такие высокие требования к мощности)
  if (SpeedNBK>0) 
  {  
    if (timeNBK>0)
     { 
        b_value[1]=SpeedNBK;
        UprNBK=bresenham_getNext(1);
        if (UprNasosNBK==0 || UprNasosNBK==2 ) UprNBK=!UprNBK;
        digitalWrite(PIN_NASOS_NBK,UprNBK);
     }
  }
  // Насосом бардоотводчика управляем всегда, если он включен.
   if (timeOpenBRD<0 && timeBRD>0)
   {
      
      b_value[2]=(int)SpeedNBKDst*(int)(-timeOpenBRD)/10;
      UprNBK=bresenham_getNext(2);
      if (UprNasosNBK==0 || UprNasosNBK==1) UprNBK=!UprNBK;
      digitalWrite(PIN_NASOS_NBK_BRD,UprNBK);  
   }
   else
   {
      UprNBK=LOW;
      if (UprNasosNBK==0 || UprNasosNBK==1) UprNBK=!UprNBK;
      digitalWrite(PIN_NASOS_NBK_BRD,UprNBK);  
   }

  
  if (StateVolts==1) return;

  if (Counter1 % 25 ==0)
  {
//    Sec5++;
  }
  
  

  #ifdef DEBUG
    if (Serial.available()) return;
  #endif
  // Сканируем клавиатуру три раза секунду или если находимся не в режиме отображения информации, то постоянно, то есть 125 раз в секунду


  if (BeepTime>1) BeepTime--;
  else
  {
    if (BeepTime==1) analogWrite(PIN_SOUND, 0);
    BeepTime=0;
  }

  //  

  // Следим за защитой от дребезга клавиш
  if (Counter1 % 20 ==0)
  {          
    if (CountKeys>0 && flScanKbd==0) CountKeys--; 
    flNeedScanKbd=1; 

  }


//  if (Counter1 % 32 ==0)
//  {
//    if (StateVolts==4) 
//    {
//      StateVolts=0; // Раз в 1/3 секунды запускаем измерение среднеквадратичного (в дальнейшем можно будет настроить, 
//      // чтобы оно делалось так часто, насколько позволяют возможности контроллера).
//      if (CorrectASC712)  flCorrASC712=!flCorrASC712; // Читаем через раз, то мощность, то напряжение если, конечно установлено что надо считать мощность.
//      
//    }
//
//  }
  
  if (Counter1>=125)
  {
//    analogWrite(PIN_LIGHT,100);
    if (CountKeys>0) 
    { 
      if (secPressKeys>0) { AddPressKeys=1;  AddPressKeys10=10;}
      if (secPressKeys>3) {AddPressKeys=5;   AddPressKeys10=20;}
      if (secPressKeys>5) {AddPressKeys=10;  AddPressKeys10=50;}
      if (secPressKeys>10) {AddPressKeys=100;AddPressKeys10=100;}
      if (secPressKeys>15) {AddPressKeys=200;AddPressKeys10=250;}
      if (secPressKeys<200) secPressKeys++;
    }
    else
    {
      secPressKeys=0;
      AddPressKeys=1;
      AddPressKeys10=10;
    }
    zPSOut=zPS;
    zPS=0;
    if (time1>0) time1--;
            else time1=0;
    if (time2>0) time2--;
            else time2=0;
    if (time3>0) time3--;
            else time3=0;
    if (timeGPRS!=0) timeGPRS--;
    if (timeBRD!=0) timeBRD--;
    
    if (timeWaitGPRS!=0) timeWaitGPRS--;
    
    if (timeRefrServer!=0) timeRefrServer--;
    else
    {
      // Если не находимся в режиме обновления сервера, тогда установим необходимость сессии GPRS  
        // Если GPRS используется и не переведено в режим отправки SMS и сессия GPRS не активирована и задан период опроса сервера.
        if (FlToGSM>1 && !SMSOnCall && timeGPRS==0 && PeriodRefrServer!=0 && (flGPRSState==0 || flGPRSState==98))
          {
            ErrGPRSConnectInernet=0;
            ErrGPRSConnectServer=0;
            if (flGPRSState==0) flGPRSState=14; // Активируем сессию GPRS
            flNeedRefrServer=1;
            timeGPRS=60;
          }
    }

    if (timeNBK>0) 
    {  
      // Задаем скорость насосу. 
      timeNBK--;
    }
    else
    {  
      // убираем насос.
      timeNBK=0;
    }

    if (timeNBK==0 || SpeedNBK==0 ) 
    {  
      // убираем насос НБК.
        UprNBK=LOW;
        if (UprNasosNBK==0 || UprNasosNBK==2 ) UprNBK=!UprNBK;
        digitalWrite(PIN_NASOS_NBK,UprNBK);
    }

    if (timeMIXER>0)
    { 
      timeMIXER--;
      digitalWrite(PIN_MIXER,RELAY_HIGH);
      if (IspReg== 108 || IspReg== 113 || IspReg== 114 || IspReg==112) // Если идет процесс разварки зерна,  то включаем мешалку, которую возможно подключить через реле на клапан голов
      {
          KlOpen[KLP_GLV_HVS]=40;
          KlClose[KLP_GLV_HVS]=0;
      }
    }
    else
    {
      timeMIXER=0;
      digitalWrite(PIN_MIXER,!RELAY_HIGH);
      if (IspReg== 108 || IspReg== 113 || IspReg== 114 || IspReg== 112) // Если идет процесс разварки зерна, то отключаем мешалку, которую возможно подключить через реле на клапан голов
      {
          KlOpen[KLP_GLV_HVS]=0;
          KlClose[KLP_GLV_HVS]=40;
      }
    }

    if (CountState>0) 
    {   
      if (CountState==1) lcd.clear();
      CountState--;
    }
    Seconds++;
    Counter1=0;
    NeedDisplaying=true;
    NeedGPRS=1;
    flNeedReadAlarm=true;
    flCrossZero++;
    if (flCrossZero>80)
    {
      // Если находимся не в режиме термостата или просмотра, и пропадание напряжения не появилось в результате нормального отключения дифавтомата,то переводим в режим тревоги.
      // Если не находимся в режиме тестирования
#ifndef TEST
#ifndef TESTRM
#ifndef DEBUG
#ifndef TESTGSM
#ifndef TESTGSM1
      if (IspReg>102 && (!USE_DIFAVTOMAT || !flAllOff))
      {
        // Выдаем ошибку детектирования нуля.
        IspReg=249;

      }
#endif
#endif
#endif
#endif
#endif
      StateVolts=0;
      flCrossZero=0;
    }
    
    
    // Если контроллер используется как термостат, или это уже чтение готового результата, или при чтении информации датчиков произошла ошибка то читаем раз в секунду.
    // Или мы сейчас вводим поправки.
    if (IspReg==102 || StepOut!=0 || FlState==300 || CntErrDs18) 
    {
      flNeedTemp=1; // Устанавливаем флаг того, что надо читать температуру
    }
    else
    {
      // Иначе запускаем расчет один раз в 5 секунд (чтоб поменьше мограл регулятор мощности).
      if (Seconds % 5==0) flNeedTemp=1; 
    }

  }
  // Выполняем процесс.

  if (CountState<=0 && flScanKbd==0) 
  {
    FlState=0;
    CountState=0;
  }  

}



// Функция вызывается по прерыванию нуля от нуля сети 220 вольт
boolean br_on; // Признак включения по брезенхему
unsigned char CntPeriodPause=0; // Число полу-периодов паузы

//void zero_crosss_int()  // function to be fired at the zero crossing to dim the light
ISR(TIMER3_COMPA_vect)
{
  #ifdef DEBUG
    if (Serial.available()) return;
  #endif

  unsigned char i;
  unsigned char flOpenClose;
  zPS++;
  flCrossZero=0;
  
  TicZero=(int) TCNT5L | (int) (TCNT5H<<8); // Расчитывааем текущее значение таймера
  // Сбрасываем 5-й таймер.
  TCNT5H=0x00;
  TCNT5L=0x00;
    
  if (StateVolts==1 ) 
   { 
    // Если за полу-период не выбрано допуустимое количество значений, тогда пропускаем эти выборки и пытаемся выбрать следующий полу-период.
    // также пропускаем выборку если управление по брезенхему и предыдущий период симистор был отключен
    
     if (index_input-prev_index_input>MAX_INDEX_BY_PERIOD || index_input-prev_index_input<MIN_INDEX_BY_PERIOD || !br_on) 
     {
        index_input=prev_index_input;
        SqNaprT=SqNaprPrevT;
     }
    else 
      CntPeriod++; // Если число в выборке допустимо, то прибавляем счетчик полу-периодов выборки
    
    // Если считали достаточное количество полу-периодов, тогда расчет закончен.
    if (CntPeriod>=CNT_PERIOD)
      {
        if (flCorrASC712==0) indexOut=index_input;
        StateVolts=2;// Если был расчет тока (напряжения), то ставим признак его окончания
        CntPeriodPause=0; 
      }
   } 
  else
   {
     CntPeriodPause++;
   } 

  if (StateVolts==4)
  {
    // Если пропустили два по необходимому числу полу-периодов, запускаем расчет.
    // Еще одно условие, чтобы мы не находились в режиме ввода клавиш, тогда расчет не запускаем, чтобы не тормозить отчаянно 
    // при вводе информации
    if (CntPeriodPause > CNT_PERIOD && CountKeys==0) 
    { 
      StateVolts=0;
      if (CorrectASC712)  flCorrASC712=!flCorrASC712; // Читаем через раз, то мощность, то напряжение если, конечно установлено что надо считать мощность.
    }
  }
   
  if (StateVolts==0) 
  {
      index_input=0;
      StateVolts=1; 
 //     MaxVolts=0;
      #if SIMPLED_VERSION!=20
      TCNT4H=0x00;
      TCNT4L=0x00;
      #endif
      CntPeriod=0;
      SqNaprT=0;
  }
    
  prev_index_input=index_input;
  SqNaprPrevT=SqNaprT;
  br_on=true;    

  #if USE_ASC712==0
  if (UstPwrPH1>=PowerPhase[0])
  {
    // Если мощность больше или равна номинальной, включаем триак всегда, при этом прерывание на закрытие никогда не выполнится.  
    TimeOpenTriac=9999;
    TimeOpenTriacFact=9999;
    // Если НПГ инициализирован, токда подаем питание на ТЭНы, чтобы их не спалить при не заполненном НПГ
    if (StateNPG>1) digitalWrite(PIN_TRIAC, HIGH);  
  }
  else
  {
    TimeOpenTriacFact=TimeOpenTriac;
  }

  // Если мощность не установлена отключена, или состояние машины конечное, то на всякий случай отключаем
  if (UstPower<=0 || StateMachine>=100) digitalWrite(PIN_TRIAC, LOW);

  if (TimeOpenTriac<=10)
  {
    // Если НПГ инициализирован, токда подаем питание на ТЭНы, чтобы их не спалить при не заполненном НПГ
    if (StateNPG>1 && UstPower>0) digitalWrite(PIN_TRIAC, HIGH);  
    TimeOpenTriacFact=9999;
  }

  //    TimeOpenTriacFact=TimeOpenTriac;
  //TimeOpenTriac=2000;
  // В прерывании нуля настраиваем, чтобы прерывание на 5-м таймере выдалось через то время, которое мы ранее рассчитали.
  OCR5AH=(char)(TimeOpenTriacFact>>8);
  OCR5AL=(char)TimeOpenTriacFact;

  if (TimeOpenKLP==0) TimeOpenKLP=9999;
  OCR5BH=(char)(TimeOpenKLP>>8);
  OCR5BL=(char)TimeOpenKLP;
  
  // Если считаем ток(напрряжение), тогда 

  
  #else
     // Устанавливаем закрытие симистора по прерыванию таймера через некоторое время (отступить от таблицы мощности на 2 шага вполе достаточно).
     TimeOpenTriacFact=tableS10[MAX_TABLE_T-2];
     TimeOpenTriacFact*=10;
     TimeOpenTriac=TimeOpenTriacFact;
     OCR5AH=(char)(TimeOpenTriacFact>>8);
     OCR5AL=(char)TimeOpenTriacFact;
    // Если процент брезенхема не рассчитан, а мощность установлена, то запускаем измерительный период
    if (UstPower>0 && b_value[0]==0 || UstPwrPH1>=PowerPhase[0]) digitalWrite(PIN_TRIAC,HIGH);
     else
       if (UstPower<=0) digitalWrite(PIN_TRIAC,LOW);  // Если мощность не установлена, то на реле подаюм нулевой сигнал
         else
           {
             br_on=bresenham_getNext(0);
             digitalWrite(PIN_TRIAC,br_on); // Иначе стандартно вызываем алгоритм брезехема.
           }
  #endif
  

// фазовое управление давигателем пока не используем 
//    TimeOpenDvigatel=5000; 
//    if (SpeedNBK>0)
//    {
//      TimeOpenDvigatel=TicZero/100;
//      TimeOpenDvigatel=TimeOpenDvigatel*SpeedNBK;
//      TimeOpenDvigatel=TicZero-TimeOpenDvigatel;
//      if (SpeedNBK>=100) digitalWrite(PIN_DVIGATEL, HIGH);  
//    }
//
//    OCR5CH=(char)(TimeOpenDvigatel>>8);
//    OCR5CL=(char)TimeOpenDvigatel;

  // В прерывании нуля управляем открытием клапанов (а их лучше открывать по 0, чтобы не было помех)
  // Синхронизацию пока отключим
  //    if (!flSyncKLP) // Если клапана не находятся в режиме синхронизации
  {
    for (i=0;i<MAX_KLP;i++)
    {
      // Разбираемся с состоянием клапана
      if (KlState[i]==0)
      {
        flOpenClose=0;
        // Состояние клапана - выключен            
        if (KlCount[i]>KlClose[i]) // Если количество тиков превышает установленное для состояния "закрыто", переводим клапан в состояние "открыто"
        {
          // Клапан переводим в состояние-включен
          KlState[i]=1;
          KlCount[i]=0;
          if(KlOpen[i]) flOpenClose=1;
        } 
      }
      else
      { 
        flOpenClose=1;
        // Состояние клапана - выключен
        if (KlCount[i]>KlOpen[i]) // Если количество тиков превышает установленное для состояния "открыто", переводим клапан в состояние "открыто"
        {
          KlState[i]=0;
          KlCount[i]=0;
          if(KlClose[i]) flOpenClose=0;
        } 
      }

      // Если клапан в принципе не должен выключаться то всегда переводим в состояние влкючен
      if (KlClose[i]==0) 
      {  
        flOpenClose=1;
        KlState[i]=1; 
      }
      // Если клапан в принципе не должен включатся,то всегда переводим  в состояние выключаем
      if (KlOpen[i]==0) 
      {
        flOpenClose=0;
        KlState[i]=0; 
        KlCount[i]=0;
      }

      // С состоянием клапана разобрались, тепер в зависимости от состояния включаем или выключаем его

      if (!flOpenClose)
      {  // Если клапан-выключен, то выключаем его в любом случае
        #if SIMPLED_VERSION<30
          digitalWrite(PIN_KLP_BEG+i, !KLP_HIGH);  
        #else
          digitalWrite(PIN_KLP_BEG+i+i, !KLP_HIGH);  
        #endif
      }
      else
      {  // Если состояние клапана - включен, включаем его только в том случае, если клапан в находится в режиме ШИМ
        // Или клапан находится в режиме фазового управления, но включен менее, чем полсекунды назад, или время фазового управления еще не рассчитано, то включаем его тоже,
        // иначе для режима фазового управления клапан включится по принципу фазового управления в прерывании. Или временно фазовое управления убрано
        if (KlReg[i]==0 || KlCount[i]<50 || TimeOpenKLP==9999 || flNoPhase)
        #if SIMPLED_VERSION<30
          digitalWrite(PIN_KLP_BEG+i, KLP_HIGH);
        #else
           digitalWrite(PIN_KLP_BEG+i+i, KLP_HIGH);
        #endif
      }

      KlCount[i]++;
    }    
  }

}


char flNumStrPrint=0;


void writeEEPROM()
{ 
  int i; 
  flAutoDetectPowerTEN=0;
  ee_addr=0;
  eeWriteChar(PR_REWRITE_EEPROM);
  eeWriteChar(IspReg);
  eeWriteInt(Power);
  eeWriteInt(UstPowerReg);
  eeWriteInt(TempTerm);
  eeWriteInt(Delta);
  eeWriteChar(FlToUSART);
  eeWriteChar(FlToGSM);
  eeWriteInt(Temp1P);
  eeWriteInt(Temp2P);
  eeWriteInt(Temp3P);
  for(i=0;i<MAX_DS1820;i++)
    eeWriteChar(ds1820_popr[i]);

  eeWriteInt(tEndRectRazgon);
  eeWriteInt(PowerRect);

  eeWriteInt(tEndRectOtbGlv);
  eeWriteInt(timeChimRectOtbGlv);
  eeWriteChar(ProcChimOtbGlv);
  eeWriteInt(timeChimRectOtbSR);
  eeWriteChar(tDeltaRect);
  eeWriteInt(tEndRectOtbSR);
  eeWriteInt(tEndRect);
  eeWriteInt(PowerGlvDistil);
  eeWriteInt(PowerDistil);

  eeWriteInt(TempDeflBegDistil);
  eeWriteInt(TempDefl);
  eeWriteChar(DeltaDefl);

  eeWriteChar(BeepEndProcess);
  eeWriteChar(BeepStateProcess);
  eeWriteChar(BeepKeyPress);

  eeWriteInt(PowerRazvZerno);

  eeWriteInt(tEndDistDefl);
  eeWriteChar(minProcChimOtbSR);
  eeWriteInt(PowerVarkaZerno);
  eeWriteInt(TempKipenZator);
  eeWriteChar(TempZSPSld);
  eeWriteChar(TempZSP);
  eeWriteInt(PowerNBK);
  eeWriteChar(SpeedNBKDst);
  eeWriteInt(PowerMinute);
  eeWriteChar(begProcChimOtbSR);
  eeWriteInt(P_MPX5010);
  eeWriteChar(PeriodRefrServer);
  eeWriteInt(NaprPeregrev);
  eeWriteInt(UrovenBarda);
  eeWriteInt(UrovenProvodimostSR);
  eeWriteInt(TimeStabKolonna);
  eeWriteChar(DecrementCHIM);
  eeWriteChar(IncrementCHIM);
  eeWriteInt(TimeAutoIncCHIM);
  eeWriteChar(CntCHIM);
  for(i=0;i<COUNT_CHIM;i++)
  {
    eeWriteInt(tempK[i]);
    eeWriteChar(CHIM[i]);
  }
  eeWriteInt(TimeRestabKolonna);
  eeWriteChar(CntPause);
  for(i=0;i<10;i++)
  {
    eeWriteChar(tempP[i]);
    eeWriteChar(timeP[i]);
  }
  eeWriteChar(CorrectASC712);
  eeWriteChar(ip[0]);
  eeWriteChar(ip[1]);
  eeWriteChar(ip[2]);
  eeWriteChar(ip[3]);
  eeWriteInt(ipPort);
  for(i=0;i<10;i++) eeWriteChar(idDevice[i]);
  for(i=0;i<12;i++) eeWriteChar(my_phone[i]);
  eeWriteInt(AlarmMPX5010);
  eeWriteChar(FlAvtonom);
  eeWriteChar(timeOpenBRD);
  eeWriteChar(PIDTemp[0]);
  eeWriteChar(PIDTemp[1]);
  eeWriteChar(PIDTemp[2]);
  for(i=0;i<11;i++) eeWriteChar(WiFiAP[i]);
  for(i=0;i<11;i++) eeWriteChar(WiFiPass[i]);
  
  eeWriteChar(CountFractionDist);
  for(i=0;i<MAX_CNT_FRACTION_DIST;i++) 
  {
    eeWriteInt(TempFractionDist[i]);
    eeWriteInt(PowerFractionDist[i]);
    eeWriteChar(AngleFractionDist[i]);
  }
  
  eeWriteChar(CountFractionRect);
  for(i=0;i<MAX_CNT_FRACTION_RECT;i++) 
  {
    eeWriteInt(TempFractionRect[i]);
    eeWriteChar(AngleFractionRect[i]);
  }
  
  eeWriteChar(TempHLDZatorBrog1);
  eeWriteInt(PowerPhase[0]);
  eeWriteInt(PowerPhase[1]);
  eeWriteInt(PowerPhase[2]);
  eeWriteChar(KtPhase[0]);
  eeWriteChar(KtPhase[1]);
  eeWriteChar(KtPhase[2]);
  
  for(i=10;i<MAX_CNT_PAUSE;i++)
  {
    eeWriteChar(tempP[i]);
    eeWriteChar(timeP[i]);
  }
  eeWriteChar(minPressNBK);
  eeWriteChar(deltaPressNBK);
  eeWriteChar(timePressNBK);  
  eeWriteChar(UprNasosNBK);  
  eeWriteChar(ProcChimOtbCP);  
}

void readEEPROM()
{
  int i; 
  unsigned char ctmp;
  ee_addr=0;
  eeReadChar();
  IspReg=eeReadChar();
  Power=eeReadInt();
  UstPowerReg=eeReadInt();
  TempTerm=eeReadInt();
  Delta=eeReadInt();
  FlToUSART=eeReadChar();
  FlToGSM=eeReadChar();
  Temp1P=eeReadInt();
  Temp2P=eeReadInt();
  Temp3P=eeReadInt();
  for(i=0;i<MAX_DS1820;i++)
    ds1820_popr[i]=eeReadChar();
  tEndRectRazgon=eeReadInt();
  PowerRect=eeReadInt();

  tEndRectOtbGlv=eeReadInt();
  timeChimRectOtbGlv=eeReadInt();
  ProcChimOtbGlv=eeReadChar();
  timeChimRectOtbSR=eeReadInt();
  tDeltaRect=eeReadChar();
  tEndRectOtbSR=eeReadInt();
  tEndRect=eeReadInt();
  PowerGlvDistil=eeReadInt();
  PowerDistil=eeReadInt();

  TempDeflBegDistil=eeReadInt();
  TempDefl=eeReadInt();
  DeltaDefl=eeReadChar();

  BeepEndProcess=eeReadChar();
  BeepStateProcess=eeReadChar();
  BeepKeyPress=eeReadChar();

  PowerRazvZerno=eeReadInt();

  tEndDistDefl=eeReadInt();
  minProcChimOtbSR=eeReadChar();
  PowerVarkaZerno=eeReadInt();
  TempKipenZator=eeReadInt();
  if (TempKipenZator>1050 || TempKipenZator<0) TempKipenZator=800;
  TempZSPSld=eeReadChar();
  if (TempZSPSld>100 || TempZSPSld<0) TempZSPSld=68;
  TempZSP=eeReadChar();
  if (TempZSP>100 || TempZSP<0) TempZSP=68;
  PowerNBK=eeReadInt();
  SpeedNBKDst=eeReadChar();
  if (PowerNBK>10000 || PowerNBK<0) PowerNBK=2400;
  if (SpeedNBKDst>254) SpeedNBKDst=0;
  PowerMinute=eeReadInt();
  if (PowerMinute>10000 || PowerMinute<0) PowerMinute=Power;
  begProcChimOtbSR=eeReadChar();
  if (begProcChimOtbSR>100) begProcChimOtbSR=40;
  P_MPX5010=eeReadInt();
  if (P_MPX5010>800 || P_MPX5010<-800) P_MPX5010=0;
  PeriodRefrServer=eeReadChar();
  timeRefrServer=PeriodRefrServer;   // Таймер для отсчета ответа соединения
  NaprPeregrev=eeReadInt();
  if (NaprPeregrev>1000 ) NaprPeregrev=300;
  UrovenBarda=eeReadInt();
  //if (UrovenBarda>2000 || UrovenBarda<0) UrovenBarda=800;
  UrovenProvodimostSR=eeReadInt();
  if (UrovenProvodimostSR>2000 || UrovenProvodimostSR==-1) 
  {
    UrovenProvodimostSR=0;
  }
  TimeStabKolonna=eeReadInt();
  if (TimeStabKolonna==-1) TimeStabKolonna=900;
  
  DecrementCHIM=eeReadChar();
  if (DecrementCHIM==-1) DecrementCHIM=10;
  IncrementCHIM=eeReadChar();
  if (IncrementCHIM==-1) IncrementCHIM=0;
  TimeAutoIncCHIM=eeReadInt();
  if (TimeAutoIncCHIM==-1) TimeAutoIncCHIM=600;
  
  CntCHIM=eeReadChar();
  for(i=0;i<COUNT_CHIM;i++)
  {
    tempK[i]=eeReadInt();
    CHIM[i]=eeReadChar();
  }
  TimeRestabKolonna=eeReadInt();
  if (TimeRestabKolonna==-1) TimeRestabKolonna=3600;

  CntPause=eeReadChar();
  for(i=0;i<10;i++)
  {
    // Если число пауз больше нуля, то считываем их
    if (CntPause>0)
    {
      tempP[i]=eeReadChar();
      timeP[i]=eeReadChar();
    }
    else
    {
      // Иначе просто читаем чтобы счетчик чтений увеличился.
      eeReadChar();
      eeReadChar();
    }
    if (timeP[i]==255) timeP[i]=0;
  }
  
  if (CntPause==-1) CntPause=8;
 
  CorrectASC712=eeReadChar();
  if (CorrectASC712==-1) CorrectASC712=0;
  
  ip[0]=eeReadChar();
  ip[1]=eeReadChar();
  ip[2]=eeReadChar();
  ip[3]=eeReadChar();
  ipPort=eeReadInt();
  
  for(i=0;i<10;i++) 
  {
    ctmp=eeReadChar();
    if (ctmp!=255)  idDevice[i]=ctmp;
  }
  idDevice[10]=0;

  for(i=0;i<12;i++) 
  {
    ctmp=eeReadChar();
    if (ctmp!=255)  my_phone[i]=ctmp;
  }
  
  my_phone[12]=0;
  AlarmMPX5010=eeReadInt();
  if (AlarmMPX5010==-1) AlarmMPX5010=0;
  
  FlAvtonom=eeReadChar();
  if (FlAvtonom==-1) FlAvtonom=0;
  timeOpenBRD=eeReadChar();
  if (timeOpenBRD==255) timeOpenBRD=2;
  

  PIDTemp[0]=eeReadChar();
  PIDTemp[1]=eeReadChar();
  PIDTemp[2]=eeReadChar();
  if (PIDTemp[0]==-1)
  {
    PIDTemp[0]=20;
    PIDTemp[1]=10;
    PIDTemp[2]=30;
  }

  for(i=0;i<11;i++) 
  {
    ctmp=eeReadChar();
    if (ctmp!=255)  WiFiAP[i]=ctmp;
  }
  WiFiAP[11]=0;
  for(i=0;i<11;i++) 
  {
    ctmp=eeReadChar();
    if (ctmp!=255)  WiFiPass[i]=ctmp;
  }
  WiFiPass[11]=0;
  CountFractionDist=eeReadChar();
  for(i=0;i<MAX_CNT_FRACTION_DIST;i++) 
  {
    TempFractionDist[i]=eeReadInt();
    if (TempFractionDist[i]==-1)  TempFractionDist[i]=0;
    PowerFractionDist[i]=eeReadInt();
    if (PowerFractionDist[i]==-1)  PowerFractionDist[i]=0;
    AngleFractionDist[i]=eeReadChar();
    if (AngleFractionDist[i]==255)  AngleFractionDist[i]=0;
  }
  
  CountFractionRect=eeReadChar();
  for(i=0;i<MAX_CNT_FRACTION_RECT;i++) 
  {
    TempFractionRect[i]=eeReadInt();
    if (TempFractionRect[i]==-1)  TempFractionRect[i]=0;
    AngleFractionRect[i]=eeReadChar();
    if (AngleFractionRect[i]==255)  AngleFractionRect[i]=0;
  }

  TempHLDZatorBrog1=eeReadChar();
  if (TempHLDZatorBrog1>100 || TempHLDZatorBrog1<0) TempHLDZatorBrog1=36;

  PowerPhase[0]=eeReadInt();
  if (PowerPhase[0]<0 || PowerPhase[0]>10000) PowerPhase[0]=Power;
  PowerPhase[1]=eeReadInt();
  if (PowerPhase[1]<0 || PowerPhase[1]>10000) PowerPhase[1]=0;
  PowerPhase[2]=eeReadInt();
  if (PowerPhase[2]<0 || PowerPhase[2]>10000) PowerPhase[2]=0;
  if ((PowerPhase[0]+PowerPhase[1]+PowerPhase[2])<=0) PowerPhase[0]=Power;
  
  KtPhase[0]=eeReadChar();
  if (KtPhase[0]>250) KtPhase[0]=100;
  KtPhase[1]=eeReadChar();
  if (KtPhase[1]>250) KtPhase[1]=100;
  KtPhase[2]=eeReadChar();
  if (KtPhase[2]>250) KtPhase[2]=100;

  for(i=10;i<MAX_CNT_PAUSE;i++)
  {
    // Если число пауз больше нуля, то считываем их
    if (CntPause>0)
    {
      tempP[i]=eeReadChar();
      timeP[i]=eeReadChar();
    }
    else
    {
      // Иначе просто читаем чтобы счетчик чтений увеличился.
      eeReadChar();
      eeReadChar();
    }
    if (timeP[i]==255) timeP[i]=0;
  }
  minPressNBK=eeReadChar();
  if (minPressNBK==-1) minPressNBK=130;
  deltaPressNBK=eeReadChar();
  if (deltaPressNBK==-1) deltaPressNBK=25;
  timePressNBK=eeReadChar();
  if (timePressNBK==-1) timePressNBK=180;
  
  UprNasosNBK=eeReadChar();
  if (UprNasosNBK==-1) UprNasosNBK=1;
  ProcChimOtbCP=eeReadChar();
  if (ProcChimOtbCP==-1) ProcChimOtbCP=0;

  #ifdef TEST
    Serial.print("maxaddr:");
    Serial.println(ee_addr);
  #endif    
  
}


void ScanKbd()
{
  char i;
  #ifdef DEBUG
    if (Serial.available()) return;
  #endif
  
  // Если режим защиты от дребезга уже выключен
  if (CountKeys==0)
  {
    flScanKbd=1; // Установим признак сканирования клавиатуры.
#if USE_LCD_KEYPAD_SHIELD     
    int key1;
    
    if (cmdGPRS==' ')
    {
        key1=analogRead(A0);
  
      //          select 720
      //          left 478
      //          up 130 ( моргает на 131)
      //          down 306
      //          rigth 0
      //          #define PRESS_SELECT KeyCode==1
      //          #define PRESS_RIGHT  KeyCode==2
      //          #define PRESS_LEFT   KeyCode==3
      //          #define PRESS_UP     KeyCode==4
      //          #define PRESS_DOWN   KeyCode==5
  
        if (key1>=0) KeyCode=2;
        if (key1>=90) KeyCode=4;
        if (key1>=200) KeyCode=5;
        if (key1>=400) KeyCode=3;
        if (key1>=600) KeyCode=1;
        if (key1>=950) KeyCode=0;
    }
    else
    {
      KeyCode=0;
      if (cmdGPRS=='R') KeyCode=2;
      if (cmdGPRS=='U') KeyCode=4;
      if (cmdGPRS=='D') KeyCode=5;
      if (cmdGPRS=='L') KeyCode=3;
      if (cmdGPRS=='S') KeyCode=1;
      #ifdef TESTGSM
          Serial.print("Go cmd=");
          Serial.print(cmdGPRS);
          Serial.print("=");          
          Serial.println((int)KeyCode);
      #endif
      cmdGPRS=' ';
    }

#endif

    if (PRESS_LEFT && PRESS_RIGHT)
    {
      if (BeepKeyPress) my_beep(2);

      lcd.begin(16, 2);
      lcd.clear();
      DisplayData();
    }


    // Нажатие кнопок одновременно ввехр и вниз позволяет перейти к следующему этапу (отладочные кнопки)
    if (PRESS_UP && PRESS_DOWN)
    {
      if (BeepKeyPress) my_beep(2);
      StateMachine++; // Состояние конечного автомата процесса ректификации
      CountKeys=10; // Большая защита от дребезга, чтоб случайно не нажали.
      DisplayData();
      timeNBK=0;
      timeMIXER=0;
      time1=0;
      time2=0;
      time3=0;

    }

    if (PRESS_SELECT)
    {            
      if (BeepKeyPress) my_beep(1);
       switch (FlState)
      {         
      case 0:   

        FlState=IspReg;
        break;
      case 100:
        FlState=200;      
        break;   
      case 101:
        FlState=0;      
        IspReg=101;   
        writeEEPROM();
        break;   
      case 102:
        if (IspReg!=102)
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=102;
        writeEEPROM();
        FlState=200;
        break;    
      case 200:
        FlState=0;
        writeEEPROM();
        break;    
      case 103:
        IspReg=103;
        writeEEPROM();
        FlState=202;
        break;    
      case 201:
        writeEEPROM();
        FlState=0;
        break;     
      case 202:
        writeEEPROM();
        FlState=0;
        break;     
      case 104:
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=104 || !StateMachine)
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=104;
        writeEEPROM(); 
        FlState=206;
        break;     
      case 105:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=105 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }                      
        if (!StateMachine) StateMachine=1;
        IspReg=105;
        writeEEPROM();
        FlState=221;
        break;     
      case 106:
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=106 || !StateMachine)
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=106;
        writeEEPROM(); 
        FlState=207;
        break;     
      case 107:
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=107 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=107;
        writeEEPROM();
        FlState=208;
        break;     
      case 108:
        if (IspReg!=108 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=108;
        FlState=259;
        writeEEPROM();
        break;
      case 109:
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=109 || !StateMachine)
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;

        IspReg=109;
        FlState=209;
        writeEEPROM();
        break;
      case 110:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=110 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;

        IspReg=110;
        writeEEPROM();
        FlState=220;
        break;     
      case 111:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=111 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;

        IspReg=111;
        writeEEPROM();
        FlState=209;
        break;     
      case 112:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=112 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
       }
        if (!StateMachine) StateMachine=1;

        IspReg=112;
        writeEEPROM();
        FlState=264;
        break;     
      case 113:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=113 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;

        IspReg=113;
        writeEEPROM();
        FlState=259;
        break;     
      case 114:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=114 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;
        
        IspReg=114;
        writeEEPROM();
        FlState=261;
        break;     
      case 115:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=115 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;
        
        IspReg=115;
        writeEEPROM();
        FlState=0;
        break;     
      case 116:  
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=116 || !StateMachine) // Если состояние автомата - не запущено, запускаем процесс
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;
        
        IspReg=116;
        writeEEPROM();
        FlState=240;
        break; 
      case 117:
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=117 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=117;
        writeEEPROM();
        FlState=257;
        break;             
      case 118:
        // Если ранее был выбран другой тип отбора, сбрасываем состояние автомата в начальное.
        if (IspReg!=118 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
          flAlarmMPX5010=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=118;
        writeEEPROM();
        FlState=258;
        break;             
      case 130:
        if (IspReg!=130 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=130;
        writeEEPROM();
        FlState=0;
        break;     
      case 129:
        if (IspReg!=129 || !StateMachine) 
        {
          Seconds=0;
          flAlarmUroven=0;
          StateMachine=0;
        }
        if (!StateMachine) StateMachine=1;
        IspReg=129;
        writeEEPROM();
        FlState=231;
        break;     
      case 203:       
      case 204:       
      case 205:       
      case 206:       
      case 207:       
      case 208:       
      case 209:       
      case 210:       
//      case 211:             
      case 212:       
      case 213:       
      case 214:       
      case 215:       
      case 216:       
      case 217:       
      case 218:       
      case 219:       
      case 220:       
      case 221:       
      case 222:       
      case 223:       
      case 224:       
      case 225:       
      case 226:       
      case 227:       
      case 228:       
      case 229:       
      case 230:       
      case 231:       
      case 232:       
      case 233:       
      case 234:       
      case 236:       
      case 237:       
      case 238:       
      case 239:       
      case 241:       
      case 243:       
      case 246:       
      case 247:       
      case 248:       
      case 250:     
      case 251:     
      case 252:     
      case 253:     
      case 254:     
      case 259:     
      case 260:     
      case 261:     
      case 264:     
      case 265:     
      case 266:     
      case 267:     
      case 268:     
        writeEEPROM();
        FlState=0;
        break;    
      case 211:
        FlState=300;
        nPopr=0;
        break;     
      case 235:
        FlState=301;
        nPopr=0;
        flPopr=0;
        break;     
      case 240:
        FlState=302;
        nPopr=0;
        flPopr=0;
        break;     
      case 242:
        FlState=303;
        nPopr=0;
        flPopr=0;
        break;
      case 244:
        FlState=304;
        nPopr=0;
        flPopr=0;
        break;
      case 245:
        FlState=305;
        nPopr=0;
        flPopr=0;
        break;
      case 249:
        FlState=309;
        nPopr=0;
        flPopr=0;
        break;
      case 255:
        FlState=310;
        nPopr=0;
        flPopr=0;
        break;
      case 256:
        FlState=311;
        nPopr=0;
        flPopr=0;
        break;
      case 257:
        FlState=312;
        nPopr=0;
        flPopr=0;
        break;
      case 258:
        FlState=313;
        nPopr=0;
        flPopr=0;
        break;
      case 262:     
        FlState=314;
        nPopr=0;
        flPopr=0;
        break;
      case 263:     
        FlState=315;
        nPopr=0;
        flPopr=0;
        break;
      case 300:       
        writeEEPROM();
        FlState=211;
        break;    
      case 301:       
        writeEEPROM();
        FlState=235;
        break;    
      case 302:       
        writeEEPROM();
        if (IspReg!=116) FlState=240;
                    else FlState=0;
        break;    
      case 303:       
        writeEEPROM();
        FlState=242;
        break;    
      case 304:       
        writeEEPROM();
        FlState=244;
        break;    
      case 305:       
        writeEEPROM();
        FlState=245;
        break;    
      case 309:       
        writeEEPROM();
        FlState=249;
        break;    
      case 310:       
        writeEEPROM();
        FlState=255;
        break;    
      case 311:       
        writeEEPROM();
        FlState=256;
        break;    
      case 312:       
        writeEEPROM();
        if (IspReg!=117) FlState=257;
                    else FlState=0;
        break;    
      case 313:       
        writeEEPROM();
        if (IspReg!=118) FlState=258;
                    else FlState=0;
        break;    
      case 314:       
        writeEEPROM();
        Power=PowerPhase[0]+PowerPhase[1]+PowerPhase[2];
        FlState=262;
        break;    
      case 315:       
        writeEEPROM();
        FlState=263;
        break;    
      }                            
      DisplayData();
      CountState=4;
      if (CountKeys==0) CountKeys=6;              
      CountState=10;

    }

    // Вторая  кнопка ( кнопка -)        
    if (PRESS_RIGHT)  
    { 
      if (BeepKeyPress) my_beep(1);
      switch (FlState)
      {                                              
      case 0:           
        if (DispPage<10) DispPage++;
        // Если перешли на третью страницу, то переинициализируем дисплей
        if (DispPage==4 || DispPage==10)
        {
          my_beep(BEEP_LONG*5);
          lcd.begin(16, 2);
          lcd.clear();
          DisplayData();
        }
        CountKeys=4;              
        break;
      case 100:
      case 101:
      case 102:
      case 103:
      case 104:
      case 105:
      case 106:
      case 107:
      case 108:
      case 109:
      case 110:
      case 111:
      case 112:
      case 113:
      case 114:
      case 115:
      case 116:
      case 117:
      case 129:
        FlState++;    
        break;   
      case 118:
        FlState=129;    
        break;   
      case 130:
        FlState=100;
        break;
      case 200:   
      case 201:
      case 202:
      case 203:
      case 204:
      case 205:
      case 206:
      case 207:
      case 208:
      case 209:
      case 210:
      case 211:
      case 212:
      case 213:
      case 214:
      case 215:
      case 216:
      case 217:
      case 218:
      case 219:
      case 220:
      case 221:
      case 222:
      case 223:
      case 224:
      case 225:
      case 226:
      case 227:
      case 228:
      case 229:
      case 230:
      case 231:
      case 232:
      case 233:
      case 234:
      case 235:
      case 236:       
      case 237:       
      case 238:       
      case 239:       
      case 240:       
      case 241:       
      case 242:       
      case 243:       
      case 244:       
      case 245:       
      case 246:       
      case 247:       
      case 248:       
      case 249:       
      case 250:
      case 251:
      case 252:
      case 253:
      case 254:
      case 255:
      case 256:
      case 257:
      case 258:
      case 259:
      case 260:
      case 261:
      case 262:
      case 263:
      case 264:
      case 265:
      case 266:
      case 267:
        FlState++;
        break;
      case 268:
        FlState=200;
        break;
      case 300:
        if (nPopr<MAX_DS1820-1) nPopr++;
        break;
      case 301:
        if (flPopr==0) flPopr=1;
        else
        {
          if (nPopr<COUNT_CHIM-1) nPopr++;
          flPopr=0;
          
        }
        break;
      case 302:
        if (flPopr==0) flPopr=1;
        else
        {
          if (nPopr<MAX_CNT_PAUSE-1) nPopr++;
          flPopr=0;
          
        }
        break;
      case 303:
        if (nPopr<3) nPopr++;
        break;
      case 304:
        if (nPopr<9) nPopr++;
        break;
      case 305:
        if (nPopr<11) nPopr++;
        break;
      case 309:
        if (nPopr<2) nPopr++;
        break;
      case 310:
      case 311:
        if (nPopr<10) nPopr++;
        break;
      case 312:
        if (flPopr<2) flPopr++;
        else
        {
          if (nPopr<MAX_CNT_FRACTION_DIST) nPopr++;
          flPopr=0;
        }
        break;
      case 313:
        if (flPopr<1) flPopr++;
        else
        {
          if (nPopr<MAX_CNT_FRACTION_RECT) nPopr++;
          flPopr=0;
        }
        break;
      case 314:
      case 315:
        if (nPopr<2) nPopr++;
        break;
        
      }

      //NeedDisplaying=1;
      DisplayData();
      if (CountKeys==0) CountKeys=6;

      CountState=4;
      CountState=40;


    }               

    // первая  кнопка  (кнопка +)      

    if (PRESS_LEFT)
    { 
      if (BeepKeyPress) my_beep(1);
      switch (FlState)
      {

      case 0:          
        if (DispPage>0) DispPage--;
        else
          { // Если на нулевой странице нажали кнопку влево, то тоже переинициализируем дисплей. 
            my_beep(BEEP_LONG*2);
            lcd.begin(16, 2);
            lcd.clear();
            DisplayData();
          }
        
        CountKeys=4;              
        break;
      case 100:
        FlState=130;   
        break;
      case 101:
      case 102:
      case 103:
      case 104:
      case 105:
      case 106:
      case 107:
      case 108:
      case 109:
      case 110:
      case 111:
      case 112:
      case 113:
      case 114:
      case 115:
      case 116:
      case 117:
      case 118:
      case 130:
        FlState--;   
        break;
      case 129:
        FlState=118;   
        break;
      case 201:   
      case 202:
      case 203:
      case 204: 
      case 205: 
      case 206: 
      case 207: 
      case 208: 
      case 209: 
      case 210: 
      case 211: 
      case 212:
      case 213:
      case 214:
      case 215:
      case 216:
      case 217:
      case 218:
      case 219: 
      case 220:
      case 221:
      case 222:
      case 223:
      case 224:
      case 225:
      case 226:
      case 227:
      case 228:
      case 229:
      case 230:
      case 231:
      case 232:
      case 233:
      case 234:
      case 235:
      case 236:
      case 237:
      case 238:
      case 239:
      case 240:
      case 241:
      case 242:
      case 243:
      case 244:
      case 245:
      case 246:
      case 247:
      case 248:
      case 249:
      case 250:
      case 251:
      case 252:
      case 253:
      case 254:
      case 255:
      case 256:
      case 257:
      case 258:
      case 259:
      case 260:
      case 261:
      case 262:
      case 263:
      case 264:
      case 265:
      case 266:
      case 267:
      case 268:
        FlState--;
        break;
      case 200:
        FlState=268;
        break;     
      case 300:
        if (nPopr>0) nPopr--;
        break;
      case 301:
        if (flPopr==1) flPopr=0;
        else
        {
          if (nPopr>0) nPopr--;
          flPopr=1;
        }
        break;
      case 302:
        if (flPopr==1) flPopr=0;
        else
        {
          if (nPopr>0) nPopr--;
          flPopr=1;
        }
        break;
      case 303:
      case 304:
      case 305:
      case 309:
      case 310:
      case 311:
      case 314:
      case 315:
        if (nPopr>0) nPopr--;
        break;
      case 312:
        if (flPopr>0) flPopr--;
        else
        {
          if (nPopr>0) nPopr--;
          flPopr=2;
        }
        break;
      case 313:
        if (flPopr>0) flPopr--;
        else
        {
          if (nPopr>0) nPopr--;
          flPopr=1;
        }
        break;
      }         

      //NeedDisplaying=1;
      DisplayData();
      flNeedAnalyse=1;
      CountState=4;
      CountState=40;
      if (CountKeys==0) CountKeys=6;
    }



    if (PRESS_UP)  
    { 
      if (BeepKeyPress) my_beep(1);
      CountState=0;
      switch (FlState)
      {                                              
      case 0:
        // В сотоянии 0 обработка клавиш происходит за выходом модели этого конечного автомата.
        if (DispPage==7) // На 8 странице вручную можно состояние процесса поменять
        {
          StateMachine++;
          break;
        }
        
        if (DispPage==5) // На 6 странице можно мощность внучную привбавить
        {
          UstPower+=10;
          break;
        }

        // Ректификация
        if (IspReg==109  || IspReg==118)
        {
          switch(StateMachine)
          {
          case 2:
            tEndRectRazgon++;
            CountKeys=1;
            break;
          case 3:
            PowerRect+=10;
            CountKeys=2;
            break;
          case 4:
            if (UrovenProvodimostSR==0) tEndRectOtbGlv++;
            else SecTempPrev+=600;
            
            
            break;
            
          case 5:
            tStabSR++;
            break;
          case 6:
            ProcChimSR++;
            break;
          case 7:
            tEndRect++;
            break;
          }
        }

        // Регулятор мощности
        if (IspReg==103)
        { CountKeys=1;
          if (SlaveON==0) 
          {
            if (DispPage==1)
            {
              // На второй странице нажатик кнопки "вверх" приводи к авто-детекции мощности ТЭНа
              flAutoDetectPowerTEN=1;
            }
            else
            {
              UstPowerReg+=10;
              flAutoDetectPowerTEN=0;
            }
          }
          if (SlaveON==2) PowerGlvDistil+=10;
          if (SlaveON==3) PowerRect+=10;
          if (SlaveON==4) PowerDistil+=10;
          
          //TekPower=(unsigned long) UstPower*220*220/ Power;
          CountKeys=3;
        }

        if (IspReg==104)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil++;
          else
          {
            if (DispPage==0)  Temp1P++;
                        else  PowerDistil+=10;
          }
        }
        if (IspReg==106)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil++;
          else
          {
            if (DispPage==0)  Temp2P++;
                         else PowerDistil+=10;
          }
        }

        if (IspReg==107)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil++;
          else
          {
            if (DispPage==0)  Temp3P++;
                        else  PowerDistil+=10;
          }
        }
        
        if (IspReg==117)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil++;
          else
          {
            if (TekFraction<CountFractionDist)
            {
              if (DispPage==0)  TempFractionDist[TekFraction]++;
                           else PowerFractionDist[TekFraction]+=10;
            }
          }
        }

        if (IspReg==105)
        {
          CountKeys=1;
          if (StateMachine==3) PowerGlvDistil+=10;
          if (StateMachine==2) TempDeflBegDistil++;
        }

        if (IspReg==114)
        {
          TempHLDZatorBrog1++;
        }
        
        if (IspReg==129)
        {
          CloseAllKLP();
          ProcChimSR++;
        }
        
        if (IspReg==115)
        { CountKeys=2;
          if (DispPage==0) timerMinute+=AddPressKeys;
                     else  PowerMinute+=AddPressKeys10;
        }
        
        if (IspReg==116)
        {
          if (DispPage==0 && StateMachine==3 && timeP[KlTek]==0) timeP[KlTek]=255; 
          else
          {
            // На второй странице можно включить насос.
            if (DispPage==1) 
            {
              time1=0;
              KlOpen[KLP_HLD]=0;
            }
          }
        }

        if (IspReg==108 || IspReg==113)
        {   CountKeys=1;
            switch (StateMachine)
            { 
              case 0: //Не запущено
                time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
                timeMIXER=(int) tempP[0] *60;        
              break;
              case 1: //Нагрев до температуры 50 градусов
              if (DispPage==0) TempZSP++;
              else 
              {
                time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
                timeMIXER=(int) tempP[0] *60;        
              }
              break;
              case 3: //Нагрев до температуры 62 градуса
                TempZSPSld++;
                break;
              case 4: //Ожидание 15 минут, поддержка температуры
                time2+=60;
              case 2: //Поддержание температуры 50 градусов, пока не произойдет ручной переход к следующему этапу.
              case 8: //ожидание засыпки солода, поддержка температуры
              // На этих этапах нажатие кнопки вверх включает мешалку, вниз - отключает.
              time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
              timeMIXER=(int) tempP[0] *60;        
              break;
              
              case 6: //Варка
              case 9: //Размешивание после засыпания солшода
              case 10: // Осахаривание
                if (DispPage==0) time2+=60;
                            else TempZSPSld++;
                break;
              case 5: //  Нагрев до закипания
                TempKipenZator++;
                break;
              case 7: //охлаждение до температуры осахаривания.
                TempZSPSld++;
                break;
              case 12: //охлаждение до температуры осахаривания.
                TempHLDZatorBrog1++;
                break;
              case 14: //охлаждение до температуры осахаривания.
              case 13: //охлаждение до температуры осахаривания.
                if (DispPage==0) TempHLDZatorBrog1++;
                else 
                {
                  time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
                  timeMIXER=(int) tempP[0] *60;        
                }
                break;
              case 100: //Окончание
                break;
            }
        }

        if (IspReg==110)
        { CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil++;
          else
            if (DispPage==0) TempDefl++;
                        else deltaPower+=10; 
        }

        // НДРФ
        if (IspReg==111)
        { CountKeys=2;
          switch(StateMachine)
          {
          case 2:
            tEndRectRazgon++;
            CountKeys=1;
            break;
          case 3:
            if (DispPage==0) SecOstatok+=60;
                      else PowerRect+=10;
            CountKeys=3;
            break;
          case 4:
            tEndRectOtbGlv++;
            break;
          case 5:
            tStabSR++;
            break;
          case 6:
            ProcChimSR++;
            break;
          case 7:
            tEndRect++;
            break;
          }
        }
        // NBK
        if (IspReg==112)
        { CountKeys=1;
          if (DispPage==0)
          {
             if (StateMachine==3) StateMachine=4;
           else
            {
              if (StateMachine==5) time2+=60;
              else
              {
                SpeedNBKDst+=1;
                SpeedNBK=SpeedNBKDst;
                CountKeys=1;
              }
            }
          }
          else
          {
            PowerNBK+=AddPressKeys10;
            UstPower=PowerNBK;
          }
        }
        if (IspReg==130) { Angle+=5; SetAngle(Angle);}
        
        break; 
      case 200:
        TempTerm+=AddPressKeys;
        CountKeys=2;
        break;
      case 201:
        Power+=AddPressKeys10;
        PowerPhase[0]=Power;
        CountKeys=3;
        break;
      case 202:
        UstPowerReg+=AddPressKeys10;
        CountKeys=3;
        break;
      case 203:
        FlToUSART=1;
        break;
      case 204:  
      
        if (FlToGSM<20) FlToGSM++;
        // Переинициализируем сотовый
        break;                     
      case 205:   
        Delta++;
        CountKeys=3;
        break;                     
      case 206:   
        Temp1P++;
        CountKeys=3;
        break;                     
      case 207:   
        Temp2P++;
        CountKeys=3;
        break;                     
      case 208:   
        Temp3P++;
        CountKeys=3;
        break;                     
      case 209:   
        tEndRectRazgon+=AddPressKeys;
        CountKeys=3;
        break;                     
      case 210:   
        PowerRect+=AddPressKeys10;
        CountKeys=3;
        break;                     
      case 212:
        tEndRectOtbGlv++;
        CountKeys=3;
        break;                     
      case 213:
        timeChimRectOtbGlv+=AddPressKeys10;
        CountKeys=3;
        break;
      case 214:
        ProcChimOtbGlv++;
        CountKeys=4;
        break;
      case 250:
        minProcChimOtbSR++;
        CountKeys=4;
        break;
      case 251:
        tStabSR++;
        CountKeys=3;
        break;
      case 252:
        begProcChimOtbSR++;
        CountKeys=4;
        break;
      case 253:
        P_MPX5010++;
        CountKeys=4;
        break;
      case 254:
        PowerNBK+=AddPressKeys10;
        CountKeys=3;
        break;
      case 215:
        timeChimRectOtbSR+=AddPressKeys10;
        CountKeys=3;
        break;
      case 216:
        tDeltaRect++;
        CountKeys=4;
        break;
      case 217:
        tEndRectOtbSR++;
        CountKeys=3;
        break;
      case 218:
        tEndRect++;
        CountKeys=3;
        break;
      case 219:
        PowerGlvDistil+=AddPressKeys10;
        CountKeys=3;
        break;
      case 220:
        PowerDistil+=AddPressKeys10;
        CountKeys=3;
        break;
      case 221:
        TempDeflBegDistil+=AddPressKeys;
        CountKeys=3;
        break;
      case 222:
        TempDefl++;
        CountKeys=3;
        break;
      case 223:
        DeltaDefl++;
        CountKeys=3;
        break;
      case 224:
        tEndDistDefl++;
        CountKeys=3;
        break;
      case 225:
        BeepEndProcess=1;
        break;
      case 226:
        BeepStateProcess++;
        CountKeys=5;
        break;
      case 227:
        BeepKeyPress=1;
        break;
      case 228:
        PowerRazvZerno+=AddPressKeys10;
        CountKeys=3;
        break;
      case 229:
        PowerVarkaZerno+=AddPressKeys10;
        CountKeys=3;
        break;
      case 230:
        PeriodRefrServer+=5;
        CountKeys=3;
        break;
      case 231:
        NaprPeregrev+=5;
        CountKeys=3;
        break;
      case 232:
        UrovenBarda+=5;
        CountKeys=3;
        break;
      case 233:
        UrovenProvodimostSR++;
        CountKeys=3;
        break;
      case 234:
        TimeStabKolonna+=60;
        CountKeys=3;
        break;
      case 235:
        CntCHIM++;
        CountKeys=3;
        break;
      case 236:
        DecrementCHIM++;
        CountKeys=3;
        break;
      case 237:
        IncrementCHIM++;
        CountKeys=3;
        break;
      case 238:
        TimeAutoIncCHIM+=60;
        CountKeys=3;
        break;
      case 239:
        TimeRestabKolonna+=60;
        CountKeys=3;
        break;
      case 240:
        if (CntPause<MAX_CNT_PAUSE-1) CntPause++;
        CountKeys=3;
        break;
      case 241:
        CorrectASC712++;
        CountKeys=3;
        break;
      case 243:
        ipPort+=AddPressKeys;
        CountKeys=2;
        break;
      case 246:
        AlarmMPX5010+=AddPressKeys;
        CountKeys=2;
        break;
      case 247:
        FlAvtonom=1;
        CountKeys=2;
        break;
      case 248:
        timeOpenBRD++;
        CountKeys=2;
        break;
      case 249:
        PIDTemp[0]+=AddPressKeys;
        CountKeys=2;
        break;
      case 257:
        if (CountFractionDist<MAX_CNT_FRACTION_DIST) CountFractionDist++;
        CountKeys=3;
        break;
      case 258:
        if (CountFractionRect<MAX_CNT_FRACTION_RECT) CountFractionRect++;
        CountKeys=3;
        break;
      case 259:
        TempZSP++;
        CountKeys=3;
        break;
      case 260:
        TempZSPSld++;
        CountKeys=3;
        break;
      case 261:
        TempHLDZatorBrog1++;
        CountKeys=3;
        break;
      case 262:
        Power+=AddPressKeys10;
        CountKeys=3;
        break;
      case 264:
        minPressNBK++;
        CountKeys=1;
        break;
      case 265:
        deltaPressNBK++;
        CountKeys=1;
        break;
      case 266:
        timePressNBK++;
        CountKeys=1;
        break;
      case 267:
        UprNasosNBK++;
        CountKeys=3;
        break;
      case 268:
        ProcChimOtbCP++;
        CountKeys=1;
        break;
      case 300:   
        ds1820_popr[nPopr]++;
        CountState=8; // При вводе поправок сохраняем состояние 4 секунды, а не две, тут спешка не нужна.                
        CountKeys=4;
        break;                     
      case 301:   
        if (flPopr==0) {  tempK[nPopr]+=AddPressKeys;
                          CountKeys=1;
                       }
                  else CHIM[nPopr]++;
        break;                     
      case 302:
        if (flPopr==0) tempP[nPopr]++;
                  else 
                  { timeP[nPopr]++;
                    if (timeP[nPopr]==255) timeP[nPopr]=0;
                  }
        CountKeys=1;
        break;                     
      case 303:
        ip[nPopr]++;
        CountKeys=1;
        break;                     
      case 304:
        idDevice[nPopr]++;
        CountKeys=1;
        break;                     
      case 305:
        my_phone[nPopr]++;
        CountKeys=1;
        break;                     
      case 309:
        PIDTemp[nPopr]+=AddPressKeys;
        CountKeys=2;
        break;                     
      case 310:
        WiFiAP[nPopr]++;
        CountKeys=1;
        break;                     
      case 311:
        WiFiPass[nPopr]++;
        CountKeys=1;
        break;                     
      case 312:
        if (nPopr==MAX_CNT_FRACTION_DIST)
        {

          if (TekFraction<MAX_CNT_FRACTION_DIST-1)
          {
              TekFraction++;
              SecTempPrev=Seconds;
              SetAngle(AngleFractionDist[TekFraction]);
          }
          CountKeys=3;
          
        }
        else
        {      
          CountKeys=2;
          if (flPopr==0) TempFractionDist[nPopr]+=AddPressKeys;
          if (flPopr==2) PowerFractionDist[nPopr]+=AddPressKeys10;
          if (flPopr==1) { AngleFractionDist[nPopr]++;  CountKeys=1;}
        }
        break;
      case 313:   
        if (nPopr==MAX_CNT_FRACTION_RECT)
        {

          if (TekFraction<MAX_CNT_FRACTION_RECT-1)
          {
              TekFraction++;
              SecTempPrev1=Seconds;
              SetAngle(AngleFractionRect[TekFraction]);
          }
          CountKeys=3;
        }
        else
        {      
          CountKeys=2;
          if (flPopr==0) TempFractionRect[nPopr]+=AddPressKeys;
          if (flPopr==1) { AngleFractionRect[nPopr]++;  CountKeys=1;}
        }
        break;
      case 314:
        PowerPhase[nPopr]+=AddPressKeys10;
        CountKeys=2;
        break;                     
      case 315:
        KtPhase[nPopr]+=AddPressKeys;
        CountKeys=2;
        break;                     
        
      }         

      //NeedDisplaying=1;
      DisplayData();
      flNeedAnalyse=1;
      if (CountState==0) CountState=4;
      if (CountKeys==0) CountKeys=6;
      CountState=40;
    }


    if (PRESS_DOWN)  
    { 
      if (BeepKeyPress) my_beep(1);
      CountState=0;
      switch (FlState)
      {                                              
      case 0:

        // В сотоянии 0 обработка клавиш происходит за выходом модели этого конечного автомата.
        if (DispPage==2) // На третьей странице сбрасываем датчик потока
        {
          break;
        }
        
        if (DispPage==7) // На 8 странице можно состояние процесс поменять.
        {
          StateMachine--;
          break;
        }
        
        if (DispPage==5) // На 6 странице можно мощность вручную изменить.
        {
          UstPower-=10;
          break;
        }

        // Ректификация
        if (IspReg==109  || IspReg==118)
        {
          switch(StateMachine)
          {
          case 2:
            tEndRectRazgon--;
            CountKeys=1;
            break;
          case 3:
            PowerRect-=10;
            CountKeys=3;
            break;
          case 4:
            if (UrovenProvodimostSR==0) tEndRectOtbGlv--;
            else SecTempPrev-=600;
            break;
          case 5:
            tStabSR--;
            break;
          case 6:
            ProcChimSR--;
            break;
          case 7:
            tEndRect--;
            break;
          }
        }

        // Регулятор мощности
        if (IspReg==103)
        {
          if (SlaveON==0) UstPowerReg-=10;
          if (SlaveON==2) PowerGlvDistil-=10;
          if (SlaveON==3) PowerRect-=10;
          if (SlaveON==4) PowerDistil-=10;
          
          //TekPower=(unsigned long) UstPower*220*220/ Power;
          CountKeys=1;
        }
        if (IspReg==104)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil--;
          else
            if (DispPage==0)  Temp1P--;
                        else  PowerDistil-=10;
        }
        if (IspReg==106)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil--;
          else
            if (DispPage==0)  Temp2P--;
                        else  PowerDistil-=10;
        }

        if (IspReg==107)
        {
          CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil--;
          else
            if (DispPage==0)  Temp3P--;
                        else  PowerDistil-=10;
        }
        
        if (IspReg==117)
        { CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil--;
          else
          {
            if (TekFraction<CountFractionDist)
            {
              if (DispPage==0)  TempFractionDist[TekFraction]--;
                           else PowerFractionDist[TekFraction]-=10;
            }
          }
        }

        if (IspReg==105)
        {
          CountKeys=1;
          if (StateMachine==3) PowerGlvDistil-=10;
          if (StateMachine==2) TempDeflBegDistil--;
        }
        
        if (IspReg==114)
        { CountKeys=1;
          TempHLDZatorBrog1--;
        }
        
        if (IspReg==129)
        {
          CloseAllKLP();
          ProcChimSR--;
        }

        if (IspReg==115)
        { CountKeys=2;
          if (DispPage==0) timerMinute-=AddPressKeys;
                     else  PowerMinute-=AddPressKeys10;
        }
        if (IspReg==108 || IspReg==113)
        {  CountKeys=1;
            switch (StateMachine)
            { 
              case 0: //Не запущено
              timeMIXER=0;  
              time1=0;
              break;      
              case 1: //Нагрев до температуры 50 градусов
              if (DispPage==0) TempZSP--;
                       else timeMIXER=0;        
              break;
              case 2: //Поддержание температуры 50 градусов, пока не произойдет ручной переход к следующему этапу.
              // Переход к следующему этапу
              StateMachine=3;
              CountKeys=3;
              break;
              case 3: //Нагрев до температуры 62 градуса
                TempZSPSld--;
                break;
              case 4: //Ожидание 15 минут, поддержка температуры
                 time2-=60;
              // На этих этапах нажатие кнопки вверх включает мешалку, вниз - отключает.
              timeMIXER=0;        
              break;
              
              case 6: //Варка
              case 9: //Размешивание после засыпания солшода
              case 10: // Осахаривание
                if (DispPage==0) 
                { if (time2>=65) time2-=60;
                            else time2=0;

                }
                         else TempZSPSld--;
              
                break;
              case 5: 
                TempKipenZator--;
                break;
              case 12: //охлаждение до температуры осахаривания.
                TempHLDZatorBrog1--;
                break;
              case 14: //охлаждение до температуры осахаривания.
              case 13: //охлаждение до температуры осахаривания.
                if (DispPage==0) TempHLDZatorBrog1--;
                else 
                {
                  timeMIXER=0;        
                }
                break;
                
              case 8:
                   // Нажатие кнопки вниз переводит на следующий этап 
                   StateMachine=9;
                   CountKeys=6;
                break;
              case 7: //охлаждение до температуры осахаривания.
                TempZSPSld--;
                break;
              case 100: //Окончание
                break;
            }
        }

        if (IspReg==110)
        { CountKeys=1;
          if (StateMachine==2) 
            TempDeflBegDistil--;
          else
            if (DispPage==0) TempDefl--;
                        else deltaPower-=10; 
            
        }

        // НДРФ
        if (IspReg==111)
        { CountKeys=2;
          switch(StateMachine)
          {  
          case 2:
            tEndRectRazgon--;
            CountKeys=1;
            break;
          case 3:
           
            if (DispPage==0) SecOstatok-=60;
                      else PowerRect-=10;
            CountKeys=3;
            break;
          case 4:
            tEndRectOtbGlv--;
            break;
          case 5:
            tStabSR--;
            break;
          case 6:
            ProcChimSR--;
            break;
          case 7:
            tEndRect--;
            break;
          }
        }
        // NBK
        if (IspReg==112)
        {  CountKeys=1;
          if (DispPage==0)
          {
              if (StateMachine==5) time2-=60;
              else
              {
                SpeedNBKDst-=1;
                SpeedNBK=SpeedNBKDst;
              }
          }
          else
          {
            PowerNBK-=AddPressKeys10;
            UstPower=PowerNBK;
           
          }
        }
        
        if (IspReg==116)
        {
            // На второй странице можно выключить насос.
            if (DispPage==1) 
            {
              time1=0;
              KlOpen[KLP_HLD]=40;
            }
        }

         if (IspReg==130) { Angle-=5; SetAngle(Angle);}      

        break; 
      case 200:
        TempTerm-=AddPressKeys;
        CountKeys=2;
        break;
      case 201:
        Power-=AddPressKeys10;
        PowerPhase[0]=Power;
        CountKeys=3;
        break;
      case 202:
        UstPowerReg-=AddPressKeys10;
        CountKeys=3;
        break;
      case 203:
        FlToUSART=0;
        break;
      case 204:   
        if (FlToGSM>0) FlToGSM--;
        break;                     
      case 205:   
        Delta--;
        CountKeys=3;
        break;                     
      case 206:   
        Temp1P--;
        CountKeys=3;
        break;                     
      case 207:   
        Temp2P--;
        CountKeys=3;
        break;                     
      case 208:   
        Temp3P--;
        CountKeys=3;
        break;                     
      case 209:   
        tEndRectRazgon-=AddPressKeys;
        CountKeys=3;
        break;                     
      case 210:   
        PowerRect-=AddPressKeys10;
        CountKeys=3;
        break;                     
      case 212:
        tEndRectOtbGlv--;
        CountKeys=3;
        break;                     
      case 213:
        timeChimRectOtbGlv-=AddPressKeys10;
        CountKeys=3;
        break;
      case 214:
        ProcChimOtbGlv--;
        CountKeys=4;
        break;
      case 250:
        minProcChimOtbSR--;
        CountKeys=4;
        break;
      case 251:
        tStabSR--;
        CountKeys=3;
        break;
      case 252:
        begProcChimOtbSR--;
        CountKeys=4;
        break;
      case 253:
        P_MPX5010--;
        CountKeys=4;
        break;
      case 254:
        PowerNBK-=AddPressKeys10;
        CountKeys=3;
        break;
      case 215:
        timeChimRectOtbSR-=AddPressKeys10;
        CountKeys=3;
        break;
      case 216:
        tDeltaRect--;
        CountKeys=4;
        break;
      case 217:
        tEndRectOtbSR--;
        CountKeys=3;
        break;
      case 218:
        tEndRect--;
        CountKeys=3;
        break;
      case 219:
        PowerGlvDistil-=AddPressKeys10;
        CountKeys=3;
        break;
      case 220:
        PowerDistil-=AddPressKeys10;
        CountKeys=3;
        break;
      case 221:
        TempDeflBegDistil-=AddPressKeys;
        CountKeys=3;
        break;
      case 222:
        TempDefl--;
        CountKeys=3;
        break;
      case 223:
        DeltaDefl--;
        CountKeys=3;
        break;
      case 224:
        tEndDistDefl--;
        CountKeys=3;
        break;
      case 225:
        BeepEndProcess=0;
        break;
      case 226:
        BeepStateProcess=0;
        break;
      case 227:
        BeepKeyPress=0;
        break;
      case 228:
        PowerRazvZerno-=AddPressKeys10;
        CountKeys=3;
        break;
      case 229:
        if (PowerVarkaZerno>10) PowerVarkaZerno-=AddPressKeys10;
        else PowerVarkaZerno=0;

        CountKeys=3;
        break;
      case 230:
        PeriodRefrServer-=5;
        CountKeys=3;
        break;
      case 231:
        NaprPeregrev-=5;
        CountKeys=3;
        break;
      case 232:
        UrovenBarda-=5;
        CountKeys=3;
        break;
      case 233:
        UrovenProvodimostSR--;
        CountKeys=3;
        break;
      case 234:
        TimeStabKolonna-=60;
        CountKeys=3;
        break;
      case 235:
        CntCHIM--;
        CountKeys=3;
        break;
      case 236:
        DecrementCHIM--;
        CountKeys=3;
        break;
      case 237:
        IncrementCHIM--;
        CountKeys=3;
        break;
      case 238:
        TimeAutoIncCHIM-=60;
        CountKeys=3;
        break;
      case 239:
        TimeRestabKolonna-=60;
        CountKeys=3;
        break;
      case 240:
        if (CntPause>=0) CntPause--;
        CountKeys=3;
        break;
      case 241:
        CorrectASC712--;
        CountKeys=3;
        break;
      case 243:
        ipPort-=AddPressKeys;
        CountKeys=2;
        break;
      case 246:
        AlarmMPX5010-=AddPressKeys;
        CountKeys=2;
        break;
      case 247:
        FlAvtonom=0;
        CountKeys=2;
        break;
      case 248:
        timeOpenBRD--;
        CountKeys=2;
        break;
      case 249:
        PIDTemp[0]-=AddPressKeys;
        CountKeys=2;
        break;
     case 257:
        if (CountFractionDist>=0) CountFractionDist--;
        CountKeys=3;
        break;
      case 258:
        if (CountFractionRect>=0) CountFractionRect--;
        CountKeys=3;
        break;        
      case 259:
        TempZSP--;
        CountKeys=3;
        break;
      case 260:
        TempZSPSld--;
        CountKeys=3;
        break;
      case 261:
        TempHLDZatorBrog1--;
        CountKeys=3;
        break;
      case 262:
        Power-=AddPressKeys10;
        CountKeys=3;
        break;
      case 264:
        minPressNBK--;
        CountKeys=1;
        break;
      case 265:
        deltaPressNBK--;
        CountKeys=1;
        break;
      case 266:
        timePressNBK--;
        CountKeys=1;
        break;
      case 267:
        UprNasosNBK--;
        CountKeys=3;
        break;
      case 268:
        ProcChimOtbCP--;
        CountKeys=1;
        break;
       case 300: 
        ds1820_popr[nPopr]--;
        CountState=8; // При вводе поправок сохраняем состояние 4 секунды, а не две, тут спешка не нужна.                
        CountKeys=4;
        break;      
      case 301:   
        if (flPopr==0) { tempK[nPopr]-=AddPressKeys;CountKeys=1; }
                  else CHIM[nPopr]--;
        break;               
      case 302:
        if (flPopr==0) tempP[nPopr]--;
                  else 
                  { timeP[nPopr]--;
                    if (timeP[nPopr]==255) timeP[nPopr]=254;
                  }
        CountKeys=1;
        break;                     
      case 303:
        ip[nPopr]--;
        CountKeys=1;
        break;                     
      case 304:
        idDevice[nPopr]--;
        CountKeys=1;
        break;                     
      case 305:
        my_phone[nPopr]--;
        CountKeys=1;
        break;                     
      case 309:
        PIDTemp[nPopr]-=AddPressKeys;
        CountKeys=2;
        break;                     
      case 310:
        WiFiAP[nPopr]--;
        CountKeys=1;
        break;                     
      case 311:
        WiFiPass[nPopr]--;
        CountKeys=1;
        break;                     
      case 312:   
        if (nPopr==MAX_CNT_FRACTION_DIST)
        {

          if (TekFraction>0)
          {
              TekFraction--;
              SecTempPrev=Seconds;
              SetAngle(AngleFractionDist[TekFraction]);
          }
          CountKeys=3;
          
        }
        else
        {
          CountKeys=2;
          if (flPopr==0) TempFractionDist[nPopr]-=AddPressKeys;
          if (flPopr==2) PowerFractionDist[nPopr]-=AddPressKeys10;
          if (flPopr==1) { AngleFractionDist[nPopr]--;  CountKeys=1;}
        }
        break;
      case 313:   
      if (nPopr==MAX_CNT_FRACTION_RECT)
        {

          if (TekFraction>0)
          {
              TekFraction--;
              SecTempPrev1=Seconds;
              SetAngle(AngleFractionRect[TekFraction]);
          }
          CountKeys=3;
        }
       else
       {
        CountKeys=2;
        if (flPopr==0) TempFractionRect[nPopr]-=AddPressKeys;
        if (flPopr==1) { AngleFractionRect[nPopr]--;  CountKeys=1;}
       }
       break;
      case 314:
        PowerPhase[nPopr]-=AddPressKeys10;
        CountKeys=2;
        break;                     
      case 315:
        KtPhase[nPopr]-=AddPressKeys;
        CountKeys=2;
        break;                     
      }         

      //NeedDisplaying=1;
      DisplayData();
      flNeedAnalyse=1;
      if (CountState==0) CountState=4;
      if (CountKeys==0) CountKeys=6;
      CountState=40;
    }
    flNeedScanKbd=0;
    flScanKbd=0; // Убираем признак того, что сканируется клавиатура.
  }
}




void CloseAllKLP()
{ 
  char i;
  // Все клапана зануляем.
  for(i=0;i<MAX_KLP;i++)
  {
    KlOpen[i]=0;    
    KlClose[i]=10;   
    KlState[i]=0;          // Начальное состояние - клапан закрыт, счетчик 0
    KlCount[i]=0;  
    KlReg[i]=PEREGREV_ON;
  }
  // Если автономная система охлаждения, то у клапана дефлегматора отключаем защиту от перегрева.
  if (FlAvtonom)
  {
    KlReg[KLP_DEFL]=0; // Клапан дефлегматора и клапан холодильника отключим от системы защиты от перегрева если используется автономка.
    KlReg[KLP_HLD]=0;
  }
  
}

void PrepareProcess()
{ 
  char i;
  // Максимальную температуру процессов зануляем.
  for(i=0;i<MAX_DS1820;i++)
  {
    Maxtemps[i]=0;
  }
  // Все таймеры зануляем.
  timeMIXER=0;
  timeNBK=0;
  time1=0;
  time2=0;
  time3=0;
  flAutoDetectPowerTEN=0;// Убираем признак авто-детектирования ТЭНОв
  flNoAlarmLowPower=0;   // Убираем признак невыдачи предупреждения о низком уровне напряжения
  // Все клапана зануляем.
  CloseAllKLP();
  //  Разгон отключаем
  digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
}


void ProcessTermostat()
{
  switch  (StateMachine)
  {
  case 0: // Температура Больше необходиомй
    // Процесс не запущен Процесс термостат заппускается автоматически, чтобы он работал даже если выключат и включат свет.
    //break;
  case 1: // Старт процесса
    digitalWrite(PIN_TERMOSTAT_OFF,LOW);// Включаем охлаждение
    digitalWrite(PIN_TERMOSTAT_ON,LOW);// Выключаем нагрев
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    StateMachine=2;
  case 2: // Температура Больше необходиомй
    if (temps[TEMP_TERMOSTAT]<=TempTerm)
    {
      digitalWrite(PIN_TERMOSTAT_OFF,LOW);// Включаем охлаждение
      digitalWrite(PIN_TERMOSTAT_ON,HIGH);// Выключаем нагрев
      digitalWrite(PIN_START_1,RELAY_HIGH);
      digitalWrite(PIN_START_2,RELAY_HIGH);
      ProcessPIDTemp(TempTerm,temps[TEMP_TERMOSTAT]);
      UstPower=(long)Power*KtT/1000;
    // Мощность ограничена мощностью варки зерна
    if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;

      break;
    }
    StateMachine=3;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    
  case 3: //
    UstPower=0; // Убираем мощность с ТЭНов
    if (temps[TEMP_TERMOSTAT]>TempTerm-Delta)
    {
      digitalWrite(PIN_TERMOSTAT_ON,LOW);// Выключаем нагрев
      digitalWrite(PIN_TERMOSTAT_OFF,HIGH);// Включаем охлаждение
      digitalWrite(PIN_START_1,!RELAY_HIGH);
      digitalWrite(PIN_START_2,!RELAY_HIGH);
      
      break;
    }
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    break;
  case 100:// Конечное состояние автомата
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    flAllOff=1;
    digitalWrite(PIN_TERMOSTAT_ON,LOW);// Выключаем нагрев
    digitalWrite(PIN_TERMOSTAT_OFF,LOW);// Включаем охлаждение
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    UstPower=0; // Убираем мощность с ТЭНов
    break;
  }
}

void ProcessTimerMaxPower()
{
  int minute;
  switch  (StateMachine)
  {
  case 0: // 
    // Процесс не запущен
    break;
  case 1: // Старт процесса
    timerMinute=0;
    StateMachine=2;
    lastSMSState=0;
  case 2: // Температура Больше необходиомй
    minute = Seconds / 60;
    if (minute<=timerMinute)
    {
      UstPower=PowerMinute; // Подаем полную мощность на ТЭНЫ
      digitalWrite(PIN_START_1,RELAY_HIGH);
      digitalWrite(PIN_START_2,RELAY_HIGH);
      break;
    }
    StateMachine=100;
    if (BeepStateProcess) my_beep(BEEP_LONG);
  case 100:// Конечное состояние автомата
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    flAllOff=1;
    UstPower=0; // Убираем мощность с ТЭНов
    break;
  }
}

// Процедура осуществляет ПИД регулирование температуры.
// В качестве результата возвращается коэффициент мощности, умноженный на 1000, то есть
// 1000 - это 100% подача мощности на ТЭНы.
void ProcessPIDTemp(int NeedTemp, int FactTemp)
{   
    NewErrTemp=(NeedTemp-FactTemp);
    DtTemp=NewErrTemp-OldErrTemp;
    ItTemp=ItTemp+NewErrTemp;
    
    if (ItTemp<-1000) ItTemp=-1000;
      else 
        if (ItTemp>1000) ItTemp=1000;
        
    
    OldErrTempOut=OldErrTemp; // Запоминаем старую ошибку.
    OldErrTemp=NewErrTemp; // Запоминаем старую ошибку.
    // Считаем коэффициент температуры
    KtT=((long)PIDTemp[0]*NewErrTemp*8 + (long)PIDTemp[1]*ItTemp/10+(long)PIDTemp[2]*DtTemp*10)/10;
    #ifdef TEST
      Serial.print("Ktt:");
      Serial.println(KtT);
    #endif
    if (KtT>1000) KtT=1000;
    if (KtT<0) KtT=0;
    // Если значение температуры превысило нужную, то сбрасываем интегральный коэффицциент
    // и убираем мощность.
    if (FactTemp>=NeedTemp) {ItTemp=0 ;KtT=0;}
    
}

void ProcessPIDPress(int NeedTemp, int FactTemp)
{   
    NewErrTemp=(NeedTemp-FactTemp);
    DtTemp=NewErrTemp-OldErrTemp;
    ItTemp=ItTemp+NewErrTemp;
    
    if (ItTemp<-1000) ItTemp=-1000;
      else 
        if (ItTemp>1000) ItTemp=1000;
        
    
    OldErrTempOut=OldErrTemp; // Запоминаем старую ошибку.
    OldErrTemp=NewErrTemp; // Запоминаем старую ошибку.
    // Считаем коэффициент температуры
    KtT=((long)PIDTemp[0]*NewErrTemp*8 + (long)PIDTemp[1]*ItTemp/10+(long)PIDTemp[2]*DtTemp*10)/10;
    #ifdef TEST
      Serial.print("Ktt:");
      Serial.println(KtT);
    #endif
    if (KtT>1000) KtT=1000;
    if (KtT<0) KtT=0;
    // Если значение температуры превысило нужную, то сбрасываем интегральный коэффицциент
    // и убираем мощность.
       
}



//#define MSerial1 Serial2

void GetPhonePDU()
{
  unsigned char i;
  for(i=1;i<=9;i+=2)
  {
    pdu_phone[i-1]=my_phone[i+1];
    pdu_phone[i]=my_phone[i];
  }
  pdu_phone[10]='F';
  pdu_phone[11]=my_phone[11];
  pdu_phone[12]=0;
}


void GetState()
{
  int tic1;
  unsigned char hour,minute;
  hour=Seconds / 3600;
  tic1=Seconds % 3600;
  minute=tic1 / 60;
  sprintf(my_tx_buffer,"%02u:%02u I=%3u S=%3u T=%3u,%3u,%3u V=%4u,%4u OFF=%1i,%4uW,U=%3u,%3imm",hour,minute,IspReg, StateMachine,temps[0],temps[1],temps[2],U_VODA,U_UROVEN, (int) flAllOff,UstPower,MaxVoltsOut,U_MPX5010);

}

void GetStateSerial()
{
  int tic1;
  unsigned char hour,minute,second;
  hour=Seconds / 3600;
  tic1=Seconds % 3600;
  minute=tic1 / 60;
  second=tic1 % 60;
  sprintf(my_tx_buffer,"%02u:%02u:%02u\tT=\t%3u\t%3u\t%3u\tGlv=%4u\tI=%3u\tS=%3u\t%4uW\tOFF=%1i\t%3imm\t%3uV",hour,minute,second,temps[0],temps[1],temps[2],U_GLV,IspReg,StateMachine,UstPower,(int) flAllOff,U_MPX5010,MaxVoltsOut);
  Serial.println(my_tx_buffer);
}

void GetStateSerialErr()
{
  sprintf(my_tx_buffer,"Err ds18b20 CntDs=%3u NumDs=%3u",CntErrDs18,NumErrDs18);
  Serial.println(my_tx_buffer);
}


void StateToSMS()
{
  

#ifndef TESTERR
  if (FlToGSM<2 || SMSOnCall==1)  
  { if (flGPRSState!=98) flGPRSState=70;
    timeGPRS=15;
//    toUSARTPDU(my_tx_buffer);
    lastSMSState=StateMachine;
  }
  else
  {
    // Активизируем разовую отправку состояния на сервер
    if (FlToGSM>1 && !SMSOnCall && timeGPRS==0 && flGPRSState==0)
      {
        flGPRSState=2; // Активируем сессию GPRS
        flNeedRefrServer=1;
        timeGPRS=60;
        lastSMSState=StateMachine;
      }
  }
  
#else
  Serial.print("SMS: ");
  Serial.println(my_tx_buffer);
#endif

  // Запоминаем последнее переданное состояние на сотовый.

}

#ifdef DEBUG
void my_debug()
{
  char s_rx;
  char *st1;
  char *stb;
  int val1;
  static char NoScan=0;
  if (!NoScan)
  {
  while (Serial.available() > 0) 
  {  //если есть доступные данные
    // считываем байт
    s_rx=Serial.read();
    delay(10);
    if (s_rx==10)
    {
      my_rx_buffer[pos_rx_buffer-1]=0;
      NoScan=1;
      pos_rx_buffer=0;
      // передаем данные в следующем формате:
      // сколькоприбавитьсекунд,Темпереатура1,Температура2, Температура3
      Serial.print("dbl:");
      Serial.println(my_rx_buffer);
      stb=my_rx_buffer;
      st1=strchr(stb,',');
      if (st1)
      {
        st1[0]=0;
        //      Serial.println(stb);
        val1=atoi(stb);
        Seconds=Seconds+val1;
        timeNBK-=val1;   // Время работы насоса НБК в секундах со скоростью SpeedNKB. По истечении этого времени насос остановится
        timeMIXER-=val1; // Время работы двигателя миксера в секундах. По истечении этого времени насос остановится
        time1-=val1;   // Таймер для отсчета секунд 1
        time2-=val1;   // Таймер для отсчета секунд 2
        time3-=val1;   // Таймер для отсчета секунд 3
        
        stb=st1+1;
      }
      st1=strchr(stb,',');
      if (st1)
      {
        st1[0]=0;
        //      Serial.println(stb);
        val1=atoi(stb);
        temps[0]=val1;
        stb=st1+1;
      }
      st1=strchr(stb,',');
      if (st1)
      {
        st1[0]=0;
        //       Serial.println(stb);
        val1=atoi(stb);
        temps[1]=val1;
        stb=st1+1;
      }
      //      Serial.println(stb);
      val1=atoi(stb);
      temps[2]=val1;
      flNeedAnalyse=1;
      NoScan=0;
    }
    else
    {
      if (pos_rx_buffer>=MY_RX_BUFFER_SIZE) pos_rx_buffer=0;
      my_rx_buffer[pos_rx_buffer]=s_rx;
      pos_rx_buffer++;
    }
  }
  }
  // Окончание анализа отладки
}
#endif

static unsigned int SecondsRing;
static char *vid_sms;   // Вид СМС - память или Сим-карта
static char num_sms[10];// Номер СМС.
static char flSMS=0; // Признак того, что приходила SMS и она еще не удалена.
static char fAnGSM=0; // Состояние анализа входящего звонка

// Установка угла на серве с помощью delay
void SetAngle(unsigned char Angl)
{
  int TAngl;
  unsigned char i;
  TAngl= map(Angl, 0, 180, 600, 2400);
  noInterrupts();
  for(i=0;i<50;i++)
  { digitalWrite(PIN_NASOS_NBK,HIGH);
    delayMicroseconds(TAngl);
    digitalWrite(PIN_NASOS_NBK,LOW);
    delayMicroseconds(10000-TAngl);
    delayMicroseconds(10000);
  }
 interrupts();

}

void my_lcdprint(char *s)
{ 
  char i;
  char len;
  len=strlen(lcd_buffer);  
  for(i=len;i<16;i++) s[i]=' ';
  s[16]=0;
  
  if (IspReg==111 && s[9]=='R' && flNumStrPrint==0) s[9]='N';
  
  lcd.print(s);

//#################################################################################################################
#ifdef MQTT_BROKER

  if (flNumStrPrint==0) {
    
    sprintf(lcd_mqtt_buf1, GetPMEM(buf_pmem,fmt_lcd1), s);
   
    // Убрать завершающие пробелы
    for(i=strlen(lcd_mqtt_buf1)-1; ;i--) {
      if (lcd_mqtt_buf1[i]==' ') lcd_mqtt_buf1[i]='\0';
      else break;
    }
  }
  else if (flNumStrPrint==1) {
 
    sprintf(lcd_mqtt_buf2, GetPMEM(buf_pmem,fmt_lcd2), s);
   
    // Убрать завершающие пробелы
    for(i=strlen(lcd_mqtt_buf2)-1; ;i--) {
      if (lcd_mqtt_buf2[i]==' ') lcd_mqtt_buf2[i]='\0';
      else break;
    }    
 }

#endif
//#################################################################################################################  
  
  // Если состояние отправки - надо отправить экран, тогда отправляем еще и на сервер.
  if (flGPRSState==20 || flGPRSState==164)
  {
    if (flNumStrPrint==0)
    {   // Первой строчкой отправляем id устройства и первую строку экрана
        sprintf(my_gprs_buffer,"%s;%s",idDevice,s); //запись в буфер текста и значений температуры в подготовленном
        Serial2.print(my_gprs_buffer);
        #ifdef TESTGSM
            Serial.println("SendDTA1:");
            Serial.println(my_gprs_buffer);
        #endif
        
    }
    else
    {   // Второй строчкой отправляем нужную часть экрана, состояние контроллера и последнюю выполненную команду. 
        sprintf(my_gprs_buffer,"%s;%3i;%3i;%c#",s,(int) IspReg,(int) StateMachine,cmdGPRSIsp);
        Serial2.println(my_gprs_buffer);
        #ifdef TESTGSM
             Serial.println("SendDTA2:");
            Serial.println(my_gprs_buffer);
        #endif
        flGPRSState++; // Переводим контроллер в состояние отправлено.
        //timeWaitGPRS=20;
        timeWaitGPRS=2; // Две секунды достаточно для отправки
    }
  }

  flNumStrPrint++;
  
  #ifdef TESTMEM
    Serial.println("\n[memCheck_print]");
    Serial.println(freeRam());
  #endif
}

unsigned char hour,minute,second;
int V1,V2,V3;

void formTRZGDistill()
{
  if (TempDeflBegDistil>0)
    #ifdef LCD_PROGMEM
      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_TRZGDistill_1),temps[TEMP_DEFL],TempDeflBegDistil,Power);
    #else
      sprintf(lcd_buffer,"tD=%3i/%3i %4iW",temps[TEMP_DEFL],TempDeflBegDistil,Power);
    #endif
  else
    #ifdef LCD_PROGMEM
      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_TRZGDistill_2),temps[TEMP_KUB],-TempDeflBegDistil,Power);
    #else
      sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],-TempDeflBegDistil,Power);
    #endif
}

void formTSAErr()
{
    #ifdef LCD_PROGMEM
      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_TSAErr_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
    #else
      sprintf(lcd_buffer,"%02u:%02u:%02u TSA!",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
    #endif
    //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
    my_lcdprint(lcd_buffer); //вывод сz`одержимого буфера на LCD
    #ifdef LCD_PROGMEM
      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_TSAErr_2),temps[TEMP_TSA],MAX_TEMP_TSA); //запись в буфер текста и значений температуры в подготовленном
    #else
      sprintf(lcd_buffer,"tT=%3i/%3i",temps[TEMP_TSA],MAX_TEMP_TSA); //запись в буфер текста и значений температуры в подготовленном
    #endif
}

void DisplayRectif()
{
         if (DispPage==0) 
          {
            switch (StateMachine)
            { 
            case 0: //Не запущено
            case 1: //Не запущено
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_1_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u Rect",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
             
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              //sprintf(lcd_buffer,"tK=%3i P=%4i",temps[TEMP_KUB],Power); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 2: //Разгон
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_2_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u R RZG",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif  
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              if (tEndRectRazgon>0) 
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_2_2),temps[TEMP_KUB],tEndRectRazgon,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],tEndRectRazgon,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #endif  
              else
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_2_3),temps[TEMP_RK20],-tEndRectRazgon,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"tN=%3i/%3i %4iW",temps[TEMP_RK20],-tEndRectRazgon,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #endif
              break;
            case 3: //Стабилицация колонны
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_3_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u R NSB",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_3_2),temps[TEMP_RK20],(int)(Seconds-SecTempPrev),PowerRect); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tN=%3i(%3i)%4iW",temps[TEMP_RK20],(int)(Seconds-SecTempPrev),PowerRect); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 4: //Отбор голов
              if (IspReg!=118)
              {
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"%02u:%02u:%02u R GLV",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
                #endif
                //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
      
                my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
                if (UrovenProvodimostSR==0)
                  #ifdef LCD_PROGMEM
                    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_2),temps[TEMP_KUB],tEndRectOtbGlv,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                  #else
                    sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],tEndRectOtbGlv,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                  #endif
                else
                  if (UrovenProvodimostSR>0)
                  {
                    if  (UrovenProvodimostSR==2)
                      #ifdef LCD_PROGMEM
                        sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_3),U_GLV,UROVEN_ALARM,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                      #else
                        sprintf(lcd_buffer,"Pr=%3i/%3i %4iW",U_GLV,UROVEN_ALARM,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                      #endif                      
                    else
                      #ifdef LCD_PROGMEM
                        sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_4),U_GLV,UrovenProvodimostSR,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                      #else
                        sprintf(lcd_buffer,"Pr=%3i/%3i %4iW",U_GLV,UrovenProvodimostSR,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                      #endif
                  }
                  else
                    #ifdef LCD_PROGMEM
                      sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_5),(int) SecOstatok,-UrovenProvodimostSR,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                    #else
                      sprintf(lcd_buffer,"Tm=%3i/%3i %4iW",(int) SecOstatok,-UrovenProvodimostSR,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                    #endif
                }
              else
              {
         
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_6),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"%02u:%02u:%02u RF GLV",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
                #endif  
                my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
               //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
                if (TekFraction<CountFractionRect) V2=TempFractionRect[TekFraction];
                if (V2>=0)
                  #ifdef LCD_PROGMEM
                    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_7),temps[TEMP_KUB],V2,UstPower);
                  #else
                    sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],V2,UstPower);
                  #endif  
                else
                  #ifdef LCD_PROGMEM
                    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_4_8),(int) SecOstatok,UstPower);
                  #else
                    sprintf(lcd_buffer,"t=%5imin %4iW",(int) SecOstatok,UstPower);
                  #endif  
                
              }  
              break;
            case 5: //Стоп, ожидание возврата температуры
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_5_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u R Stop",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif  
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_5_2),temps[TEMP_RK20],tStabSR,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tN=%3i/%3i %2i%%",temps[TEMP_RK20],tStabSR,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 6: //Ректификация
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_6_1),hour,minute,second,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u RSt %2i%%",hour,minute,second,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
              #endif  
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_6_2),temps[TEMP_RK20],tStabSR+tDeltaRect,tDeltaRect); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tN=%3i/%3i/d%2i",temps[TEMP_RK20],tStabSR+tDeltaRect,tDeltaRect); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 7: //Отбор хвостов
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_7_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u R Hvost",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_7_2),temps[TEMP_KUB],IspReg!=118 ? tEndRect:TekTemp); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i",temps[TEMP_KUB],IspReg!=118 ? tEndRect:TekTemp); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 8: //Отбор ожидание 3 минут
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_8_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u R Wait",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_8_2),temps[TEMP_KUB],IspReg!=118 ? tEndRect : TekTemp); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i",temps[TEMP_KUB],IspReg!=118 ? tEndRect : TekTemp); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 9: //Ожидание датчика уровня
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_9_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u R Wait",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_9_2),U_UROVEN,UROVEN_ALARM); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"by Urov=%3i/%3i",U_UROVEN,UROVEN_ALARM); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 100: //Окончание
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_100_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u Rec End",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_100_2),temps[TEMP_KUB],tEndRect); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i End",temps[TEMP_KUB],tEndRect); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 101: //Температура в ТСА превысила предельную
              formTSAErr();
              break;
            case 102: //Давление
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_102_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u Pres",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод сz`одержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_case_102_2),U_MPX5010,AlarmMPX5010); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"MPX5010=%3i/%3i",U_MPX5010,AlarmMPX5010); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
    
            }
          }
          else
          {
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_1),temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #endif
            my_lcdprint(lcd_buffer);
            if (StateMachine!=5) 
            {   if (IspReg!=118)
                  #ifdef LCD_PROGMEM
                    sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_2),U_MPX5010,U_UROVEN,U_GLV); //запись в буфер текста и значений температуры в подготовленном
                  #else
                    sprintf(lcd_buffer,"%3immV=%4u,%4u",U_MPX5010,U_UROVEN,U_GLV); //запись в буфер текста и значений температуры в подготовленном
                  #endif
                  else 
                   if (TekFraction<CountFractionRect)
                   {
                     if (TempFractionRect[TekFraction]>0) 
                       #ifdef LCD_PROGMEM
                         sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_3),U_MPX5010,TempFractionRect[TekFraction],(int) TekFraction+1,(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
                       #else
                         sprintf(lcd_buffer,"%3imm t=%3i F%1i/%1i",U_MPX5010,TempFractionRect[TekFraction],(int) TekFraction+1,(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
                       #endif
                     else
                       #ifdef LCD_PROGMEM
                         sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_4),U_MPX5010,(int) SecOstatok,(int) TekFraction+1,(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
                       #else
                         sprintf(lcd_buffer,"%3imm %4im F%1i/%1i",U_MPX5010,(int) SecOstatok,(int) TekFraction+1,(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
                       #endif
                   }
                   else
                      #ifdef LCD_PROGMEM
                        sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_5),U_MPX5010,U_UROVEN,(int) TekFraction+1,(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
                      #else
                        sprintf(lcd_buffer,"%3immV=%4u F%1i/%1i",U_MPX5010,U_UROVEN,(int) TekFraction+1,(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
                      #endif
            }
            else
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_rect_6),U_MPX5010,time1); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%3imm reSt=%5i",U_MPX5010,time1); //запись в буфер текста и значений температуры в подготовленном
              #endif
          }
   
}

void DisplayDistDefl()
{
            if (DispPage==0) 
          {
            switch (StateMachine)
            { 
            case 0: //Не запущено
            case 1: //Нагрев до температуры 50 градусов
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_1_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              formTRZGDistill();
              break;
            case 2: //Ожидание закипания (прогреется дефлегматор)
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_2_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD Rzg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_2_2),temps[TEMP_DEFL],TempDeflBegDistil,UstPower);
              #else
                sprintf(lcd_buffer,"tD=%3i/%3i %4iW",temps[TEMP_DEFL],TempDeflBegDistil,UstPower);
              #endif
              break;
            case 3: //Ожидание закипания (прогреется куб дефлегматор)
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_3_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD Rzg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_3_2),temps[TEMP_KUB],TempDeflBegDistil,UstPower); 
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],TempDeflBegDistil,UstPower); 
              #endif
              break;
            case 4: //Работа без дефлегматора
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_4_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD BezD",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif  
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_4_2),temps[TEMP_DEFL],TempDefl,UstPower); 
              #else
                sprintf(lcd_buffer,"tD=%3i/%3i %4iW",temps[TEMP_DEFL],TempDefl,UstPower); 
              #endif
              break;
            case 5: //Работа с 50% дефлегмацией
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_5_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD 50%%",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_5_2),temps[TEMP_DEFL],TempDefl-DeltaDefl,UstPower); 
              #else
                sprintf(lcd_buffer,"tD=%3i/%3i %4iW",temps[TEMP_DEFL],TempDefl-DeltaDefl,UstPower); 
              #endif
              break;
            case 6: //Работа с 100% дефлегмацией
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_6_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD 100%%",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_6_2),temps[TEMP_DEFL],TempDefl-DeltaDefl,UstPower); 
              #else
                sprintf(lcd_buffer,"tD=%3i/%3i %4iW",temps[TEMP_DEFL],TempDefl-DeltaDefl,UstPower); 
              #endif
              break;
            case 7: //Ожидание для охлаждения
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_7_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD Wait",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_7_2),temps[TEMP_KUB],SecondsEnd); //запись в буфер текста и значений температуры в подготовленном
              #else
              sprintf(lcd_buffer,"tK=%3i Wait=%4i",temps[TEMP_KUB],SecondsEnd); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 100: //Окончание
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_100_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u DD end",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_case_100_2),temps[TEMP_KUB]); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i end",temps[TEMP_KUB]); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            }
          }
          else
          {
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_1),temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном#
            #else
              sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #endif
            my_lcdprint(lcd_buffer);
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_DDefl_2),U_VODA,U_UROVEN,deltaPower); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"V%4u,%4u-%4iW",U_VODA,U_UROVEN,deltaPower); //запись в буфер текста и значений температуры в подготовленном
            #endif
          }

}
/*
void DisplaySimpleGLV()
{
          
          if (DispPage==0)
          {
            switch (StateMachine)
            { 
            case 0: //Не запущено
            case 1: //Нагрев до температуры 50 градусов
              sprintf(lcd_buffer,"%02u:%02u:%02u sGl Not",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              formTRZGDistill(); 
              break;
            case 2: //Ожидание закипания (прогреется дефлегматор)
              sprintf(lcd_buffer,"%02u:%02u:%02u sGl Rzg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              formTRZGDistill(); 
              break;
            case 3: //Ожидание закипания (прогреется куб дефлегматор)
              sprintf(lcd_buffer,"%02u:%02u:%02u sGl Rzg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],TempDeflBegDistil,Power); 
              break;
            case 4: //Отбор голов
              sprintf(lcd_buffer,"%02u:%02u:%02u sGl Otb",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],Temp3P,PowerGlvDistil); 
              break;
            case 5: //Ожидание 60 секунд для охлаждения
              sprintf(lcd_buffer,"%02u:%02u:%02u sOtbGlv",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i Wait=%4i",temps[TEMP_KUB],SecondsEnd); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 100: //Окончание
              sprintf(lcd_buffer,"%02u:%02u:%02u sOtbGlv",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i end",temps[TEMP_KUB]); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 101: //Температура в ТСА превысила предельную
              formTSAErr();
              break;
            }
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"V=%4u,%4u,%4u",U_VODA,U_UROVEN,U_GLV); //запись в буфер текста и значений температуры в подготовленном
          }
}*/

void DisplaySimpleDistill()
{
            if (DispPage==0) 
          {
            if (IspReg==104) 
            { 
              V1=1;
              V2=Temp1P;   
            }
            if (IspReg==106) 
            { 
              V1=2;
              V2=Temp2P;   
            }
            if (IspReg==107   || IspReg==105 ) 
            { 
              V1=3;
              V2=Temp3P;   
            }
    
            if (IspReg!=105)
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_SDist_1),hour,minute,second,V1); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u %1u-Otbor",hour,minute,second,V1); //запись в буфер текста и значений температуры в подготовленном
              #endif
            else
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_SDist_2),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u sGl",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              
            switch (StateMachine)
            { 
            case 0: //Не запущено
            case 1: //Нагрев до температуры 50 градусов
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              break;
            case 2: //Ожидание закипания (прогреется дефлегматор)
              if (IspReg!=105)
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_SDist_case_2_1),hour,minute,second,V1); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"%02u:%02u:%02u %1u-O Rzg",hour,minute,second,V1); //запись в буфер текста и значений температуры в подготовленном
                #endif
              else
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_SDist_case_2_2),hour,minute,second,V1); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"%02u:%02u:%02u sGl Rzg",hour,minute,second,V1); //запись в буфер текста и значений температуры в подготовленном
                #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              formTRZGDistill();
              break;
            case 3: //Дистилляция
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],V2,UstPower); 
              break;
            case 4: //Ожидание 60 секунд для охлаждения
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i Wait=%4i",temps[TEMP_KUB],SecondsEnd); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 100: //Окончание
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i end",temps[TEMP_KUB]); //запись в буфер текста и значений температуры в подготовленном
              break;                        
            case 101: //Температура в ТСА превысила предельную
              formTSAErr();
              break;
            }
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"V%4u,%4u %4uW",U_VODA,U_UROVEN,UstPower); //запись в буфер текста и значений температуры в подготовленном
          }

}
void DisplayFracionDistill()
{
           if (DispPage==0) 
          {
            sprintf(lcd_buffer,"%02u:%02u:%02u DsF %1i/%1i",hour,minute,second,(int) (TekFraction<CountFractionDist) ? TekFraction+1: CountFractionDist,(int) CountFractionDist); //запись в буфер текста и значений температуры в подготовленном
    
            switch (StateMachine)
            { 
            case 0: //Не запущено
            case 1: //Нагрев до температуры 50 градусов
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              break;
            case 2: //Ожидание закипания (прогреется дефлегматор)
              sprintf(lcd_buffer,"%02u:%02u:%02u DsF Rzg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              formTRZGDistill();
              break;
            case 3: //Дистилляция
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              
              if (TekFraction<CountFractionDist) V2=TempFractionDist[TekFraction];
              if (V2>=0) sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],V2,UstPower);
                    else sprintf(lcd_buffer,"t=%5imin %4iW",(int) SecOstatok,UstPower);
                    
              break;
            case 4: //Ожидание 60 секунд для охлаждения
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i Wait=%4i",temps[TEMP_KUB],time3); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 100: //Окончание
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i end",temps[TEMP_KUB]); //запись в буфер текста и значений температуры в подготовленном
              break;                        
            case 101: //Температура в ТСА превысила предельную
              formTSAErr();
              break;
            }
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"V%4u,%4u %4uW",U_VODA,U_UROVEN,UstPower); //запись в буфер текста и значений температуры в подготовленном
          }

}
void DisplayRazvar()
{
           if (DispPage==0) 
          {
            if (IspReg==114)
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u HLD Ztr",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif                
            else
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_2),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u Razvar",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
            my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
            
            switch (StateMachine)
            { 
            case 0: //Не запущено
//            case 1: //Нагрев до температуры 50 градусов
//              sprintf(lcd_buffer,"t=%3i P=%4i",temps[TEMP_RAZVAR],UstPower); //запись в буфер текста и значений температуры в подготовленном
//              break;
            case 1: //Нагрев до температуры 50 градусов
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_1_1),temps[TEMP_RAZVAR],(int)TempZSP*10,UstPower); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i/%3i %4iW",temps[TEMP_RAZVAR],(int)TempZSP*10,UstPower); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 2: //Поддержание температуры 50 градусов, пока не произойдет ручной переход к следующему этапу.
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_2_1),temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i ZASYP & Dn",temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 3: //Нагрев до температуры 64 градуса
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_3_1),temps[TEMP_RAZVAR],(int)TempZSPSld*10,UstPower); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i/%3i %4iW",temps[TEMP_RAZVAR],(int)TempZSPSld*10,UstPower); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 4: //Ожидание 15 минут, поддержка температуры
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_4_1),temps[TEMP_RAZVAR],UstPower,time2); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i %4iW %4i",temps[TEMP_RAZVAR],UstPower,time2); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 5: //Нагрев до кипения
              if (ds1820_devices<2)
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_5_1),temps[TEMP_RAZVAR],TempKipenZator,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_RAZVAR],TempKipenZator,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #endif
              else
                #ifdef LCD_PROGMEM
                  sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_5_2),temps[TEMP_DEFL],TempKipenZator,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #else
                  sprintf(lcd_buffer,"tD=%3i/%3i %4iW",temps[TEMP_DEFL],TempKipenZator,UstPower); //запись в буфер текста и значений температуры в подготовленном
                #endif
              break;
            case 6: //Варка
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_6_1),temps[TEMP_RAZVAR],PowerRazvZerno,time2); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i %4iW %4i",temps[TEMP_RAZVAR],PowerRazvZerno,time2); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 7: //Охлаждение до температыры осахаривания
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_7_1),temps[TEMP_RAZVAR],(int)TempZSPSld*10); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i HLD-1",temps[TEMP_RAZVAR],(int)TempZSPSld*10); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 8: //Поддержание температуры 50 градусов, пока не произойдет ручной переход к следующему этапу.
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_8_1),temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i SOLOD & Dn",temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 9: //Мешаем 10 минут
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_9_1),temps[TEMP_RAZVAR],time2); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i MIX %4i",temps[TEMP_RAZVAR],time2); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 10: //Осахаривание
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_10_1),temps[TEMP_RAZVAR],time2); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i OSH %4i",temps[TEMP_RAZVAR],time2); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 11: //Охлаждение до температуры первичного внесения дрожжей осахаривания
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_11_1),temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/400 HLD-2",temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 12: //Охлаждение до температыры осахаривания
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_12_1),temps[TEMP_RAZVAR],(int)TempHLDZatorBrog1*10); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i HLD-3",temps[TEMP_RAZVAR],(int)TempHLDZatorBrog1*10); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 13: //Поддержка брожения, ничего не делаем, только мешаем периодически
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_13_1),temps[TEMP_RAZVAR],(int)TempHLDZatorBrog1*10); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i BROG",temps[TEMP_RAZVAR],(int)TempHLDZatorBrog1*10); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 14: //Поддержка брожения - охлаждение
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_14_1),temps[TEMP_RAZVAR],(int)TempHLDZatorBrog1*10-5); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i/%3i BR-H",temps[TEMP_RAZVAR],(int)TempHLDZatorBrog1*10-5); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 100: //Окончание
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_case_100_1),temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"t=%3i end",temps[TEMP_RAZVAR]); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            }
          }
          else
          {
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_3),temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #endif
            my_lcdprint(lcd_buffer);
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_razv_4),(int)TempZSPSld*10,U_NPG); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"OSH=%3u NPG=%4u",(int)TempZSPSld*10,U_NPG); //запись в буфер текста и значений температуры в подготовленном
            #endif
          }
 
}
/*
void DisplayNDRF()
{
            if (DispPage==0) 
          {
            switch (StateMachine)
            { 
            case 0: //Не запущено
            case 1: //Не запущено
              sprintf(lcd_buffer,"%02u:%02u:%02u NDRF",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i P=%4i",temps[TEMP_KUB],Power); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 2: //Разгон
              sprintf(lcd_buffer,"%02u:%02u:%02u ND RZG",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              if (tEndRectRazgon>0) sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],tEndRectRazgon,UstPower); //запись в буфер текста и значений температуры в подготовленном
                               else sprintf(lcd_buffer,"tN=%3i/%3i %4iW",temps[TEMP_RK20],-tEndRectRazgon,UstPower); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 3: //Стабилицация колонны
              sprintf(lcd_buffer,"%02u:%02u:%02u ND NSB",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"t=%3i(%4i)%4iW",temps[TEMP_RK20],(int)(SecOstatok),PowerRect); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 4: //Отбор голов
              sprintf(lcd_buffer,"%02u:%02u:%02u ND GLV",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              if (UrovenProvodimostSR==0)
                sprintf(lcd_buffer,"tK=%3i/%3i %4iW",temps[TEMP_KUB],tEndRectOtbGlv,PowerRect); //запись в буфер текста и значений температуры в подготовленном
              else
                if (UrovenProvodimostSR>0)
                  sprintf(lcd_buffer,"Pr=%3i/%3i %4iW",U_GLV,UrovenProvodimostSR,PowerRect); //запись в буфер текста и значений температуры в подготовленном
                else
                  sprintf(lcd_buffer,"Tm=%3i/%3i %4iW",(int) SecOstatok,-UrovenProvodimostSR,PowerRect); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 5: //Стоп, ожидание возврата температуры
              sprintf(lcd_buffer,"%02u:%02u:%02u ND Stop",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tN=%3i/%3i %2i%%",temps[TEMP_RK20],tStabSR,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 6: //Ректификация
              sprintf(lcd_buffer,"%02u:%02u:%02u NDst %2i%%",hour,minute,second,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tN=%3i/%3i/d%2i",temps[TEMP_RK20],tStabSR+tDeltaRect,tDeltaRect); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 7: //Отбор хвостов
              sprintf(lcd_buffer,"%02u:%02u:%02u ND Hvst",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i/%3i",temps[TEMP_KUB],tEndRect); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 8: //Отбор ожидание 3 минут
              sprintf(lcd_buffer,"%02u:%02u:%02u ND Wait",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i/%3i",temps[TEMP_KUB],tEndRect); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 9: //Ожидание датчика уровня
              sprintf(lcd_buffer,"%02u:%02u:%02u R Wait",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"by Urov=%3i/%3i",U_UROVEN,UROVEN_ALARM); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 100: //Окончание
              sprintf(lcd_buffer,"%02u:%02u:%02u ND End",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"tK=%3i/%3i",temps[TEMP_KUB],tEndRect); //запись в буфер текста и значений температуры в подготовленном
              break;
            case 101: //Температура в ТСА превысила предельную
              formTSAErr();
              break;
            case 102: //Давление
              sprintf(lcd_buffer,"%02u:%02u:%02u Pres",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод сz`одержимого буфера на LCD
              sprintf(lcd_buffer,"MPX5010=%3i/%3i",U_MPX5010,AlarmMPX5010); //запись в буфер текста и значений температуры в подготовленном
              break;
    
            }
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"%3immV=%4u,%4u",U_MPX5010,U_UROVEN,U_GLV); //запись в буфер текста и значений температуры в подготовленном
          }

}*/

void DisplayNBK()
{
          if (DispPage==0) 
          {
            switch (StateMachine)
            { 
            case 0: //Не запущено
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_0_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u NBK",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_0_2),temps[TEMP_KUB],U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tK=%3i,%3imm",temps[TEMP_KUB],U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 1: //Ожидание закипания (прогреется дефлегматор)
            case 2: //Ожидание закипания (прогреется дефлегматор)
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_2_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u NBK Rzg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              formTRZGDistill();
            break;
            case 3: //Ожидание запуска
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_3_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u NBK Beg",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_3_2),U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"UP to Beg %3imm",U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 4: //Запущено
            case 6: //Запущено, подача браги остановлена
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_6_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u NBK Wrk",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_6_2),(int) SpeedNBK,U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"Sp=%4i,%3imm",(int) SpeedNBK,U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 5: //Превышение температуры вверху
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_5_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u NBK tmp",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_5_2),temps[TEMP_DEFL],time2); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"tD=%3i/970,s%4i",temps[TEMP_DEFL],time2); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            case 100: //Окончание
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_100_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"%02u:%02u:%02u NBK End",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
              #endif
              //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #ifdef LCD_PROGMEM
                sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_case_100_2),(int) SpeedNBK,U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #else
                sprintf(lcd_buffer,"Sp=%4i,%3imm",(int) SpeedNBK,U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
              #endif
              break;
            }
          }
          else
          {
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_1),temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            #endif
            my_lcdprint(lcd_buffer);
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_nbk_2),UstPower,U_MPX5010,MaxPressByPeriod); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"%4uW,%3imm %3i",UstPower,U_MPX5010,MaxPressByPeriod); //запись в буфер текста и значений температуры в подготовленном
            #endif
          }
}
void DisplayTestKLP()
{
    if (DispPage==0) 
    {
      sprintf(lcd_buffer,"%02u:%02u:%02u tst Dev",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
      //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf(lcd_buffer,"St=%3i CHIM=%3i",StateMachine,ProcChimSR); //запись в буфер текста и значений температуры в подготовленном
    }
    else
    {
      sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"%3u,%3u Gas=%4u",temps[3],temps[4],U_GAS); //запись в буфер текста и значений температуры в подготовленном
    }
 
}

void DisplayBeer()
{
    if (DispPage==0) 
    {
      #ifdef LCD_PROGMEM
        sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_1),hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
      #else
        sprintf(lcd_buffer,"%02u:%02u:%02u beer",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
      #endif
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      if (StateMachine==2)   
        #ifdef LCD_PROGMEM
          sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_2),temps[TEMP_TERMOSTAT],(int) tempP[KlTek]*10,(int) KlTek,(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
        #else
          sprintf(lcd_buffer,"T=%3i/%3i P%2i/%2i",temps[TEMP_TERMOSTAT],(int) tempP[KlTek]*10,(int) KlTek,(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
        #endif
      if (StateMachine==3)  
       {  if (timeP[KlTek]!=0)
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_3),time2,(int) KlTek,(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"Ts=%5i P=%2i/%2i",time2,(int) KlTek,(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
            #endif
          else
            #ifdef LCD_PROGMEM
              sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_4),(int) KlTek+1,(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
            #else
              sprintf(lcd_buffer,"Press UP to%2i/%2i",(int) KlTek+1,(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
            #endif      
     
     }             
      if (StateMachine==100)
        #ifdef LCD_PROGMEM
          sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_5)); //запись в буфер текста и значений температуры в подготовленном
        #else
          sprintf(lcd_buffer,"end"); //запись в буфер текста и значений температуры в подготовленном
        #endif
    }
    else
    {
      #ifdef LCD_PROGMEM
        sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_6),temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
      #else
        sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
      #endif  
      my_lcdprint(lcd_buffer);
      #ifdef LCD_PROGMEM
        sprintf(lcd_buffer,GetPMEM(buf_pmem,lcd_beer_7),time1,UstPower); //запись в буфер текста и значений температуры в подготовленном
      #else
        sprintf(lcd_buffer,"mix=%5i %4iW",time1,UstPower); //запись в буфер текста и значений температуры в подготовленном
      #endif
    }
}

void DisplayData()
{
  int tic1;
  static int PrevState=0;
  static char flDspDop=0; // Флаг для отображения информации (через раз)
  char j; // Временная переменная для цикла
  flNumStrPrint=0;
  flDspDop=!flDspDop;

  hour=Seconds / 3600;
  tic1=Seconds % 3600;
  minute=tic1 / 60;
  second=tic1 % 60;   

  if (FlState==0) 
    lcd.setCursor(0, 0);
  else 
  { 
    // Если предыдущее состояние было 0, то даем задерку в 1/10 секунды, чтобы не слишком часто обновлялся
    // дисплей.
    if (PrevState==0)  delay(100);

    if (FlState!=PrevState)
    {
      lcd.clear();  //очистка дисплея   
      PrevState=FlState;
    }

    lcd.setCursor(0, 0);
  }

  #ifdef TESTMEM
    Serial.println("\n[memCheck_displ]");
    Serial.println(freeRam());
  #endif
    
  switch (FlState)
  {
  case 0:
  
    if (DispPage<2) // Первые две страницы отображаются по-разному, остальные всегда одни и те же
      {
        switch (IspReg)
        {
        case 101: // Displaying
          //sprintf(lcd_buffer,"S=%5u TT=%5u",Seconds,MaxTimeTimer); //запись в буфер текста и значений температуры в подготовленном
    
          //  Выводим сетевое напряжение по тому же принципу, что и работают дешевые вольтметры, то есть максимальное количество вольт в сети умножаем на 0,707
          //  это напряжение нужно для калибровки нашего измерителя
    
          if (DispPage==0) 
          {
            sprintf(lcd_buffer,"%02u:%02u:%02u DSP 1/2",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"Zr=%4u U=%3i",TicZero,MaxVoltsOut); //запись в буфер текста и значений температуры в подготовленном
          }
    
          if (DispPage==1) 
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"V=%4u,%4u,%4u",U_VODA,U_UROVEN,U_GLV); //запись в буфер текста и значений температуры в подготовленном
          }
//          lcd.setCursor(0, 1);
//          my_lcdprint(lcd_buffer);                      
    
          break;
        case 102: // Термостат         
          if (DispPage==0) 
          {
            sprintf(lcd_buffer,"%02u:%02u:%02u TermSt",hour,minute,second,Delta); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
            sprintf(lcd_buffer,"t=%3i/%3i(%2i) %1i",temps[TEMP_TERMOSTAT],TempTerm,Delta,StateMachine); //запись в буфер текста и значений температуры в подготовленном
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"%3u,%3u P=%3imm",temps[3],temps[4],U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
          }
          break;
        case 115: // Таймер
          if (DispPage==0) 
          {
            sprintf(lcd_buffer,"%02u:%02u:%02u Timer",hour,minute,second,Delta); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
            minute=Seconds / 60;
            sprintf(lcd_buffer,"tm=%3i/%3i %4iW",minute,timerMinute,UstPower); //запись в буфер текста и значений температуры в подготовленном
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            sprintf(lcd_buffer,"%3u,%3u %4iW",temps[3],temps[4],PowerMinute); //запись в буфер текста и значений температуры в подготовленном
          }
          break;
        case 116: // Пивоварня - клон браумастера
          DisplayBeer();
          break;
        case 103: // Регулятор мощности
          if (DispPage==0) 
          {
            #ifndef USE_SLAVE
              sprintf(lcd_buffer,"%02u:%02u:%02u RMU=%3u",hour,minute,second,MaxVoltsOut); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              #if SIMPLED_VERSION==20
              sprintf(lcd_buffer,"P=%3u/%3u%% c%3u",UstPowerReg/10,Power/10,indexOut); //запись в буфер текста и значений температуры в подготовленном
              #else
              sprintf(lcd_buffer,"P=%4u/%4u c%3u",UstPowerReg,Power,indexOut); //запись в буфер текста и значений температуры в подготовленном
              #endif
              #ifdef TEST
              if (NUM_PHASE>1) sprintf(lcd_buffer,"%s",my_rx_buffer); //запись в буфер текста и значений температуры в подготовленном
              #endif
            #else
              sprintf(lcd_buffer,"%02u:%02u:%02u RMU=%3u",hour,minute,second,MaxVoltsOut); //запись в буфер текста и значений температуры в подготовленном
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf(lcd_buffer,"PSlv=%4u/%4u",UstPower,Power); //запись в буфер текста и значений температуры в подготовленном
            #endif
          }
          else
          {
            sprintf(lcd_buffer,"D2 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
            my_lcdprint(lcd_buffer);
            #ifndef USE_SLAVE
            sprintf(lcd_buffer,"Up to auto=%4uW",Power); //запись в буфер текста и значений температуры в подготовленном
            #else
            if (flAutoDetectPowerTEN)
              sprintf(lcd_buffer,"Up to auto=%4uW",Power); //запись в буфер текста и значений температуры в подготовленном
            else
              sprintf(lcd_buffer,"SlaveOn (Up)=%1i",(int) SlaveON); //запись в буфер текста и значений температуры в подготовленном
            #endif
          }
          break;
        case 105: // Отбор голов33
//          DisplaySimpleGLV();
//          break;
        case 104: // Перевый (недробный) отбор
        case 106: // Второй дробный отбор
        case 107: // Третий дробный отбор
          DisplaySimpleDistill();
          break;
    
       case 117: // Фракционная перегонка
          DisplayFracionDistill();
          break;
    
        case 108: // Разваривание
        case 113: // Разваривание мучно-солодового затора (без варки).
        case 114: // Разваривание
          DisplayRazvar();
          break;
        case 109: // Ректификация
        case 118: // Ректификация
          DisplayRectif();
          break;
          #ifndef TEST
          #ifndef DEBUG // Это отображение информации выкусываем, если в режиме теста, потому что не хватает памяти
          #ifndef TESTGSM
          #ifndef TESTGSM1
        case 110: // Дистилляция с дефлегматором
          DisplayDistDefl();
          break;
        case 111: // НДРФ
         DisplayRectif();
         break;
          #endif
          #endif
          #endif
          #endif
             
        case 112: // NBK
          DisplayNBK();
          break;
    
        case 130: // Резерв
          sprintf(lcd_buffer,"%02u:%02u:%02u Reserv",hour,minute,second); //запись в буфер текста и значений температуры в подготовленном
          //sprintf(lcd_buffer,"%u%u%u%u  %u%u%u%u",PINB.4,PINB.3,PINB.2,PINA.7, PINA.6,PINA.5,PINA.4,PINA.3); //запись в буфер текста и значений температуры в подготовленном                    
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"KLP %3i %3i /%1i",KlOpen[KlTek],KlClose[KlTek],Angle); //запись в буфер текста и значений температуры в подготовленном
          break;
        case 129: // тест клапанов
          DisplayTestKLP();
          break;
        case 248:
//          sprintf(lcd_buffer,"LOW LEVEL VOLTS!"); //запись в буфер текста и значений температуры в подготовленном
//          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
//          sprintf(lcd_buffer,"Umax=%i Sec=%3i",MaxVoltsOut,(int) ErrCountIndex*12); //запись в буфер текста и значений температуры в подготовленном
          break;
        case 249:
          sprintf(lcd_buffer,"NO DETECT ZERO!"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"Umax=%i",MaxVoltsOut); //запись в буфер текста и значений температуры в подготовленном
          break;
        case 250:
          sprintf(lcd_buffer,"ALARM VODA !"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          break;
        case 251:
          sprintf(lcd_buffer,"ERR Ds18b20!"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"Num=%i",(int) NumErrDs18); //запись в буфер текста и значений температуры в подготовленном
    
          break;
        case 252:
          sprintf(lcd_buffer,"ALARM GAS SENS!"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          break;
        case 253:
          sprintf(lcd_buffer,"NPG OSUHENIE!"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"U_NPG=%i CNT=%i",U_NPG, countAlrmNPG); //запись в буфер текста и значений температуры в подготовленном
          break;
        case 254:
          sprintf(lcd_buffer,"NPG PEREPOLN!"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"U_NPG=%i CNT=%i",U_NPG, countAlrmNPG); //запись в буфер текста и значений температуры в подготовленном
          break;
    
        }   
        // Это обычное состояние
        //
        if (StateNPG==1 && DispPage==0) sprintf(lcd_buffer,"Napolnenie NPG.."); //запись в буфер текста и значений температуры в подготовленном
    
    }
     // Третья страница отображения универсальна, там показываем напряжение, дистанцию и возможное число ошибок
     // расчета среднеквадратичного. 
     
    if (DispPage==2)
    {
      sprintf(lcd_buffer,"D3 %3i,w%3u,i%3i",(int)flGPRSState,(int)timeWaitGPRS,(int)timeGPRS); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"%4iW,%3uV,%3uA",FactPower,MaxVoltsOut,MaxIOut); //запись в буфер текста и значений температуры в подготовленном
    }
    // На четвертой странице показываем всю температуру и давление
    if (DispPage==3)
    {
        sprintf(lcd_buffer,"D4 T=%3u,%3u,%3u",temps[0],temps[1],temps[2]); //запись в буфер текста и значений температуры в подготовленном
        my_lcdprint(lcd_buffer);
        sprintf(lcd_buffer,"%3u,%3u P=%3imm",temps[3],temps[4],U_MPX5010); //запись в буфер текста и значений температуры в подготовленном
    }
    
    // На пятой странице показываем состояние датчиков уровней
    if (DispPage==4)
    {
      sprintf(lcd_buffer,"D5 VODA=%4u", U_VODA); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"URV=%4u GL=%4u",U_UROVEN,U_GLV); //запись в буфер текста и значений температуры в подготовленном
    }

    // На шестой странице показываем состояние датчиков уровней
    if (DispPage==5)
    {
      sprintf(lcd_buffer,"D6 GAS=%4u", U_GAS); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"zPS=%3u,%4uW",(int) zPSOut,UstPwrPH1); //запись в буфер текста и значений температуры в подготовленном
    }

    // На седьмой странице показываем состояние регулятора мощности
    if (DispPage==6)
    {
      #if NUM_PHASE==1
          sprintf(lcd_buffer," %4i,%4i,%4iW",UstPwrPH1,UstPwrPH2,UstPwrPH3); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer);
          sprintf(lcd_buffer,"D7 %3i,%3i,%3i %%",KtPhase[0],KtPhase[1],KtPhase[2]); //запись в буфер текста и значений температуры в подготовленном
      #else
          sprintf(lcd_buffer,"D7 T=%4u,Z=%4u", TimeOpenTriac,TicZero); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer);
          sprintf(lcd_buffer,"KLP=%4u,Bz=%3u",TimeOpenKLP,(int)b_value[0]); //запись в буфер текста и значений температуры в подготовленном
      #endif
    }

    // На восьмой можно сменить состояние процесса.
    if (DispPage==7)
    {
      sprintf(lcd_buffer,"D8 State Debug"); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"StateMacine=%i",(int)StateMachine); //запись в буфер текста и значений температуры в подготовленном
    }
    
    // На девятой состояние ПИД-регулятора
    if (DispPage==8)
    {
      sprintf(lcd_buffer,"D9 N=%4i O=%4i",NewErr,OldErrOut); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"Dt=%4i It=%4i",Dt,It); //запись в буфер текста и значений температуры в подготовленном
    }
    // На Десятой странице показываем максимальные температуры за процесс температуру и давление
    if (DispPage==9)
    {
        sprintf(lcd_buffer,"D10 MaxT=%3u,%3u",Maxtemps[0],Maxtemps[1]); //запись в буфер текста и значений температуры в подготовленном
        my_lcdprint(lcd_buffer);
        sprintf(lcd_buffer,"%3u,%3u,%3u",Maxtemps[2],Maxtemps[3],Maxtemps[4]); //запись в буфер текста и значений температуры в подготовленном
    }
    // На 11  странице показываем параметры расчета ПИД-регулятора
    if (DispPage==10)
    {
      sprintf(lcd_buffer,"D11 N,O=%3i %3i",NewErrTemp,OldErrTempOut); //запись в буфер текста и значений температуры в подготовленном
      my_lcdprint(lcd_buffer);
      sprintf(lcd_buffer,"KId%4i %4i %3i",KtT,ItTemp,DtTemp); //запись в буфер текста и значений температуры в подготовленном
    }

    if (flDspDop && DispDopInfo>0) // Если отображаем дополнительную информацию, тогда проверяем ее статус
    {

      if (BeepStateProcess>1) 
      {
        if (DispDopInfo==3 || DispDopInfo==4)
        {
          if (!(BeepStateProcess&B00000010) && DispDopInfo==3) my_beep(BEEP_LONG); // Не пищим, если установлена маска в 2 разряде
          if (!(BeepStateProcess&B00000100) && DispDopInfo==4) my_beep(BEEP_LONG); // Не пищим, если установлена маска в 3 разряде
        }
        else
          my_beep(BEEP_LONG); // Сначала пищим, предупреждая.
      }
      else
        my_beep(BEEP_LONG); // Сначала пищим, предупреждая.
      
      
      if(DispDopInfo==1) sprintf(lcd_buffer,"Smena TARA!(%3i)",(IspReg==117 || IspReg==118) ? COUNT_ALARM_UROVEN_FR-CountAlarmUroven :COUNT_ALARM_UROVEN-CountAlarmUroven); //Выводим на экран инфомацию о необходимости замены тары.
      if(DispDopInfo==2) sprintf(lcd_buffer,"VODA! (%3i)",COUNT_ALARM_VODA-CountAlarmVoda); //Выводим на экран инфомацию о необходимости замены тары.
      if(DispDopInfo==3) sprintf(lcd_buffer,"Low Power! %3iV",MaxVoltsOut); //Выводим на экран инфомацию о необходимости замены тары.
      if(DispDopInfo==6) sprintf(lcd_buffer,"HIGH Power! %3iV",MaxVoltsOut); //Выводим на экран инфомацию о необходимости замены тары.
      if(DispDopInfo==4) sprintf(lcd_buffer,"Err temp! ds=%2i",(int) NumErrDs18 ); //Выводим на экран инфомацию о ошибке чтения датчика.
      if(DispDopInfo==5) sprintf(lcd_buffer,"!PRESS=%3i/%3i",U_MPX5010,AlarmMPX5010); //Выводим на экран инфомацию о ошибке чтения датчика.
      
      // Активизируем разовую отправку состояния на сервер
      if (FlToGSM>1 && flGPRSState!=20  && flGPRSState!=164 && (timeGPRS==0 || flNeedCall==1) )
        {
          if (flNeedCall==1) 
           { 
              flGPRSState=40; // Активируем звонок если в этом есть необходимость
           }
           else  
           {   // Если в данный момент не производится дозвон, тогда активируем GPRS сессию. 
               if (flNeedCall!=2) flGPRSState=2;
           }
           // Активируем длительную сессию GPRS
          flNeedRefrServer=0;
          timeGPRS=250;
        }
    }
    
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD

    break;
  case 100:
    sprintf(lcd_buffer,"UstParam",FlState); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 101:
    sprintf(lcd_buffer,"IsR = Displaying"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 102:
    sprintf(lcd_buffer,"IsR = Termostat"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 103:
    sprintf(lcd_buffer,"IsR = Reg Power"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 104:
    sprintf(lcd_buffer,"IsR = 1 ND otbor"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 105:
    sprintf(lcd_buffer,"IsR = otbor GLV"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 106:
    sprintf(lcd_buffer,"IsR = 2 DR otbor"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 107:
    sprintf(lcd_buffer,"IsR = 3 DR otbor"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 108:
    sprintf(lcd_buffer,"IsR = Razvar"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Zerno"); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 109:
    sprintf(lcd_buffer,"IsR = Rectif"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 110:// Дистилляция с дефлегматором
    sprintf(lcd_buffer,"IsR = Dist Defl"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 111:// НДРФ
    sprintf(lcd_buffer,"IsR = NDRF"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 112:// НБК
    sprintf(lcd_buffer,"IsR = NBK"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 113:// солодо-мучной затор (без варки
    sprintf(lcd_buffer,"IsR = Zator"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Solod - Muka",PowerRect); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 114:// Охлаждение затора
    sprintf(lcd_buffer,"IsR = HLD Zator"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"by Chiller&Mixer",PowerRect); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 115:// Таймер
    sprintf(lcd_buffer,"IsR = Timer"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"ust Power"); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 116:// Варка пива
    sprintf(lcd_buffer,"IsR = Brewing"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Beer"); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
   case 117:// Фракционная дистилляция
    sprintf(lcd_buffer,"IsR = Distill"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Fractional"); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
   case 118:// Фракционная ректификация
    sprintf(lcd_buffer,"IsR = Rectif"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Fractional"); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
 case 130:
    sprintf(lcd_buffer,"IsR = Reserv"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"(%i)",FlState); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 129:
    sprintf(lcd_buffer,"IsR = Test"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Device (%i)",FlState); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 200:
    sprintf(lcd_buffer,"Max_t_Tst=%5i",TempTerm); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 201:
    sprintf(lcd_buffer,"Power TEN=%5u",Power); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 202:
    sprintf(lcd_buffer,"Power Reg=%5u",UstPowerReg); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 203:
    sprintf(lcd_buffer,"ParamUSART=%u",FlToUSART); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 204:
    sprintf(lcd_buffer,"ParamGSM=%u",FlToGSM); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    if(FlToGSM==1 || FlToGSM==0) sprintf(lcd_buffer,"GSM (SMS)"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==2) sprintf(lcd_buffer,"GPRS Megafon"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==3) sprintf(lcd_buffer,"GPRS BeeLine"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==4) sprintf(lcd_buffer,"GPRS MTS"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==5) sprintf(lcd_buffer,"GPRS Rostelecom"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==6) sprintf(lcd_buffer,"GPRS Tele 2"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM>=7) sprintf(lcd_buffer,"GPRS Reserv %i", (int) FlToGSM); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==10) sprintf(lcd_buffer,"Wi-Fi (UDP)"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM==11) sprintf(lcd_buffer,"Wi-Fi (TCP)"); //запись в буфер текста и значений температуры в подготовленном
    if(FlToGSM>11) sprintf(lcd_buffer,"Err! 0..11 (%i)",(int) FlToGSM); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 205:
    sprintf(lcd_buffer,"dtTermostat=%3i",Delta); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 206:
    sprintf(lcd_buffer,"Temp 1 Nedrobn"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Distill=%3i",Temp1P); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 207:
    sprintf(lcd_buffer,"Temp 2 Drobn"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Distill=%3i",Temp2P); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 208:
    sprintf(lcd_buffer,"Temp 3 Drobn"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Distill=%3i",Temp3P); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 209:
    sprintf(lcd_buffer,"Temp Razgon Rect"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"(+Kub,-Kol)=%3i",tEndRectRazgon); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 210:
    sprintf(lcd_buffer,"Power"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%3i",PowerRect); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 211:
    sprintf(lcd_buffer,"Vvod Popravok"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"ds18b20 "); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 212:
    sprintf(lcd_buffer,"Temp Okon Otbor"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Glv Rectif=%3i",tEndRectOtbGlv ); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 213:
    sprintf(lcd_buffer,"CHIM Otbor GLV"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%5u",timeChimRectOtbGlv); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 214:
    sprintf(lcd_buffer,"%% CHIM Otbor GLV"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%3i",(int) ProcChimOtbGlv); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 215:
    sprintf(lcd_buffer,"CHIM Otbor SR"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%5u",timeChimRectOtbSR); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 216:
    sprintf(lcd_buffer,"Delta Otbor SR"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%3u",tDeltaRect); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 217:
    sprintf(lcd_buffer,"Temp Okon Otbor"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"SR Rectif=%3i",tEndRectOtbSR); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 218:
    sprintf(lcd_buffer,"Temp Okon"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%3i",tEndRect); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 250:
    sprintf(lcd_buffer,"min %% CHIM"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer," Otbor SR=%2i",minProcChimOtbSR); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 251:
    sprintf(lcd_buffer,"Edit Temp Stab"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Rectif=%3i",tStabSR); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 252:
    sprintf(lcd_buffer,"Beg %% CHIM"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer," Otbor SR=%2i",begProcChimOtbSR); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 253:
    sprintf(lcd_buffer,"Popr MPX=%4i",P_MPX5010); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%4i/%4i",U_MPX5010,U_MPX5010+P_MPX5010); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 219:
    sprintf(lcd_buffer,"Power GLV simple"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Distill=%4i",PowerGlvDistil); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 220:
    sprintf(lcd_buffer,"Power simple"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Distill=%4i",PowerDistil); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 221:
    sprintf(lcd_buffer,"Temp Begin Dist"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"(+Def -Kub)=%3i",TempDeflBegDistil); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 222:
    sprintf(lcd_buffer,"Temp Distill"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"With Defl=%3i",TempDefl); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 223:
    sprintf(lcd_buffer,"Delta Distill"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"With Defl=%3i",DeltaDefl); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 224:
    sprintf(lcd_buffer,"Temp Kub Okon"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"DistWithDefl=%3i",tEndDistDefl); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 225:
    sprintf(lcd_buffer,"BeepEndProc=%1u",BeepEndProcess); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 226:
    sprintf(lcd_buffer,"BeepStateProc=%1u",BeepStateProcess); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 227:
    sprintf(lcd_buffer,"BeepKeyPress=%1u",BeepKeyPress); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 228:
    sprintf(lcd_buffer,"Power Razvar"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Zerno=%4i",PowerRazvZerno); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 229:
    sprintf(lcd_buffer,"Power Varka"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Zerno=%4i",PowerVarkaZerno); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 230:
    sprintf(lcd_buffer,"Period Refresh");
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Server(sec)=%3u",(unsigned int) PeriodRefrServer); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer);
    break;
  case 231:
    sprintf(lcd_buffer,"U Peregrev=%3uV",(unsigned int) NaprPeregrev); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer);
    break;
  case 232:
    sprintf(lcd_buffer,"Urv Barda=%4i",UrovenBarda); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer);
    sprintf(lcd_buffer,"Barda(%4u)",U_GLV); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer);
    break;
  case 233:
    sprintf(lcd_buffer,"Provod SR=%4i",UrovenProvodimostSR); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer);
    break;
  case 234:
    sprintf(lcd_buffer,"Time Stab (+/-)"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Kolonna=%5isec",TimeStabKolonna); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 235:
    sprintf(lcd_buffer,"Edit T & CHIM"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Count CHIM=%3i",(int) CntCHIM); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 236:
    sprintf(lcd_buffer,"Auto - CHIM=%3i",(int)DecrementCHIM); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer);
    break;
  case 237:
    sprintf(lcd_buffer,"Auto + CHIM=%3i",(int)IncrementCHIM); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer);
    break;
  case 238:
    sprintf(lcd_buffer,"Time Auto +"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"CHIM=%5isec",TimeAutoIncCHIM); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 239:
    sprintf(lcd_buffer,"Time reStab(+/-)"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Kolonna=%5isec",TimeRestabKolonna); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 240:
    sprintf(lcd_buffer,"Beer Pause"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Count =%3i",(int) CntPause); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 241:
    sprintf(lcd_buffer,"Power correct"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"ASC712 =%3i",(int) CorrectASC712); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 242:
    sprintf(lcd_buffer,"Server adr="); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%3u.%3u.%3u.%3u",(unsigned int)ip[0],(unsigned int)ip[1],(unsigned int)ip[2],(unsigned int)ip[3]); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 243:
    sprintf(lcd_buffer,"Server port="); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%u",ipPort); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 244:
    sprintf(lcd_buffer,"ID Device="); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%s",idDevice); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 245:
    sprintf(lcd_buffer,"My Phone="); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%s",my_phone); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 246:
    sprintf(lcd_buffer,"Alarm Pressure"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"MPX5010=%i",AlarmMPX5010); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 247:
    sprintf(lcd_buffer,"Use Avtonom"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"HLD=%i",FlAvtonom); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 248:
    sprintf(lcd_buffer,"Time Open"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"BRD=%i",(int)timeOpenBRD); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 249:
    sprintf(lcd_buffer,"PID Paramters"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%4i %4i %4i",(int)PIDTemp[0],(int)PIDTemp[1],(int)PIDTemp[2]); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 254:
    sprintf(lcd_buffer,"Power NBK=%4i",PowerNBK); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 255:
    sprintf(lcd_buffer,"Wi-Fi AP="); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%s",WiFiAP); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 256:
    sprintf(lcd_buffer,"Wi-Fi Password="); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%s",WiFiPass); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 257:
    sprintf(lcd_buffer,"Fraction Dist"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Count =%3i",(int) CountFractionDist); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 258:
    sprintf(lcd_buffer,"Fraction Rectif"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Count =%3i",(int) CountFractionRect); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 259:
    sprintf(lcd_buffer,"Temp Zasyp"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Zator=%3i",(int) TempZSP*10); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 260:
    sprintf(lcd_buffer,"Temp Osahariv"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Zator=%3i",(int) TempZSPSld*10); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 261:
    sprintf(lcd_buffer,"Temp Brogenia"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Zator=%3i",(int) TempHLDZatorBrog1*10); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 262:
    sprintf(lcd_buffer,"PhasePW %5i=",Power); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%4i+%4i+%4i",PowerPhase[0],PowerPhase[1],PowerPhase[2]); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 263:
    sprintf(lcd_buffer,"Phase%% %3i=",(int)KtPhase[0]+KtPhase[1]+KtPhase[2]); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"%3i + %3i + %3i",(int)KtPhase[0],(int)KtPhase[1],(int)KtPhase[2]);
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;   
  case 264:
    sprintf(lcd_buffer,"min Pressure"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"NBK=%3i+%3i=%3i",(int) minPressNBK*5,(int) deltaPressNBK,(int) minPressNBK*5+(int) deltaPressNBK); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;     
  case 265:
    sprintf(lcd_buffer,"delta Pressure"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"NBK=%3i+%3i=%3i",(int) minPressNBK*5,(int) deltaPressNBK,(int) minPressNBK*5+(int) deltaPressNBK); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;     
  case 266:
    sprintf(lcd_buffer,"time Pressure"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"NBK=%3i",(int) timePressNBK*5); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;     
  case 267:
    sprintf(lcd_buffer,"Upravl Nasos"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"NBK=%3i",(int)UprNasosNBK); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;     
  case 268:
    sprintf(lcd_buffer,"%% otbor Tsarga"); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Paster(+/-)=%3i",(int)ProcChimOtbCP); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;     
  case 300:
    sprintf(lcd_buffer,"Temp%1i=%3u",nPopr,temps[nPopr]); //запись в буфер текста и значений температуры в подготовленном
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"Popr=%3i T=%3u",ds1820_popr[nPopr], (temps[nPopr]+ds1820_popr[nPopr])); //запись в буфер текста и значений температуры в подготовленном
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 301:
    sprintf(lcd_buffer,"TempK(%1i)=%4i",(int) nPopr, tempK[nPopr]); //запись в буфер текста и значений температуры в подготовленном
    if (flPopr==0) 
     {  
        sprintf(lcd_buffer,"%s %c",lcd_buffer,'*');
     }
     
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    sprintf(lcd_buffer,"CHIM(%1i)=%3i", (int) nPopr, (int) CHIM[nPopr]); //запись в буфер текста и значений температуры в подготовленном
    if (flPopr==1) 
     {  
        sprintf(lcd_buffer,"%s %c",lcd_buffer,'*');
     }
    
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 302:
    // Первое значение это время работы насоса
    if (nPopr==0) 
      sprintf(lcd_buffer,"Work Pump =%2im",(int) tempP[nPopr]); //запись в буфер текста и значений температуры в подготовленном 
    else
      sprintf(lcd_buffer,"TemP(%1i)=%3i.0",(int) nPopr, (int) tempP[nPopr]); //запись в буфер текста и значений температуры в подготовленном
    
    if (flPopr==0) 
     {  
        sprintf(lcd_buffer,"%s %c",lcd_buffer,'*');
     }
     
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    if (nPopr==0) 
      sprintf(lcd_buffer,"Pause Pump=%2im",(int) timeP[nPopr]); //запись в буфер текста и значений температуры в подготовленном 
    else
      sprintf(lcd_buffer,"TimP(%1i)=%3i m", (int) nPopr, (int) timeP[nPopr]); //запись в буфер текста и значений температуры в подготовленном
      
    if (flPopr==1) 
     {  
        sprintf(lcd_buffer,"%s %c",lcd_buffer,'*');
     }
    
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 303:
    sprintf(lcd_buffer,"%3u.%3u.%3u.%3u",(unsigned int)ip[0],(unsigned int)ip[1],(unsigned int)ip[2],(unsigned int)ip[3]);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    j=-1;
    while (j<nPopr*4)
    {
      j++;
      lcd_buffer[j]=' ';
    }
    lcd_buffer[j+1]='*';
    lcd_buffer[j+2]=0;
    
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    
    break;
  case 304:
    sprintf(lcd_buffer,"%s (%3u)",idDevice,(unsigned int)idDevice[nPopr] );
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    j=0;
    while (j<nPopr)
    {
      lcd_buffer[j]=' ';
      j++;
    }
    lcd_buffer[j]='*';
    lcd_buffer[j+1]=0;
    
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    
    break;

  case 305:
    sprintf(lcd_buffer,"%s/%3u)",my_phone,(unsigned int)my_phone[nPopr]);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    j=0;
    while (j<nPopr)
    {
      lcd_buffer[j]=' ';
      j++;
    }
    lcd_buffer[j]='*';
    lcd_buffer[j+1]=0;
    
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    
    break;
  case 309:
    sprintf(lcd_buffer,"%4i %4i %4i",(int)PIDTemp[0],(int)PIDTemp[1],(int)PIDTemp[2]);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    if (nPopr==0) sprintf(lcd_buffer,"   P");
    if (nPopr==1) sprintf(lcd_buffer,"        I");
    if (nPopr==2) sprintf(lcd_buffer,"             D");
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;

  case 310:
    sprintf(lcd_buffer,"%s /%3u",WiFiAP,(unsigned int)WiFiAP[nPopr] );
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    j=0;
    while (j<nPopr)
    {
      lcd_buffer[j]=' ';
      j++;
    }
    lcd_buffer[j]='*';
    lcd_buffer[j+1]=0;
    
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    
    break;
  case 311:
    sprintf(lcd_buffer,"%s /%3u",WiFiPass,(unsigned int)WiFiPass[nPopr] );
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    j=0;
    while (j<nPopr)
    {
      lcd_buffer[j]=' ';
      j++;
    }
    lcd_buffer[j]='*';
    lcd_buffer[j+1]=0;
    
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    
    break;
  case 312:
    // последнее значение это тест фракционника
    if (nPopr==MAX_CNT_FRACTION_DIST)
      { 
          sprintf(lcd_buffer,"Fraction Dist"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"Test =%3i",(int) TekFraction+1); //запись в буфер текста и значений температуры в подготовленном
      }
    else
    {
      if (TempFractionDist[nPopr]>=0)
        sprintf(lcd_buffer,"tFrDst(%1i)=%4i",(int) nPopr+1, (int) TempFractionDist[nPopr]); //запись в буфер текста и значений температуры в подготовленном
      else
        sprintf(lcd_buffer,"timFD(%1i)=%5im",(int) nPopr+1, (int) -TempFractionDist[nPopr]); //запись в буфер текста и значений температуры в подготовленном        
      
      if (flPopr==0) 
       {  
          sprintf(lcd_buffer,"%s%c",lcd_buffer,'*');
       }
       
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      
      if (flPopr==1)  sprintf(lcd_buffer,"Angl=%3i*,%4iW",(int) AngleFractionDist[nPopr] , PowerFractionDist[nPopr]); //запись в буфер текста и значений температуры в подготовленном
                else  sprintf(lcd_buffer,"Angl=%3i ,%4iW",(int) AngleFractionDist[nPopr] , PowerFractionDist[nPopr]); //запись в буфер текста и значений температуры в подготовленном 
     
      if (flPopr==2)  sprintf(lcd_buffer,"%s%c",lcd_buffer,'*');
    }
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 313:
    // последнее значение это тест фракционника
    if (nPopr==MAX_CNT_FRACTION_RECT)
      { 
          sprintf(lcd_buffer,"Fraction Rect"); //запись в буфер текста и значений температуры в подготовленном
          my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
          sprintf(lcd_buffer,"Test =%3i",(int) TekFraction+1); //запись в буфер текста и значений температуры в подготовленном
      }
    else
    {
      if (TempFractionRect[nPopr]>=0)
          sprintf(lcd_buffer,"tFrRect(%1i)=%4i",(int) nPopr+1, (int) TempFractionRect[nPopr]); //запись в буфер текста и значений температуры в подготовленном
      else
        sprintf(lcd_buffer,"timFR(%1i)=%5im",(int) nPopr+1,(int) -TempFractionRect[nPopr]); //запись в буфер текста и значений температуры в подготовленном        
         
      if (flPopr==0) 
       {  
          sprintf(lcd_buffer,"%s%c",lcd_buffer,'*');
       }
       
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf(lcd_buffer,"Angle=%3i",(int) AngleFractionRect[nPopr]); //запись в буфер текста и значений температуры в подготовленном 
      
      if (flPopr==1)  sprintf(lcd_buffer,"%s%c",lcd_buffer,'*');
    }
    lcd.setCursor(0, 1);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;    
  case 314:
    sprintf(lcd_buffer,"%4i %4i %4i",(int)PowerPhase[0],(int)PowerPhase[1],(int)PowerPhase[2]);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    if (nPopr==0) sprintf(lcd_buffer," PH1");
    if (nPopr==1) sprintf(lcd_buffer,"      PH2");
    if (nPopr==2) sprintf(lcd_buffer,"           PH3");
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  case 315:
    sprintf(lcd_buffer,"%3i %3i %3i=%3i",(int)KtPhase[0],(int)KtPhase[1],(int)KtPhase[2],(int)KtPhase[0]+KtPhase[1]+KtPhase[2]);
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    lcd.setCursor(0, 1);
    if (nPopr==0) sprintf(lcd_buffer,"PH1");
    if (nPopr==1) sprintf(lcd_buffer,"    PH2");
    if (nPopr==2) sprintf(lcd_buffer,"        PH3");
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
    break;
  default:
    break;
  }

  //    
  //  lcd.setCursor(0, 0);
  //  my_lcdprint(Seconds);
  //  lcd.setCursor(5, 0);
  //  my_lcdprint(MaxVoltsOut);
  //  lcd.setCursor(9, 0);
  //  my_lcdprint((long)MaxVoltsOut*707/1000);
  //  lcd.setCursor(0, 1);
  //  my_lcdprint(index_input);
  //  lcd.setCursor(5, 1);
  //  my_lcdprint(TimeOpenTriac);
  //  NeedDisplaying=false;
  //  lcd.setCursor(10, 1);
  //  my_lcdprint(TicZero);
  // предупреждения теперь выводим на всех страницах.
  
  if (flNumStrPrint<2) 
  {
    lcd_buffer[0]=0;
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
  }
  NeedDisplaying=false;

}
//    case 116:

void ProcessBeerCloneBrau()
{
  // Управление мешалкой;
  if (StateMachine>0 && StateMachine<100)
  {
    if (time1==0 || temps[TEMP_TERMOSTAT]>=880)
    {  
        // Насос включаем только 
        if (KlOpen[KLP_HLD]==0 && temps[TEMP_TERMOSTAT]<880)
        {
            KlOpen[KLP_HLD]=40;
            KlClose[KLP_HLD]=0;
            KlOpen[KLP_GLV_HVS]=40;
            KlClose[KLP_GLV_HVS]=0;
            time1=(int) tempP[0] *60; // Взводим насос на десять минут работы 
            timeMIXER=(int) tempP[0]*60;

        }
        else
        {
            KlOpen[KLP_HLD]=0;
            KlClose[KLP_HLD]=40;
            KlOpen[KLP_GLV_HVS]=0;
            KlClose[KLP_GLV_HVS]=40;
            time1=(unsigned int) timeP[0]*60; // Взводим насос на две минуты простоя 
            timeMIXER=0;
        }
    }
  } 
  
   
  switch  (StateMachine)
  {
  case 0: // 
    // Процесс не запущен
    time1=0;
    break;
  case 1: // Старт процесса
    KlTek=1; 
    time1=0;
    PrepareProcess();
    flNoAlarmLowPower=1; // Предупреждение о низкой мощности отключим, здесь оно не важно
    StateMachine=2;
    
  case 2: // Температура Больше необходиомй
    ProcessPIDTemp((int)tempP[KlTek] * 10,temps[TEMP_TERMOSTAT]);
    UstPower=(long)Power*KtT/1000;
    // Мощность ограничена мощностью варки зерна
    if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
    // Если нужная температура меньше 100 градусов, то проверяем пока она не превысит заданную
    // Если нужная температура более 100 градусов, тогда пропускаем этот режим и считаем что началась варка. 
    if (temps[TEMP_TERMOSTAT]<(int)tempP[KlTek] * 10 && tempP[KlTek]<=100) break;
    // Если темперетура больше либо равно 110, то закипание определяется по второму датчику температуры.
  //  if (temps[TEMP_DEFL]<TempDeflBegDistil && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_DEFL]<-TempDeflBegDistil) && tempP[KlTek]>=110 )  break;
    
    StateMachine=3;
    time2=(int)timeP[KlTek]*60;
  case 3:
    if (tempP[KlTek]>100) 
    { UstPower=PowerVarkaZerno;
      // Если температура вверху заторника меньше заданной и температура варки больше 110, тогда считаем, что еще не закипело.
      // Проверка на закипание проводится только один раз в начале кипения, то есть если предыдущий этап не был кипением.
      if (temps[TEMP_DEFL]<TempDeflBegDistil && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_DEFL]<-TempDeflBegDistil) && tempP[KlTek]>=110)  time2=(int)timeP[KlTek]*60;
      
    }
    else
    {   if (temps[TEMP_TERMOSTAT]<(int)tempP[KlTek] * 10) 
          {
              ProcessPIDTemp((int)tempP[KlTek] * 10,temps[TEMP_TERMOSTAT]);
              UstPower=(long)Power*KtT/1000;
              if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
          }
        else
          UstPower=0;
    }
    
    if (timeP[KlTek]==0) 
      my_beep(2*BEEP_LONG); // Если время ожидания равно нулю, тогда пищим, ожидая действия пользователя
    else
    {
      if (timeP[KlTek]==255) timeP[KlTek]=0; 
      if (time2==0) 
      {
          KlTek++; // Если все паузы выполнены, то переходим к окончанию процесса, иначе к следующей паузе.
          if (KlTek>CntPause) StateMachine=100;
                         else StateMachine=2;
          if (BeepStateProcess) my_beep(BEEP_LONG);
      }
    }
    break;
  case 4:
    // Это системное состояние к нему только через debug перейти можно, просто ручной переход на следующую паузу.
      KlTek++; // Если все паузы выполнены, то переходим к окончанию процесса, иначе к следующей паузе.
      if (KlTek>CntPause) StateMachine=100;
                     else StateMachine=2;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    break;
  case 100:// Конечное состояние автомата
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    CloseAllKLP();
    flAllOff=1;
    UstPower=0; // Убираем мощность с ТЭНов
    break;
  }
}

//    case 110:
// Процесс дистилляции с подключаемым дефлегматором
// Процесс поддерживат температуру в дефлегматоре не более заданной + дельта и не менее заданной - дельта
void ProcessDistilDefl()
{

  // Если достигнут уровень в приемной емкости или температуре в кубе превысыла допустимую температуру для этого режима

  if ((flAlarmUroven || temps[TEMP_KUB]>=tEndDistDefl || deltaPower>=Power) && StateMachine<7) 
  { // Переводим автомат в стадию завершения 
    StateMachine=7;        
    SecondsEnd=2*60;// Через одну минуту отключим воду
    SecTempPrev=Seconds;
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }
  // Проверяем температуру в ТСА
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
    UstPower=0;
  }
  if (flAlarmMPX5010) StateMachine=100; // Переводим в режим тревоги по датчику давления
  
  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    //          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;
    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
    StateMachine=2;//Этот процесс пока заблокирован, сразу идем на окончание.
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    flAllOff=0;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    deltaPower=0;
  case 2: // Ожидание, пока прогреется термометр в дефлегматоре

    UstPower=Power; // Устанавливаем максимальную мощность для разгона

    // Если температура в дефлегматоре меньше 70 градусов и датчик дефлегматора в принципе подключен, тогда ждем, пока
    // дефлегматор прогреется, чтобы запустить воду в холодильник (это для некоторой экономии воды)
    if (temps[TEMP_DEFL]<TempDeflBegDistil && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_KUB]<-TempDeflBegDistil) ) break;

    // Подаем питание на клапан подачи воды холодильника
    StateMachine=3;        
    if (BeepStateProcess) my_beep(BEEP_LONG);

  case 3:
    // Ждем, пока температура в кубе не достигнет температуры начала дистилляции (если есть датчик дефлегматора, тогда этот пункт сразу
    // проскочится, поскольку темперетура в кубе не может быть меньше, чем температура в дефлегматоре
    if (temps[TEMP_KUB]<TempDeflBegDistil) break;
    // Подаем питание на клапан подачи воды холодильника

    // Подаем питание на клапан подачи воды холодильника
    // для этого синхронизируем его с клапаном общей подачи воды (возможно, при этом будет меньше гидроударов, хотя я их и так не наблюдаю)
    KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
    KlClose[KLP_HLD]=KlClose[KLP_VODA];
    KlState[KLP_HLD]=KlState[KLP_VODA];
    KlCount[KLP_HLD]=KlCount[KLP_VODA];

    StateMachine=4;     
    time2=15*60; // Взводиим таймер на 15 минут   
    if (BeepStateProcess) my_beep(BEEP_LONG);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);

  case 4: // Дефлегматор отключен.
    // Если более 10 минут не вышли за пределы температуры, тогда увеличиваем мощность.
    if (time2==0)
    {
      deltaPower=deltaPower-(int) (PowerDistil/20);
      if (deltaPower<0) deltaPower=0;
      time2=10*60;
    }

    UstPower=PowerDistil-deltaPower; // Устанавливаем мощность дистилляции.

    if (temps[TEMP_DEFL]>TempDefl)
    {  
      // Включаем дефлегматор и холодильник
      
      KlOpen[KLP_DEFL_D]=KlOpen[KLP_VODA];
      KlClose[KLP_DEFL_D]=KlClose[KLP_VODA];
      KlState[KLP_DEFL_D]=KlState[KLP_VODA];
      KlCount[KLP_DEFL_D]=KlCount[KLP_VODA];
      
      KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
      KlClose[KLP_HLD]=KlClose[KLP_VODA];
      KlState[KLP_HLD]=KlState[KLP_VODA];
      KlCount[KLP_HLD]=KlCount[KLP_VODA];
      StateMachine=5; // Переходим к этапу поддержания температуры путем частичной подачи воды в дефлегмато
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else    
      break;    

  case 5:// 50% воды из холодильника идет в дефлегматор.
    // Если более 10 минут не вышли за пределы температуры, тогда увеличиваем мощность.
    time2=10*60;
    if (time3==0)
    {
      deltaPower=deltaPower-(int) (PowerDistil/30);
      if (deltaPower<0) deltaPower=0;
      time3=10*60;
    }
    UstPower=PowerDistil-deltaPower; // Устанавливаем мощность дистилляции.
    // Если температура превысила температуру, которую надо поддерживать в дефлегматоре+дельта, тогда переходим к режиму, когда
    // вся вода из холодильника идет через дефлегматор
    if (temps[TEMP_DEFL]>TempDefl+DeltaDefl)
    {  
      // Отключаем прямой выход из холодильника
      KlOpen[KLP_HLD]=0;
      KlClose[KLP_HLD]=10;
      // Включаем дефлегматор
      KlOpen[KLP_DEFL_D]=KlOpen[KLP_VODA];
      KlClose[KLP_DEFL_D]=KlClose[KLP_VODA];
      KlState[KLP_DEFL_D]=KlState[KLP_VODA];
      KlCount[KLP_DEFL_D]=KlCount[KLP_VODA];

      StateMachine=6; // Переходим к этапу поддержания температуры путем 100% направления воды в дефлегматор        
      time1=0;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else
      if (temps[TEMP_DEFL]<TempDefl-DeltaDefl) // Если в результате работы с 50% дефлегматором, температура понизилась переходим в режим работы без дефлегматора
      {
        
        KlOpen[KLP_DEFL_D]=0;
        KlClose[KLP_DEFL_D]=10;
        
        KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
        KlClose[KLP_HLD]=KlClose[KLP_VODA];
        KlState[KLP_HLD]=KlState[KLP_VODA];
        KlCount[KLP_HLD]=KlCount[KLP_VODA];
        StateMachine=4; // Переходим к этапу без дефлегмации
        if (BeepStateProcess) my_beep(BEEP_LONG);
        break;
      }
      else    
        break;

  case 6: // Вся вода из холодильника идет в дефлегматор.
    // Взводиим таймер на 10 минут
    time3=10*60;
    UstPower=PowerDistil-deltaPower; // Устанавливаем мощность дистилляции.
    // Если температура превысила требуемую + дельта более, чем на полдельта за три минуты, тогда уменьшааем мощность 
    if (temps[TEMP_DEFL]>TempDefl+DeltaDefl+DeltaDefl/2)
    {
      if (time1==0)
      {
        deltaPower=deltaPower+(int) (PowerDistil/10);
        time1=3*60;// Взводим таймер на три минуты.
      }
    }
    
    if (temps[TEMP_DEFL]<TempDefl) // Если в результате работы с 100% дефлегматором, температура понизилась ниже установленной температуры переходим в режим работы с 50% дефлегма
    {
      // Включаем прямой выход из холодильника.
      KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
      KlClose[KLP_HLD]=KlClose[KLP_VODA];
      KlState[KLP_HLD]=KlState[KLP_VODA];
      KlCount[KLP_HLD]=KlCount[KLP_VODA];
      
      KlOpen[KLP_DEFL_D]=KlOpen[KLP_VODA];
      KlClose[KLP_DEFL_D]=KlClose[KLP_VODA];
      KlState[KLP_DEFL_D]=KlState[KLP_VODA];
      KlCount[KLP_DEFL_D]=KlCount[KLP_VODA];
      StateMachine=5; // Переходим к этапу поддержания температуры 50% дефлегмацией
      time2=15*60; // Взводиим таймер на 15 минут   
      if (BeepStateProcess) my_beep(BEEP_LONG);
      break;
    }
    else    
      break;

  case 7:// Ждем минуту для окончания подачи воды
    UstPower=0;
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);

    SecondsEnd=SecondsEnd-(Seconds-SecTempPrev);
    SecTempPrev=Seconds;

    if (SecondsEnd<=0)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 100:// Конечное состояние автомата
    if (BeepEndProcess) my_beep(BEEP_LONG);
  case 101:
    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    flAllOff=1;

    break;
  }
}
//    case 117:

void ProcessDistillFractional()
{
  if (flAlarmUroven && StateMachine<4) 
  { // Если сработал датчик уровня, то переходим на следующую фракцию.
    TekFraction++; SecTempPrev=Seconds; 
    flAlarmUroven=0;
    CountAlarmUroven=0;
    if (TekFraction<CountFractionDist) SetAngle(AngleFractionDist[TekFraction]);
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }

  // Проверяем температуру в ТСА
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
    UstPower=0;
  }

  if (flAlarmMPX5010) StateMachine=100; // Переводим в режим тревоги по датчику давления
  
  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    //          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;

    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    flAllOff=0;

  case 2: // Ожидание, пока прогреется термометр в дефлегматоре

    UstPower=Power; // Устанавливаем максимальную мощность для разгона
    // Подчиненные переводим в разгон
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);

    // Если температура в дефлегматоре меньше 70 градусов и датчик дефлегматора в принципе подключен, тогда ждем, пока
    // дефлегматор прогреется, чтобы запустить воду в холодильник (это для некоторой экономии воды)
    if (temps[TEMP_DEFL]<TempDeflBegDistil && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_KUB]<-TempDeflBegDistil) ) break;

    // Подаем питание на клапан подачи воды холодильника
    // для этого синхронизируем его с клапаном общей подачи воды (возможно, при этом будет меньше гидроударов, хотя я их и так не наблюдаю)
    KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
    KlClose[KLP_HLD]=KlClose[KLP_VODA];
    KlState[KLP_HLD]=KlState[KLP_VODA];
    KlCount[KLP_HLD]=KlCount[KLP_VODA];
    StateMachine=3;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    TekFraction=0;
    SetAngle(AngleFractionDist[TekFraction]);
    SecTempPrev=Seconds;

  case 3:// Ждем, пока температура в кубе не превысит установленную
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 1); // Переводим подчиненный в режим дистилляции
    digitalWrite(PIN_SLAVE_3, 0);

    if (TekFraction<CountFractionDist)
    {    
      if (TekFraction==0 && CountFractionDist==2) // Если число фракций равно ровно две, тогда отбираем первую фракцию через клапан голов, остальные через клапан тела
      {
          // Для первой фракции (считаем ее головами) открываем клапан отбора голов, клапан тела закрываем.
          KlOpen[KLP_SR]=0;
          KlClose[KLP_SR]=10;
          KlOpen[KLP_GLV_HVS]=PER_KLP_OPEN;
          KlOpen[KLP_GLV_HVS]=PER_KLP_CLOSE;  
      }
      else
      {   // Для последующих фракций (считаем их телом) клапан тела открываем, клапан голов закрываем.
          KlOpen[KLP_SR]=PER_KLP_OPEN;
          KlClose[KLP_SR]=PER_KLP_CLOSE;
          KlOpen[KLP_GLV_HVS]=0;
          KlOpen[KLP_GLV_HVS]=10;          
      }
      UstPower=PowerFractionDist[TekFraction]; // Устанавливаем максимальную мощность для разгона
      TekTemp=TempFractionDist[TekFraction]; //Текущее состояние - первый недробный перегон
      if (TekTemp>=0) // Значение больше нуля, значит это температура
      {
        if (temps[TEMP_KUB]>=TekTemp)
        {  
           TekFraction++; SecTempPrev=Seconds; 
           if (TekFraction<CountFractionDist) SetAngle(AngleFractionDist[TekFraction]);
        }
      }
      else
      {
        // Значение меньше нуля, значит это время в минутах
          SecOstatok=-((Seconds-SecTempPrev) / 60 + TekTemp);
        // Если времени не осталось, переходим на следующую фракцию  
        if (SecOstatok<=0) 
         { 
           TekFraction++; SecTempPrev=Seconds; 
           if (TekFraction<CountFractionDist) SetAngle(AngleFractionDist[TekFraction]);
         }
        
      }    
     break; 
    }
    else
    {
      StateMachine=4;        
      time3=2*60;// Через две минуты отключим воду
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    
  case 4:// Ждем минуту для окончания подачи воды
    UstPower=0;
    // Закрываем клапана на отбор.
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    KlOpen[KLP_GLV_HVS]=0;
    KlOpen[KLP_GLV_HVS]=10;      
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    // Отключаем подчиненные
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0); 
    digitalWrite(PIN_SLAVE_3, 0);
//    SecondsEnd=SecondsEnd-(Seconds-SecTempPrev);
//    SecTempPrev=Seconds;
//
//    if (SecondsEnd<=0)
    if (time3==0)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 101: // Превышение температуры в ТСА!!!
    my_beep(3*BEEP_LONG);
  case 100:// Конечное состояние автомата
    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    // Отключаем подчиненные
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0); 
    digitalWrite(PIN_SLAVE_3, 0);
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    flAllOff=1;

    break;
  }

}
//    case 112:

 
void ProcessNBK()
{

  //#define KLP_NPG  0      // Номер клапана для управления НПГ (при дистилляции) 
  //#define KLP_VODA 1      // Номер клапана для управления общей подачей воды в систему
  //#define KLP_DEFL 2      // Номер клапана для подачи воды в дейфлегматор
  //#define KLP_GLV_HVS  3       // Номер клапана отбора головных и хвостовых фракций
  //#define KLP_SR   4      // Номер клапана отбора ректификата
  //#define TEMP_KUB  0      // Номер датчика  термометра в кубе
  //#define TEMP_RK20 1      // Номер датчика термометра в РК 20 см от насадки
  //#define TEMP_TSA  2      // Номер термометра в трубке связи с атмосферой
  if (flAlarmUroven && StateMachine<100) 
  { // Переводим автомат в стадию завершения 
    StateMachine=100;        
  }
  if (flAlarmMPX5010) StateMachine=100; // Переводим в режим тревоги по датчику давления

  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    lastSMSState=0;
    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    MaxPressByPeriod=0;
  case 2:
    UstPower=Power;
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    if (temps[TEMP_DEFL]<TempDeflBegDistil  && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_KUB]<-TempDeflBegDistil)) break;
    KlOpen[KLP_HLD]=PER_KLP_OPEN;
    KlClose[KLP_HLD]=PER_KLP_CLOSE;
    StateMachine=3;
    if (BeepStateProcess) my_beep(BEEP_LONG);
  case 3:
    SpeedNBK=0;
    UstPower=PowerNBK;
    time1=20*60; // Задаем время 20 минут с начала старта
    time2=0;
    my_beep(BEEP_LONG*3);
    break;
  case 4:
    UstPower=PowerNBK;
    timeNBK=20;
    // Работаем в следующем режиме - 2 минуты пауза, 1 минута работает миксер.
    if (time3==0) 
    {  
       time3=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
       timeMIXER=(int) tempP[0] *60;        
    }
    
    if (time2==0)
    { 
      time2=(int) timePressNBK*5;
      ProcessPIDPress((int) minPressNBK*5,MaxPressByPeriod);
      if (MaxPressByPeriod>(int) minPressNBK*5+(int)deltaPressNBK)
        { // Если превышено давление, то переходим в режим прекращения подачи браги.
          StateMachine=6;
          time2=((int) timePressNBK*5)/8;  
          if (BeepStateProcess) my_beep(BEEP_LONG/2);
          break;
        }
      if (MaxPressByPeriod<(int) minPressNBK*5 && deltaPressNBK>0) {SpeedNBKDst++; if (BeepStateProcess) my_beep(BEEP_LONG/2);}
      // Взводим таймер на время, определяемое мощностью регулятора мощности.
      MaxPressByPeriod=U_MPX5010;
    }
    // Если превысило двойную дельту, то не дожидаемся, пока окончится время и сразу переходим в режим превышения по давлению.
    if (MaxPressByPeriod>(int) minPressNBK*5+(int)deltaPressNBK +(int)deltaPressNBK  && deltaPressNBK>0)
    {
        StateMachine=6;
        time2=((int) timePressNBK*5)/8;  
        if (BeepStateProcess) my_beep(BEEP_LONG/2);
        break;
    }
    
    SpeedNBK=SpeedNBKDst;

    if  (temps[TEMP_DEFL]>=980 && time1<=0)
    {
      //Если температура превысила максимальную и с момента старта прошло более 20 минут, то считаем, что произошла авария - заклинил насос или кончилась брага.
      // Ждем 5 минут и завершаем процесс.
      time2=5*60;
      StateMachine=5;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 5:
    SpeedNBK=SpeedNBKDst;
    timeNBK=20;

    if  (temps[TEMP_DEFL]<970)
    {  
       StateMachine=4;
    }
    my_beep(BEEP_LONG*3);
    if (time2<=0)
    {
      StateMachine=100;
    }
    break;
  case 6: // Превышение давления, ожидаем снижения давления, подача браги при этом прекращается.
    UstPower=PowerNBK;
    timeNBK=20;
    // Работаем в следующем режиме - 2 минуты пауза, 1 минута работает миксер.
    if (time3==0) 
    {  
       time3=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
       timeMIXER=(int) tempP[0] *60;        
    }
    
    if (time2==0)
    { 
  
      time2=(int) timePressNBK*5/8;
      ProcessPIDPress((int) minPressNBK*5,MaxPressByPeriod);
      if (MaxPressByPeriod<(int) minPressNBK*5+(int)deltaPressNBK ) // Если давление вернулось в норму, уменьшаем подачу и переходим в режим подачи браги
        { if (deltaPressNBK>0) SpeedNBKDst-=1;
          // Время анализа ставим вдвое меньше, чем на превышение.
          time2=(int) timePressNBK*5/4;
          StateMachine=4;
          if (BeepStateProcess) my_beep(BEEP_LONG/2);
        }
      // Взводим таймер на время, определяемое настройками
      MaxPressByPeriod=U_MPX5010;
    }
    else
     if (U_MPX5010<=(int) minPressNBK*5) // Если текущее давление стало меньше или равно минимальному, тогда заканчиваем прекращать подачу браги
     {
        if (deltaPressNBK>0) SpeedNBKDst-=1; 
        // Время анализа ставим вдвое меньше, чем на превышение.
          time2=(int) timePressNBK*5/4;
         StateMachine=4;
         if (BeepStateProcess) my_beep(BEEP_LONG/2);
         MaxPressByPeriod=U_MPX5010;
    }
    
    SpeedNBK=0;
    break;
  
  case 101: // Превышение температуры в ТСА!!!
    break;
    my_beep(3*BEEP_LONG);
  case 100:// Конечное состояние автомата
    SpeedNBK=0;
    timeNBK=0;
    if (StateMachine==100 && BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    flAllOff=1;
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    break;
  }

}
//    case 111:

void ProcessNDRF()
{

  //#define KLP_NPG  0      // Номер клапана для управления НПГ (при дистилляции) 
  //#define KLP_VODA 1      // Номер клапана для управления общей подачей воды в систему
  //#define KLP_DEFL 2      // Номер клапана для подачи воды в дейфлегматор
  //#define KLP_GLV_HVS  3       // Номер клапана отбора головных и хвостовых фракций
  //#define KLP_SR   4      // Номер клапана отбора ректификата
  //#define TEMP_KUB  0      // Номер датчика  термометра в кубе
  //#define TEMP_RK20 1      // Номер датчика термометра в РК 20 см от насадки
  //#define TEMP_TSA  2      // Номер термометра в трубке связи с атмосферой
  static char Count_Provodimost=0; // Количество срабатываний датчика проводимости для окончания отбора голов.
  static char OldStateMachine=0; // Количество срабатываний датчика проводимости для окончания отбора голов.
  int V3; // Текущий уровень проводимости;

  if (flAlarmUroven && StateMachine<8 ) 
  { // Переводим автомат в стадию завершения 
    if (USE_ALARM_UROVEN==1)
    {
      StateMachine=8; // Переводим в режим окончания
      SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else
    {
      OldStateMachine=StateMachine;
      StateMachine=9;  // Переводим в режим ожидания
      if (BeepStateProcess) my_beep(BEEP_LONG);
      Count_Provodimost=0;
      KlOpen[KLP_GLV_HVS]=0;
      KlClose[KLP_GLV_HVS]=100;
      KlOpen[KLP_SR]=0;
      KlClose[KLP_SR]=100;
    }
  }

  // Проверяем температуру в ТСА
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
  }

  if (flAlarmMPX5010) StateMachine=102; // Переводим в режим тревоги по датчику давления
  
 // Проверяем окончание температуры окончания ректификации
  if (temps[TEMP_KUB]>=tEndRect  && StateMachine<8 && IspReg!=118 )
  {  
    StateMachine=8;        
    SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }
  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    KlReg[KLP_GLV_HVS]=PEREGREV_ON;
    KlReg[KLP_SR]=PEREGREV_ON;

    //          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;

    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    flAllOff=0;
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    Count_Provodimost=0;
  case 2: // Разгон
    UstPower=Power; // Устанавливаем максимальную мощность для разгона

    // Учитываем как температуру в кубе, так и температуру в колонне, в зависимости от того, положительные или отрицательные значения температур.
    if (temps[TEMP_KUB]<tEndRectRazgon  && tEndRectRazgon>0) break;
    if (temps[TEMP_RK20]<-tEndRectRazgon && tEndRectRazgon<=0) break;

    // Подаем питание на клапан дефлегматора
    KlOpen[KLP_DEFL]=PER_KLP_OPEN;
    KlClose[KLP_DEFL]=PER_KLP_CLOSE;
    TempPrev=temps[TEMP_RK20];
    SecTempPrev=Seconds; // Запоминаем дату последнего изменения температуры
    SecOstatok=1800; // Стаблилизация колонный по-умолчанию полчаса
    StateMachine=3;        
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    if (BeepStateProcess) my_beep(BEEP_LONG);

  case 3:// Стабилизация температуры (строго по времени)
    UstPower=PowerRect;
    // Польский буфер откроем.
    KlOpen[KLP_PB]=200;
    KlClose[KLP_PB]=0;
    
    if (temps[TEMP_RK20]>500) // Если температура больше 70 градусов (вдруг куб прогрелся, а колонна еще нет, нелогично, конечно, но на всякий случай учтем)
    {
      // Стабилизация строго по температуре 
      SecOstatok=SecOstatok-(Seconds-SecTempPrev);
      SecTempPrev=Seconds;
      if (SecOstatok<=0) 
      {
        StateMachine=4;
        SecTempPrev=Seconds;
      }
      else
        break;
    }
    else
    { 
      SecTempPrev=Seconds; // Запоминаем дату последнего изменения температуры
      break;
    }  

  case 4: // Отбор голов
    UstPower=PowerRect;
    //unsigned int timeChimRectOtbGlv=2000;  // Шим отбора голов (в полупериодах сетевого напряжения в данном случае это около 20 секунд)
    //unsigned char ProcChimOtbGlv=5;  // Процент ШИМ отбора голов (в данном случае это 5 процентов)
    // Устанавливаем клапан отбора хвостов и голов в соответвии с установленнным ШИМ
    //int tEndRectOtbGlv=854;     // Температура окончания отбора голов 85.4 С
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    // Польский буфер закроем.
    KlOpen[KLP_PB]=0;
    KlClose[KLP_PB]=10;
    
    if (ProcChimOtbGlv>0)
        KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
    else
        KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
         
    KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор


    if (UrovenProvodimostSR==0) // Если проводимость спирта не используется, головы отбираем по температуре
    {
      if (temps[TEMP_KUB]<tEndRectOtbGlv)
      {  // Пока температура в кубе не выросла выше окончания отбора голов, продолжаем это состояни
        break;       
      }
    }
    else
    {

      if (UrovenProvodimostSR>0) // Если проводимость больше нуля, используем проводимость в том или ином виде
      {
        V3=U_GLV; 
        // Если проводимость менее заданного уровня, и проводимость вообще сущетвует, то увеличиваем счетчик срабатываний
        // или ести проводимость больше уровня тревоги и установлен признак отбирать головы по уровню в приемной емкости.
  
        if ((V3<UrovenProvodimostSR && V3>=2 && UrovenProvodimostSR>=5) || (V3>=UROVEN_ALARM && UrovenProvodimostSR<=2 )) 
        {
          my_beep(BEEP_LONG); // Сначала пищим, предупреждая.
          Count_Provodimost++;
        }
        else 
          Count_Provodimost=0;
  
        // Если счетчик срабатываний не превысил максимальное число раз, тогда продолжаем процесс отбора голов.
        if (Count_Provodimost<MAX_COUNT_PROVODIMOST_SR) break;
      } 
      else // Если уровень проводимости SR меньше нуля, тогда это означает, что отбор голов производится по времени (в десятках минут).
      {
        // Анализируем, сколько минут прошло с начала отбора голов.
        SecOstatok=-((Seconds-SecTempPrev) / 600 + UrovenProvodimostSR);
        if (SecOstatok>0) break;
      }
    }

    // Если температура превысила указанный рубеж, то переходим в состояние отбора спирта-ректификата.
    // Клапан отбора голов и хвостов отключаем
    tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
    KlOpen[KLP_GLV_HVS]=0;
    KlClose[KLP_GLV_HVS]=10;
    ProcChimSR=begProcChimOtbSR;
    StateMachine=5;        
    if (BeepStateProcess) my_beep(BEEP_LONG);

  case 5: // Ожидание, пока температура не вернется к состоянию стабилизации
    // Это для отладки страт-стопа, на самом деле температура 28,5 градуса в этом режиме в колонне быть не может
    if (tStabSR==0) tStabSR=285;

    KlOpen[KLP_GLV_HVS]=0;
    KlClose[KLP_GLV_HVS]=10;
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    if (temps[TEMP_KUB]>=tEndRectOtbSR)
    {  
      StateMachine=7;        
      if (BeepStateProcess) my_beep(BEEP_LONG);
      break;
    }

    if (temps[TEMP_RK20]>tStabSR)
    {
      break;
    }
    SecTempPrev=Seconds;// Запомним время, когда стабилизировалась температура
    StateMachine=6;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
  case 6: // Отбор СР
    //unsigned int timeChimRectOtbSR=1000;  // Шим отбора СР (в полупериодах сетевого напряжения в данном случае это около 10 секунд)
    //unsigned char tDeltaRect=10;  // Дельта ректификации (в десятых градуса Цельсия) в данном случае - это 10 десятых, то есть 1 градус Цельсия
    //unsigned int tEndRectOtbSR=965;  // Температура окончания отбора спирта-ректификата и начала отбора хвостов (в данном случе 96.5 С)
    // Устанавливаем ШИМ отбора СР
    UstPower=PowerRect;
    if (ProcChimOtbCP>=0)
    {
          if (ProcChimOtbGlv>0)
              KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
          else
              KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
              
          KlOpen[KLP_GLV_HVS]=(KlOpen[KLP_GLV_HVS]*ProcChimOtbCP)/100;
          KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
      
    }
    else
    {
      KlOpen[KLP_GLV_HVS]=-ProcChimOtbCP;
      KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    }
    // Польский буфер закроем.
    KlOpen[KLP_PB]=0;
    KlClose[KLP_PB]=10;


    KlOpen[KLP_SR]=((timeChimRectOtbSR/10)*ProcChimSR)/10;
    KlClose[KLP_SR]=timeChimRectOtbSR-KlOpen[KLP_SR];

    if (temps[TEMP_RK20]>=tStabSR+tDeltaRect) // Если текущая температура превысила базовую, тогда останавливаем отбор
    {
      StateMachine=5;        
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }

    // Если температура в кубе превысила температуру при которой надо отбирать СР
    if (temps[TEMP_KUB]>=tEndRectOtbSR)
    {  
      StateMachine=7;        
      KlOpen[KLP_SR]=0;// Отключаем клапана отбора СР
      KlClose[KLP_SR]=10;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else    
      break;    
  case 7: // Отбор Хвостов
    UstPower=PowerRect;
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*90;
    KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    if (temps[TEMP_KUB]>=tEndRect)
    {  
      StateMachine=8;        
      SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else    
      break;
  case 8:// Ждем три минуты для окончания подачи воды
    UstPower=0;
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    KlOpen[KLP_GLV_HVS]=0;  // Отключаем клапана отбора хвостов
    KlClose[KLP_GLV_HVS]=10;

    if  (Seconds-SecTempPrev>180)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
      V3=U_UROVEN; 
      if (V3<UROVEN_ALARM )
      {
        Count_Provodimost++;
        #ifdef TEST
                sprintf(my_tx_buffer,"ALARM VOZVRAT U_UROVEN=%i cnt=%i",U_UROVEN,Count_Provodimost);
                Serial.println(my_tx_buffer);
        #endif      
        // По прошествии минуты устанавливаем флаг сработки.
        if (Count_Provodimost>2) 
        {
          flAlarmUroven=0;
          StateMachine=OldStateMachine;
          Count_Provodimost=0;
        }
      }
      else 
      {
        Count_Provodimost=0;
      }
      break;
  case 101: // Превышение температуры в ТСА!!!
    my_beep(2*BEEP_LONG);
  case 100:// Конечное состояние автомата

    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();

    if (StateMachine==100 && BeepEndProcess) my_beep(BEEP_LONG);

    UstPower=0;
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    flAllOff=1;

    break;
  }

}
//    case 108:
//    case 113:

void ProcessRazvarZerno()
{
  switch (StateMachine)
  { 
  case 0: //Не запущено
  
    break;
  case 1: //Нагрев до температуры 50 градусов
//    TempHLDZatorBrog1=36;
//    if (TempZSP<40) TempZSP=40;
//    if (TempZSPSld<55) TempZSPSld=55;
    timeNBK=0;
    SpeedNBK=0;
    // Этот пункт пропускаем, потому что и так работает мешалка.
    //      digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    //  
    //      // Если необходимо отправлять состояние на сотовый и это состояние еще не отправлялось, тогда информируем о состоянии процесса
    //  //    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;
    //  
    //  
      if (temps[TEMP_RAZVAR]<(int)TempZSP*10)
      {
        ProcessPIDTemp((int)TempZSP*10,temps[TEMP_RAZVAR]);
        UstPower=(long)Power*KtT/1000;
        // Мощность ограничена мощностью варки зерна
        if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
          break;
        }
      StateMachine=2;
    //      if (BeepStateProcess) my_beep(BEEP_LONG);

  case 2: //Поддержание температуры 50 градусов, пока не произойдет ручной переход к следующему этапу.
    // Из этого режима можно выйти только вручную, нажав кнопку вверх или отладочной функцией одновременно нажав вверх и вниз.
    // Пищим, чтобы привлечь внимание
    if (BeepStateProcess) my_beep(BEEP_LONG);
    ProcessPIDTemp(500,temps[TEMP_RAZVAR]);
    UstPower=(long)Power*KtT/1000;
    // Мощность ограничена мощностью варки зерна
    if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
    break;
    StateMachine=3;
    time1=0;
    timeMIXER=0;
  case 3: //Нагрев до температуры 64 градуса
    if (temps[TEMP_RAZVAR]<(int)TempZSPSld*10)
    {
      // Работаем согласно настроек пивоварения (по умолчанию 10 минут работы, 2 минуты пауза).
      if (time1==0) 
      {  
            time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
            timeMIXER=(int) tempP[0] *60;        
      }
      ProcessPIDTemp((int)TempZSPSld*10,temps[TEMP_RAZVAR]);
      UstPower=(long)Power*KtT/1000;
      // Мощность ограничена мощностью варки зерна
      if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
      break;
    }
    time1=0;
    timeMIXER=0;
    StateMachine=4;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    SecTempPrev=Seconds;
    time2=15*60;  // Взводим таймер на 15 секунд паузы
    if (IspReg==113)
    {
      // Если затор из муки и солода, переходим сразу к размешиванию и последующему осахариванию.
      time2=10*60; // Устанавливаем время на работу мешалки после засыпки солода - 10 минут
      time1=0;
      StateMachine=9; 
      break;
    }

  case 4: //Ожидание 15 минут, поддержка температуры
    timeNBK=0;
    SpeedNBK=0;

    // Работаем в следующем режиме - 2 минуты пауза, 1 минута работает миксер.
    if (time1==0) 
    {  
       time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
       timeMIXER=(int) tempP[0] *60;        
    }

    ProcessPIDTemp((int)TempZSPSld*10,temps[TEMP_RAZVAR]);
    UstPower=(long)Power*KtT/1000;
    // Мощность ограничена мощностью варки зерна
    if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;

    // Закончилось время паузы
    if (time2==0)
    {
      StateMachine=5;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else
      break;

  case 5: 
    timeNBK=0;
    SpeedNBK=0;
    //  Нагрев до закипания закипание определяется по преввшениею температуры либо в заторнике, либо в верхней части заторника.
    //  Миксер Работаем в следующем режиме - 5 минут пауза, 1 минута работает миксер.
    if (time1==0) 
    {  
       time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
       timeMIXER=(int) tempP[0] *60;        

    }

    UstPower=PowerVarkaZerno;
    
    if ((temps[TEMP_RAZVAR]>=TempKipenZator && ds1820_devices<2) || temps[TEMP_DEFL]>=TempKipenZator) // Если температура затора, либо температура дефлегматора больше заданной
    {
          // Переходим к следующему этапу - варка 1 часа. 
          StateMachine=6;
          time2=1*60*60; // Это 1.5 часа в секундах 
          if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else
      break;

  case 6: //Варка
    timeNBK=0;
    SpeedNBK=0;
    UstPower=PowerRazvZerno;
    //  Миксер Работаем в следующем режиме - 5 минут пауза, 1 минута работает миксер.
    if (time1==0) 
    {  
       time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
       timeMIXER=(int) tempP[0] *60;        
    }
    
    if (time2==0)
    {
      StateMachine=7;
      if (BeepStateProcess) my_beep(BEEP_LONG);
      time1=0;
    }
    else
      break;
  case 7: //охлаждение до температуры осахаривания.
    timeNBK=0;
    SpeedNBK=0;
    UstPower=0;
    // Мешалку и воду включаем одновременно, две минуты работаем, минуту отдыхаем.
    if (time1==0) 
    {  
        time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
        timeMIXER=(int) tempP[0] *60;        
        KlOpen[KLP_HLD]=PER_KLP_OPEN;
        KlClose[KLP_HLD]=PER_KLP_CLOSE;
    }
    // Если миксер закончил вращаться, то холодильник тоже отрубаем.
    if (timeMIXER==0) 
    {
      KlOpen[KLP_HLD]=0;
      KlClose[KLP_HLD]=100;
    }
    
    if (temps[TEMP_RAZVAR]<=(int)TempZSPSld*10)
    {
      // Если температура меньше либо равна температуре засыпки солода, то переходим к следующему этапу
      // - ожидание засыпки солода.
      StateMachine=8;
    }
    else
      break;
  case 8: //ожидание засыпки солода, поддержка температуры
    timeNBK=0;
    SpeedNBK=0;
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    KlOpen[KLP_HLD]=0;
    KlClose[KLP_HLD]=100;
    timeMIXER=0;
    // Из этого режима можно выйти только вручную, нажав кнопку вверх или отладочной функцией одновременно нажав вверх и вниз.
    // Пищим, чтобы привлечь внимание
    if (BeepStateProcess) my_beep(BEEP_LONG);
    ProcessPIDTemp((int)TempZSPSld*10,temps[TEMP_RAZVAR]);
    UstPower=(long)Power*KtT/1000;
    // Мощность ограничена мощностью варки зерна
    if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
    time2=10*60; // Устанавливаем время на работу мешалки после засыпки солода - 10 минут
    time1=0;
    break;
  case 9: // Мешаем затор 10 минут.
    timeNBK=0;
    SpeedNBK=0;
    
    KlOpen[KLP_HLD]=0;
    KlClose[KLP_HLD]=100;
    
    ProcessPIDTemp((int)TempZSPSld*10,temps[TEMP_RAZVAR]);
    UstPower=(long)Power*KtT/1000;
    // Мощность ограничена мощностью варки зерна
    if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;

    if (time2>0)
    {
      // Время работы - 1.5 минуты работы, 30 секунд отдых
      if (time1==0) 
      { 
         time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
         timeMIXER=(int) tempP[0] *60;        
      }
      
      break;
      
    }
    StateMachine=10;
    time2=60*60; // Время осахаривания - 1 час.
    if (IspReg==113)
    {
      time2=60*60*2/3; // Время осахаривания - 1.5 часа.
    }

  case 10: // осахаривание затора.
    timeNBK=0;
    SpeedNBK=0;
    KlOpen[KLP_HLD]=0;
    KlClose[KLP_HLD]=100;
    if (time2>0) 
    {
      // Время работы - 1 минуты работы, 10 минут отдых, то есть при осахаривании меняем время работы и время отдыха
      if (time1==0) 
      { 
        time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
        timeMIXER=(int) timeP[0] *60;        
      }

      ProcessPIDTemp((int)TempZSPSld*10,temps[TEMP_RAZVAR]);
      UstPower=(long)Power*KtT/1000;
      // Мощность ограничена мощностью варки зерна
      if (UstPower>PowerVarkaZerno) UstPower=PowerVarkaZerno;
     
      break;
    }
    StateMachine=11;
    time1=0;
  case 11: // Остужение до 40 градусов
      // Мешалку и воду включаем одновременно, две минуты работаем, минуту отдыхаем.
    UstPower=0;
    if (time1==0) 
    {  
      time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
      timeMIXER=(int) tempP[0] *60;        
      KlOpen[KLP_HLD]=PER_KLP_OPEN;
      KlClose[KLP_HLD]=PER_KLP_CLOSE;
    }
    // Если миксер закончил вращаться, то холодильник тоже отрубаем.
    if (timeMIXER==0) 
    {
      KlOpen[KLP_HLD]=0;
      KlClose[KLP_HLD]=100;
    }
    
    if (temps[TEMP_RAZVAR]<=400)
    {
      // Если температура меньше 40 градосов, то зададим небольшую порцию дрожжей 
      StateMachine=12;
      SpeedNBK=125;  
      timeNBK=5;    // Запускаем насос для первоначальной задачи дрожжей
      time1=0;
      my_beep(BEEP_LONG*3);

    }
    else
      break;
      
  case 12: // Остужение до 355 градусов
    UstPower=0;
    // Мешалку и воду включаем одновременно, две минуты работаем, 60 секунд отдыхаем.
    if (time1==0) 
    {  
      time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
      timeMIXER=(int) tempP[0] *60;        
      KlOpen[KLP_HLD]=PER_KLP_OPEN;
      KlClose[KLP_HLD]=PER_KLP_CLOSE;
    }
    // Если миксер закончил вращаться, то холодильник тоже отрубаем.
    if (timeMIXER==0) 
    {
      KlOpen[KLP_HLD]=0;
      KlClose[KLP_HLD]=100;
    }
    
    if (temps[TEMP_RAZVAR]<=(int)TempHLDZatorBrog1*10)
    {
      // переходи  к следующему этапу - поддержка брожения.
      StateMachine=13;
      SpeedNBK=125;  
      timeNBK=25;    // Запускаем насос для окончательной задачи дрожжей 
      time1=0;
      my_beep(BEEP_LONG*3);
    }
    else
      break;
  case 13: // Поддержка брожения - температура нормальна
    timeNBK=0;
    SpeedNBK=0;
      // Поперла пена - мешаем чаще и пищим
      if (flAlarmUroven) 
      { 
          
       my_beep(BEEP_LONG*5);
       if (time1==0 || time1>3*60)
        {
            time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
            timeMIXER=(int) tempP[0] *60;        
        }
        flAlarmUroven=0;
      }
    
  
    UstPower=0;
    // Мешалку и воду включаем одновременно, две минуты работаем, 60 секунд отдыхаем.
    if (temps[TEMP_RAZVAR]<=(int)TempHLDZatorBrog1*10)
    {
        KlOpen[KLP_HLD]=0;
        KlClose[KLP_HLD]=1000;
      //Если температура меньше 36 градусов, тогда периодически помешиваем, раз в 20 минут на 10 секунд
        if (time1==0) 
        {  
          time1=10*60;
          timeMIXER=30;
        }
        break;
    }
    else
    {
      if (BeepStateProcess) my_beep(BEEP_LONG);
      StateMachine=14;
      time1=0;
    }
    
  case 14: // Поддержка брожения температура превышена
    // Поперла пена
    if (flAlarmUroven)  
    { 
      my_beep(BEEP_LONG*5);
      if (time1==0)
      {
          // Звоним на сотовый, что поперла пена раз в три минуты - заливает!!!
          // На сотовый будет звонок при предупреждении
      }
      flAlarmUroven=0;
    }
    
    if (temps[TEMP_RAZVAR]>(int)TempHLDZatorBrog1*10-5)
    {
      //Если температура меньше 36 градусов, тогда интенсивно мешаем, раз в 15 минут на 60 секунд
        if (time1==0) 
        {  
          time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
          timeMIXER=(int) timeP[0] *60;                
          KlOpen[KLP_HLD]=PER_KLP_OPEN;
          KlClose[KLP_HLD]=PER_KLP_CLOSE;
        }
        
        if (timeMIXER==0) 
        {
          KlOpen[KLP_HLD]=0;
          KlClose[KLP_HLD]=100;
        }
        break;    
    }
    else
    {  
        if (BeepStateProcess) my_beep(BEEP_LONG);
        StateMachine=13;
        time1=0;
    }
    break;

  case 100: //Окончание
    // Если необходимо отправлять состояние на сотовый и это состояние еще не отправлялось, тогда информируем о состоянии процесса
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    flAllOff=1;

    break;
  }
}

//case 114:

void ProcessHLDZatorByChiller()
{
  switch (StateMachine)
  { 
  case 0: //Не запущено
    break;
  case 1: //Инициализация
    timeNBK=0;
    SpeedNBK=0;
    // Этот пункт пропускаем, потому что и так работает мешалка.
//      
//  
//      // Если необходимо отправлять состояние на сотовый и это состояние еще не отправлялось, тогда информируем о состоянии процесса
//  //    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
      lastSMSState=0;
//  
//  
//      if (temps[TEMP_RAZVAR]<500)
//      {
//        UstPower=Power;
//        break;
//      }
    StateMachine=12;
    //      if (BeepStateProcess) my_beep(BEEP_LONG);
     time1=0;
  case 12: // Остужение до 355 градусов
    UstPower=0;
    // Мешалку и воду включаем одновременно, две минуты работаем, 60 секунд отдыхаем.
    if (time1==0) 
    {  
      time1=4*60;
      timeMIXER=3*60;
      KlOpen[KLP_HLD]=PER_KLP_OPEN;
      KlClose[KLP_HLD]=PER_KLP_CLOSE;
    }
    // Если миксер закончил вращаться, то холодильник тоже отрубаем.
    if (timeMIXER==0) 
    {
      KlOpen[KLP_HLD]=0;
      KlClose[KLP_HLD]=100;
    }
    
    if (temps[TEMP_RAZVAR]<=(int)TempHLDZatorBrog1*10)
    {
      // переходи  к следующему этапу - поддержка брожения.
      StateMachine=13;
      time1=0;
      my_beep(BEEP_LONG*5);
    }
    else
      break;
  case 13: // Поддержка брожения - температура нормальна
    timeNBK=0;
    SpeedNBK=0;
      // Поперла пена - мешаем чаще и пищим
      if (flAlarmUroven) 
      { 
          
       my_beep(BEEP_LONG*5);
       if (time1==0 || time1>3*60)
        {
            time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
            timeMIXER=(int) tempP[0] *60;        
        }
        flAlarmUroven=0;
      }
    
  
    UstPower=0;
    // Мешалку и воду включаем одновременно, две минуты работаем, 60 секунд отдыхаем.
    if (temps[TEMP_RAZVAR]<=(int)TempHLDZatorBrog1*10)
    {
        KlOpen[KLP_HLD]=0;
        KlClose[KLP_HLD]=1000;
      //Если температура меньше 36 градусов, тогда периодически помешиваем, раз в 20 минут на 10 секунд
        if (time1==0) 
        {  
          time1=10*60;
          timeMIXER=30;
        }
        break;
    }
    else
    {
      if (BeepStateProcess) my_beep(BEEP_LONG);
      StateMachine=14;
      time1=0;
    }
    
  case 14: // Поддержка брожения температура превышена
    // Поперла пена
    if (flAlarmUroven)  
    { 
      my_beep(BEEP_LONG*5);
      if (time1==0)
      {
          // Звоним на сотовый, что поперла пена раз в три минуты - заливает!!!
          // На сотовый будет звонок при предупреждении
      }
      flAlarmUroven=0;
    }
    
    if (temps[TEMP_RAZVAR]>(int)TempHLDZatorBrog1*10-5)
    {
      //Если температура меньше 36 градусов, тогда интенсивно мешаем, раз в 15 минут на 60 секунд
        if (time1==0) 
        {  
          time1=((int) tempP[0]+ (int) timeP[0])*60; // Взводим насос на десять минут работы
          timeMIXER=(int) timeP[0] *60;                
          KlOpen[KLP_HLD]=PER_KLP_OPEN;
          KlClose[KLP_HLD]=PER_KLP_CLOSE;
        }
        
        if (timeMIXER==0) 
        {
          KlOpen[KLP_HLD]=0;
          KlClose[KLP_HLD]=100;
        }
        break;    
    }
    else
    {  
        if (BeepStateProcess) my_beep(BEEP_LONG);
        StateMachine=13;
        time1=0;
    }
    break;


  case 100: //Окончание
    // Если необходимо отправлять состояние на сотовый и это состояние еще не отправлялось, тогда информируем о состоянии процесса
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    flAllOff=1;

    break;
  }

}
//    case 118: //Текущее состояние - фракционная ректификация
//    case 109: //Текущее состояние - ректификация

// Функция возвращает значение ШИМ для отборв по "шпоре" (температуре в кубе)
char GetCHIMOtbor()
{
  // Процент отбора задается в табличном виде, например в данном случае 
  // от 0 до 88 в кубе процент отбора не меняем, от 88.0 до 96.0 в кубе процент отбора ставить в промежутке от 68 до 25
  // от 960 до 1000 процент отбора понижаем до нуля.
  // здесь храним количество значений в таблице;
  char FindCHIM;
  char i;
  #ifdef TEST
    Serial.print("KUB=");
    Serial.print(temps[TEMP_KUB]);
  #endif
  
  FindCHIM=ProcChimSR;
  for(i=1;i<CntCHIM;i++)
  {
    if (tempK[i-1]<=temps[TEMP_KUB] && tempK[i]>temps[TEMP_KUB])
    {
      // Если отбор при этом диапазоне температур производим
      if (CHIM[i-1]>0) 
      {
        FindCHIM=(temps[TEMP_KUB]-tempK[i-1])*(int) (CHIM[i]-CHIM[i-1])/(tempK[i]-tempK[i-1])+(int) CHIM[i-1];
        #ifdef TEST
          Serial.print(" CHIM=");
          Serial.println((int)FindCHIM);
        #endif
        return FindCHIM; // Теперь возвращаем новое значение ШИМ в любом случае. 
      }
      
    }
  }
  
  #ifdef TEST
  Serial.println(" noFindCHIM");
  #endif

  return ProcChimSR; 
}

void ProcessRectif()
{

  //#define KLP_NPG  0      // Номер клапана для управления НПГ (при дистилляции) 
  //#define KLP_VODA 1      // Номер клапана для управления общей подачей воды в систему
  //#define KLP_DEFL 2      // Номер клапана для подачи воды в дейфлегматор
  //#define KLP_GLV_HVS  3       // Номер клапана отбора головных и хвостовых фракций
  //#define KLP_SR   4      // Номер клапана отбора ректификата
  //#define TEMP_KUB  0      // Номер датчика  термометра в кубе
  //#define TEMP_RK20 1      // Номер датчика термометра в РК 20 см от насадки
  //#define TEMP_TSA  2      // Номер термометра в трубке связи с атмосферой
  static char Count_Provodimost=0; // Количество срабатываний датчика проводимости для окончания отбора голов.
  static char OldStateMachine=0; // Количество срабатываний датчика проводимости для окончания отбора голов.
  int V3; // Текущий уровень проводимости;

  if (flAlarmUroven && StateMachine<8 ) 
  { // Переводим автомат в стадию завершения 
    
    if (IspReg!=118)
    {  // Не фракционная перегонка, все как обычно, останавливаем процесс.
      if (USE_ALARM_UROVEN==1)
        {
          StateMachine=8; // Переводим в режим окончания
          SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
          if (BeepStateProcess) my_beep(BEEP_LONG);
        }
        else
        {
          OldStateMachine=StateMachine;
          StateMachine=9;  // Переводим в режим ожидания
          if (BeepStateProcess) my_beep(BEEP_LONG);
          Count_Provodimost=0;
          KlOpen[KLP_GLV_HVS]=0;
          KlClose[KLP_GLV_HVS]=100;
          KlOpen[KLP_SR]=0;
          KlClose[KLP_SR]=100;
        }
   }
   else
   {
    // Фракционная ректификация - переходим на следующую фракцию
        TekFraction++; SecTempPrev1=Seconds;
       flAlarmUroven=0; 
       CountAlarmUroven=0;
       if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
   }   
  }

  // Проверяем температуру в ТСА
/*
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
  }
*/
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA && StateMachine !=101) 
  {
    OldStateMachine = StateMachine; // BMV сохраним предыдущее состояние
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
  }  
  else {
    if (StateMachine==101 && temps[TEMP_TSA]<MAX_TEMP_TSA) {
     StateMachine = OldStateMachine;
    }
  }


  // Проверяем окончание температуры окончания ректификации
  if (temps[TEMP_KUB]>=tEndRect  && StateMachine<8 && IspReg!=118 )
  {  
    StateMachine=8;        
    SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }

  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    KlReg[KLP_GLV_HVS]=PEREGREV_ON;
    KlReg[KLP_SR]=PEREGREV_ON;

    //          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;
    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    
    flAllOff=0;
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    Count_Provodimost=0;
    if (CntCHIM<0) CntCHIM=-1;
    
  case 2: // Разгон
    UstPower=Power; // Устанавливаем максимальную мощность для разгона
    // Подчиненные контроллеры переводим в разгон
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    // Учитываем как температуру в кубе, так и температуру в колонне, в зависимости от того, положительные или отрицательные значения температур.
    if (temps[TEMP_KUB]<tEndRectRazgon  && tEndRectRazgon>0) break;
    if (temps[TEMP_RK20]<-tEndRectRazgon && tEndRectRazgon<=0) break;

    // Подаем питание на клапан дефлегматора
    KlOpen[KLP_DEFL]=PER_KLP_OPEN;
    KlClose[KLP_DEFL]=PER_KLP_CLOSE;
    TempPrev=temps[TEMP_RK20];
    SecTempPrev=Seconds; // Запоминаем дату последнего изменения температуры
    StateMachine=3;        
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    if (BeepStateProcess) my_beep(BEEP_LONG);
    TekFraction=0;
    SetAngle(AngleFractionRect[TekFraction]);
    SecTempPrev=Seconds;
  case 3:// Стабилизация температуры
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    // Польский буфер откроем.
    KlOpen[KLP_PB]=200;
    KlClose[KLP_PB]=0;
    
    if (temps[TEMP_RK20]>500) // Если температура больше 70 градусов (вдруг куб прогрелся, а колонна еще нет, нелогично, конечно, но на всякий случай учтем)
    {
      if (TimeStabKolonna>=0)  // Если время стабилизации колонны больше нуля, то тогда считаем время относительно последнего изменения температуры
      {
        if (abs(temps[TEMP_RK20]-TempPrev)<2) //Если текущая температура колонны равна температуре, запомненной ранее за исключением погрешности 0,2 градуса
        {
          if (Seconds-SecTempPrev>TimeStabKolonna) //Если с момента последнего измерения прошло более 600 секунд (10 минут), тогда считаем, что температура в колоенне стабилизировалась
          {
            // Переходим к следующему этапу - отбору голов. 
            StateMachine=4;        
            if (BeepStateProcess) my_beep(BEEP_LONG);
            SecTempPrev=Seconds;
          }
          else
            break;
  
        }
        else
        {
          TempPrev=temps[TEMP_RK20];// Запоминаем температуру
          SecTempPrev=Seconds; // Запоминаем время последнего изменения температуры
          break;
        }
      }
      else
      {
        // Если время стабилизации колонны меньше нуля, то тогда считаем абсолютное время 
        if (Seconds-SecTempPrev>-TimeStabKolonna) //Если с момента начала стабилизации прошло больше заданного количества секунд
          {
            // Переходим к следующему этапу - отбору голов. 
            StateMachine=4;        
            if (BeepStateProcess) my_beep(BEEP_LONG);
            SecTempPrev=Seconds;
          }
          else
            break;
      }
    }
    else
      break;
  case 4: // Отбор голов
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    //unsigned int timeChimRectOtbGlv=2000;  // Шим отбора голов (в полупериодах сетевого напряжения в данном случае это около 20 секунд)
    //unsigned char ProcChimOtbGlv=5;  // Процент ШИМ отбора голов (в данном случае это 5 процентов)
    // Клапан отбора ректификата закроем на всякий случай
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    // Польский буфер закроем.
    KlOpen[KLP_PB]=0;
    KlClose[KLP_PB]=10;
    
    // Устанавливаем клапан отбора хвостов и голов в соответвии с установленнным ШИМ
    //int tEndRectOtbGlv=854;     // Температура окончания отбора голов 85.4 С
    if (ProcChimOtbGlv>0)
        KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
    else
        KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
            
    KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
    
    if (IspReg!=118)
    { // Обычная ректификация
      if (UrovenProvodimostSR==0) // Если проводимость спирта не используется, головы отбираем по температуре
      {
        if (temps[TEMP_KUB]<tEndRectOtbGlv)
        {  // Пока температура в кубе не выросла выше окончания отбора голов, продолжаем это состояни
          break;       
        }
      }
      else
      {
  
        if (UrovenProvodimostSR>0) // Если проводимость больше нуля, используем проводимость в том или ином виде
        {
          V3=U_GLV; 
          // Если проводимость менее заданного уровня, и проводимость вообще сущетвует, то увеличиваем счетчик срабатываний
          // или ести проводимость больше уровня тревоги и установлен признак отбирать головы по уровню в приемной емкости.
    
          if ((V3<UrovenProvodimostSR && V3>=2 && UrovenProvodimostSR>=5) || (V3>=UROVEN_ALARM && UrovenProvodimostSR<=2 ))
           { 
             my_beep(BEEP_LONG); // Сначала пищим, предупреждая.
             Count_Provodimost++;
           }
          else 
            Count_Provodimost=0;
    
          // Если счетчик срабатываний не превысил максимальное число раз, тогда продолжаем процесс отбора голов.
          if (Count_Provodimost<MAX_COUNT_PROVODIMOST_SR) break;
        } 
        else // Если уровень проводимости SR меньше нуля, тогда это означает, что отбор голов производится по времени (в десятках минут).
        {
          // Анализируем, сколько минут прошло с начала отбора голов.
          SecOstatok=-((Seconds-SecTempPrev) / 600 + UrovenProvodimostSR);
          if (SecOstatok>0) break;
        }
      }
    }
    else
    {
      // При фракционном отборе отбор продукта идет через клапан СР.
      KlOpen[KLP_SR]=KlOpen[KLP_GLV_HVS];
      KlClose[KLP_SR]=KlClose[KLP_GLV_HVS];

      if (TekFraction==0) // Текущая фракция при перегонке всегда равна нулю, если вдруг она сменилась, например по наполнению приемной емкости (или вручную ее поменяли), тогда перейдем на отбор СР
      {
        // Фракционная ректификация - там немного по-другому
        TekTemp=TempFractionRect[TekFraction]; //Текущее состояние - первый недробный перегон
        if (TekTemp>=0) // Значение больше нуля, значит это температура
        {
          if (temps[TEMP_KUB]>=TekTemp)
          {  
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
          }
          else 
            break;
        }
        else
        {
          // Значение меньше нуля, значит это время в минутах
            SecOstatok=-((Seconds-SecTempPrev) / 60 + TekTemp);
          // Если времени не осталось, переходим на следующую фракцию  
          if (SecOstatok<=0) 
           { 
             TekFraction++; SecTempPrev1=Seconds; 
             
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
             
           }
           else 
             break;
        }
      }    
    }
    // Если температура превысила указанный рубеж, то переходим в состояние отбора спирта-ректификата.
    // Клапан отбора голов и хвостов отключаем
    tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
    KlOpen[KLP_GLV_HVS]=0;
    KlClose[KLP_GLV_HVS]=10;
    ProcChimSR=begProcChimOtbSR;
    StateMachine=5;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
    SecTempPrev1=Seconds;// Запомним время, когда произошел переход с фракции голов на отбор тела

  case 5: // Ожидание, пока температура не вернется к состоянию стабилизации
    // Это для отладки страт-стопа, на самом деле температура 28,5 градуса в этом режиме в колонне быть не может
    if (tStabSR==0) tStabSR=285;
    // Если колонна слишком долго находится в режиме стопа, то температуру стабилизации примем за новую
    if (time1==0 && TimeRestabKolonna!=0) 
     { 
       tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
       if (BeepStateProcess) my_beep(BEEP_LONG);
       // Если время рестабилизации колонны меньше нуля, тогда при долгом простое переходим на отбор хвостов.
       if (TimeRestabKolonna<0)
       {
            if (IspReg!=118)// Если не фракционная перегонка, то все как обычно.
            {
               // При не фракционной перегонке
                StateMachine=7;        
                KlOpen[KLP_SR]=0;// Отключаем клапана отбора СР
                KlClose[KLP_SR]=10;
                if (BeepStateProcess) my_beep(BEEP_LONG);        
            }
            else
            {
               // Переходим на последнюю фракцию, то есть отбор хвостов.
               SecTempPrev1=Seconds; 
               TekFraction=CountFractionRect-1;
               if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
               StateMachine=7;                     
            }
       }
     }
     
    KlOpen[KLP_GLV_HVS]=0;
    KlClose[KLP_GLV_HVS]=10;
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    if (temps[TEMP_KUB]>=tEndRectOtbSR)
    {  
      StateMachine=7;        
      if (BeepStateProcess) my_beep(BEEP_LONG);
      break;
    }

    if (temps[TEMP_RK20]>tStabSR)
    {
      break;
    }
    SecTempPrev=Seconds;// Запомним время, когда стабилизировалась температура
    StateMachine=6;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
  case 6: // Отбор СР
    //unsigned int timeChimRectOtbSR=1000;  // Шим отбора СР (в полупериодах сетевого напряжения в данном случае это около 10 секунд)
    //unsigned char tDeltaRect=10;  // Дельта ректификации (в десятых градуса Цельсия) в данном случае - это 10 десятых, то есть 1 градус Цельсия
    //unsigned int tEndRectOtbSR=965;  // Температура окончания отбора спирта-ректификата и начала отбора хвостов (в данном случе 96.5 С)
    // Работа Клапана отбора голов зависит от работы царги пастеризации

    if (ProcChimOtbCP>=0)
    {
          if (ProcChimOtbGlv>0)
              KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
          else
              KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
              
          KlOpen[KLP_GLV_HVS]=(KlOpen[KLP_GLV_HVS]*ProcChimOtbCP)/100;
          KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
      
    }
    else
    {
      KlOpen[KLP_GLV_HVS]=-ProcChimOtbCP;
      KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    }
    
    // Польский буфер закроем.
    KlOpen[KLP_PB]=0;
    KlClose[KLP_PB]=10;

    // Устанавливаем ШИМ отбора СР
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    ProcChimSR=GetCHIMOtbor();  // Реализуется отбор по-шпоре, что в функции прописано то и будет возвращено.
    
    KlOpen[KLP_SR]=((timeChimRectOtbSR/10)*ProcChimSR)/10;
    KlClose[KLP_SR]=timeChimRectOtbSR-KlOpen[KLP_SR];
    
    if (temps[TEMP_RK20]>=tStabSR+tDeltaRect) // Если текущая температура превысила базовую 
    {
      //        //Если температура превысилась менее, чем за одну минуту
      //        if (Seconds-SecTempPrev<60)
      //          {
      // произошел стоп - если значение количества температур в кубе меньше нуля, тогда запомним температуру в кубе за вычетом 0.1 градуса и процент ШИМ, при котором это произошло.
      if (CntCHIM<0)
      {
        // Запоминаем температуру, когда произошел стоп за вычетом 0.1 градуса.
        tempK[-CntCHIM]=temps[TEMP_KUB]-1;
        CHIM[-CntCHIM]=ProcChimSR;
        if (-CntCHIM<COUNT_CHIM-1) CntCHIM--;
      }

      if (DecrementCHIM>=0) ProcChimSR=ProcChimSR-DecrementCHIM;  // Тогда уменьшаем  ШИМ указанное число процентов в абсолютном выражении
         else  
         {
           V3=((int) ProcChimSR* (int)-DecrementCHIM)/100;
           // Процентное отношение может быть очень мало, поэтому если получилось нулевое значение, то вычтем единицу.
           if (V3<=0) V3=1;
           ProcChimSR=ProcChimSR-(char) V3; // Тогда увеличиваем ШИМ на число процентов в относительном выражении
         }

      if (ProcChimSR<minProcChimOtbSR) ProcChimSR=minProcChimOtbSR;
      
      StateMachine=5;        
      // Взводим таймер для возможной рестабилизации колонны
      if (TimeRestabKolonna<0) time1=-TimeRestabKolonna;
                          else time1=TimeRestabKolonna; 
                          
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else // Если температура не выросла более, чем за 10 минут, прибавим ШИМ на 5%
    {
      if (Seconds-SecTempPrev>TimeAutoIncCHIM) // Если не стабилизировалось более, чем за 10 минут, тогда прибавляем ШИМ на 5%
      {

        // Шим прибавляем только если не дошли до минимального шима, если дошли, то считаем, что хватит играть ШИМом
        if (ProcChimSR>minProcChimOtbSR) 
        {
          if (IncrementCHIM>=0)   ProcChimSR=ProcChimSR+IncrementCHIM; // Тогда увеличиваем ШИМ указанное число процентов в абсолютном выражении
                          else    ProcChimSR=ProcChimSR+((int) ProcChimSR* (int)-IncrementCHIM)/100 ; // Тогда увеличиваем ШИМ на число процентов в относительном выражении
          
          if (ProcChimSR>95) ProcChimSR=95;
        }
        SecTempPrev=Seconds;// Запомним время, когда стабилизировалась температура
      }
    }
   
    
    if (IspReg!=118)// Если не фракционная перегонка, то все как обычно.
    {
      // Если температура в кубе превысила температуру при которой надо отбирать СР
      if (temps[TEMP_KUB]>=tEndRectOtbSR)
      {  
        StateMachine=7;        
        KlOpen[KLP_SR]=0;// Отключаем клапана отбора СР
        KlClose[KLP_SR]=10;
        if (BeepStateProcess) my_beep(BEEP_LONG);
      }
      else    
        break;    
    }
    else
    {
      if (TekFraction<CountFractionRect-1) // смотрим на количество фракций, а оно может измениться при сработке датчика уровня.
      {
        TekTemp=TempFractionRect[TekFraction]; //Текущее состояние - первый недробный перегон
        if (TekTemp>=0) // Значение больше нуля, значит это температура
        {
          if (temps[TEMP_KUB]>=TekTemp)
          {  
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
          }
        }
        else
        {
          // Значение меньше нуля, значит это время в минутах
            SecOstatok=-((Seconds-SecTempPrev1) / 60 + TekTemp);
          // Если времени не осталось, переходим на следующую фракцию  
          if (SecOstatok<=0) 
           { 
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
           }
          
        }    
       break;        
      }
      else
      {  // Если это последняя фракция, то переводим в режим отбора хвостов.
         SecTempPrev1=Seconds; 
         if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
         StateMachine=7;        
      }
    }
  case 7: // Отбор Хвостов
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    if (IspReg!=118) // Если не фракционный отбор, то все как обычно
    {
      KlOpen[KLP_SR]=0;
      KlClose[KLP_SR]=10;
      KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*90;
      KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
      
      if (temps[TEMP_KUB]>=tEndRect)
      {  
        StateMachine=8;        
        SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
        if (BeepStateProcess) my_beep(BEEP_LONG);
      }
      else    
        break;
    }
    else
    { // Отбор фракционный, все через клапан отбора СР
      KlOpen[KLP_SR]=(timeChimRectOtbGlv/100)*90;
      KlClose[KLP_SR]=timeChimRectOtbGlv-KlOpen[KLP_SR];

      if (TekFraction<CountFractionRect)
      { // Если количество фракций не превышает заданную. 
        TekTemp=TempFractionRect[TekFraction]; //Текущее состояние - первый недробный перегон
        if (TekTemp>=0) // Значение больше нуля, значит это температура
        {
          if (temps[TEMP_KUB]>=TekTemp)
          {  
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
          }
        }
        else
        {
          // Значение меньше нуля, значит это время в минутах
            SecOstatok=-((Seconds-SecTempPrev1) / 60 + TekTemp);
          // Если времени не осталось, переходим на следующую фракцию  
          if (SecOstatok<=0) 
           { 
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
           }
        }    
       break;        
      }
      else
      {
        // Иначе переходим к завершению процесса
        StateMachine=8;        
        SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
        if (BeepStateProcess) my_beep(BEEP_LONG);
      }     
      
    }
  case 8:// Ждем три минуты для окончания подачи воды
    UstPower=0;
    // Подчиненные контроллеры отключаем
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    KlOpen[KLP_GLV_HVS]=0;  // Отключаем клапана отбора хвостов
    KlClose[KLP_GLV_HVS]=10;
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;

    if  (Seconds-SecTempPrev>180)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 9:// Ожидание, пока датчик уровня отменяет сработку.
      V3=U_UROVEN; 
      if (V3<UROVEN_ALARM )
      {
        Count_Provodimost++;
        #ifdef TEST
                sprintf(my_tx_buffer,"ALARM VOZVRAT U_UROVEN=%i cnt=%i",U_UROVEN,Count_Provodimost);
                Serial.println(my_tx_buffer);
        #endif      
        // По прошествии минуты устанавливаем флаг сработки.
        if (Count_Provodimost>2) 
        {
          flAlarmUroven=0;
          StateMachine=OldStateMachine;
          Count_Provodimost=0;
        }
      }
      else 
      {
        Count_Provodimost=0;
      }
      break;
 
  case 101: // Превышение температуры в ТСА!!!
  case 102: // Превышение давления
    my_beep(2*BEEP_LONG);
  case 100:// Конечное состояние автомата

    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();

    if (StateMachine==100 && BeepEndProcess) my_beep(BEEP_LONG);

    UstPower=0;
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    flAllOff=1;
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    // Отключаем подчиненные контроллеры
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);

    break;
  }
  #ifdef TESTMEM
    Serial.println("\n[memCheck_Rectif]");
    Serial.println(freeRam());
  #endif
}

char flNeedSendGPRS=0;
//char flStatusGPRS=0;      // Флаг выполнения опроса состояния.
// 0  - неопределено
// 1  + CREG: 0,1 - зарегестрировано в сети
// 2  + CREG: 0,1 - не зарегестрировано в сети.
// 3 +XIIC:    1, 10.104.82.242 успешное подключение к интернету
// 4 +XIIC:    1, 0.0.0.0       неуспешное подключение к интернету.
void ClearBuffer()
{
    while (Serial2.available() > 0) Serial2.read();
    pos_rx_buffer=0;
    my_rx_buffer[pos_rx_buffer]=0;
}

void toUSARTPDU(char *s)
{ 
  char ot[20];
  unsigned char i; 
  char pdu_header[]="0011000B91"; // Заголовок ПДУ.
  char pdu_poster[]="0008FF"; // Заключительная часть ПДУ.

  // Выводим заголовок ПДУ
  Serial2.print(pdu_header);
  Serial2.print(pdu_phone);
  Serial2.print(pdu_poster);
  // Выводим длину сообщения в шестнадцатеричном формате.
  sprintf(ot,"%02X",strlen(s)*2);
  Serial2.print(ot);
  // Выводим само сообщение

  for(i=0;s[i];i++)
  {
    sprintf(ot,"00%02X",s[i]);
    Serial2.print(ot);
  }

  Serial2.write(0x1A);

}

void SendWiFi8266()
{
  switch(flGPRSState)
  {
    case 0:  // Состояние не запущено
      break;
    case 1: // Активируем отправку SMS
  
    case 2: // С состояний, соответсвующих отправке GPRS активируем wi-fi соединение
    case 14: 
        flGPRSState=158;
        break;
    case 40: // Состояние 40 - это дозвон до телефона, но у wi-fi телефон не подключен, поэтому просто передаем информацию 
        flGPRSState=158;
        flNeedCall=3; // ставим признак что дозвонились
    case 98: // Инициализация GSM
      // Включаем уведомления о SMS
      timeWaitGPRS=10;
      Serial2.println("AT");
      flGPRSState=99;    
    case 99:
      if (timeWaitGPRS==0) flGPRSState=98;
      break;
    case 150:
      timeWaitGPRS=10;
      Serial2.println("AT+CWMODE=1");
      flGPRSState=151;    
    case 151:
      if (timeWaitGPRS==0) flGPRSState=150;
      break;
    case 152:
      // Отключаемся от сети
      timeWaitGPRS=10;
      Serial2.println("AT+CWQAP");
      flGPRSState=153;
    case 153:
      if (timeWaitGPRS==0) flGPRSState=152;
      break;
    case 154:
      // Проверка подключения
      timeWaitGPRS=10;
      Serial2.println("AT+CIFSR");
      flGPRSState=155;
    case 155:
      if (timeWaitGPRS==0) flGPRSState=154;
      break;
    case 168: // Промежуточное состояние - к серверу подключены, идет ожидание OK
      if (timeWaitGPRS==0) flGPRSState=154;
      break;
      
    case 156:
      timeWaitGPRS=10;
      // Подключение
      sprintf(my_gprs_buffer,"AT+CWJAP=\"%s\",\"%s\"",WiFiAP,WiFiPass);
      Serial2.println(my_gprs_buffer); 
      flGPRSState=157;
    case 157:
      if (timeWaitGPRS==0) flGPRSState=156;
      break;
    case 158:
      timeWaitGPRS=10;
      // проверка соединения с сервером.
      Serial2.println("AT+CIPSTATUS");
      flGPRSState=159;
    case 159:
      if (timeWaitGPRS==0) flGPRSState=158;
      break;
    case 167: // Промежуточное состояние - к серверу подключены, идет ожидание OK
      if (timeWaitGPRS==0) flGPRSState=158;
      break;
      
    
    case 160:
      timeWaitGPRS=10;
      if (FlToGSM==10) sprintf(my_gprs_buffer,"AT+CIPSTART=\"UDP\",\"%u.%u.%u.%u\",%u",(unsigned int)ip[0],(unsigned int)ip[1],(unsigned int)ip[2],(unsigned int)ip[3],ipPort);
      if (FlToGSM==11) sprintf(my_gprs_buffer,"AT+CIPSTART=\"TCP\",\"%u.%u.%u.%u\",%u",(unsigned int)ip[0],(unsigned int)ip[1],(unsigned int)ip[2],(unsigned int)ip[3],ipPort);
      // Подключение
      Serial2.println(my_gprs_buffer);
      flGPRSState=161;
    case 161:
      if (timeWaitGPRS==0) flGPRSState=160;
      break;
    case 162:
      // Отправка
      timeWaitGPRS=10;
      //Serial2.println("AT+CIPSEND=54");
      Serial2.println("AT+CIPSEND=56");
      flGPRSState=163;
    case 163:
      if (timeWaitGPRS==0) flGPRSState=162;
      break;
    case 164:
      timeWaitGPRS=20;
      DisplayData();
      break;
    case 165:
      if (timeWaitGPRS==0) flGPRSState=158;
      // Сообщение отправлено
      break;
    
      
    case 180: // Закрываем TCP-UDP соединение и завершаем сеанс
      timeWaitGPRS=10;
      Serial2.println("AT+CIPCLOSE");
      flGPRSState=181;    
    case 181:
      flGPRSState=0; // Ничего не делаем, ждем.
      break;
   default:
     // Если состояние неизвестно, то переходим на инициализацию.
     flGPRSState=98;
     break;
     
  
  }
  
}
void SendGPRS()
{
  if (FlToGSM==10 || FlToGSM==11)  SendWiFi8266();
#ifdef TESTGSM
  //     if (timeWaitGPRS % 5 ==0)
  //     {
  //      Serial.print("Sec:");
  //      Serial.print(Seconds);
  //      Serial.print(" StGPRS:");
  //      Serial.println((int)flGPRSState);
  //     }
#endif

  switch(flGPRSState)
  {
  case 0:  // Состояние не запущено
    break;
  case 1: // Активируем отправку SMS


  case 2: // Активирукм GPRS
//    InitGSM();
    // Проверка подключения
    if (FlToGSM<2)
    {
      flGPRSState=0;
      break;
    }     
    Serial2.println("AT+XISP=0");
    timeWaitGPRS=15;
    flGPRSState=3;
  case 3:
    if (timeWaitGPRS==0) flGPRSState=2;
    break;
  case 4:
    timeWaitGPRS=10;
    Serial2.println("AT+CLCC");
    flGPRSState=5;
  case 5:
    if (timeWaitGPRS==0) flGPRSState=4;
    break;
  case 6:
    timeWaitGPRS=10;
    #ifdef TESTGSM
        Serial.println("SendCMD:AT+CHUP");
    #endif
    Serial2.println("AT+CHUP"); // Отклоняем вызов.
    flGPRSState=7;
  case 7:
   if (timeWaitGPRS==0) flGPRSState=6;
    break;
  case 10:  // Пытаемся подключить интернет
    if (FlToGSM<2)
    {
      flGPRSState=0;
      break;
    }     
    if (FlToGSM==2)  sprintf(my_gprs_buffer,"at+cgdcont=1,\"IP\",\"internet\"");
    else
      if (FlToGSM==3)  sprintf(my_gprs_buffer,"at+cgdcont=1,\"IP\",\"internet.beeline.ru\"");
      else
        if (FlToGSM==4)  sprintf(my_gprs_buffer,"at+cgdcont=1,\"IP\",\"internet.mts.ru\"");
        else
          if (FlToGSM==5)  sprintf(my_gprs_buffer,"at+cgdcont=1,\"IP\",\"internet.rt.ru\"");
          else
            if (FlToGSM==6)  sprintf(my_gprs_buffer,"at+cgdcont=1,\"IP\",\"internet.tele2.ru\"");

    Serial2.println(my_gprs_buffer);
#ifdef TESTGSM
    Serial.print("SendCMD:");
    Serial.println(my_gprs_buffer);
#endif
    timeWaitGPRS=15;
    flGPRSState=11; // Ничего не делаем, ждем.
  case 11: // Ждем выполнения команды.
    if (timeWaitGPRS==0) flGPRSState=10;
    break;
  case 12: // Не Подключено к интернет - посылаем команду подключения
    Serial2.println("at+xiic=1"); // Подключаем IP
#ifdef TESTGSM
    Serial.print("SendCMD:");
    Serial.println("at+xiic=1");
#endif
    timeWaitGPRS=15;
    flGPRSState=13; // Ничего не делаем, ждем.
  case 13: // Ждем выполнения команды IP
    if (timeWaitGPRS==0) flGPRSState=12;
    break;
  case 14: // Не Подключено к интернет - посылаем команду подключения
    // Проверка подключения
    if (FlToGSM<2)
    {
      flGPRSState=0;
      break;
    }     
    
    // Проверка подключения
    Serial2.println("at+xiic?"); // Проверка подключения
#ifdef TESTGSM
    Serial.print("SendCMD:");
    Serial.println("at+xiic?");
#endif
    timeWaitGPRS=15;
    flGPRSState=15; // Ничего не делаем, ждем.
  case 15: // Ждем выполнения команды IP
    if (timeWaitGPRS==0) flGPRSState=14;
    break;
  case 16: // Не Подключено к интернет - посылаем команду подключения к серверу

    #ifndef UDP_PROTOKOL     
      sprintf(my_gprs_buffer,"at+tcpsetup=0,%u.%u.%u.%u,%u",(unsigned int)ip[0],(unsigned int)ip[1],(unsigned int)ip[2],(unsigned int)ip[3],ipPort);
    #else
      sprintf(my_gprs_buffer,"AT+UDPSETUP=0,%u.%u.%u.%u,%u",(unsigned int)ip[0],(unsigned int)ip[1],(unsigned int)ip[2],(unsigned int)ip[3],ipPort);
    #endif
    
    Serial2.println(my_gprs_buffer);

#ifdef TESTGSM
    Serial.print("SendCMD:");
    Serial.println(my_gprs_buffer);
#endif

    timeWaitGPRS=15;
    flGPRSState=17; // Ничего не делаем, ждем.
  case 17: // Ждем выполнения команды IP
    if (timeWaitGPRS==0) flGPRSState=16;
    break;
  case 50: // Закрываем TCP соединение
    
    #ifndef UDP_PROTOKOL     
    Serial2.println("AT+TCPCLOSE=0");
    #else
    Serial2.println("AT+UDPCLOSE=0");
    #endif
    
    timeWaitGPRS=15;
    flGPRSState=51; // Ничего не делаем, ждем.
  case 51:
    if (timeWaitGPRS==0) flGPRSState=50;
    break;
  case 18: // Посылаем команду для подключения к серверу.
    if (NeedGPRS==0) break;
    
    #ifndef UDP_PROTOKOL     
      Serial2.println("at+tcpsend=0,54");
    #else
      Serial2.println("AT+UDPSEND=0,54");
    #endif
    timeWaitGPRS=15;
    flGPRSState=19; // Ничего не делаем, ждем.
#ifdef TESTGSM
    Serial.print("SendCMD:");
    Serial.println("at+TcUdSend=0,54");
#endif
    NeedGPRS=0;
  case 19:
    if (timeWaitGPRS==0) flGPRSState=30;
    break;
  case 20:
    timeWaitGPRS=20;
    DisplayData();
    // Можно отправлять сообщение
    // Отправляем сообщение оно состоит из
    // <id устройства 10 символов>:<содержимое экрана 32 символа>:<состояние контроллера 3 символа># - признак окончания вывода.
    // Итого 48 символов.
    break;
  case 21:
    if (timeWaitGPRS==0) flGPRSState=30;
    // Сообщение отправлено
    break;
  case 25:
    // Отправлено, готовим следующюю отправку
    flGPRSState=18;
    break;
  case 30: // Проверяем доступность соединения
    if (FlToGSM<2)
    {
      flGPRSState=0;
      break;
    }     
    timeWaitGPRS=15;
    Serial2.println("AT+IPSTATUS=0");
#ifdef TESTGSM
    Serial.print("Status:");
    Serial.println("AT+IPSTATUS=0");
#endif
    flGPRSState=31;
    break;
  case 31: // Проверяем доступность соединения
    if (timeWaitGPRS==0) flGPRSState=30;
    break;
  case 40: // Звонок по аварии
    //Если телефон на задан или первый знак в номере не, тогда считаем что дозвонились.
    if (!strcmp(my_phone,"+77777777777") || my_phone[0]!='+')
    {  
        flNeedCall=3;       // Ставив признак дозвона.
        flGPRSState=2;      // Переходим в состояние отправки информации по GPRS.  
        break;
    }
    
    Serial2.print("ATD ");
    Serial2.print(my_phone);
    Serial2.println(";");
    flGPRSState=41;
    flNeedCall=2; // Ставим признак дозвона
    timeWaitGPRS=20; // Надо дозвониться за 20 секунд
 case 41:
 
   if (timeWaitGPRS==0) 
    { 
       flGPRSState=0; // Не дозвонились за указанное время, тогда
       Serial2.println("ATH"); // Сбрасываем звонок.
       flNeedCall=1;  // Устанавливаем признак необходимости дозвона.
     }; // Ставим признак дозвона
   break;
  case 60: // Чтение SMS сообщения
    timeWaitGPRS=10;
    Serial2.print("AT+CPMS=");
    Serial2.println(vid_sms);
    flGPRSState=61;
  case 61: // Ждем чтения сообщения
    if (timeWaitGPRS==0) flGPRSState=60;
    break;
  case 62: // Чтение SMS сообщения
    // Очищаем буфер приемки
    pos_rx_buffer=0;
    timeWaitGPRS=5;
    Serial2.print("AT+CMGR=");
    Serial2.println(num_sms);
    flGPRSState=63;
  case 63: // Ждем чтения сообщения
    if (timeWaitGPRS==0) flGPRSState=62;
    break;
  case 64: // Удаление сообщения
    #ifdef TESTGSM
          Serial.print("DELETING:");
          Serial.println(num_sms);
    #endif 

    Serial2.print("AT+CMGD=");
    Serial2.println(num_sms);
    timeWaitGPRS=10;
    flGPRSState=65;
  case 65: // Удаление сообщения
    if (timeWaitGPRS==0) flGPRSState=64;
    break;
  case 66: // Отправка сообщения - перевод в PDU режим - пока пользуемся старой функцией отправки SMS
  case 67:
  case 68: // Отправка сообщения - перевод в формат GSM
  case 69:
    flGPRSState=0;
    break;
  case 70: // Отправка сообщения - команда на отправку, ожидание >
    GetState();
    sprintf(my_gprs_buffer,"AT+CMGS=%2i",14+strlen(my_tx_buffer)*2);// выводим количество окетов
    Serial2.println(my_gprs_buffer);
    timeWaitGPRS=15;
    flGPRSState=71;
  case 71:
    // Ждем >
    if (timeWaitGPRS==0) flGPRSState=70;
    break;
  case 72: // Отправка сообщения - собственно отправка
//    GetStateTXT();
    toUSARTPDU(my_tx_buffer);
    timeWaitGPRS=10;
    flGPRSState=73;
  case 73:
    flGPRSState=0;
    if (timeWaitGPRS==0) flGPRSState=0;
    break;
  case 90: // Закрываем TCP соединение
    timeWaitGPRS=10;
    
    #ifndef UDP_PROTOKOL     
    Serial2.println("AT+TCPCLOSE=0");
    #else
    Serial2.println("AT+UDPCLOSE=0");
    #endif
    
    flGPRSState=91;    
  case 91:
    flGPRSState=0; // Ничего не делаем, ждем.
    break;
  case 98: // Инициализация GSM
    // Включаем уведомления о SMS
    timeWaitGPRS=10;
    Serial2.println("AT");
    flGPRSState=99;    
  case 99:
    if (timeWaitGPRS==0) flGPRSState=98;
    break;
  case 100: // Инициализация GSM
    // Включаем уведомления о SMS
    timeWaitGPRS=10;
    Serial2.println("AT+CNMI=1,1");
    flGPRSState=101;    
  case 101:
    if (timeWaitGPRS==0) flGPRSState=100;
    break;
  case 102: // Инициализация GSM
    // Включаем внутренний стек модема.
    timeWaitGPRS=10;
    Serial2.println("AT+XISP=0");
    flGPRSState=103;    
  case 103:
    if (timeWaitGPRS==0) flGPRSState=102;
    break;
  case 104: // Инициализация GSM
    // Включаем уведомления о SMS
    timeWaitGPRS=10;
    sprintf(my_gprs_buffer,"AT+CSCS=\"GSM\"");
    Serial2.println(my_gprs_buffer);
    flGPRSState=105;    
  case 105:
    if (timeWaitGPRS==0) flGPRSState=104;
    break;
  case 106: // Инициализация GSM
    // Включаем формат SMS в виде PDU
    timeWaitGPRS=10;
    Serial2.println("AT+CMGF=0");
    flGPRSState=107;    
  case 107:
    if (timeWaitGPRS==0) flGPRSState=106;
    break;
  case 108: // Инициализация GSM
    // Включаем формат SMS в виде PDU
    timeWaitGPRS=10;
    Serial2.println("AT+CMGD=1");
    flGPRSState=109;    
  case 109:
    if (timeWaitGPRS==0) flGPRSState=108;
    break;
  case 110: // Инициализация GSM
    // Включаем формат SMS в виде PDU
    timeWaitGPRS=10;
    Serial2.println("AT+CMGD=2");
    flGPRSState=111;  
    stateAfterDelete=10;  
  case 111:
    if (timeWaitGPRS==0) flGPRSState=110;
    break;
  // Это системное состояние - пауза нужное колчичество времени, после паузы программа перейдет на заданое состояние
  case 80:
    if (timeWaitGPRS==0) flGPRSState=stateAfterDelete;
    break;
   // Это состояние переводит в паузу перед повторным вызовом состояния подключения к серверу
  case 81:
    timeWaitGPRS=2;
    flGPRSState=80;
    stateAfterDelete=14;
    break;
   
  }
}


void  StateToGPRS()
{ 
  flNeedSendGPRS=1;
}

void AnalyseCommand()
{
    #ifdef TESTGSM
      // Дублируем информацию на Serial1
      Serial.print("GetComm:");
      Serial.println(my_rx_buffer);
      Serial.print(" State: ");
      Serial.print((int)flGPRSState);
      Serial.print(" Time: ");
      Serial.println((int)timeWaitGPRS);
    #endif
    
    #ifdef TESTGSM1
//      Serial.print("S:");
//      Serial.println(Seconds);
    #endif
    
    

    if(!strcmp("OK",my_rx_buffer)) // Получили результат OK, значит анализируем состояние автомата.
    {
  
      #ifdef TESTGSM
          Serial.print("Get OK flGPRSState:");
          Serial.println((int)flGPRSState);
      #endif
  
      switch(flGPRSState)
      {

        case 3:
          flGPRSState=10;
           break;
        case 5:
          flGPRSState=6; // 
          break;
        case 7:
          // Если сотояние сети зарегестрирован, то считаем успешно, если нет, то тоже успешно, поскольку пока не знаю, что с этим состоянием делать
          // А если состояние другое, значит что-то не определилось, пробуем заново.
          // получили ОК после выполнения команды 
          //at+creg?
          // Перед ОК было вывод состояния в виде результата
          // +CREG: 0,1
          // Отклонили вызов если звонили с нужного номера, то либо активизируем сессию GPRS либо отправляем информацию по SMS
          if (flRing)
          {
            if (FlToGSM<2 || SMSOnCall==1) 
            {

              flGPRSState=70; // Переводим в режим отрпавки SMS.
              timeGPRS=30;
              #ifdef TESTGSM
                  Serial.print("Actvate SMS:");
                  Serial.println((int)flGPRSState);
              #endif
            }
            else
            { 
              flGPRSState=14; // Активируем сессию GPRS
              
              timeGPRS=120; // Установив внемя для отправки GPRS сообщений
              flNeedRefrServer=0; // Снмаем флаг необходимости разовой отправки сведений на сервер
              ErrGPRSConnectInernet=0;
              ErrGPRSConnectServer=0;
            }
            
          }
          else 
            flGPRSState=0; 
    
          break;
        
        case 8:
          // Если сотояние сети зарегестрирован, то считаем успешно, если нет, то тоже успешно, поскольку пока не знаю, что с этим состоянием делать
          // А если состояние другое, значит что-то не определилось, пробуем заново.
          // получили ОК после выполнения команды 
          //at+creg?
          // Перед ОК было вывод состояния в виде результата
          // +CREG: 0,1
          flGPRSState=10;
          break;
        case 11:
          // получили ОК после выполнения команды 
          //at+cgdcont=1,"IP","internet"
          flGPRSState=12;
          break;
        case 13:
          // получили ОК после выполнения команды 
          //at+xiic=1
          // После команды посылки связи с интернетом ждем 2 секунды
          timeWaitGPRS=2; // Делаем паузу 2 секунды, затем переходим на проверку связи.
          flGPRSState=80;
          stateAfterDelete=14;
          break;
        case 15:
          // получили ОК после выполнения команды 
          //at+xiic?
          // Перед ОК было вывод состояния в виде результата
          // +XIIC:    1, 10.104.82.242
          // ErrGPRSConnectInernet=0;
          break;
        case 17:
          // получили ОК после выполнения команды 
          // at+tcpsetup=0,09.23.22.16,1211
          // Но состояние после этой команды не изменяем, поскольку ОК идет до того, как ответит сервер.
          flGPRSState=17;
          break;
        case 21:
          //flGPRSState=25;
          break;
        case 65: // Удалили SMS сообщение
          flGPRSState=stateAfterDelete;
          break;
        case 67: // 
          flGPRSState=68;
          break;
        case 69: // 
          flGPRSState=70;
          break;
        case 73: // 
          flGPRSState=0;
          break;
        case 99:
          flGPRSState=100;
          //
          if (FlToGSM==10 || FlToGSM==11) flGPRSState=150; 
            
          break;
        case 101:
          flGPRSState=102;
          break;
        case 103:
          flGPRSState=104;
          break;
        case 105:
          flGPRSState=106;
          break;
        case 107:
          flGPRSState=108;
          break;
        case 109:
          flGPRSState=110;
          break;
        case 111:
          flGPRSState=stateAfterDelete;
          if (flGPRSState==0) timeGPRS=10;          
          break;
        case 151:
          flGPRSState=152;
          break;
        case 153:
          flGPRSState=154;
          break;
        case 157: // Подключились
          flGPRSState=154; // Переход на передачу информации.
          break;
        case 159: // Проверка подключения с серверу, если дошли до ОК, значит не подключены, вызываем подключение
         // Дошли до 
          flGPRSState=160; // Переход на подключение к серверу.
          break;
        case 161: // Подключены к серверу по TCP или UDP
         // Дошли до 
          flGPRSState=162; // Переход на передачу данных.
          break;
        case 167: // Проверка подключения с серверу выполнена вызываем отправку данных
         // Дошли до 
          flGPRSState=160; // Переход на отправку данных.
          break;
        case 168: // Проверка подключения к wi-fi выполнена вызываем отправку данных
         // Дошли до 
          flGPRSState=stateAfterDelete; // Переход на отправку данных.
          break;

        default:
          break;
      }
      return;
    }

    if(!strcmp("ERROR",my_rx_buffer)) // Получили результат OK, значит анализируем состояние автомата.
     {
       ClearBuffer(); 
       #ifdef TESTGSM
            Serial.print("Get ERROR State:");
            Serial.println((int)flGPRSState);
        #endif
    
        switch(flGPRSState)
        {
          case 65:
            flGPRSState=0;
            break;
          case 101: // при этих ошибках возвращаемся снова на команду AT
          case 103:
          case 105:
          case 107:
          case 109:
          case 111:
            flGPRSState=98;
            break;
          case 157:
            flGPRSState=156;
            break;
          case 162:
            flGPRSState=158;
            break;
          case 168: // прогнозируемая ошибка, например повторное подключение к серверу.
            flGPRSState=stateAfterDelete; // Переход на отправку данных.
            break;
          default:
            ClearBuffer();
            if (FlToGSM<2)
            {
              break;
            }     
            flGPRSState=30;
            if (FlToGSM==10 || FlToGSM==11)  flGPRSState=150;
            break;
        }
    
        #ifdef TESTGSM
            Serial.print("Change ERROR State:");
            Serial.println((int)flGPRSState);
        #endif

      return;
    }
    
    if (strstr(my_rx_buffer,"+CREG: ")) // Информация о состоянии подключения
    {  
        #ifdef TESTGSM
            Serial.print("State +CREG:");
            Serial.println((int)flGPRSState);
        #endif
      return;
    }


  if (strstr(my_rx_buffer,"+XIIC:")) // Информация о состоянии подключения
  {  

    if (strstr(my_rx_buffer,"0.0.0.0"))  
      {  ErrGPRSConnectInernet++; stateAfterDelete=81; flGPRSState=168; }// Неуспешное подключение к инернету - переодим в режим ожидания 2 сек перед повторым вызовом состояния подклчения
    else  
      {  
        stateAfterDelete=16; flGPRSState=168;ErrGPRSConnectInernet=0;  
      } //  Успешное подключение к интернету
    

    if (ErrGPRSConnectInernet>40)
    {
      #ifdef TESTGSM
          Serial.print("Err conn failed internet:");
          Serial.println((int)ErrGPRSConnectInernet);
      #endif
      flGPRSState=0;
    }
    else
      if (ErrGPRSConnectInernet>20)
      {
        #ifdef TESTGSM
            Serial.print("Err conn internet:");
            Serial.println((int)ErrGPRSConnectInernet);
        #endif
        stateAfterDelete=2;
        flGPRSState=168;
      }

#ifdef TESTGSM
    Serial.print("State +XIIC:");
    Serial.println((int)flGPRSState);
#endif
    return;
  }

  if (strstr(my_rx_buffer,"+CIFSR:STAIP")) // Информация о состоянии подключения wi-fi
  {  

    if (strstr(my_rx_buffer,"0.0.0.0"))  {  flGPRSState=168; stateAfterDelete=156; ErrGPRSConnectInernet++;} // Неуспешное подключение к wi-fi
    else  {flGPRSState=168;stateAfterDelete=158; ErrGPRSConnectInernet=0;} //  Успешное подключение к wi-fi, переход к ожиданию ОК.

    if (ErrGPRSConnectInernet>40)
    {
      #ifdef TESTGSM
          Serial.print("Err conn failed Wi-FI:");
          Serial.println((int)ErrGPRSConnectInernet);
      #endif
      flGPRSState=0;
    }
    else
      if (ErrGPRSConnectInernet>20)
      {
        #ifdef TESTGSM
            Serial.print("Err conn Wi-Fi:");
            Serial.println((int)ErrGPRSConnectInernet);
        #endif
        flGPRSState=168; stateAfterDelete=150;
      }

#ifdef TESTGSM
    Serial.print("+CIFSR:STAIP");
    Serial.println((int)flGPRSState);
#endif
    return;
  }


  #ifndef UDP_PROTOKOL
  if (strstr(my_rx_buffer,"+TCPSETUP:")) // Информация о состоянии к серверу
  #else
  if (strstr(my_rx_buffer,"+UDPSETUP:")) // Информация о состоянии к серверу
  #endif  
  
  {
    if (strstr(my_rx_buffer,":0,OK"))  { 
      flGPRSState=30; 
      ErrGPRSConnectServer=0;
    }// Успешно подключено к серверу
    else 
      if (strstr(my_rx_buffer,":Error")) { 
      flGPRSState=50; 
      ErrGPRSConnectServer++;
    }// Ошибка подключения, закрываем соединение и пробуем еще раз
    else  
    { 
      flGPRSState=14; 
      ErrGPRSConnectServer++;
    }// Неуспешно подключено пробуем подключение еще раз.

    if (ErrGPRSConnectServer>20) 
    {
#ifdef TESTGSM
      Serial.print("Error connect to Server count=");
      Serial.println((int)ErrGPRSConnectServer);
#endif
      flGPRSState=2;
    } 

   return;
  }

  if (strstr(my_rx_buffer,"+IPSTATUS:")) // Информация о состоянии к серверу
  {
    //            +IPSTATUS:0,CONNECT,TCP,2047
    if (strstr(my_rx_buffer,":0,CONNECT,"))  { 
      flGPRSState=18; 
    }// Успешно подключено к серверу, можно отправлять данные. 
    else 
    { 
      flGPRSState=16;
    }// Неуспешно подключено пробуем подключение еще раз.
   return;
  }

  if (strstr(my_rx_buffer,"+CIPSTATUS:")) // Информация о состоянии к серверу для wi-fi
  {
     if (strstr(my_rx_buffer,":0,"))  
    { 
      // состояние Ожидание ОК
      flGPRSState=167; 
    }// Успешно подключено к серверу, можно отправлять данные. 
    else 
    { 
      flGPRSState=160;
    }// Неуспешно подключено пробуем подключение еще раз.
   return;
  }


  #ifndef UDP_PROTOKOL
  if (strstr(my_rx_buffer,"+TCPSEND:")) // Информация о состоянии к серверу - отправка ошибка, проверяем коннект к серверу и пробуем еще.
  #else
  if (strstr(my_rx_buffer,"+UDPSEND:")) // Информация о состоянии к серверу - отправка ошибка, проверяем коннект к серверу и пробуем еще.
  #endif
  {
    
    if (strstr(my_rx_buffer,":Error")) 
    {
      // Если ошибка, тогда идем на проверку связи
      flGPRSState=30; 
    }
    else
    {
        // Если состояние - сообщение отправлено, решаем что делать дальше
        if (flGPRSState==21) 
        { 
          if (flNeedRefrServer) 
            {  // Если это разовая отправка
              // Постоим в режиме ожидания, не разывая соединения, ожидая возможной команыд от серверка
              timeWaitGPRS=10;
              timeGPRS=5;
              flNeedRefrServer=0; // Снмаем флаг необходимости отправки сведений на сервер
              timeRefrServer=PeriodRefrServer;// Взводим таймер на следующую отправку
            }   
          else
            flGPRSState=25;  // Если отправка не разовая, тогда переводим в режим следующей отправки.
        }
    }
    return;
  }


  if (strstr(my_rx_buffer,"SEND OK")) // информация отправлена.
  {
    
        // Если состояние - сообщение отправлено, решаем что делать дальше
        if (flGPRSState==165) 
        { 
          if (flNeedRefrServer) 
            {  // Если это разовая отправка
              // Постоим в режиме ожидания, не разывая соединения, ожидая возможной команыд от серверка
              timeWaitGPRS=10;
              timeGPRS=5;
              flNeedRefrServer=0; // Снмаем флаг необходимости отправки сведений на сервер
              timeRefrServer=PeriodRefrServer;// Взводим таймер на следующую отправку
            }   
          else
            flGPRSState=162;  // Если отправка не разовая, тогда переводим в режим следующей отправки.
        }
    return;
  }



    #ifndef UDP_PROTOKOL
    if (strstr(my_rx_buffer,"+TCPRECV:")) // Информация о состоянии к серверу
    #else
    if (strstr(my_rx_buffer,"+UDPRECV:")) // Информация о состоянии к серверу
    #endif
    {
      // Команда состоит из двух частей первая - это символ команды, вторая это номер по порядку
      // например U1 - нажата кнопка вверх, номер команды 1
      cmdGPRS=my_rx_buffer[strlen(my_rx_buffer)-2];
      
      cmdGPRSIsp=my_rx_buffer[strlen(my_rx_buffer)-1]; // э

      #ifdef TESTGSM
          Serial.print("Get cmd keys:");
          Serial.print(cmdGPRS);
          Serial.println(cmdGPRSIsp);
      #endif
      
      if (cmdGPRS>'A')
      {
        
        if (cmdGPRS=='T') StateMachine=1;        // Старт процесса
        if (cmdGPRS=='P') StateMachine=100;     // Стоп процесса
        if (cmdGPRS=='E') flNeedRefrServer=1;     // Если подана команда прекратить постоянный обмен,то включаем режим разовой отправки, после которой прекратится постоянный обмен.
                    else  flNeedRefrServer=0;     // Иначе отключаем режим разовой отправки.
        ScanKbd();
        timeGPRS=120;

      }
      
      // Пробуем закрывать сессию после получения данных
      return;
      
    }


    if (strstr(my_rx_buffer,"+IPD,")==my_rx_buffer) // получили команду от сервера (учитываем команду только если она начинается на +IPD,
    {
      // Команда состоит из двух частей первая - это символ команды, вторая это номер по порядку
      // например U1 - нажата кнопка вверх, номер команды 1
//      cmdGPRS=my_rx_buffer[strlen(my_rx_buffer)-2];
//      
//      cmdGPRSIsp=my_rx_buffer[strlen(my_rx_buffer)-1]; // э
      cmdGPRS=my_rx_buffer[7];
      cmdGPRSIsp=my_rx_buffer[8]; // э

      #ifdef TESTGSM
          Serial.print("Get cmd keys:");
          Serial.print(cmdGPRS);
          Serial.println(cmdGPRSIsp);
      #endif
      
      if (cmdGPRS>'A')
      {
        
        if (cmdGPRS=='T') StateMachine=1;        // Старт процесса
        if (cmdGPRS=='P') StateMachine=100;     // Стоп процесса
        if (cmdGPRS=='E') flNeedRefrServer=1;     // Если подана команда прекратить постоянный обмен,то включаем режим разовой отправки, после которой прекратится постоянный обмен.
                    else  flNeedRefrServer=0;     // Иначе отключаем режим разовой отправки.
        ScanKbd();
        if (flGPRSState==0) flGPRSState=14;
        timeGPRS=250;
      }
      return;
    }


   if(!strcmp("+PBREADY",my_rx_buffer) || !strcmp("MODEM:STARTUP",my_rx_buffer)) // Получили готовность модема, значит все сначала.
    {
      timeGPRS=240;
      stateAfterDelete=0;
       // Если сотояние не начальное,то переходим на инициализацию модема, после инициализации активируем отправку GPRS.
      if (flGPRSState>0)  stateAfterDelete=2;
      // Переходим на инициализацию модема.
      flGPRSState=98;
      return;
    }  

   if(!strcmp("invalid",my_rx_buffer)) // Пере-включение Wi-Fi
   {
      timeGPRS=120;
      stateAfterDelete=0;
      // Переходим на инициализацию GPRS.
      flGPRSState=98;
      return;
   }
    
    if(!strcmp("NO CARRIER",my_rx_buffer)) // Получили отказ связи
    {
      if (flGPRSState==41) // Если состояние - дозвон
      {
        flNeedCall=3;       // Ставив признак дозвона.
        flGPRSState=2;      // Переходим в состояние отправки информации по GPRS.  
      }
    return;
    }  

    if(!strcmp("ALREADY CONNECTED",my_rx_buffer)) // Получили готовность модема, значит все сначала.
    {
      // Переходим на отправку данных 
      if (flGPRSState>150)   
      { 
        stateAfterDelete=162; 
        flGPRSState=168;
      }
      return;
    }  


    
    #ifndef UDP_PROTOKOL
    if (strstr(my_rx_buffer,"+TCPCLOSE:")) // Сервер отключился.
    #else
    if (strstr(my_rx_buffer,"+UDPCLOSE:")) // Сервер отключился.
    #endif
    
    {
      // Если находимся в режиме закрытия
      if (flGPRSState!=0) flGPRSState=16; // 
      #ifdef TESTGSM
          Serial.print("State +TCP_UDP_CLOSE:");
          Serial.println((int)flGPRSState);
      #endif
      return;
      
    }

    if (strstr(my_rx_buffer,"CLOSED")) // Сервер отключился.
    {
      // Если находимся в режиме закрытия
      if (flGPRSState!=0) flGPRSState=160; // 
      #ifdef TESTGSM
          Serial.print("State WI-FI CLOSE:");
          Serial.println((int)flGPRSState);
      #endif
      return;
    }


  //+CLCC: 1,1,4,0,0,"+77777777777",145
    if (strstr(my_rx_buffer,"+CLCC:")) // получили номер телефона
    {
      flRing=0;
      if (strstr(my_rx_buffer,my_phone)) flRing=1;
      #ifdef TESTGSM
          Serial.print("State +CLCC:");
          Serial.println((int)flRing);
      #endif
      return;
      
  
    }

    if (strstr(my_rx_buffer,"+CPMS:")) // получили номер телефона
    {
      flRing=0;
      flGPRSState=62; // 
      #ifdef TESTGSM
          Serial.print("State +CMPS:");
          Serial.println((int)flRing);
      #endif
      return;
    }

    // Если ожидаем SMS, тогда это SMS
    if (flGPRSState==63 && strlen(my_rx_buffer)>40)
    {
        // Если сообщение больше, чем 60 символом, считмаем, что это SMS, полученный из телефона.
       #ifdef TESTGSM
        Serial.print("GETSMS:");
        Serial.println(my_rx_buffer);
       #endif 
        // Ищем в СМС свой номер телефона
        vid_sms=strstr(my_rx_buffer,pdu_phone);
  
        #ifdef TESTGSM
        Serial.print("addr:");
        Serial.println((int)vid_sms,HEX);
        #endif 
  
        if (vid_sms)
        {// Если найден номер телефона, тогда выделяем информацию из него
          // Информацией считаем последние четыре принятых байта (то есть один символ в формате Unicode)
          //   
          stateAfterDelete=0;
          vid_sms=my_rx_buffer+strlen(my_rx_buffer)-4;
         #ifdef TESTGSM
          Serial.print("INFOSMS:");
          Serial.println(vid_sms);
         #endif 
          // Если получили команду на включение котроллера, устанавливаем состояние машины в первое рабочее.
          if(!strcmp(sms_start,vid_sms)) 
          {
        #ifdef TESTGSM
            Serial.println("Start on SMS");
        #endif 
            StateMachine=1;
          }
          // Если получили команду на включение котроллера, устанавливаем состояние машины в конечное.
          if(!strcmp(sms_stop,vid_sms)) 
          {
            StateMachine=100;
            digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
            flAllOff=1;
  
            #ifdef TESTGSM
              Serial.println("Stop on SMS");
            #endif 
          }
          if(!strcmp(call_sms,vid_sms)) 
          {
            SMSOnCall=1;
            #ifdef TESTGSM
                Serial.println("Sms on call");
            #endif 
          }
          
          if(!strcmp(call_gprs,vid_sms)) 
          {
            SMSOnCall=0;
            #ifdef TESTGSM
                Serial.println("GPRS on call");
            #endif 
          }
          
        }
        // Переходим в состояние удаления SMS.
       flGPRSState=64;
      return;
    }

    if (!strcmp("RING",my_rx_buffer)) // Если входящий звонок, то определяем номер.
    {
      
      timeGPRS=60;  
      flGPRSState=4; // 
      #ifdef TESTGSM
          Serial.println("Get RING:");
      #endif
      return;

    }
    else
    {
        vid_sms=strstr(my_rx_buffer,"+CMTI:");
        if (vid_sms)
        {
            vid_sms+=6;
            vid_sms[5]=0;
            // Запоминаем номер СМС.
            strcpy(num_sms,&vid_sms[6]);
            // Выдаем запрос на чтение смс из того места, откуда она пришла.
              #ifdef TESTGSM
                  Serial.print("Get num SMS:");
                  Serial.print(num_sms); 
                  Serial.print("Vid SMS:");                  
                  Serial.println(vid_sms); 
              #endif
            timeGPRS=40;
            flGPRSState=60;
            flSMS=1;
        }
      return;
    }
}

void ProcessGSM()
{   
  char s_rx;

  while (Serial2.available() > 0)     
  {
    s_rx=Serial2.read();


#ifdef TESTGSM1
    // Дублируем информацию на Serial1
    Serial.write(s_rx);
#endif

    // Получили приглашение ко вводу SMS или GPRS или Wi-Fi
    if (s_rx=='>' && (flGPRSState==19 || flGPRSState==71 || flGPRSState==163 || flGPRSState==73)) 
    {
      #ifdef TESTGSM
          // Дублируем информацию на Serial1
          Serial.print("Get>:");
          Serial.println((int)flGPRSState);
      #endif
      #ifdef TESTGSM1
        Serial.println("Get>:");
        Serial.print("S:");
        Serial.println(Seconds);
      #endif
      
      if (flGPRSState!=73)  flGPRSState++;
                      else  flGPRSState=72;
      SendGPRS();
    }
    else
      if (s_rx==10)
      {
        my_rx_buffer[pos_rx_buffer-1]=0;
        pos_rx_buffer=0;
        AnalyseCommand();
        SendGPRS();
      }
      else
      {
        if (pos_rx_buffer>=MY_RX_BUFFER_SIZE) pos_rx_buffer=0;
        my_rx_buffer[pos_rx_buffer]=s_rx;
        pos_rx_buffer++;
      }
  }

  //если есть доступные данные
  // считываем байт
  // Если время выдачи информацции закончилось и информация выдана
  if (timeGPRS!=0)
  {
    // Если слишком долго ничего не получаем, и в буфере что-то есть, то анализируем то, что есть.
//    if (timeWaitGPRS==0 && pos_rx_buffer>0)
//    {
//      my_rx_buffer[pos_rx_buffer-1]=0;
//      AnalyseCommand();
//    }
    SendGPRS();
  }
  else
  {
    if  (flGPRSState==20 || flGPRSState==164)  SendGPRS();
    else
      if  (flGPRSState!=0) 
      {
        // Если соединение не закрыто, то закроем его, но только в том случае если модем прошел инициализацию.
        if (flGPRSState==98 || flGPRSState==99)
        {
          flGPRSState=98;
        }
        else
        {
          flGPRSState=90;
          if (FlToGSM==10 || FlToGSM==11) flGPRSState=180;
          SendGPRS();
        }
      } 
  }
  
  // Окончание анализа сотового  
}

void ScanDS18b20Slave()
{
  char maxDs;
  maxDs=0;
  // Первый этап-читаем датчики три раза.
  while (nPopr<3)
  {
    nPopr++;
    delay(1500);
    ds1820_devices=0;
    while(ds.search(ds1820_rom_codes[ds1820_devices])) {
      ds1820_devices++;
      if (ds1820_devices>=MAX_DS1820) break;
    }
    ds.reset_search();
    if (ds1820_devices>maxDs) maxDs=ds1820_devices;
    if (ds1820_devices>=3) return; 
  }
  // Если за время чтения количество датчиков больше 1,то читаем датчики еще 7 раз
  if (maxDs>0)
  {
     while (nPopr<10)
    {
      nPopr++;
      delay(1500);
      ds1820_devices=0;
      while(ds.search(ds1820_rom_codes[ds1820_devices])) {
        ds1820_devices++;
        if (ds1820_devices>=MAX_DS1820) break;
      }
      ds.reset_search();
      if (ds1820_devices>maxDs) maxDs=ds1820_devices;
      if (ds1820_devices>=3) return; 
    }    
  }
  
}

void RaspredPowerByPhase()
{
  #if NUM_PHASE==0
    if (PowerPhase[0]<=0) PowerPhase[0]=Power;
    // Корректировка мощности если установлено ограничение мощности по фазе
    if (UstPower>Power) UstPower=Power;
    UstPwrPH1=UstPower;  
  #else
  #endif
}

//    case 104: //Текущее состояние - первый недробный перегон
//    case 106: //Текущее состояние - второй дробный
//    case 107: //Текущее состояние - третий дробный
//    case 105: //Текущее состояние - отбор голов

// Процесс отбора голов (без дефлегматора)
void ProcessSimpleGlv()
{

  if (flAlarmUroven && StateMachine<4) 
  { // Переводим автомат в стадию завершения 
    StateMachine=4;        
    SecondsEnd=60;// Через одну минуту отключим воду
    SecTempPrev=Seconds;
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }
  
  // Проверяем температуру в ТСА
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
    UstPower=0;
  }
  if (flAlarmMPX5010) StateMachine=100; // Переводим в режим тревоги по датчику давления
  
  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    //         if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;

    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
    StateMachine=2;
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    flAllOff=0;

    if (BeepStateProcess) my_beep(BEEP_LONG);
  case 2: // Ожидание, пока прогреется термометр в дефлегматоре

    UstPower=Power; // Устанавливаем максимальную мощность для разгона
    // Подчиненные в разгон
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    

    // Если температура в дефлегматоре меньше 70 градусов и датчик дефлегматора в принципе подключен, тогда ждем, пока
    // дефлегматор прогреется, чтобы запустить воду в холодильник (это для некоторой экономии воды)
    if (temps[TEMP_DEFL]<TempDeflBegDistil && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_KUB]<-TempDeflBegDistil)) break;

    // Подаем питание на клапан подачи воды холодильника
    StateMachine=3;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
    // Ждем, пока температура в кубе не достигнет температуры начала дистилляции (если есть датчик дефлегматора, тогда этот пункт сразу
    // проскочится, поскольку темперетура в кубе не может быть меньше, чем температура в дефлегматоре
    //if (temps[TEMP_KUB]<TempDeflBegDistil) break;

    // Подаем питание на клапан подачи воды холодильника
    KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
    KlClose[KLP_HLD]=KlClose[KLP_VODA];
    KlState[KLP_HLD]=KlState[KLP_VODA];
    KlCount[KLP_HLD]=KlCount[KLP_VODA];
    if (BeepStateProcess) my_beep(BEEP_LONG);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
  case 3:// Работа в режиме отбра голов идет исключетельно по наполнению приемной емкости, но все-таки температуру
    // на всякий случай анализируем, ведь температура в кубе при отборе голов, в любом случае не может быть больше, чем температура окончания
    // третьей дробной перегонки

    UstPower=PowerGlvDistil; // Устанавливаем мощность отбора голов.
    // Клапан отбора голов устанавливаем, поскольку по совместительству этот режим может использоваться и для эпюрации
    // Устанавливаем клапан отбора хвостов и голов в соответвии с установленнным ШИМ
    //int tEndRectOtbGlv=854;     // Температура окончания отбора голов 85.4 С
    if (ProcChimOtbGlv>0)
        KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
    else
        KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
    
    KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    
    // Подчиненные в режим отбора голов
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);

    if (temps[TEMP_KUB]>=Temp3P)
    {  
      StateMachine=4;        
      SecondsEnd=1*60;// Через две минуты отключим воду
      SecTempPrev=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);

    }
    else    
      break;    
  case 4:// Ждем минуту для окончания подачи воды
    UstPower=0;
    // Подчиненные отключаем
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    SecondsEnd=SecondsEnd-(Seconds-SecTempPrev);
    SecTempPrev=Seconds;

    if (SecondsEnd<=0)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 101: // Превышение температуры в ТСА!!!
    my_beep(3*BEEP_LONG);
  case 100:// Конечное состояние автомата
    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    // Подчиненные отключаем
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    flAllOff=1;

    break;
  }
}

void ProcessSimpleDistill()
{

  if (flAlarmUroven && StateMachine<4) 
  { // Переводим автомат в стадию завершения 
    StateMachine=4;        
    SecondsEnd=2*60;// Через две минуты отключим воду
    SecTempPrev=Seconds;
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }

  // Проверяем температуру в ТСА
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
    UstPower=0;
  }

  if (flAlarmMPX5010) StateMachine=100; // Переводим в режим тревоги по датчику давления
 
  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    //          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;

    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    flAllOff=0;

  case 2: // Ожидание, пока прогреется термометр в дефлегматоре

    UstPower=Power; // Устанавливаем максимальную мощность для разгона
    // Подчиненные переводим в разгон
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);

    // Если температура в дефлегматоре меньше 70 градусов и датчик дефлегматора в принципе подключен, тогда ждем, пока
    // дефлегматор прогреется, чтобы запустить воду в холодильник (это для некоторой экономии воды)
    if (temps[TEMP_DEFL]<TempDeflBegDistil && ds1820_devices>1 || (TempDeflBegDistil<0 &&  temps[TEMP_KUB]<-TempDeflBegDistil) ) break;

    // Подаем питание на клапан подачи воды холодильника
    // для этого синхронизируем его с клапаном общей подачи воды (возможно, при этом будет меньше гидроударов, хотя я их и так не наблюдаю)
    KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
    KlClose[KLP_HLD]=KlClose[KLP_VODA];
    KlState[KLP_HLD]=KlState[KLP_VODA];
    KlCount[KLP_HLD]=KlCount[KLP_VODA];
    StateMachine=3;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);

  case 3:// Ждем, пока температура в кубе не превысит установленную
    UstPower=PowerDistil; // Устанавливаем максимальную мощность для разгона
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 1); // Переводим подчиненный в режим дистилляции
    digitalWrite(PIN_SLAVE_3, 0);

    if (IspReg==104) TekTemp=Temp1P; //Текущее состояние - первый недробный перегон
    if (IspReg==106) TekTemp=Temp2P; //Текущее состояние - второй дробный 
    if (IspReg==107) TekTemp=Temp3P; //Текущее состояние - третий дробный

    if (temps[TEMP_KUB]>=TekTemp)
    {  
      StateMachine=4;        
      SecondsEnd=2*60;// Через две минуты отключим воду
      SecTempPrev=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else    
      break;    
  case 4:// Ждем минуту для окончания подачи воды
    UstPower=0;
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    // Отключаем подчиненные
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0); 
    digitalWrite(PIN_SLAVE_3, 0);
    SecondsEnd=SecondsEnd-(Seconds-SecTempPrev);
    SecTempPrev=Seconds;

    if (SecondsEnd<=0)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 101: // Превышение температуры в ТСА!!!
    my_beep(3*BEEP_LONG);
  case 100:// Конечное состояние автомата
    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    if (BeepEndProcess) my_beep(BEEP_LONG);
    UstPower=0;
    // Отключаем подчиненные
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0); 
    digitalWrite(PIN_SLAVE_3, 0);
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    flAllOff=1;

    break;
  }

}
// процесс 129
// Процедура по-очереди открывает клапана на 5 секунд
void ScanKLP()
{
    int i;
    #if SIMPLED_VERSION<30
      if (SIMPLED_VERSION!=5 && SIMPLED_VERSION!=20)
      {
        lcd.clear();
        lcd.print("KLP NPG ON");
        for(i=0;i<10000;i++)
        { digitalWrite(PIN_KLP_BEG+KLP_NPG,KLP_HIGH);
          delay(1);
        }
        
        lcd.clear();
        lcd.print("KLP VODA ON");
        time3=10;
        for(i=0;i<10000;i++)
        { digitalWrite(PIN_KLP_BEG+KLP_VODA,KLP_HIGH);
          delay(1);
        }
        lcd.clear();
        lcd.print("KLP HLD DIST ON");
        for(i=0;i<10000;i++)
        { digitalWrite(PIN_KLP_BEG+KLP_HLD,KLP_HIGH);
          delay(1);
        }
      }
          
      lcd.clear();
      lcd.print("KLP DEFL ON");
      for(i=0;i<10000;i++)
      { digitalWrite(PIN_KLP_BEG+KLP_DEFL,KLP_HIGH);
        delay(1);
      }
  
      lcd.clear();
      lcd.print("KLP GLV HVST ON");
      for(i=0;i<10000;i++)
      { digitalWrite(PIN_KLP_BEG+KLP_GLV_HVS,KLP_HIGH);
        delay(1);
      }
  
      lcd.clear();
      lcd.print("KLP SR ON");
      for(i=0;i<10000;i++)
      { digitalWrite(PIN_KLP_BEG+KLP_SR,KLP_HIGH);
        delay(1);
      }
 #else
      lcd.clear();
      lcd.print("KLP DEFL ON");
      for(i=0;i<3000;i++)
      { digitalWrite(PIN_KLP_BEG+KLP_DEFL+KLP_DEFL,KLP_HIGH);
        delay(1);
      }
  
      lcd.clear();
      lcd.print("KLP GLV HVST ON");
      for(i=0;i<3000;i++)
      { digitalWrite(PIN_KLP_BEG+KLP_GLV_HVS+KLP_GLV_HVS,KLP_HIGH);
        delay(1);
      }
  
      lcd.clear();
      lcd.print("KLP SR ON");
      for(i=0;i<3000;i++)
      { digitalWrite(PIN_KLP_BEG+KLP_SR+KLP_SR,KLP_HIGH);
        delay(1);
      }
  #endif 
}

// процесс 129
void ProcessTestKLP()
{
  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    ProcChimSR=2;
    ScanKLP();
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    SecTempPrev=Seconds; // Запоминаем дату последнего изменения температуры
    if (SIMPLED_VERSION==0 || SIMPLED_VERSION>=5) ProcChimSR=95;
                       else ProcChimSR=4; // Если используется простая версия, то ШИМ всегда 50%, а процент ШИМ-Это номер тестируемого в данный момент клапана.
                                          // Чтобы при одновременном включении реле не происходил перезапуск контроллера. 
  case 2: // Клапана открыты
    SpeedNBK=5;
    timeNBK=20;   
    UstPower=500; 
    if (SIMPLED_VERSION==0 || SIMPLED_VERSION>=5) // В обычной версии тестируем клапана одновременно, с разным ШИМ
    {
      // Клапан общей подачи воды откырт
      KlOpen[KLP_VODA]=((timeChimRectOtbSR/10)*ProcChimSR)/10;
      KlClose[KLP_VODA]=timeChimRectOtbSR-KlOpen[KLP_SR];
      // Клапан НПГ откырт
      KlOpen[KLP_NPG]=KlOpen[KLP_VODA];
      KlClose[KLP_NPG]=KlClose[KLP_VODA];
      // Клапан Холодильника откырт
      KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
      KlClose[KLP_HLD]=KlClose[KLP_VODA];
      // Клапан дефлегматора откырт
      KlOpen[KLP_DEFL]=KlOpen[KLP_VODA];
      KlClose[KLP_DEFL]=KlClose[KLP_VODA];
      // Клапан отбора спирта откырт
      KlOpen[KLP_GLV_HVS]=KlOpen[KLP_VODA];
      KlClose[KLP_GLV_HVS]=KlClose[KLP_VODA];
      // Клапан отбора спирта откырт
      KlOpen[KLP_SR]=KlOpen[KLP_VODA];
      KlClose[KLP_SR]=KlClose[KLP_VODA];
      
    }
    else
    { // В упрощенной версии тестируем клапана по-очереди
      // ШИМ всегда 50%, в проценте ШИМ записан номер клапана.
      KlOpen[ProcChimSR]=300;
      KlClose[ProcChimSR]=300;
    }
    if (Seconds-SecTempPrev>600)  // Если с момента запуска прошло более 10 минут, то переходим в режим паузы - 1 минута.
    {
      if (BeepStateProcess) my_beep(BEEP_LONG);
      SecTempPrev=Seconds; // Запоминаем время паузы.
      StateMachine=3;
      // Закрываем все клапана.
      if (SIMPLED_VERSION==0 || SIMPLED_VERSION>=5)
      {
        ProcChimSR-=5;
        if (ProcChimSR<=0) ProcChimSR=100;
      }
      else
      { // Тестируем клапана.
        ProcChimSR++;
        if (ProcChimSR>4) ProcChimSR=2;
      }
      CloseAllKLP();
    }
    break;
  case 3: // Клапана закрыты, ждем 1 минуту
    if (Seconds-SecTempPrev>60)  // Если с момента запуска прошло более 20 минут, то переходим в режим паузы - 1 минута.
    {
      if (BeepStateProcess) my_beep(BEEP_LONG);
      StateMachine=2;
    }
    break;
  case 100:
    CloseAllKLP();
    break;
  }
}




void loop() 
{
  int i,j;
  unsigned long SqNapr=0, SqNaprPrev=0,FindPower,FindPowerKLP;
  int FindKtKLP;
  byte data[12];
  byte present = 0;
  unsigned int raw;
  int raw1;
  byte ds18b20CRC;
//  static char  FlFindPower;
//  static char  FlFindPowerKLP;
  char flErrDs18;
  //unsigned int tic;
  static char trig_on=true; //Признак того, что выдаем в данном цикле - 0 или 1

  #ifdef DEBUG
  my_debug();
  #endif

  //  static int PowerPrev=0;
  //  static int UstPowerPrev=0;
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  // print the number of seconds since reset:
  // Если есть необходимость произвести отображениеш  информации на дисплее и среднеквадратичное уже рассчитано.
  // И контроллен находится в режиме "лазанья" по меню, тогда отображаем информацию.
  // Или контроллер не находится в режиме ошибки.
  // Обработка уровней и НПГ
  if (flNeedReadAlarm && StateVolts!=1)
  {
//    #ifndef DEBUG
    ReadAlarm();
//    #endif
    flNeedReadAlarm=0;
  }


//#################################################################################################################
#ifdef MQTT_BROKER

  // Отправка оператвных данных
  SendOperData();

  // Получение команды от клиентов
  RecvSerial3();

#endif
//#################################################################################################################

  
  // Для обеспечения равномерности выборки делаем это в цикле подряд.
  while (StateVolts==1)
  {
    #ifdef DEBUG
      if (Serial.available()) return;
    #endif

    #if USE_ASC712==0 // если датчик тока не используем, тогда читаем значение и делим его на два
      if (!flCorrASC712)  resultU=(analogRead(PIN_READU)>>1); // читаем показания с pin А3 и делим их на два, для увеличения измеряемого выходного напряжения.
      else
      {
        if (StateVolts==1)
        {
          if (CorrectASC712==3)
          {
            resultU=analogRead(PIN_READI)-512; // читаем показания с pin А3 и делим их на два, для увеличения измеряемого выходного напряжения.
            if (resultU<0) resultU=-resultU;
          }
          else
          {
            resultU=analogRead(PIN_READI)-512;  // читаем показания с датчика тока.
            if (resultU<0) resultU=-resultU; 
          }
        }       
      }
      
    #else
      // Если используем датчик тока, тогда читаем значение, если оно больше 512, значит приводим его к 512;
      resultU=analogRead(PIN_READI)-512;  // читаем показания с датчика тока.
      if (resultU<0) resultU=-resultU;
    #endif                  
                      

//      data_adc[index_input]=resultU;  
      // Пробуем расчет квадратов на лету.
      SqNaprT=SqNaprT+(unsigned long) resultU* (unsigned long) resultU;
      #ifdef TESTRM
//        tic=(int) TCNT4L | (int) (TCNT4H<<8); // Расчитывааем текущее значение таймера
//        tic_adc[index_input]=tic;
      #endif
      if (index_input<MAX_INDEX_INPUT) index_input++;
      else StateVolts=0; // Если превышер размер массива, ставим флаг расчета заново.
      
  }

  if (NeedDisplaying && (StateVolts!=1 || IspReg>=240) && FlState==0)
  {
    DisplayData();
  }

  // Сканируем клавиатуру, если установлен флаг.
  if (flNeedScanKbd==1 )
  {
    ScanKbd();
    flNeedScanKbd=0;
  }

  if (flNeedTemp==1 && IspReg>=240)
  {
    flNeedAnalyse=1;
  }

  if (StateVolts!=1)                 
  {
    if (flNeedTemp==1)
    {
      flNoPhase=1; // временно убираем с фазового управления клапана, чтобы они не закрылись
      OpenKLP(); // открываем на всякий случай все клапана, которые необходимо открыть.
      if (StepOut==0)
      {
        ds.reset();
        ds.write(0xCC,1);         // start conversion, with parasite power on at the end
        ds.write(0x44,1);         // start conversion, with parasite power on at the end
        flNeedTemp=0;
        StepOut=1;
      }
      else
      {

        flErrDs18=0; // Сбрасываем флаг nошибки датчика ds18b20
        for (j=0;j<ds1820_devices;j++)
        {
          present=ds.reset();

#ifdef TIMING
          TCNT4H=0x00;
          TCNT4L=0x00;
#endif            

          // Здесь вообще засада, передача кода датчика занимает около 1600 тиков таймера, что гарантирует пропуск регуляции как минимум одного полупериода.
          // Возможно решение - посадить каждый датчик на отдельный ПИН, тогда код датчика передавать не нужно.
          ds.select(ds1820_rom_codes[j]);    

#ifdef TIMING
          unsigned int Tic2=(int) TCNT4L | (int) (TCNT4H<<8); // Расчитывааем текущее значение таймера
          sprintf(my_tx_buffer,"TicDs18Select=%i",Tic2);
          FlUsart=1;
#endif

          ds.write(0xBE);         // Read Scratchpad
          //          Serial.print(present,HEX);
          //          Serial.print(" ");
          //        Читаем пока только 2 байта из 9, остальные не трогаем.
          for ( i = 0; i < 9; i++) 
          {  
            data[i] = ds.read();
            //            Serial.print(data[i], HEX);
            //            Serial.print(" ");
          }
          //          Serial.print(" CRC=");
          ds18b20CRC=OneWire::crc8(data, 8);

          if (ds18b20CRC!=data[8]) //  Ошибка чтения, записываем признак ошибки 
          {
#ifndef DEBUG          
            if (!flErrDs18) // Если ранее в этом сеансе не было ошибок чтения датчика, то прибавляем количество ошибок чтения на единицу
            {             
              flErrDs18=1; 
              CntErrDs18++;
              NumErrDs18=j+1; // Запоминаем номер сглючившего датчика.
              if (CntErrDs18>20) DispDopInfo=4;  // Устанавливаем признак вывода информации об ошибке если ошибок больше 4
              if (FlToUSART) GetStateSerialErr();
            }
#endif
          }
          else // Если контрольная сумма совпадает, то считаем, что все нормально.
          {
            raw = (data[1] << 8) | data[0];

            raw1=raw*5/8;
            // Температуру равной 850 считаем ошибочной.
            if (raw1==850 || raw1>1200)
            {
              // Если температура скаканула больше чем на 10 градусов, то считаем ошибкой.
              if (abs(raw1-temps[j])>100 && temps[j]!=0 || raw1>1200)
              {
                if (!flErrDs18) // Если ранее в этом сеансе не было ошибок чтения датчика, то прибавляем количество ошибок чтения на единицу
                {             
                  flErrDs18=1; 
                  CntErrDs18++;
                  NumErrDs18=j+1; // Запоминаем номер сглючившего датчика.
                  if (CntErrDs18>20) DispDopInfo=4;  // Устанавливаем признак вывода информации об ошибке если ошибок больше 4
                  if (FlToUSART) GetStateSerialErr();
                }
              }
            }
            else 
            
            {
              #ifndef DEBUG          
                temps[j]=raw1;
                // Если мы сейчас не находимся в режиме ввода поправок к датчикам, то прибавляем к температуре значение поправки
    
                if (FlState!=300)
                {
                  temps[j]=temps[j]+(int) ds1820_popr[j];
                }
                // Запоминаем максимальную температуру датчиков
                if (Maxtemps[j]<temps[j]) Maxtemps[j]=temps[j];
              #endif
            }
          }
        } 

        StepOut=0;
        flNeedTemp=0;
        
#ifdef DEBUG
        flErrDs18=0;
#endif

        flNoPhase=0; // восстанавливаем фазовое управление клапанами.
        
        if (!flErrDs18) // Если не было ошибок, то сбрасываем счетчики.
        {
          flNeedAnalyse=1;   // Все необходимые данные прочитаны устанавливаем флаг того, что нужно анализировать состояние выполнения процесса.
          CntErrDs18=0;      
          NumErrDs18=0;
          // Сбрасываем признак вывода информации об ошибке
          if (DispDopInfo==4) DispDopInfo=0;
        }
      }

    }
  }

  if (CntErrDs18>MAX_ERR_DS18)
  {
    flNeedAnalyse=1;
#ifndef TEST
    IspReg=251;      // Переводим контроллер в режим ошибки датчиков.
#endif
  }



  if (flNeedAnalyse==1)
  {

    trig_on=!trig_on;         
    digitalWrite(PIN_RST_WDT,trig_on); // Выдаем признак работы для внешнего контроля
    
    
    #ifdef USE_SLAVE
     SlaveON=digitalRead(PIN_SLAVE_3)*8+digitalRead(PIN_SLAVE_2)*4+digitalRead(PIN_SLAVE_1)*2+digitalRead(PIN_SLAVE_0);
    #endif
    
#ifdef TEST
    Serial.println("BEGIN PROSSES");
#endif      

    switch(IspReg)
    {
    case 102: //Текущее состояние - термостат
      //UstPower=0;
      ProcessTermostat();
      break;
    case 103: //Текущее состояние - регулятор мощности
      #ifndef USE_SLAVE
      if (StateMachine<100) UstPower=UstPowerReg;
      else UstPower=0;
      #else
        if (SlaveON==0) UstPower=0;
        if (SlaveON==1) UstPower=Power;
        if (SlaveON==2) UstPower=PowerGlvDistil;
        if (SlaveON==3) UstPower=PowerRect;
        if (SlaveON==4) UstPower=PowerDistil;
      #endif
      // Если стоит признак авто-детектирования ТЭНа, то мощность устанавливаем максимально возможную.
      if (flAutoDetectPowerTEN==1 && StateMachine<100) UstPower=9999;
      break;
    case 108:
    case 113:
      ProcessRazvarZerno();
      break;
    case 114:
      ProcessHLDZatorByChiller();
      break;
    case 115:
      ProcessTimerMaxPower();
      break;
    case 116:
      ProcessBeerCloneBrau();
      break;
    case 117:
      ProcessDistillFractional();
      break;
    case 118: //Текущее состояние - фракционная ректификация
    case 109: //Текущее состояние - ректификация
      ProcessRectif();
      break;
    case 104: //Текущее состояние - первый недробный перегон
    case 106: //Текущее состояние - второй дробный
    case 107: //Текущее состояние - третий дробный
      ProcessSimpleDistill();
      break;
    case 105: //Текущее состояние - отбор голов
      ProcessSimpleGlv();
      break;
    case 110:
      ProcessDistilDefl();
      break;
    case 111:
      ProcessNDRF();
      break;
    case 112:
      ProcessNBK();
      break;
    case 129:
      ProcessTestKLP();
      break;
    case 130:
      if (StateMachine==1) // Если состояние автомата - не запущено
      {
        //            flRing=1;
        KlOpen[KLP_VODA]=900;
        KlClose[KLP_VODA]=100;
        KlOpen[KLP_HLD]=KlOpen[KLP_VODA];
        KlClose[KLP_HLD]=KlClose[KLP_VODA];
        KlState[KLP_HLD]=KlState[KLP_VODA];
        KlCount[KLP_HLD]=KlCount[KLP_VODA];
        KlOpen[KLP_DEFL]=KlOpen[KLP_VODA];
        KlClose[KLP_DEFL]=KlClose[KLP_VODA];
        KlState[KLP_DEFL]=KlState[KLP_VODA];
        KlCount[KLP_DEFL]=KlCount[KLP_VODA];
        KlOpen[KLP_SR]=KlOpen[KLP_VODA];
        KlClose[KLP_SR]=KlClose[KLP_VODA];
        KlState[KLP_SR]=KlState[KLP_VODA];
        KlCount[KLP_SR]=KlCount[KLP_VODA];
        digitalWrite(PIN_TERMOSTAT_ON,HIGH);
        digitalWrite(PIN_TERMOSTAT_OFF,HIGH);
        //             digitalWrite(30,HIGH);
        //             digitalWrite(31,HIGH);
        //             digitalWrite(32,HIGH);

        StateMachine=100; // Переводим процесс в состояние выполнения
      }
      break; 

    case 248:
    case 249:                 
    case 250:     
    case 251:
    case 252:
    case 253:
    case 254:
      #if NUM_PHASE>1
          KlOpen[KLP_HLD]=100;
          KlClose[KLP_HLD]=0;
          digitalWrite(PIN_KLP_BEG+KLP_HLD,HIGH);
      #endif
      StateMachine=100;
      if (flNeedCall==0) flNeedCall=1;
      if (flNeedCall==1)
      {
         // Звонок на телефон в обязательном порядке.
        flGPRSState=40; // Активируем звонок если в этом есть необходимость
        flNeedRefrServer=1;
        timeGPRS=60;
        
      }
      else
      {
        // Если в данный момент не дозавниваемся, то отправляем SMS или GPRS
        if (flNeedCall!=2)
          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
      }
      // Тревога либо ошибки протечка воды, отключаем воду, отключаем мощность.
      UstPower=0;
      CloseAllKLP();
      // Пищим, независимо от настроек.
#ifndef TEST
      my_beep(BEEP_LONG*5);
#endif

      digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
      digitalWrite(PIN_TRIAC,LOW);
      digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
      flAllOff=1;
      break;

    }

    if (FlToUSART) GetStateSerial();

#ifdef TEST
    Serial.println("END PROSSES");
#endif      

    flNeedAnalyse=0;
    RaspredPowerByPhase();
  }

if (StateVolts!=1)
{
  #if NUM_PHASE<2
  //  анализ сотового   
  ProcessGSM();
  #else
  ProcessSlavePhase();
  #endif
  
}
  //  анализ отладки

  if (FlUsart==1)
  {
    // Признак того, что надо выдать информацию в ком-порт (есл ее вдруг невозможно сразу выдать, например, из прерывания).
    Serial.println(my_tx_buffer);
    FlUsart=0;    
  }       


  // Начало расчета среднеквадратичного
  
  if (StateVolts==2)        
  {
   
    #ifdef TESTRM
        Serial.println("BEGIN SQNAPR");
        TCNT4H=0x00;
        TCNT4L=0x00;
    #endif      

    StateVolts=3;

    // поскольку тут идет преобразование и умножение 32 разрядных чисел, делаем это по необходимости, а не каждый раз (хотя какая разница, тут и таких умножений дохрена)
    SqNapr=(long) 220*220*20/PowerPhase[0]; // Рассчитали сопротивление ТЭНа, умноженное на 20
    R_TEN20=  SqNapr;

    #ifdef TESTRM
        Serial.print("UstPower=");
        Serial.println(UstPower);
        Serial.print("Power=");
        Serial.println(Power);
        Serial.print("NaprPeregrev=");
        Serial.println(NaprPeregrev);
        Serial.print("MaxVoltsOut=");
        Serial.println(MaxVoltsOut);
        Serial.print("R_TEN20=");
        Serial.println(R_TEN20);
    #endif

   
   SqNapr=0;
   SqNaprPrev=0;
//   MaxVolts=0;
   
   #ifdef TESTRM
        TCNT4H=0x00;
        TCNT4L=0x00;
   #endif

    
    #ifdef TESTRM
        Serial.print("SqNapr=");
        Serial.println(SqNaprT);
        //Считаем напряжение в сети исходя из среднеквадратичного, средневыпрямленного, максимального
        Serial.print("RMS/AVG/MAX");
        Serial.write(9);
        Serial.print((long) my_sqrt(SqNaprT/index_input));
        Serial.write(9);
        Serial.println((long)MaxVolts*707/1000);
    #endif

    
    #if USE_ASC712==0
      if (!flCorrASC712) // Расчет напряжения
      {
 
        TekPower=SqNaprT/index_input;     // Определили квадрат напряжения
        
        // Но фактически мощность подаваемая на ТЭНы меньше из-за падения на симисторе.
        
        

        MaxVoltsOut=my_sqrt(TekPower);  // Рассчитываем среднеквадратичное
        
        #ifdef TESTRM
            Serial.print("index_input=");
            Serial.println(index_input);
            Serial.print("TekPower2=");
            Serial.println(TekPower);
        #endif
        
        
        TekPower=(TekPower*20)/R_TEN20; // Определили фактическую мощность в зависимости от сопротивления ТЭНа
        
        #ifdef TESTRM
            Serial.print("TekPower2=");
            Serial.println(TekPower);
        #endif
        
        

        TekPowerKLP=(unsigned long) NaprPeregrev*NaprPeregrev;

        #ifdef TESTRM
            Serial.print("UstPower=");
            Serial.println(UstPwrPH1);
            Serial.print("FactPower=");
            Serial.println(FactPower);
            Serial.print("MaxVoltsOut=");
            Serial.println(MaxVoltsOut);
            Serial.print("TekPowerKLP=");
            Serial.println(TekPowerKLP);
        #endif
        
      
        if (FactPower>15 && CorrectASC712>=2 && !(UstPwrPH1>=PowerPhase[0])) // Если фактическая мощность рассчитана и мы не находимся в режиме подачи полного напряжения на ТЭНы
        {
          // Это формулы, для ПИД-регулировани, предоставленные коллегой m16 c homedistiller.ru
          //          ;******************************************************************************
          //          ;*
          //          ;*	new_err = Ps_r - Pv			;вычисляем новую ошибку
          //          ;*
          //          ;*
          //          ;*	It = It + ki_r * new_err	        ;считаем интегральную составляющую
          //          ;*
          //          ;*		if ( It < 0.0 ) It = 0.0        ;если интегральная отрицательна то присваиваем ей ноль
          //          ;*		else  if( It > Sc_r ) It = Sc_r	;если интегральная превысит установленный предел то присваиваем значение предела 
          //          ;*
          //          ;*	Dt =  kd_r * ( new_err - old_err )	;считаем дифференциальную
          //          ;*
          //          ;*	Ut = ( new_err*10 + It + Dt ) /  Sc_r	;вычисляем % от максимальной мощности
          //          ;*
          //          ;*		if ( Ut < 0.0 ) Ut = 0.0
          //          ;*		else if ( Ut > 100 ) Ut = 100
          //          ;*
          //          ;*	old_err = new_err                       ;присваиваем значению новой ошибке предыдущую
          //          ;******************************************************************************
          //          где
          //          Ps_r - мощность установленная
          //          Pv   - мощность измеренная
          //          new_err- новая ошибка рассогласования
          //          old_err- предыдущая ошибка рассогласования
          //          Sc_r   - максимальное накопление интегральной составляющей
          //          It     - интегральная
          //          ki_r   - коэф. интегральной
          //          Dt     - дифю составляющая 
          //          kd_r   - коэф. дифференциальной
          //          Ut     - % от макс. мощности
          // Считаем ошибку рассоглавования.

          // Ошибку рассогласования учитвыаем, если она не более 1/8 мощности ТЭНа.
          NewErr=((long)UstPwrPH1-(long)FactPower)*200/(long)Power;
          if (NewErr<-30) NewErr=-30;
          else
            if (NewErr>30) NewErr=30;
            
          Dt=NewErr-OldErr;
          It=It+NewErr;
          // Накопленную ошибку не более 1/2 мощности ТЭНа.
          if (It<-100) It=-100;
            else if (It>100) It=100;
            
          OldErrOut=OldErr; // Старая ошибка для отображения на экране
          OldErr=NewErr; // Запоминаем старую ошибку.
//          if (UstPower>FactPower) It=It+5;
//          else  
//            if (UstPower<FactPower) It=It-5;
        }
        else
        {
          // Если мощность не рассчитана, то считаем, что находимся в режиме отладки и не подключили ТЭНы В этом случае просто делаем фазовое регулирование исходя из
          // входного напряжения, не учитывая фактическую мощность, рассчитанную исходя из датчика тока на ТЭНах
          NewErr=0;
          OldErr=0;
          OldErrOut=0;
          It=0;
          Dt=0;
        }

    
        // Пока без коэффиццииенов
        FindKt=(long)(UstPwrPH1)*200/ TekPower;        // Установили коэффициент поиска мощности для регулятора мощности как соотношение необходимой мощности к имеющейся в данный момент.
        FindKt=FindKt+It;                              // Прибавляем коэффициент.
        
        
        

    
                                                      
        if (FindKt>200 || It==-100) 
        {
          CntErrPower++; 
          if (UstPwrPH1<PowerPhase[0] && UstPwrPH1>15 && CntErrPower>100 && !flNoAlarmLowPower) 
          {   if (FindKt>200)  { DispDopInfo=3; FindKt=200; } // Выводим на экран информацию о низком напряжении но только в том случае, когда не в режиме разгона и установленная мощность больше 15
              else DispDopInfo=6;              // и в том случае когда не установлено, что не нужно выводить предупреждение о низком уровне напряжения
              #if NUM_PHASE>1
                if (FindKt>=200)
                {
                 KlOpen[KLP_HLD]=100; // Раз в секунду
                 KlClose[KLP_HLD]=100;
                }
                else
                {
                 KlOpen[KLP_HLD]=15; // Два раза в секунду
                 KlClose[KLP_HLD]=15;
                }
                
              #endif
          }
       }
        else
          {
            CntErrPower=0; 
            #if NUM_PHASE>1
               KlOpen[KLP_HLD]=0;
               KlClose[KLP_HLD]=50;
            #endif
           
            if (DispDopInfo==3 || DispDopInfo==6) 
            { 
              DispDopInfo=0; // Убираем информацию о низком напряжении, если она была ранее установлена
            }
            
          }
          
          #if NUM_PHASE>1
            if (IspReg>=240)
             {
               KlOpen[KLP_HLD]=100;
               KlClose[KLP_HLD]=0;
             }
          #endif
          
 
         // В клапанах тоже учитываем поправку мощности
        //TekPowerKLP=(TekPowerKLP+It)*1000/( (long) MaxVoltsOut* (long)MaxVoltsOut);
        //Пока убираем поправку мощности в клапанах, возможно из-за этого "пробивает" симистор.
        TekPowerKLP=(TekPowerKLP)*1000/( (long) MaxVoltsOut* (long)MaxVoltsOut);
        
        FindKtKLP=TekPowerKLP;// Установили коэффициент поиска мощности для клапанов
        
        // Проверка минимума и максимума
        if (FindKtKLP>1000) FindKtKLP=1000;
        if (FindKtKLP<5) FindKtKLP=5;

        #ifdef TESTRM
            Serial.print("NewErr=");
            Serial.println(NewErr);
            Serial.print("OldErr=");
            Serial.println(OldErrOut);
            Serial.print("FindKt=");
            Serial.println(FindKt);
            Serial.print("FindKtNoErr=");
            Serial.println((long)(UstPower)*1000/ TekPower);
            Serial.print("NaprPeregrev=");
            Serial.println(NaprPeregrev);
            Serial.print("FindKtKLP=");
            Serial.println(FindKtKLP);
            Serial.print("SqNaprT=");
            Serial.println(SqNapr);
            Serial.print("SqNaprPrev=");
            Serial.println(SqNaprPrev);
        #endif

        if (FindKt<1) FindKt=1;
        if (FindKt>200) FindKt=200;

        // Поскольку таблица разбита по мощности равномерно в пределах 5/1000 доли, то нет необходимости высчитывать и искать по таблице, достаточно просто поделить.
        TimeOpenTriac=tableS10[FindKt - 1];
        TimeOpenTriac*=10;
        // Учиываем предельно допустимый угол открытия триака
        if (FindKtKLP<1000)
        {
          TimeOpenKLP  =tableS10[FindKtKLP/5 - 1];
          // Защита от "дребезга клапана" на малых углах открытия симистора.
          if (TimeOpenKLP<70) TimeOpenKLP=0; 
          TimeOpenKLP*=10;
          
        }
        else
         TimeOpenKLP=0;
       
        
        if (TimeOpenKLP<=10) TimeOpenKLP=9999; // Если не нашли нужную мощность, то устанавливаем максимальную.
        if (flAutoDetectPowerTEN) TimeOpenTriac=9999;
      }
     else // Это расчет фактического тока и фактической мощности
      {

        if (CorrectASC712==3) // Расчет по напряжениию
        {
             
        }
        else
        {
          // Здесь определили квадрат тока умноженный на 10.
          SqNaprPrev=SqNaprT*100/ ((unsigned long) index_input*SENSITIVE_ASC712);
          // Определили ток, умноженный на 10 (для вывода на экран).
          MaxIOut=my_sqrt((long)SqNaprPrev*100/SENSITIVE_ASC712);
          if (flAutoDetectPowerTEN)
          {
              FindPower=(long) SqNaprPrev*100/(SENSITIVE_ASC712);
  //            Serial.print("I^2=");
  //            Serial.println(my_sqrt(FindPower));
              TekPower=(unsigned long) 220*22*my_sqrt(FindPower)/MaxVoltsOut;
              Power=(int)(TekPower/10)*10;
              PowerPhase[0]=Power;
          }
          
          
          FindPower=(long) SqNaprPrev*R_TEN20/ (20*SENSITIVE_ASC712);
          FactPower=FindPower;
         
          #ifdef TESTRM
            Serial.print("SqNaprT=");
            Serial.println(SqNaprT);
            Serial.print("index_input=");
            Serial.println(index_input);
            Serial.print("index_input*SENSITIVE_ASC712=");
            Serial.println((unsigned long)index_input*SENSITIVE_ASC712);
            Serial.print("SqNaprPrev=");
            Serial.println(SqNaprPrev);
            Serial.print("FindPower=");
            Serial.println(FindPower);
            Serial.print("FactPower=");
            Serial.println(FactPower);
          #endif
        } 
       }   

    #endif
    
    
//   Сумму квадратов измерений определили, она хранится в переменной SqNapr.
//   Теперь надо перевести это в фактический ток.
  #if USE_ASC712==1

      // Рассчитали сумму, пересчитанную в фактические значения тока.
      SqNaprPrev=SqNaprT*100/ ( (unsigned long) index_input*SENSITIVE_ASC712);
      #ifdef TESTRM
      Serial.print("SqNapr=");
      Serial.println(SqNapr);
      Serial.print("SqNaprPrev=");
      Serial.println(SqNaprPrev);
      #endif
      
      

      FindPower=(long) SqNaprPrev*R_TEN20/ (20*SENSITIVE_ASC712);
      FactPower=FindPower;
      
      MaxVoltsOut=my_sqrt(FindPower*R_TEN20/20);

      // Теперь зная мощность рассчитываем процент для брезенхема
      b_value[0]=(long) UstPower*b_size[0]/(long) FindPower;

      if (b_value[0]>b_size[0] && UstPower<Power) DispDopInfo=3; // Выводим на экран информацию о низком напряжении если находимся не в режиме разгона.
      else
        if (DispDopInfo==3) DispDopInfo=0; // Убираем информацию о низком напряжении, если она была ранее установлена
      
      
      #ifdef TESTRM
        Serial.print("MaxVoltsOut=");
        Serial.println(MaxVoltsOut);
        Serial.print("FindPower=");
        Serial.println(FindPower);
        Serial.print("FactPower=");
        Serial.println(FactPower);
        Serial.print("b_value[0]=");
        Serial.println((int)b_value[0]);
      #endif
  #endif

#ifdef TESTRM

    Serial.println("VOLTS");
    Serial.print("j=");
    Serial.println(j);
    Serial.print("SqNapr=");
    Serial.println(SqNapr);
    Serial.print("SqNaprPrev=");
    Serial.println(SqNaprPrev);
    Serial.print("FindPower=");
    Serial.println(FindPower);
    Serial.print("b_value=");
    Serial.println(b_value[0]);
    Serial.print("OpenTriac=");
    Serial.println(TimeOpenTriac);
    Serial.print("OpenKLP=");
    Serial.println(TimeOpenKLP);

    Serial.print("COUNT=");
    Serial.println(index_input);

//    for(i=0;i<index_input;i++)
//    {            
//      // Сканируем клавиатуру, если установлен флаг.
//      if (flNeedScanKbd==1 )
//      {
//        ScanKbd();
//        flNeedScanKbd=0;
//      }
//
//      Serial.print(tic_adc[i]);
//      Serial.write(9);
//      Serial.println(data_adc[i]);
//    }                                                    
    Serial.print("TICZERO=");
    Serial.write(9);
    Serial.println(TicZero);
    
//    Serial.println("TABLET");
//    for(i=0;i<MAX_TABLE_T;i++)
//    {            
//      // Сканируем клавиатуру, если установлен флаг.
//      if (flNeedScanKbd==1 )
//      {
//        ScanKbd();
//        flNeedScanKbd=0;
//      }
//
//      Serial.print(tableK[i]);
//      Serial.write(9);
//      Serial.println(tableT[i]);
//    }                                                    

    Serial.println("ENDVOLTS");


#endif      

    StateVolts=4;
  }
  
}


//#################################################################################################################
#ifdef MQTT_BROKER

void printSerial3(char *s) {
  
  // Ждем освобождения буфера отправки
  while (Serial3.availableForWrite() < strlen(s)+1)
    yield();

  Serial3.write(s, strlen(s)+1);
     
}


// Отправка оперативных данных
void SendOperData() {

  char buf[30];


  // этот блок требуется для возможности расчета времени работы на текущем этапе и количества СТОПОВ
  // если произошла смена этапа
  if (CurrentStage != StateMachine) {

    //  если перешли в СТОП
    if (StateMachine == 5) {
            
      // увеличим счетчик стопов
      CntStop++;
    }
    
    // Непосредственная отправка оперативных данных
    SendOper();
    
    // сохраним новый этап
    CurrentStage = StateMachine;
      
    // сохраним время перехода на новый этап
    TimeStage = Seconds;
        
  }
  
  // если наступило время отправки и еще не отправляли в эту секунду и разрешено обновление в online режиме
  // ИЛИ требуется обновление по нажатию кнопки и дисплей успел обновиться (задержка 1 сек)
  if ( (Seconds % 2 == 0 && Seconds != SecPrevBuf && FlagRefreshOnline) || (NeedRefresh && Seconds > SecCmdDisp+1) ) {
   
    // Вывод LCD экрана
    printSerial3(lcd_mqtt_buf1);
    printSerial3(lcd_mqtt_buf2);

    // состояние клапана отбора
    sprintf(buf, GetPMEM(buf_pmem,fmt_klp), KlState[KLP_GLV_HVS] | KlState[KLP_SR]);
    printSerial3(buf);

    // сохраним время отправки
    SecPrevBuf = Seconds;

    // сбросим признак обновления
    NeedRefresh = false;

  }
  
  // если наступило время отправки и еще не отправляли в эту секунду
  if (Seconds % 30 == 0 && Seconds != SecPrevOper) {
    
    // Непосредственная отправка оперативных данных
    SendOper();   
    
    // сохраним время отправки
    SecPrevOper = Seconds;
  }  
}

// Непосредственная отправка оперативных данных
void SendOper() {

  char buf[30];

    // этап процесса
    switch (StateMachine) {
      case 0:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_0));    break;
      case 1:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_1));    break;
      case 2:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_2));    break;
      case 3:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_3));    break;
      case 4:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_4));    break;
      case 5:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_5));    break;
      case 6:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_6));    break;
      case 7:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_7));    break;
      case 8:   sprintf(buf, GetPMEM(buf_pmem,fmt_stage_8));    break;
      case 100: sprintf(buf, GetPMEM(buf_pmem,fmt_stage_100));  break;
      case 101:
      case 102: sprintf(buf, GetPMEM(buf_pmem,fmt_stage_101));  break;
      default:  sprintf(buf, GetPMEM(buf_pmem,fmt_stage_def));  break;
    }
    printSerial3(buf);
    
    // температура в кубе
    sprintf(buf, GetPMEM(buf_pmem,fmt_t_kub), temps[TEMP_KUB]/10,temps[TEMP_KUB]%10);
    printSerial3(buf);
               
    // температура в колонне
    sprintf(buf, GetPMEM(buf_pmem,fmt_t_col), temps[TEMP_RK20]/10,temps[TEMP_RK20]%10);
    printSerial3(buf);
                
    // температура в TSA
    sprintf(buf, GetPMEM(buf_pmem,fmt_t_tsa), temps[TEMP_TSA]/10,temps[TEMP_TSA]%10);
    printSerial3(buf);
            
    // время с момента включения системы/начала процесса
    sprintf(buf, GetPMEM(buf_pmem,fmt_work_time), hour,minute,second);
    printSerial3(buf);

    // текущий % отбора спирта
    sprintf(buf, GetPMEM(buf_pmem,fmt_ProcShimSR), KLP_GLV_HVS != KLP_SR
                                                  // условие с двумя клапанами отбора
                                                  ? (KlOpen[KLP_GLV_HVS] != 0 ? KlOpen[KLP_GLV_HVS]/(timeChimRectOtbGlv/100) : KlOpen[KLP_SR]/(timeChimRectOtbSR/100))
                                                  // условие с одним клапаном отбора
                                                  : (StateMachine !=6 ? KlOpen[KLP_GLV_HVS]/(timeChimRectOtbGlv/100) : KlOpen[KLP_SR]/(timeChimRectOtbSR/100)) );
    printSerial3(buf);

    // температура стабилизации колонны
    sprintf(buf, GetPMEM(buf_pmem,fmt_tStabSR), tStabSR/10, tStabSR%10);
    printSerial3(buf);

    // количество СТОПОВ
    sprintf(buf, GetPMEM(buf_pmem,fmt_CntStop), CntStop);
    printSerial3(buf);

    // время текущего этапа
    sprintf(buf, GetPMEM(buf_pmem,fmt_TimeStage), (int)((Seconds-TimeStage)/3600), (int)((Seconds-TimeStage)%3600)/60, (int)((Seconds-TimeStage)%3600)%60);
    printSerial3(buf);
      
}  


// Отправка статических данных (настроек)
void SendStaticData() {

  char buf[30];
  
      //### отправляем данные ###
    
      // звук при нажатии клавиш
      sprintf(buf, GetPMEM(buf_pmem,fmt_BeepKeyPress), BeepKeyPress);
      printSerial3(buf);
      
      // звук при окончании процесса
      sprintf(buf, GetPMEM(buf_pmem,fmt_BeepEndProc), BeepEndProcess);
      printSerial3(buf);
      
      // звук при переходе между этапами процесса
      sprintf(buf, GetPMEM(buf_pmem,fmt_BeepStateProc), BeepStateProcess);
      printSerial3(buf);
              
      // флаг вывода в USART
      sprintf(buf, GetPMEM(buf_pmem,fmt_FlToUSART), FlToUSART);
      printSerial3(buf);

      // температура окончания режима разгона
      sprintf(buf, GetPMEM(buf_pmem,fmt_tEndRectRazgon), tEndRectRazgon/10, abs(tEndRectRazgon)%10);
      printSerial3(buf);

      // температура окончания отбора голов
      sprintf(buf, GetPMEM(buf_pmem,fmt_tEndRectOtbGlv), tEndRectOtbGlv/10, tEndRectOtbGlv%10);
      printSerial3(buf);

      // температура окончания отбора спирта
      sprintf(buf, GetPMEM(buf_pmem,fmt_tEndRectOtbSR), tEndRectOtbSR/10, tEndRectOtbSR%10);
      printSerial3(buf);

      // температура окончания ректификации
      sprintf(buf, GetPMEM(buf_pmem,fmt_tEndRect), tEndRect/10, tEndRect%10);
      printSerial3(buf);

      // дельта ректификации
      sprintf(buf, GetPMEM(buf_pmem,fmt_tDeltaRect), tDeltaRect/10, tDeltaRect%10);
      printSerial3(buf);      
      
      // ШИМ отбора голов
      sprintf(buf, GetPMEM(buf_pmem,fmt_ShimGlv), timeChimRectOtbGlv/100);
      printSerial3(buf);

      // % ШИМ отбора голов
      sprintf(buf, GetPMEM(buf_pmem,fmt_ProcShimGlv), ProcChimOtbGlv);
      printSerial3(buf);

      // ШИМ отбора спирта
      sprintf(buf, GetPMEM(buf_pmem,fmt_ShimSR), timeChimRectOtbSR/100);
      printSerial3(buf);

      // минимальный % отбора спирта
      sprintf(buf, GetPMEM(buf_pmem,fmt_MinProcShimSR), minProcChimOtbSR);
      printSerial3(buf);

      // начальный % отбора спирта
      sprintf(buf, GetPMEM(buf_pmem,fmt_BegProcShimSR), begProcChimOtbSR);
      printSerial3(buf);

      // режим отбора голов
      sprintf(buf, GetPMEM(buf_pmem,fmt_ProvodSR), UrovenProvodimostSR);
      printSerial3(buf);

}


// Получение данных от ESP
void RecvSerial3() {

  char ch;
  String pub_topic="";
  String val="";
  int pos_delim;
  int ret;
  
    while (Serial3.available()) {

      ch = (char)Serial3.read();

      if (ch == '\r') continue;

      if (ch == '\n') {

        // Проверим наличие разделителя в строке
        pos_delim = RecvBufSerial.indexOf("=");

        // Если разделитель присутствует
        if (pos_delim != -1 ) {
          
          // Выделим свойство и значение
          pub_topic = RecvBufSerial.substring(0,pos_delim);
          val = RecvBufSerial.substring(pos_delim+1);

          // удалить десятичные разделители
          val.replace(".","");  val.replace(",","");
                  
          // звук кнопок
          if ( pub_topic == GetPMEM(buf_pmem,val_BeepKeyPress) )
            BeepKeyPress = (char)val.toInt();
          
          // звук окончания процесса
          else if ( pub_topic == GetPMEM(buf_pmem,val_BeepEndProc) )
            BeepEndProcess = (char)val.toInt();
          
          // звук смены этапа
          else if ( pub_topic == GetPMEM(buf_pmem,val_BeepStateProc) )
            BeepStateProcess = (char)val.toInt();    
          
          // вывод в UART
          else if ( pub_topic == GetPMEM(buf_pmem,val_FlToUSART) )
            FlToUSART = (char)val.toInt();
          
          // нажатие кнопки дисплея
          else if ( pub_topic == GetPMEM(buf_pmem,val_key_u) || pub_topic == GetPMEM(buf_pmem,val_key_d) ||
                    pub_topic == GetPMEM(buf_pmem,val_key_l) || pub_topic == GetPMEM(buf_pmem,val_key_r) ||
                    pub_topic == GetPMEM(buf_pmem,val_key_s) ) {
            cmdGPRS = (char)val.charAt(0);
            // сохранить время получения команды
            SecCmdDisp = Seconds;
            // установить признак необходимости обновления показаний дисплея
            NeedRefresh = true;
          }
          
          // настройка обновления дисплея
          else if ( pub_topic == GetPMEM(buf_pmem,val_online) ) {
            if (val.toInt() == 0) FlagRefreshOnline = false;
            else FlagRefreshOnline = true;
          }
          
          // сохранение в EEPROM
          else if ( pub_topic == GetPMEM(buf_pmem,val_save) )
            writeEEPROM();
          
          // считывание значений настроек
          else if ( pub_topic == GetPMEM(buf_pmem,val_refresh) )  
            SendStaticData();

          // температура окончания разгона
          else if ( pub_topic == GetPMEM(buf_pmem,val_tEndRectRazgon) )  {
            ret = val.toInt();
            if ( ret != 0 and abs(ret) <= 999 )
              tEndRectRazgon = ret;
          }  

          // температура окончания отбора голов
          else if ( pub_topic == GetPMEM(buf_pmem,val_tEndRectOtbGlv) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 999 )
              tEndRectOtbGlv = ret;
          }

          // температура окончания отбора тела
          else if ( pub_topic == GetPMEM(buf_pmem,val_tEndRectOtbSR) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 999 )
              tEndRectOtbSR = ret;
          }

          // температура окончания ректификации
          else if ( pub_topic == GetPMEM(buf_pmem,val_tEndRect) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 999 )
              tEndRect = ret;
          }

          // ШИМ отбора голов
          else if ( pub_topic == GetPMEM(buf_pmem,val_ShimGlv) )  {
            ret = val.toInt()*100;
            if ( ret != 0 and ret >= 0 )
              timeChimRectOtbGlv = ret;
          }

          // % ШИМ отбора голов
          else if ( pub_topic == GetPMEM(buf_pmem,val_ProcShimGlv) )  {
            ret = val.toInt();
            if ( ret != 0 and abs(ret) <= 100 )
              ProcChimOtbGlv = ret;
          }

          // ШИМ отбора тела
          else if ( pub_topic == GetPMEM(buf_pmem,val_ShimSR) )  {
            ret = val.toInt()*100;
            if ( ret != 0 and ret >= 0 )
              timeChimRectOtbSR = ret;
          }

          // минимальный % ШИМ отбора тела
          else if ( pub_topic == GetPMEM(buf_pmem,val_MinProcShimSR) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              minProcChimOtbSR = ret;
          }

          // начальный % ШИМ отбора тела
          else if ( pub_topic == GetPMEM(buf_pmem,val_BegProcShimSR) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              begProcChimOtbSR = ret;
          }

          // дельта ректификации
          else if ( pub_topic == GetPMEM(buf_pmem,val_tDeltaRect) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              tDeltaRect = ret;
          }

          // настройка окончания отбора голов
          else if ( pub_topic == GetPMEM(buf_pmem,val_ProvodSR) )  {
            ret = val.toInt();
            if ( ret != 0 )
              UrovenProvodimostSR = ret;
          }

          // установка текущего процесса
          else if ( pub_topic == GetPMEM(buf_pmem,val_IspReg) )  {
            ret = val.toInt();
            if ( ret != 0 )
              IspReg = ret;
          }

          // установка текущего этапа процесса
          else if ( pub_topic == GetPMEM(buf_pmem,val_StateMachine) )  {
            ret = val.toInt();
            if ( ret != 0 )
              StateMachine = ret;
          }          
          
        }    

        RecvBufSerial = "";  
        break;
      }
        
      RecvBufSerial += ch;
    }
    
}


#endif
//#################################################################################################################


